#
#  @file   BaseStateMachine.py
#  @author Francis Moreau
#  date    Mon May 5 2008
#  
#  Description:
#
#  Type your description here
#
#  Copyright (C) 2008 Paraxip Technologies.
#  All Rights Reserved.
#

from SipCallEngineExtending import PySipCallEngineSM
from netborderPyInfra import TraceScope
import ObjectWithId
import ObjectProxyCounted
import weakref
from time import time

# Force declaration of class attributes in __init__ method
__pychecker__ = "initattr"

#
# Derived from C++ base class
#

class BaseStateMachine(PySipCallEngineSM,
                       ObjectWithId.ObjectWithId,
                       ObjectProxyCounted.ObjectProxyCounted):
  def __init__(self):
    PySipCallEngineSM.__init__(self)
    ObjectWithId.ObjectWithId.__init__(self)
    ObjectProxyCounted.ObjectProxyCounted.__init__(self)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "BaseStateMachine.__init__")

    self._ApplicationWR = None
    self._GlobalContextWR = None

    self._startTime = None

  def initializeWithObjAndCallId(self, in_refApplication, in_refGlobalContext,
                 in_szObjectId, in_szCallId):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "BaseStateMachine." 
      "initializeWithObjAndCallId")

    self._ApplicationWR = weakref.ref(in_refApplication)
    self._GlobalContextWR = weakref.ref(in_refGlobalContext)

    self._startTime = time()

    ObjectWithId.ObjectWithId.setId(self, in_szObjectId)

    # Start the CallLogger
    self.getLogger().callStart(in_szCallId)
    
    return True
  
  def reset(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "BaseStateMachine.reset")
    
    # Stop the CallLogger
    self.getLogger().callEnd()

    ObjectWithId.ObjectWithId.resetId(self)
    ObjectProxyCounted.ObjectProxyCounted.resetProxyCount(self)

    self._startTime = None

    self._GlobalContextWR = None
    self._ApplicationWR = None

  def _get_Application(self):
    return self._ApplicationWR()
  application = property(_get_Application, None)

  def _get_GlobalContext(self):
    return self._GlobalContextWR()
  globalContext = property(_get_GlobalContext, None)

  def getDurationStr(self):
    if (self._startTime is None):
      return "Unknown"

    return str(time() - self._startTime)

## !!MIRROR FILE, DO NOT MODIFY!!
