#
#  @file   ObjectPool.py
#  @author Francis Moreau
#  date    Wed Apr 16 2008
#  
#  Description:
#
#  Type your description here
#
#  Copyright (C) 2008 Paraxip Technologies.
#  All Rights Reserved.
#

from netborderPyInfra import Logger, TraceScope

# Force declaration of class attributes in __init__ method
__pychecker__ = "initattr"

fileScopeLogger = Logger("netborder.ObjectPool.py")

class ObjectPool:
  def __init__(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "ObjectPool.__init__")
    
    # _dictOfStacks is a dictionary containing the list of available state
    # machines. The key is the class name, and the value is a stack containing
    # the available instance (the stack is a list used as a stack)
    self._dictOfStacks = {}
  
  # Input parameter: the Python class wanted
  # Output parameter: an instance of class
  # Triggers exceptions if class cannot be imported, is not defined or 
  #   construction fails
  def get(self, in_Class):
    stackObj = None
    if (in_Class.__name__ in self._dictOfStacks):
      stackObj = self._dictOfStacks[in_Class.__name__]
    
    if ((stackObj is not None) and (len(stackObj) > 0)):
      # Remove an instance from the stack and return it
      return stackObj.pop()
    else:
      # Create a new instance of the class and return it
      return in_Class()
  
  def takeBack(self, in_Instance):
    stackObj = None
    if (in_Instance.__class__.__name__ in self._dictOfStacks):
      stackObj = self._dictOfStacks[in_Instance.__class__.__name__]
    
    if (stackObj is not None):
      stackObj.append(in_Instance)
    else:
      stackObj = []
      stackObj.append(in_Instance)
      self._dictOfStacks[in_Instance.__class__.__name__] = stackObj

  def clear(self):
    self._dictOfStacks.clear()
## !!MIRROR FILE, DO NOT MODIFY!!
