#
#  @file   SipCallEngineEvent.py
#  @author Francis Moreau
#  date    Thu May 29 2007
#  
#  Description:
#
#  Type your description here
#
#  Copyright (C) 2007 Paraxip Technologies.
#  All Rights Reserved.
#

from SipCallEngineExtending import PySipCallEngineEvent
from netborderPyInfra import TraceScope, Logger

fileScopeLogger = Logger("netborder.SipCallEngineEvent.py")

class SipCallEngineEvent(PySipCallEngineEvent):
  def __init__(self, in_szDestObjectId):
    PySipCallEngineEvent.__init__(self, in_szDestObjectId)
  
  def __str__(self):
    r_str = "%s with members: %s" % (self.__class__.__name__, self.__dict__)
    return r_str
  
#
# CallToDialog events
#

class CallToDialogEvent(SipCallEngineEvent):
  def __init__(self, in_szDestObjectId):
    SipCallEngineEvent.__init__(self, in_szDestObjectId)

class CallToDialogRingEvent(CallToDialogEvent):
  def __init__(self, in_szDestObjectId):
    CallToDialogEvent.__init__(self, in_szDestObjectId)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "CallToDialogRingEvent.__init__")

class CallToDialogSendProvisionalEvent(CallToDialogEvent):
  def __init__(self, in_szDestObjectId, in_statusCode, in_reasonText):
    CallToDialogEvent.__init__(self, in_szDestObjectId)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "CallToDialogSendProvisionalEvent.__init__")

    self._statusCode = in_statusCode
    self._reasonText = in_reasonText

  def getStatusCode(self):
    return self._statusCode

  def getReasonText(self):
    return self._reasonText

class CallToDialogEarlyMediaEvent(CallToDialogEvent):
  def __init__(self, in_szDestObjectId, in_sdpContents):
    CallToDialogEvent.__init__(self, in_szDestObjectId)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "CallToDialogEarlyMediaEvent.__init__")
    
    self._sdpContents = in_sdpContents
  
  def getSdpContents(self):
    return self._sdpContents

class CallToDialogProvideSdpAnswerEvent(CallToDialogEvent):
  def __init__(self, in_szDestObjectId, in_sdpAnswer):
    CallToDialogEvent.__init__(self, in_szDestObjectId)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger,
      "CallToDialogProvideSdpAnswerEvent.__init__")
    
    self._sdpAnswer = in_sdpAnswer
  
  def getSdpAnswer(self):
    return self._sdpAnswer

class CallToDialogProvideSdpOfferEvent(CallToDialogEvent):
  def __init__(self, in_szDestObjectId, in_sdpOffer):
    CallToDialogEvent.__init__(self, in_szDestObjectId)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "CallToDialogProvideSdpOfferEvent.__init__")
    
    self._sdpOffer = in_sdpOffer
  
  def getSdpOffer(self):
    return self._sdpOffer

class CallToDialogAnswerEvent(CallToDialogEvent):
  def __init__(self, in_szDestObjectId, in_sdpContents, in_bSdpIsOffer,
               in_sipMessage):
    CallToDialogEvent.__init__(self, in_szDestObjectId)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "CallToDialogAnswerEvent.__init__")
    
    self._sdpContents = in_sdpContents
    self._bSdpIsOffer = in_bSdpIsOffer
    self._sipMessage = in_sipMessage
  
  def getSdpContents(self):
    return self._sdpContents
  
  def getSdpIsOffer(self):
    return self._bSdpIsOffer
  
  def getSipMessage(self):
    return self._sipMessage

class CallToDialogRedirectEvent(CallToDialogEvent):
  def __init__(self, in_szDestObjectId, in_contacts):
    CallToDialogEvent.__init__(self, in_szDestObjectId)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "CallToDialogRedirectEvent.__init__")
    
    self._contacts = in_contacts
  
  def getContacts(self):
    return self._contacts

class CallToDialogRejectEvent(CallToDialogEvent):
  def __init__(self,
               in_szDestObjectId,
               in_statusCode,
               in_sipMessage = None,
               in_szReason = None):
    CallToDialogEvent.__init__(self, in_szDestObjectId)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "CallToDialogRejectEvent.__init__")
    
    self._statusCode = in_statusCode 
    self._sipMessage = in_sipMessage
    self._strReason  = in_szReason
  
  def getStatusCode(self):
    return self._statusCode
  
  def getSipMessage(self):
    return self._sipMessage

  def getReason(self):
    return self._strReason

class CallToDialogReInviteEvent(CallToDialogEvent):
  def __init__(self, in_szDestObjectId, in_sdpOffer):
    CallToDialogEvent.__init__(self, in_szDestObjectId)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "CallToDialogReInviteEvent.__init__")
    
    self._sdpOffer = in_sdpOffer
  
  # sdp Offer can be None (re-INVITE w/o SDP
  def getSdpOffer(self):
    return self._sdpOffer

class CallToDialogSendInfoMsgEvent(CallToDialogEvent):
  def __init__(self, in_szDestObjectId, strMimeType, strMimeSubType, strBodyText):
    CallToDialogEvent.__init__(self, in_szDestObjectId)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "CallToDialogSendInfoMsgEvent.__init__")
    
    self._strMimeType = strMimeType
    self._strMimeSubType = strMimeSubType
    self._strBodyText = strBodyText 
  
  def getMimeType(self):
    return self._strMimeType
  
  def getMimeSubType(self):
    return self._strMimeSubType
  
  def getBodyText(self):
    return self._strBodyText

class CallToDialogHangupEvent(CallToDialogEvent):
  def __init__(self, in_szDestObjectId):
    CallToDialogEvent.__init__(self, in_szDestObjectId)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "CallToDialogHangupEvent.__init__")

#
# DialogToCall base class event
#

class DialogToCallEvent(SipCallEngineEvent):
  def __init__(self, in_szDialogId, in_szDestObjectId):
    SipCallEngineEvent.__init__(self, in_szDestObjectId)
    
    self._strDialogId = in_szDialogId
  
  def getDialogId(self):
    return self._strDialogId

#
# Common In/Out DialogToCall events
#

class DialogToCallSdpOfferedEvent(DialogToCallEvent):
  def __init__(self, in_szDialogId, in_szDestObjectId, in_sipMessage, in_sdpContents):
    DialogToCallEvent.__init__(self, in_szDialogId, in_szDestObjectId)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "DialogToCallSdpOfferedEvent.__init__")

    self._sipMessage = in_sipMessage
    self._sdpContents = in_sdpContents
  
  def getSipMessage(self):
    return self._sipMessage
  
  def getSdpContents(self):
    return self._sdpContents

class DialogToCallSdpAnsweredEvent(DialogToCallEvent):
  def __init__(self, in_szDialogId, in_szDestObjectId, in_sipMessage, in_sdpContents):
    DialogToCallEvent.__init__(self, in_szDialogId, in_szDestObjectId)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "DialogToCallSdpAnsweredEvent.__init__")
    
    self._sipMessage = in_sipMessage
    self._sdpContents = in_sdpContents
  
  def getSipMessage(self):
    return self._sipMessage
  
  def getSdpContents(self):
    return self._sdpContents

class DialogToCallFailedEvent(DialogToCallEvent):
  def __init__(self, in_szDialogId, in_szDestObjectId, in_sipMessage):
    DialogToCallEvent.__init__(self, in_szDialogId, in_szDestObjectId)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "DialogToCallFailedEvent.__init__")
    
    self._sipMessage = in_sipMessage
  
  def getSipMessage(self):
    return self._sipMessage

class DialogToCallInfoEvent(DialogToCallEvent):
  def __init__(self, in_szDialogId, in_szDestObjectId, in_sipMessage):
    DialogToCallEvent.__init__(self, in_szDialogId, in_szDestObjectId)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "DialogToCallInfoEvent.__init__")
    
    self._sipMessage = in_sipMessage
  
  def getSipMessage(self):
    return self._sipMessage

class DialogToCallInfoSentSucceededEvent(DialogToCallEvent):
  def __init__(self, in_szDialogId, in_szDestObjectId, in_sipMessage):
    DialogToCallEvent.__init__(self, in_szDialogId, in_szDestObjectId)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "DialogToCallInfoSentSucceededEvent.__init__")
    
    self._sipMessage = in_sipMessage
  
  def getSipMessage(self):
    return self._sipMessage

class DialogToCallInfoSentFailedEvent(DialogToCallEvent):
  def __init__(self, in_szDialogId, in_szDestObjectId, in_sipMessage):
    DialogToCallEvent.__init__(self, in_szDialogId, in_szDestObjectId)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "DialogToCallInfoSentFailedEvent.__init__")
    
    self._sipMessage = in_sipMessage
  
  def getSipMessage(self):
    return self._sipMessage

class DialogToCallAckNotReceivedEvent(DialogToCallEvent):
  def __init__(self, in_szDialogId, in_szDestObjectId):
    DialogToCallEvent.__init__(self, in_szDialogId, in_szDestObjectId)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "DialogToCallAckNotReceivedEvent.__init__")

class DialogToCallSdpOfferRequiredEvent(DialogToCallEvent):
  def __init__(self, in_szDialogId, in_szDestObjectId, in_sipMessage):
    DialogToCallEvent.__init__(self, in_szDialogId, in_szDestObjectId)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "DialogToCallSdpOfferRequiredEvent.__init__")

    self._sipMessage = in_sipMessage
  
  def getSipMessage(self):
    return self._sipMessage

  # INVITE or re-INVITE w/o SDP, provide a getSdpContents() so the event can 
  #   be treated the same as an SdpOfferedEvent (INVITE/re-INVITE with SDP)
  def getSdpContents(self):
    return None

class DialogToCallNotifyEvent(DialogToCallEvent):
  def __init__(self, in_szDialogId, in_szDestObjectId, in_eventName):
    DialogToCallEvent.__init__(self, in_szDialogId, in_szDestObjectId)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "DialogToCallNotifyEvent.__init__")

    self._eventName = in_eventName

  def getEventName(self):
    return self._eventName

class DialogToCallTerminatedEvent(DialogToCallEvent):
  def __init__(self, in_szDialogId, in_szDestObjectId):
    DialogToCallEvent.__init__(self, in_szDialogId, in_szDestObjectId)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "DialogToCallTerminatedEvent.__init__")

class DialogToCallConnectedEvent(DialogToCallEvent):
  def __init__(self, in_szDialogId, in_szDestObjectId, in_sipMessage):
    DialogToCallEvent.__init__(self, in_szDialogId, in_szDestObjectId)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "DialogToCallConnectedEvent.__init__")

    self._sipMessage = in_sipMessage
  
  def getSipMessage(self):
    return self._sipMessage

#
# InDialogToCall events
#

class InDialogToCallCalledEvent(DialogToCallEvent):
  def __init__(self, in_szDialogId, in_szDestObjectId, in_sipMessage):
    DialogToCallEvent.__init__(self, in_szDialogId, in_szDestObjectId)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "InDialogToCallCalledEvent.__init__")
    
    self._sipMessage = in_sipMessage
  
  def getSipMessage(self):
    return self._sipMessage

#
# OutDialogToCall events
#

class OutDialogToCallCallingEvent(DialogToCallEvent):
  def __init__(self, in_szDialogId, in_szDestObjectId, in_sipMessage):
    DialogToCallEvent.__init__(self, in_szDialogId, in_szDestObjectId)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "OutDialogToCallCallingEvent.__init__")
    
    self._sipMessage = in_sipMessage
  
  def getSipMessage(self):
    return self._sipMessage

class OutDialogToCallEarlyMediaEvent(DialogToCallEvent):
  def __init__(self, in_szDialogId, in_szDestObjectId,
               in_sipMessage, in_sdpContents):
    DialogToCallEvent.__init__(self, in_szDialogId, in_szDestObjectId)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "OutDialogToCallEarlyMediaEvent.__init__")
    
    self._sipMessage = in_sipMessage
    self._sdpContents = in_sdpContents
  
  def getSipMessage(self):
    return self._sipMessage
  
  def getSdpContents(self):
    return self._sdpContents

class OutDialogToCallProvisionalEvent(DialogToCallEvent):
  def __init__(self, in_szDialogId, in_szDestObjectId, in_sipMessage):
    DialogToCallEvent.__init__(self, in_szDialogId, in_szDestObjectId)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "OutDialogToCallProvisionalEvent.__init__")
    
    self._sipMessage = in_sipMessage
  
  def getSipMessage(self):
    return self._sipMessage

#
# Timeout events
#

class TimeoutBaseEvent(SipCallEngineEvent):
  def __init__(self, in_szDestObjectId):
    SipCallEngineEvent.__init__(self, in_szDestObjectId)
    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "TimeoutBaseEvent.__init__")
    
    self._timerId = None
  
  def _get_timerId(self):
    return self._timerId
  def _set_timerId(self, in_timerId):
    self._timerId = in_timerId
  timerId = property(_get_timerId, _set_timerId)

class TimeoutEvent(TimeoutBaseEvent):
  def __init__(self, in_szDestObjectId = None):
    TimeoutBaseEvent.__init__(self, in_szDestObjectId)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "TimeoutEvent.__init__")

class TimeoutCancelledEvent(TimeoutBaseEvent):
  def __init__(self, in_szDestObjectId = None):
    TimeoutBaseEvent.__init__(self, in_szDestObjectId)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "TimeoutCancelledEvent.__init__")

## !!MIRROR FILE, DO NOT MODIFY!!
