#
#  @file   SipCallEngineSMCall.py
#  @author Francis Moreau
#  date    Thu Apr 26 2007
#  
#  Description:
#
#  Type your description here
#
#  Copyright (C) 2007 Paraxip Technologies.
#  All Rights Reserved.
#

"""
 Call state machine
"""

from netborderPyInfra import TraceScope
import paraxip.infra.PyLoggingFunctions as PyLoggingFunctions
import paraxip.infra.CallStack as CallStack

from BaseStateMachine import BaseStateMachine

import copy

# Force declaration of class attributes in __init__ method
__pychecker__ = "initattr"

class SipCallEngineSMCall(BaseStateMachine):

  #@override
  def __init__(self):
    BaseStateMachine.__init__(self)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "SipCallEngineSMCall.__init__")
    
    self._dialogInDict = {}
    self._dialogOutDict = {}
    # Holds both types of dialogs
    self._dialogDict = {}

    self._strCallId = None

    self._refGlobalContext = None

    self.config = None
    
    # Holds names and initial values of members that must be re-initialized to 
    #   their original initial value each time reset() is called
    # Keys: name of member, value: initial value
    self._perCallMembers = {}

  def configure(self, in_dictConfig):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "SipCallEngineSMCall.configure")

    self.config = in_dictConfig
    return True
  
  def initialize(self, in_refApplication, in_refGlobalContext, in_szCallId):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "SipCallEngineSMCall.initialize")
    
    # Using call ID as both Object ID and Call ID
    BaseStateMachine.initializeWithObjAndCallId(self,
                                in_refApplication,
                                in_refGlobalContext,
                                in_szCallId,
                                in_szCallId)

    self._strCallId = in_szCallId

    self._refGlobalContext = in_refGlobalContext
    
    return True
  
  #@override
  def reset(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "SipCallEngineSMCall.reset")
    
    BaseStateMachine.reset(self)

    self._strCallId = None
    
    self._dialogInDict.clear()
    self._dialogOutDict.clear()
    self._dialogDict.clear()

    # Reset per call members to their initial values
    for (memberName, memberInitialValue) in self._perCallMembers.items():
      setattr(self, memberName, copy.deepcopy(memberInitialValue))

  def declarePerCallMembers(self, *memberNames):
    """
    Declare members that must be re-initialized to their initial value
    each time reset() is called.
    
    @input memberNames: sequence of names of members (attributes of self), for 
                        which then current value is stored as initial value to 
                        restore when reset() is called. 
    
    @note Can be called multiple times for a Call class, as long as the member 
          names are unique.
    """
    for memberName in memberNames: 
      initialValue = getattr(self, memberName)
      # Must not already hold a value for this name
      assert not memberName in self._perCallMembers
      self._perCallMembers[memberName] = copy.deepcopy(initialValue)

  def getCallId(self):
    return self._strCallId

  def hasDialogs(self):
    if (len(self._dialogDict) > 0):
      return True
    return False

  def isInDialog(self, in_szDialogId):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "SipCallEngineSMCall.isInDialog")

    return in_szDialogId in self._dialogInDict

  def isOutDialog(self, in_szDialogId):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "SipCallEngineSMCall.isOutDialog")

    return in_szDialogId in self._dialogOutDict

  def addInDialog(self, in_szDialogId, in_sipCallEngineSMIn):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "SipCallEngineSMCall.addInDialog")

    self._dialogInDict[in_szDialogId] = in_sipCallEngineSMIn
    self._dialogDict[in_szDialogId] = in_sipCallEngineSMIn
  
  def addOutDialog(self, in_szDialogId, in_sipCallEngineSMOut):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "SipCallEngineSMCall.addOutDialog")

    self._dialogOutDict[in_szDialogId] = in_sipCallEngineSMOut
    self._dialogDict[in_szDialogId] = in_sipCallEngineSMOut
  
  def removeInDialog(self, in_szDialogId):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "SipCallEngineSMCall.removeInDialog")
    
    del self._dialogInDict[in_szDialogId]
    del self._dialogDict[in_szDialogId]
  
  def removeOutDialog(self, in_szDialogId):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "SipCallEngineSMCall.removeOutDialog")
    
    del self._dialogOutDict[in_szDialogId]
    del self._dialogDict[in_szDialogId]
  
  def getInDialog(self, in_szDialogId):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "SipCallEngineSMCall.getInDialog")
    
    # If the value is not in the dictionary:
    #   A dialog could have been removed while there are still events in the
    #   queue from the dialog to its call. This means the dialog has been
    #   released by the SIP stack and the last event from this dialog is
    #   DialogToCallTerminatedEvent and it's pending in the queue.
    return self._dialogInDict.get(in_szDialogId)

  
  def getOutDialog(self, in_szDialogId):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "SipCallEngineSMCall.getOutDialog")
    
    # If the value is not in the dictionary:
    #   A dialog could have been removed while there are still events in the
    #   queue from the dialog to its call. This means the dialog has been
    #   released by the SIP stack and the last event from this dialog is
    #   DialogToCallTerminatedEvent and it's pending in the queue.
    return self._dialogOutDict.get(in_szDialogId)
  
  def getDialog(self, in_szDialogId):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "SipCallEngineSMCall.getDialog")
    
    # If the value is not in the dictionary:
    #   A dialog could have been removed while there are still events in the
    #   queue from the dialog to its call. This means the dialog has been
    #   released by the SIP stack and the last event from this dialog is
    #   DialogToCallTerminatedEvent and it's pending in the queue.
    return self._dialogDict.get(in_szDialogId)
  
  # In dialog action functions
  
  def ringInDialog(self, in_szDialogId):
    dialogSM = self.getInDialog(in_szDialogId)
    if (dialogSM is not None):
      if (not dialogSM.ring()):
        (strMethodName, strFileName, argClass) = CallStack.getCallerInfo()
        PyLoggingFunctions.log_error(self.getLogger(),
                  "ringInDialog failed in state: %s::%s",
                  strFileName, strMethodName)
        return False
    else:
      PyLoggingFunctions.log_info(self.getLogger(),
               "ringInDialog failed because dialog id %s "
               "does not exist or has been released", in_szDialogId)
    return True
  
  def sendProvisionalToInDialog(self, in_szDialogId, in_statusCode, 
                              in_reasonText=None):
    dialogSM = self.getInDialog(in_szDialogId)
    if (dialogSM is not None):
      if (not dialogSM.sendProvisional(in_statusCode, in_reasonText)):
        (strMethodName, strFileName, argClass) = CallStack.getCallerInfo()
        PyLoggingFunctions.log_error(self.getLogger(),
                  "sendProvisionalToInDialog failed in state: %s::%s",
                  strFileName, strMethodName)
        return False
    else:
      PyLoggingFunctions.log_info(self.getLogger(),
               "sendProvisionalToInDialog failed because dialog id %s "
               "does not exist or has been released", in_szDialogId)
    return True

  def sendEarlyMediaToInDialog(self, in_szDialogId, in_sdpContents):
    dialogSM = self.getInDialog(in_szDialogId)
    if (dialogSM is not None):
      if (not dialogSM.sendEarlyMedia(in_sdpContents)):
        (strMethodName, strFileName, argClass) = CallStack.getCallerInfo()
        PyLoggingFunctions.log_error(self.getLogger(),
                  "sendEarlyMediaToInDialog failed in state: %s::%s",
                  strFileName, strMethodName)
        return False
    else:
      PyLoggingFunctions.log_info(self.getLogger(),
               "sendEarlyMediaToInDialog failed because dialog id %s "
               "does not exist or has been released", in_szDialogId)
    return True
  
  def answerInDialog(self, in_szDialogId, in_sdpContents,
                     in_bSdpIsOffer=False, in_sipMessage=None):
    dialogSM = self.getInDialog(in_szDialogId)
    if (dialogSM is not None):
      if (not dialogSM.answer(in_sdpContents, in_bSdpIsOffer, in_sipMessage)):
        (strMethodName, strFileName, argClass) = CallStack.getCallerInfo()
        PyLoggingFunctions.log_error(self.getLogger(),
                  "answerInDialog failed in state: %s::%s",
                  strFileName, strMethodName)
        return False
    else:
      PyLoggingFunctions.log_info(self.getLogger(),
               "answerInDialog failed because dialog id %s "
               "does not exist or has been released", in_szDialogId)
    return True
  
  def redirectInDialog(self, in_szDialogId, in_contacts):
    dialogSM = self.getInDialog(in_szDialogId)
    if (dialogSM is not None):
      if (not dialogSM.redirect(in_contacts)):
        (strMethodName, strFileName, argClass) = CallStack.getCallerInfo()
        PyLoggingFunctions.log_error(self.getLogger(),
                  "redirectInDialog failed in state: %s::%s",
                  strFileName, strMethodName)
        return False
    else:
      PyLoggingFunctions.log_info(self.getLogger(),
               "redirectInDialog failed because dialog id %s "
               "does not exist or has been released", in_szDialogId)
    return True

  def constructAnswerMessageInDialog(self, in_szDialogId, out_sipMessage):
    dialogSM = self.getInDialog(in_szDialogId)
    if (dialogSM is not None):
      dialogSM.constructAnswerMessage(out_sipMessage)
      return True
    else:
      PyLoggingFunctions.log_info(self.getLogger(),
               "constructAnswerMessageInDialog failed because dialog id %s "
               "does not exist or has been released", in_szDialogId)
      return False


  def constructRejectMessageInDialog(self, in_szDialogId, out_sipMessage):
    dialogSM = self.getInDialog(in_szDialogId)
    if (dialogSM is not None):
      dialogSM.constructRejectMessage(out_sipMessage)
      return True
    else:
      PyLoggingFunctions.log_info(self.getLogger(),
               "constructRejectMessageInDialog failed because dialog id %s "
               "does not exist or has been released", in_szDialogId)
      return False


  # Common In/Out dialog action functions

  def sendInfoMsgToDialog(self, in_szDialogId,
                        in_szMimeType, in_szMimeSubType, in_szBodyText):
    dialogSM = self.getDialog(in_szDialogId)
    if (dialogSM is not None):
      if (not dialogSM.sendInfoMsg(in_szMimeType,
                                     in_szMimeSubType,
                                     in_szBodyText)):
        (strMethodName, strFileName, argClass) = CallStack.getCallerInfo()
        PyLoggingFunctions.log_error(self.getLogger(),
                  "sendInfoMsgToDialog failed in state: %s::%s",
                  strFileName, strMethodName)
        return False
    else:
      PyLoggingFunctions.log_info(self.getLogger(),
               "sendInfoMsgToDialog failed because dialog id %s "
               "does not exist or has been released", in_szDialogId)
    return True

  def provideSdpOfferToDialog(self, in_szDialogId, in_sdpOffer):
    dialogSM = self.getDialog(in_szDialogId)
    if (dialogSM is not None):
      if (not dialogSM.provideSdpOffer(in_sdpOffer)):
        (strMethodName, strFileName, argClass) = CallStack.getCallerInfo()
        PyLoggingFunctions.log_error(self.getLogger(),
                  "provideSdpOfferToDialog failed in state: %s::%s",
                  strFileName, strMethodName)
        return False
    else:
      PyLoggingFunctions.log_info(self.getLogger(),
               "provideSdpOfferToDialog failed because dialog id %s "
               "does not exist or has been released", in_szDialogId)
    return True
  
  def provideSdpAnswerToDialog(self, in_szDialogId, in_sdpAnswer):
    dialogSM = self.getDialog(in_szDialogId)
    if (dialogSM is not None):
      if (not dialogSM.provideSdpAnswer(in_sdpAnswer)):
        (strMethodName, strFileName, argClass) = CallStack.getCallerInfo()
        PyLoggingFunctions.log_error(self.getLogger(),
                  "provideSdpAnswerToDialog failed in state: %s::%s",
                  strFileName, strMethodName)
        return False
    else:
      PyLoggingFunctions.log_info(self.getLogger(),
               "provideSdpAnswerToDialog failed because dialog id %s "
               "does not exist or has been released", in_szDialogId)
    return True
  
  def reInviteDialog(self, in_szDialogId, in_sdpOffer=None):
    dialogSM = self.getDialog(in_szDialogId)
    if (dialogSM is not None):
      if (not dialogSM.reInvite(in_sdpOffer)):
        (strMethodName, strFileName, argClass) = CallStack.getCallerInfo()
        PyLoggingFunctions.log_error(self.getLogger(),
                  "reInviteDialog failed in state: %s::%s",
                  strFileName, strMethodName)
        return False
    else:
      PyLoggingFunctions.log_info(self.getLogger(),
               "reInviteDialog failed because dialog id %s "
               "does not exist or has been released", in_szDialogId)
    return True
  
  def rejectDialog(self,
                   in_szDialogId,
                   in_uiSipErrorCode,
                   in_sipMessage = None,
                   in_szReason = None):
    dialogSM = self.getDialog(in_szDialogId)
    if (dialogSM is not None):
      if (not dialogSM.reject(in_uiSipErrorCode, in_sipMessage, in_szReason)):
        (strMethodName, strFileName, argClass) = CallStack.getCallerInfo()
        PyLoggingFunctions.log_error(self.getLogger(),
                  "rejectDialog failed in state: %s::%s",
                  strFileName, strMethodName)
        return False
    else:
      PyLoggingFunctions.log_info(self.getLogger(),
               "rejectDialog failed because dialog id %s "
               "does not exist or has been released", in_szDialogId)
    return True  
   
  def hangupDialog(self, in_szDialogId):
    dialogSM = self.getDialog(in_szDialogId)
    if (dialogSM is not None):
      if (not dialogSM.hangup()):
        (strMethodName, strFileName, argClass) = CallStack.getCallerInfo()
        PyLoggingFunctions.log_error(self.getLogger(),
                  "hangupDialog failed in state: %s::%s",
                  strFileName, strMethodName)
        return False
    else:
      PyLoggingFunctions.log_info(self.getLogger(),
               "hangupDialog failed because dialog id %s "
               "does not exist or has been released", in_szDialogId)
    return True

## !!MIRROR FILE, DO NOT MODIFY!!
