#
#  @file   SipCallEngineSMIn.py
#  @author Francis Moreau
#  date    Thu Apr 26 2007
#  
#  Description:
#
#  Type your description here
#
#  Copyright (C) 2007 Paraxip Technologies.
#  All Rights Reserved.
#

from paraxip.sipua.internal.SipCallEngineStateIn import *
from paraxip.sipua.internal.SipCallEngineEvent import *
import paraxip.infra.PyLoggingFunctions as PyLoggingFunctions
from netborderPyInfra import *
from paraxip.sipua.internal.SipCallEngineSMInOutBaseClass import *
from paraxip.sipua.internal.SipCallEngineStateInOutCommon import *

#
# Incoming call state machine
#

class SipCallEngineSMIn(SipCallEngineSMInOutBaseClass):
  def __init__(self):
    SipCallEngineSMInOutBaseClass.__init__(self)

    self.setLogger(CallLogger("netborder.incoming-dialog.py"))

    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "SipCallEngineSMIn.__init__") 

    # Create and register the IDLE state
    idleState = SipCallEngineIdleStateIn(self)
    self.setInitialStateName(idleState.getName())
    self.setInitialState(idleState)

    # Create and register the intermediate states
    callNotifiedCalledState = SipCallEngineCallNotifiedCalledStateIn(self)
    self.addState(callNotifiedCalledState.getName(),
                  callNotifiedCalledState)
    
    callNotifiedSdpOfferedState = SipCallEngineCallNotifiedSdpOfferedStateIn(self)
    self.addState(callNotifiedSdpOfferedState.getName(),
                  callNotifiedSdpOfferedState)
    
    ringingState = SipCallEngineRingingStateIn(self)
    self.addState(ringingState.getName(), ringingState)
    
    earlyMediaState = SipCallEngineEarlyMediaStateIn(self)
    self.addState(earlyMediaState.getName(), earlyMediaState)
    
    answeringState = SipCallEngineAnsweringStateIn(self)
    self.addState(answeringState.getName(), answeringState)
    
    connectedState = SipCallEngineConnectedStateInOut(self)
    self.addState(connectedState.getName(), connectedState)
    
    reinvitingState = SipCallEngineReinvitingStateInOut(self)
    self.addState(reinvitingState.getName(), reinvitingState)
    
    reinvitedState = SipCallEngineReinvitedStateInOut(self)
    self.addState(reinvitedState.getName(), reinvitedState)
    
    disconnectingState = SipCallEngineDisconnectingStateInOut(self)
    self.addState(disconnectingState.getName(), disconnectingState)
    
    # Create and register the final state
    disconnectedState = SipCallEngineDisconnectedStateInOut(self)
    self.addFinalState(disconnectedState.getName(), disconnectedState)
  
  def ring(self):
    return self.processPythonEvent(CallToDialogRingEvent(self.getId()))

  def sendProvisional(self, in_statusCode, in_reasonText):
    return self.processPythonEvent(
      CallToDialogSendProvisionalEvent(self.getId(), in_statusCode, in_reasonText))

  def sendEarlyMedia(self, in_sdpContents):
    return self.processPythonEvent(
      CallToDialogEarlyMediaEvent(self.getId(), in_sdpContents))
  
  def constructAnswerMessage(self, out_sipMessage):
    self.sipLeg.constructAnswerMessage(out_sipMessage)
  
  def answer(self, in_sdpContents, in_bSdpIsOffer=False, in_sipMessage=None):
    return self.processPythonEvent(
      CallToDialogAnswerEvent(self.getId(),
                              in_sdpContents,
                              in_bSdpIsOffer,
                              in_sipMessage))

  # @input in_contacts List of URL strings
  def redirect(self, in_contacts):
    return self.processPythonEvent(
      CallToDialogRedirectEvent(self.getId(), in_contacts))

## !!MIRROR FILE, DO NOT MODIFY!!
