#
#  @file   SipCallEngineSMInOutBaseClass.py
#  @author Francis Moreau
#  date    Thu Aug 2 2007
#  
#  Description:
#
#  Type your description here
#
#  Copyright (C) 2007 Paraxip Technologies.
#  All Rights Reserved.
#

from paraxip.infra import PyLoggingFunctions

from paraxip.sipua.internal.BaseStateMachine import BaseStateMachine
from netborderPyInfra import TraceScope
from SipCallEngineExtending import PySipLeg
from paraxip.sipua.internal.SipCallEngineEvent import \
    CallToDialogSendInfoMsgEvent, \
    CallToDialogProvideSdpAnswerEvent, \
    CallToDialogProvideSdpOfferEvent, \
    CallToDialogReInviteEvent, \
    CallToDialogRejectEvent, \
    CallToDialogHangupEvent

# Force declaration of class attributes in __init__ method
__pychecker__ = "initattr"

#
# Base class for incoming and outgoing state machine
#

class SipCallEngineSMInOutBaseClass(BaseStateMachine):
  def __init__(self):
    BaseStateMachine.__init__(self)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "SipCallEngineSMInOutBaseClass.__init__")
    
    self._strCallId = None
    self._strDialogId = None
    
    self.__sipLeg = None
  
  def initialize(self, in_refApplication, in_refGlobalContext,
                 in_szCallId, in_szDialogId):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "SipCallEngineSMInOutBaseClass.initialize")
    
    BaseStateMachine.initializeWithObjAndCallId(self,
                                in_refApplication,
                                in_refGlobalContext,
                                in_szDialogId,
                                in_szCallId)

    self._strCallId = in_szCallId
    self._strDialogId = in_szDialogId
    
    return True
  
  def reset(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "SipCallEngineSMInOutBaseClass.reset")
    
    BaseStateMachine.reset(self)

    self._strCallId = None
    self._strDialogId = None

    self.__sipLeg = None

  def _get_dialogId(self):
    return self._strDialogId
  dialogId = property(_get_dialogId)
  
  def _get_callId(self):
    return self._strCallId
  callId = property(_get_callId)
  
  def _get_sipLeg(self):
    return self.__sipLeg
  
  def _set_sipLeg(self, in_sipLeg):
    assert isinstance(in_sipLeg, PySipLeg)
    PyLoggingFunctions.log_debug(
      self.getLogger(),('Sit leg changing: _set_sipLeg %s -> %s.' % (self.__sipLeg, in_sipLeg)))
    self.__sipLeg = in_sipLeg
  
  #Public property
  sipLeg = property(_get_sipLeg, _set_sipLeg)

  def enqueueEventForCall(self, in_Event):
    return self.globalContext.enqueuePythonEvent(in_Event.getDestObjectId(),
                                                 in_Event)

  # Commands common to In/Out state machines 
  def sendInfoMsg(self, in_szMimeType, in_szMimeSubType, in_szBodyText):
    return self.processPythonEvent(
      CallToDialogSendInfoMsgEvent(self.getId(),
                                   in_szMimeType,
                                   in_szMimeSubType,
                                   in_szBodyText)) 

  def provideSdpAnswer(self, in_sdpAnswer):
    return self.processPythonEvent(
      CallToDialogProvideSdpAnswerEvent(self.getId(), in_sdpAnswer))
  
  def provideSdpOffer(self, in_sdpOffer):
    return self.processPythonEvent(
      CallToDialogProvideSdpOfferEvent(self.getId(), in_sdpOffer))
  
  def reInvite(self, in_sdpOffer=None):
    return self.processPythonEvent(
      CallToDialogReInviteEvent(self.getId(), in_sdpOffer))

  def constructRejectMessage(self, out_sipMessage):
    self.sipLeg.constructRejectMessage(out_sipMessage)
  
  def reject(self,
             in_uiSipErrorCode,
             in_sipMessage = None,
             in_szReason = None):
    
    if in_uiSipErrorCode is None:
      recoverySipErrorCode = 603
      PyLoggingFunctions.log_warn(self.getLogger(), 
                                  "Rejecting a call without a valid SIP error code, setting "
                                  "default error code of: %s." % recoverySipErrorCode)
      in_uiSipErrorCode = recoverySipErrorCode
                                     
    return self.processPythonEvent(
      CallToDialogRejectEvent(self.getId(),
                              in_uiSipErrorCode,
                              in_sipMessage,
                              in_szReason))

  def hangup(self):
    return self.processPythonEvent(
      CallToDialogHangupEvent(self.getId()))

def test_module():
  sipCA = SipCallEngineSMInOutBaseClass()
  sipCA.reset()
  #Cannot be called; requires the C++ stack to be initialized?
  #sipCA.reject(None)
  
  #Test that the 'property-bug-when-not-inheriting-from-object' is not present
  sipCA = SipCallEngineSMInOutBaseClass()
  sipCA.reset()
  gotAssert = False
  try:
    sipCA.sipLeg = 3
  except AssertionError:
    gotAssert = True
  assert gotAssert
  

if __name__ == '__main__':
  test_module()
## !!MIRROR FILE, DO NOT MODIFY!!
