#
#  @file   SipCallEngineSMOut.py
#  @author Francis Moreau
#  date    Thu Apr 26 2007
#  
#  Description:
#
#  Type your description here
#
#  Copyright (C) 2007 Paraxip Technologies.
#  All Rights Reserved.
#

from paraxip.sipua.internal.SipCallEngineStateOut import *
from paraxip.sipua.internal.SipCallEngineEvent import *
import paraxip.infra.PyLoggingFunctions as PyLoggingFunctions
from netborderPyInfra import *
from paraxip.sipua.internal.SipCallEngineSMInOutBaseClass import *
from paraxip.sipua.internal.SipCallEngineStateInOutCommon import *

#
# Outgoing call state machine
#

class SipCallEngineSMOut(SipCallEngineSMInOutBaseClass):
  def __init__(self):
    SipCallEngineSMInOutBaseClass.__init__(self)
    
    self.setLogger(CallLogger("netborder.outgoing-dialog.py"))

    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "SipCallEngineSMOut.__init__")
    
    # IMPORTANT: This member is to prevent a race condition; between the time
    #            this State Machine is initialized and the time the
    #            ResipCallingEvent is processed there is a possibility to
    #            receive a CallToDialogHangupEvent. But we won't be able to
    #            call hangup() function on self.sipLeg since this last one
    #            is only got upon the reception of ResipCallingEvent.
    self.bHangupOnResipCallingEvent = False

    # Create and register the IDLE state
    idleState = SipCallEngineIdleStateOut(self)
    self.setInitialStateName(idleState.getName())
    self.setInitialState(idleState)
    
    # Create and register the intermediate states
    callingState = SipCallEngineCallingStateOut(self)
    self.addState(callingState.getName(), callingState)
    
    callNotifiedEarlyMediaState = \
      SipCallEngineCallNotifiedEarlyMediaStateOut(self)
    self.addState(callNotifiedEarlyMediaState.getName(),
                  callNotifiedEarlyMediaState)
    
    callNotifiedSdpReceivedState = \
      SipCallEngineCallNotifiedSdpReceivedStateOut(self)
    self.addState(callNotifiedSdpReceivedState.getName(),
                  callNotifiedSdpReceivedState)
    
    connectedState = SipCallEngineConnectedStateInOut(self)
    self.addState(connectedState.getName(), connectedState)
    
    reinvitingState = SipCallEngineReinvitingStateInOut(self)
    self.addState(reinvitingState.getName(), reinvitingState)
    
    reinvitedState = SipCallEngineReinvitedStateInOut(self)
    self.addState(reinvitedState.getName(), reinvitedState)
    
    disconnectingState = SipCallEngineDisconnectingStateInOut(self)
    self.addState(disconnectingState.getName(), disconnectingState)
    
    # Create and register the final state
    disconnectedState = SipCallEngineDisconnectedStateInOut(self)
    self.addFinalState(disconnectedState.getName(), disconnectedState)

  def reset(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "SipCallEngineSMOut.reset")

    SipCallEngineSMInOutBaseClass.reset(self)

    self.bHangupOnResipCallingEvent = False

## !!MIRROR FILE, DO NOT MODIFY!!
