#
#  @file   SipCallEngineState.py
#  @author Francis Moreau
#  date    Tue Mar 13 2007
#  
#  Description:
#
#  Type your description here
#
#  Copyright (C) 2007 Paraxip Technologies.
#  All Rights Reserved.
#

import weakref

from SipCallEngineExtending import PySipCallEngineState

import paraxip.infra.PyLoggingFunctions as PyLoggingFunctions
from netborderPyInfra import TraceScope, Logger
import paraxip.infra.CallStack

eventProcessorLogger = Logger('netborder.processevent')

class SipCallEngineState(PySipCallEngineState):
  """Base class for states, derived directly from C++."""
  
  def __init__(self, in_stateMachine):
    PySipCallEngineState.__init__(self)

    self._stateMachineWR = weakref.ref(in_stateMachine)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "SipCallEngineState.__init__")
  
  def getName(self):
    return ""
  
  def entryAction(self, in_event, out_pystrNewStateName):
    __pychecker__ = 'unusednames=in_event'
    out_pystrNewStateName.set(self.getName())
  
  def processEvent_fromCpp(self, in_event, out_pystrNewStateName):
    """
    This function should never be overwritten. It should be used to marshal
    calls to 'processEvent' (allow easy debug statements, filter events, ...).
    (@see Non-Virtual Interface Idiom)
    """
    
    PyLoggingFunctions.log_info(eventProcessorLogger,
            "processEvent(in_event= %s, out_pystrNewStateName=%s)." %
            (in_event, out_pystrNewStateName))
    
    #Call the 'virtual' function
    rv = self.processEvent(in_event, out_pystrNewStateName)
    
    PyLoggingFunctions.log_info(eventProcessorLogger,
            "processEvent returned %s." % rv)
    
    return rv
  
  def processEvent(self, in_event, out_pystrNewStateName):    
    """
    This function should be overwritten by children to customize behavior.
    """
    
    __pychecker__ = 'unusednames=in_event'
    out_pystrNewStateName.set(self.getName())
    return True
  
  def onStateExit(self, in_event, in_strDestStateName):
    pass
  
  def _get_stateMachine(self):
    return self._stateMachineWR()

  def getLogger(self):
    return self._get_stateMachine().getLogger()

  def warnUnexpectedEvent(self, in_strStateName, in_event):
    PyLoggingFunctions.log_warn(self.getLogger(),
             '%s::processEvent received an invalid event %s',
             (in_strStateName, in_event.__class__.__name__))

  _stateMachine = property(_get_stateMachine, None)

class SipCallEngineStateInOut(SipCallEngineState):
  """Base class for incoming and outgoing call states."""
  
  def __init__(self, in_stateMachine):
    SipCallEngineState.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "SipCallEngineStateInOut.__init__")
  
  def enqueueEventForCall(self, in_event):
    if (not self._stateMachine.enqueueEventForCall(in_event)):
      (strMethodName, strFileName, argClass) = \
        paraxip.infra.CallStack.getCallerInfo()
      PyLoggingFunctions.log_error(self.getLogger(),
                "enqueueEventForCall(%s) failed in state: %s::%s",
                in_event.__class__.__name__,
                strFileName,
                strMethodName)
      return False
    return True

## !!MIRROR FILE, DO NOT MODIFY!!
