#
#  @file   SipCallEngineStateIn.py
#  @author Francis Moreau
#  date    Thu Apr 26 2007
#  
#  Description:
#
#  Type your description here
#
#  Copyright (C) 2007 Paraxip Technologies.
#  All Rights Reserved.
#

from netborderPyInfra import TraceScope
from SipCallEngineExtending import InitialEvent, ResipCalledEvent, \
    ResipSdpOfferedEvent, ResipSdpOfferRequiredEvent, \
    ResipTerminatedEvent, ResipSdpAnsweredEvent, ResipAckNotReceivedEvent, \
    ResipConnectedEvent, PyGetCallIdResult

import paraxip.infra.PyLoggingFunctions as PyLoggingFunctions 

from paraxip.sipua.internal.SipCallEngineState import SipCallEngineStateInOut
from paraxip.sipua.internal.SipCallEngineEvent import *

# Force declaration of class attributes in __init__ method
__pychecker__ = "initattr"

#
# Incoming call states
#

#
# Base class
#

class SipCallEngineStateIn(SipCallEngineStateInOut):
  def __init__(self, in_stateMachine):
    SipCallEngineStateInOut.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "SipCallEngineStateIn.__init__")
  
#
# Idle state
#

class SipCallEngineIdleStateIn(SipCallEngineStateIn):
  def __init__(self, in_stateMachine):
    SipCallEngineStateIn.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "SipCallEngineIdleStateIn.__init__")
  
  def getName(self):
    return "IDLE"
  
  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    out_pystrNewStateName.set(self.getName())
    
    if (isinstance(in_event, InitialEvent)):
      return True
    
    elif (isinstance(in_event, ResipCalledEvent)):
      # Keep a copy of the sip in leg in the state machine instance
      self._stateMachine.sipLeg = in_event.getSipInLegCopy()
      
      toCallEvent = InDialogToCallCalledEvent(self._stateMachine.dialogId,
                                              self._stateMachine.callId,
                                              in_event.getSipMessageCopy())
      if (not self.enqueueEventForCall(toCallEvent)):
        return False
      
      out_pystrNewStateName.set("CALL_NOTIFIED_CALLED")
      return True
    
    # Sent by SipCallEngine on incoming calls that were rejected immediately
    elif (isinstance(in_event, ResipTerminatedEvent)):
      toCallEvent = DialogToCallTerminatedEvent(self._stateMachine.dialogId,
                                                self._stateMachine.callId)
      if (not self.enqueueEventForCall(toCallEvent)):
        return False

      out_pystrNewStateName.set("DISCONNECTED")
      return True

    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)
    
    return True

#
# CallNotifiedCalled state
#

class SipCallEngineCallNotifiedCalledStateIn(SipCallEngineStateIn):
  def __init__(self, in_stateMachine):
    SipCallEngineStateIn.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(),
               "SipCallEngineCallNotifiedCalledStateIn.__init__")
  
  def getName(self):
    return "CALL_NOTIFIED_CALLED"
  
  def processEvent(self, in_event, out_pystrNewStateName):
    __pychecker__ = 'maxreturns=0'

    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    out_pystrNewStateName.set(self.getName())
    
    if (isinstance(in_event, CallToDialogRingEvent)):
      # Start ringing
      self._stateMachine.sipLeg.ringing()
      
      out_pystrNewStateName.set("RINGING")
      return True

    elif (isinstance(in_event, CallToDialogSendProvisionalEvent)):
      statusCode = in_event.getStatusCode()
      reasonText = in_event.getReasonText()
      PyLoggingFunctions.log_debug(self.getLogger(),
          "SipCallEngineCallNotifiedCalledStateIn::SEND_PROVISIONAL "
          "statusCode=%s, reasonText=%s", statusCode, reasonText)
      self._stateMachine.sipLeg.sendProvisional(statusCode, reasonText)
      return True

    elif (isinstance(in_event, CallToDialogAnswerEvent)):
      self._stateMachine.sipLeg.answer(in_event.getSdpContents(),
                                       in_event.getSdpIsOffer(),
                                       in_event.getSipMessage())
      
      out_pystrNewStateName.set("ANSWERING")
      return True

    elif (isinstance(in_event, CallToDialogRedirectEvent)):
      return self._stateMachine.sipLeg.redirect(in_event.getContacts())

    elif (isinstance(in_event, CallToDialogEarlyMediaEvent)):
      # Send 183 with SDP
      self._stateMachine.sipLeg.earlyMedia(in_event.getSdpContents())
      
      out_pystrNewStateName.set("EARLY_MEDIA")
      return True
    
    elif (isinstance(in_event, CallToDialogRejectEvent)):
      # Reject incoming call
      self._stateMachine.sipLeg.reject(in_event.getStatusCode(),
                                       in_event.getSipMessage(),
                                       in_event.getReason())
      # Wait for ResipTerminatedEvent event
      return True
    
    elif (isinstance(in_event, CallToDialogSendInfoMsgEvent)):
      self._stateMachine.sipLeg.sendInfoMsg(
          in_event.getMimeType(),
          in_event.getMimeSubType(),
          in_event.getBodyText())

      # Stay in the same state
      return True

    elif (isinstance(in_event, ResipSdpOfferedEvent)):
      # IMPORTANT boost::python converter for resip::SDP is defined in 
      #   PyResipStack extension module (src: sip/resiprocate/resip/stack)
      #   MUST be imported before calling getSdpContentsCopy()
      __pychecker__ = 'unusednames=PyResipStack'
      import PyResipStack
      toCallEvent = DialogToCallSdpOfferedEvent(self._stateMachine.dialogId,
                                                self._stateMachine.callId,
                                                in_event.getSipMessageCopy(),
                                                in_event.getSdpContentsCopy())
      if (not self.enqueueEventForCall(toCallEvent)):
        return False
      
      out_pystrNewStateName.set("CALL_NOTIFIED_SDP_OFFERED")
      return True
    
    elif (isinstance(in_event, ResipSdpOfferRequiredEvent)):
      toCallEvent = DialogToCallSdpOfferRequiredEvent(
          self._stateMachine.dialogId,
          self._stateMachine.callId,
          in_event.getSipMessageCopy())
      if (not self.enqueueEventForCall(toCallEvent)):
        return False
      
      # Stay in the same state
      return True
    
    elif (isinstance(in_event, ResipTerminatedEvent)):
      toCallEvent = DialogToCallTerminatedEvent(self._stateMachine.dialogId,
                                                self._stateMachine.callId)
      if (not self.enqueueEventForCall(toCallEvent)):
        return False
      
      out_pystrNewStateName.set("DISCONNECTED")
      return True
    
    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)
    
    return True

#
# CallNotifiedSdpOffered state
#

class SipCallEngineCallNotifiedSdpOfferedStateIn(SipCallEngineStateIn):
  def __init__(self, in_stateMachine):
    SipCallEngineStateIn.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(),
               "SipCallEngineCallNotifiedSdpOfferedStateIn.__init__")
  
  def getName(self):
    return "CALL_NOTIFIED_SDP_OFFERED"
  
  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    out_pystrNewStateName.set(self.getName())
    
    if (isinstance(in_event, CallToDialogRingEvent)):
      self._stateMachine.sipLeg.ringing()
      
      out_pystrNewStateName.set("RINGING")
      return True
    
    elif (isinstance(in_event, CallToDialogSendProvisionalEvent)):
      statusCode = in_event.getStatusCode()
      reasonText = in_event.getReasonText()
      PyLoggingFunctions.log_debug(self.getLogger(),
          "SipCallEngineCallNotifiedSdpOfferedStateIn::SEND_PROVISIONAL "
          "statusCode=%s, reasonText=%s", statusCode, reasonText)
      self._stateMachine.sipLeg.sendProvisional(statusCode, reasonText)
      return True

    elif (isinstance(in_event, CallToDialogEarlyMediaEvent)):
      # Send 183 with SDP
      self._stateMachine.sipLeg.earlyMedia(in_event.getSdpContents())
      
      out_pystrNewStateName.set("EARLY_MEDIA")
      return True
    
    elif (isinstance(in_event, CallToDialogAnswerEvent)):
      self._stateMachine.sipLeg.answer(in_event.getSdpContents(),
                                       in_event.getSdpIsOffer(),
                                       in_event.getSipMessage())
      
      out_pystrNewStateName.set("ANSWERING")
      return True
    
    elif (isinstance(in_event, CallToDialogRedirectEvent)):
      return self._stateMachine.sipLeg.redirect(in_event.getContacts())

    elif (isinstance(in_event, CallToDialogRejectEvent)):
      # Reject incoming call
      self._stateMachine.sipLeg.reject(in_event.getStatusCode(),
                                       in_event.getSipMessage(),
                                       in_event.getReason())
      # then wait for ResipTerminatedEvent event
      return True
    
    elif (isinstance(in_event, CallToDialogSendInfoMsgEvent)):
      self._stateMachine.sipLeg.sendInfoMsg(
          in_event.getMimeType(),
          in_event.getMimeSubType(),
          in_event.getBodyText())

      # Stay in the same state
      return True

    elif (isinstance(in_event, ResipTerminatedEvent)):
      toCallEvent = DialogToCallTerminatedEvent(self._stateMachine.dialogId,
                                                self._stateMachine.callId)
      if (not self.enqueueEventForCall(toCallEvent)):
        return False
      
      out_pystrNewStateName.set("DISCONNECTED")
      return True
    
    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)
    
    return True

#
# Ringing state
#

class SipCallEngineRingingStateIn(SipCallEngineStateIn):
  def __init__(self, in_stateMachine):
    SipCallEngineStateIn.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "SipCallEngineRingingStateIn.__init__")
  
  def getName(self):
    return "RINGING"
  
  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    out_pystrNewStateName.set(self.getName())
    
    if (isinstance(in_event, CallToDialogSendProvisionalEvent)):
      statusCode = in_event.getStatusCode()
      reasonText = in_event.getReasonText()
      PyLoggingFunctions.log_debug(self.getLogger(),
          "SipCallEngineRingingStateIn::SEND_PROVISIONAL "
          "statusCode=%s, reasonText=%s", statusCode, reasonText)
      self._stateMachine.sipLeg.sendProvisional(statusCode, reasonText)
      return True

    elif (isinstance(in_event, CallToDialogEarlyMediaEvent)):
      # Send 183 with SDP
      self._stateMachine.sipLeg.earlyMedia(in_event.getSdpContents())
      
      out_pystrNewStateName.set("EARLY_MEDIA")
      return True
    
    elif (isinstance(in_event, CallToDialogAnswerEvent)):
      # Answer incoming call
      self._stateMachine.sipLeg.answer(in_event.getSdpContents(),
                                       in_event.getSdpIsOffer(),
                                       in_event.getSipMessage())
      
      out_pystrNewStateName.set("ANSWERING")
      return True
    
    elif (isinstance(in_event, CallToDialogRedirectEvent)):
      return self._stateMachine.sipLeg.redirect(in_event.getContacts())

    elif (isinstance(in_event, CallToDialogRejectEvent)):
      # Reject incoming call
      self._stateMachine.sipLeg.reject(in_event.getStatusCode(),
                                       in_event.getSipMessage(),
                                       in_event.getReason())
      # Wait for ResipTerminatedEvent event
      return True
    
    elif (isinstance(in_event, CallToDialogSendInfoMsgEvent)):
      self._stateMachine.sipLeg.sendInfoMsg(
          in_event.getMimeType(),
          in_event.getMimeSubType(),
          in_event.getBodyText())

      # Stay in the same state
      return True

    elif (isinstance(in_event, ResipTerminatedEvent)):
      toCallEvent = DialogToCallTerminatedEvent(self._stateMachine.dialogId,
                                                self._stateMachine.callId)
      if (not self.enqueueEventForCall(toCallEvent)):
        return False
      
      out_pystrNewStateName.set("DISCONNECTED")
      return True
    
    elif (isinstance(in_event, CallToDialogRingEvent)):
      # No retransmission of 180 Ringing should be sent: ignore (bug 1018) 
      PyLoggingFunctions.log_info(self.getLogger(),
          "SipCallEngineRingingStateIn.processEvent: "\
          "Ignoring duplicate request to send 180 Ringing")
      # Stay in same state
      return True

    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)
    
    return True

#
# EarlyMedia state
#

class SipCallEngineEarlyMediaStateIn(SipCallEngineStateIn):
  def __init__(self, in_stateMachine):
    SipCallEngineStateIn.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "SipCallEngineEarlyMediaStateIn.__init__")
  
  def getName(self):
    return "EARLY_MEDIA"
  
  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    out_pystrNewStateName.set(self.getName())
    
    if (isinstance(in_event, CallToDialogAnswerEvent)):
      # Answer incoming call
      self._stateMachine.sipLeg.answer(in_event.getSdpContents(),
                                       in_event.getSdpIsOffer(),
                                       in_event.getSipMessage())
      
      out_pystrNewStateName.set("ANSWERING")
      return True
    
    elif (isinstance(in_event, CallToDialogRejectEvent)):
      # Reject incoming call
      self._stateMachine.sipLeg.reject(in_event.getStatusCode(),
                                       in_event.getSipMessage(),
                                       in_event.getReason())
      # Wait for ResipTerminatedEvent event
      return True
    
    elif (isinstance(in_event, CallToDialogSendInfoMsgEvent)):
      self._stateMachine.sipLeg.sendInfoMsg(
          in_event.getMimeType(),
          in_event.getMimeSubType(),
          in_event.getBodyText())

      # Stay in the same state
      return True

    elif (isinstance(in_event, ResipTerminatedEvent)):
      toCallEvent = DialogToCallTerminatedEvent(self._stateMachine.dialogId,
                                                self._stateMachine.callId)
      if (not self.enqueueEventForCall(toCallEvent)):
        return False
      
      out_pystrNewStateName.set("DISCONNECTED")
      return True
    
    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)
    
    return True

#
# Answering state
#

class SipCallEngineAnsweringStateIn(SipCallEngineStateIn):
  def __init__(self, in_stateMachine):
    SipCallEngineStateIn.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "SipCallEngineAnsweringStateIn.__init__")
  
  def getName(self):
    return "ANSWERING"
  
  def processEvent(self, in_event, out_pystrNewStateName):
    __pychecker__ = 'maxreturns=0'
    
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())
    
    if (isinstance(in_event, CallToDialogHangupEvent)):
      # Happens when hanging-up upon timeout waiting for an ACK
      self._stateMachine.sipLeg.hangup()
      out_pystrNewStateName.set("DISCONNECTING")
      return True
    
    elif (isinstance(in_event, CallToDialogSendInfoMsgEvent)):
      self._stateMachine.sipLeg.sendInfoMsg(
          in_event.getMimeType(),
          in_event.getMimeSubType(),
          in_event.getBodyText())

      # Stay in the same state
      return True

    elif (isinstance(in_event, ResipSdpAnsweredEvent)):
      # IMPORTANT boost::python converter for resip::SDP is defined in 
      #   PyResipStack extension module (src: sip/resiprocate/resip/stack)
      #   MUST be imported before calling getSdpContentsCopy()
      __pychecker__ = 'unusednames=PyResipStack'
      import PyResipStack
      toCallEvent = DialogToCallSdpAnsweredEvent(self._stateMachine.dialogId,
                                                 self._stateMachine.callId,
                                                 in_event.getSipMessageCopy(),
                                                 in_event.getSdpContentsCopy())
      if (not self.enqueueEventForCall(toCallEvent)):
        return False
      
      # Stay in the same state
      return True
    
    elif (isinstance(in_event, ResipAckNotReceivedEvent)):
      toCallEvent = DialogToCallAckNotReceivedEvent(self._stateMachine.dialogId,
                                                    self._stateMachine.callId)
      if (not self.enqueueEventForCall(toCallEvent)):
        return False
      
      # Stay in the same state
      return True
    
    elif (isinstance(in_event, ResipConnectedEvent)):
      toCallEvent = DialogToCallConnectedEvent(self._stateMachine.dialogId,
                                               self._stateMachine.callId,
                                               in_event.getSipMessageCopy())
      if (not self.enqueueEventForCall(toCallEvent)):
        return False
      
      out_pystrNewStateName.set("CONNECTED")
      return True
    
    elif (isinstance(in_event, ResipTerminatedEvent)):
      toCallEvent = DialogToCallTerminatedEvent(self._stateMachine.dialogId,
                                                self._stateMachine.callId)
      if (not self.enqueueEventForCall(toCallEvent)):
        return False
      
      out_pystrNewStateName.set("DISCONNECTED")
      return True
    
    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)
    
    return True

## !!MIRROR FILE, DO NOT MODIFY!!
