#
#  @file   SipCallEngineStateOut.py
#  @author Francis Moreau
#  date    Thu Apr 26 2007
#  
#  Description:
#
#  Type your description here
#
#  Copyright (C) 2007 Paraxip Technologies.
#  All Rights Reserved.
#

from paraxip.sipua.internal.SipCallEngineState import *
from paraxip.sipua.internal.SipCallEngineEvent import *
import paraxip.infra.PyLoggingFunctions as PyLoggingFunctions
from netborderPyInfra import *
from SipCallEngineExtending import *

#
# Outgoing call states
#

#
# Base class
#

class SipCallEngineStateOut(SipCallEngineStateInOut):
  def __init__(self, in_stateMachine):
    SipCallEngineStateInOut.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "SipCallEngineStateOut.__init__")

#
# Idle state
#

class SipCallEngineIdleStateOut(SipCallEngineStateOut):
  def __init__(self, in_stateMachine):
    SipCallEngineStateOut.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "SipCallEngineIdleStateOut.__init__")
  
  def getName(self):
    return "IDLE"
  
  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    out_pystrNewStateName.set(self.getName())
    
    if (isinstance(in_event, InitialEvent)):
      return True
    
    elif (isinstance(in_event, CallToDialogHangupEvent)):
      self._stateMachine.bHangupOnResipCallingEvent = True
      # ResipCallingEvent is coming...

      # Stay in the same state
      return True
    
    elif (isinstance(in_event, ResipCallingEvent)):
      # Keep a copy of the sip out leg in the state machine instance
      self._stateMachine.sipLeg = in_event.getSipOutLegCopy()
      
      if (not self._stateMachine.bHangupOnResipCallingEvent):
        toCallEvent = OutDialogToCallCallingEvent(self._stateMachine.dialogId,
                                                  self._stateMachine.callId,
                                                  in_event.getSipMessageCopy())
        if (not self.enqueueEventForCall(toCallEvent)):
          return False
        
        out_pystrNewStateName.set("CALLING")

      else:
        # See the comment in SipCallEngineSMOut.py about
        # bHangupOnResipCallingEvent
        self._stateMachine.sipLeg.hangup()
        
        out_pystrNewStateName.set("DISCONNECTING")

      return True
    
    elif (isinstance(in_event, ResipFailedEvent)):
      toCallEvent = DialogToCallFailedEvent(self._stateMachine.dialogId,
                                            self._stateMachine.callId,
                                            in_event.getSipMessageCopy())
      if (not self.enqueueEventForCall(toCallEvent)):
        return False
      
      out_pystrNewStateName.set("DISCONNECTING")
      return True
    
    elif (isinstance(in_event, ResipTerminatedEvent)):
      toCallEvent = DialogToCallTerminatedEvent(self._stateMachine.dialogId,
                                                self._stateMachine.callId)
      if (not self.enqueueEventForCall(toCallEvent)):
        return False
      
      out_pystrNewStateName.set("DISCONNECTED")
      return True
    
    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)
    
    return True

#
# Calling state
#

class SipCallEngineCallingStateOut(SipCallEngineStateOut):
  def __init__(self, in_stateMachine):
    SipCallEngineStateOut.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "SipCallEngineCallingStateOut.__init__")
  
  def getName(self):
    return "CALLING"
  
  def processEvent(self, in_event, out_pystrNewStateName):
    __pychecker__ = 'maxreturns=0'
    
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    out_pystrNewStateName.set(self.getName())
    
    if (isinstance(in_event, ResipFailedEvent)):
      toCallEvent = DialogToCallFailedEvent(self._stateMachine.dialogId,
                                            self._stateMachine.callId,
                                            in_event.getSipMessageCopy())
      if (not self.enqueueEventForCall(toCallEvent)):
        return False
      
      out_pystrNewStateName.set("DISCONNECTING")
      return True
    
    elif (isinstance(in_event, CallToDialogHangupEvent)):
      self._stateMachine.sipLeg.hangup()
      
      out_pystrNewStateName.set("DISCONNECTING")
      return True
    
    elif (isinstance(in_event, ResipProvisionalEvent)):
      toCallEvent = OutDialogToCallProvisionalEvent(self._stateMachine.dialogId,
                                                    self._stateMachine.callId,
                                                    in_event.getSipMessageCopy())
      if (not self.enqueueEventForCall(toCallEvent)):
        return False
      
      # Stay in the same state
      return True
    
    elif (isinstance(in_event, ResipEarlyMediaEvent)):
      # IMPORTANT boost::python converter for resip::SDP is defined in 
      #   PyResipStack extension module (src: sip/resiprocate/resip/stack)
      #   MUST be imported before calling getSdpContentsCopy()
      __pychecker__ = 'unusednames=PyResipStack'
      import PyResipStack
      toCallEvent = OutDialogToCallEarlyMediaEvent(self._stateMachine.dialogId,
                                                   self._stateMachine.callId,
                                                   in_event.getSipMessageCopy(),
                                                   in_event.getSdpContentsCopy())
      if (not self.enqueueEventForCall(toCallEvent)):
        return False
      
      out_pystrNewStateName.set("CALL_NOTIFIED_EARLY_MEDIA")
      return True
    
    elif (isinstance(in_event, ResipSdpAnsweredEvent)):
      # IMPORTANT boost::python converter for resip::SDP is defined in 
      #   PyResipStack extension module (src: sip/resiprocate/resip/stack)
      #   MUST be imported before calling getSdpContentsCopy()
      __pychecker__ = 'unusednames=PyResipStack'
      import PyResipStack
      toCallEvent = DialogToCallSdpAnsweredEvent(self._stateMachine.dialogId,
                                                 self._stateMachine.callId,
                                                 in_event.getSipMessageCopy(),
                                                 in_event.getSdpContentsCopy())
      if (not self.enqueueEventForCall(toCallEvent)):
        return False
      
      out_pystrNewStateName.set("CALL_NOTIFIED_SDP_RECEIVED")
      return True
    
    elif (isinstance(in_event, ResipSdpOfferedEvent)):
      # IMPORTANT boost::python converter for resip::SDP is defined in 
      #   PyResipStack extension module (src: sip/resiprocate/resip/stack)
      #   MUST be imported before calling getSdpContentsCopy()
      __pychecker__ = 'unusednames=PyResipStack'
      import PyResipStack
      toCallEvent = DialogToCallSdpOfferedEvent(self._stateMachine.dialogId,
                                                self._stateMachine.callId,
                                                in_event.getSipMessageCopy(),
                                                in_event.getSdpContentsCopy())
      if (not self.enqueueEventForCall(toCallEvent)):
        return False
      
      out_pystrNewStateName.set("CALL_NOTIFIED_SDP_RECEIVED")
      return True
    
    elif (isinstance(in_event, ResipConnectedEvent)):
      toCallEvent = DialogToCallConnectedEvent(self._stateMachine.dialogId,
                                               self._stateMachine.callId,
                                               in_event.getSipMessageCopy())
      if (not self.enqueueEventForCall(toCallEvent)):
        return False
      
      out_pystrNewStateName.set("CONNECTED")
      return True
    
    elif (isinstance(in_event, ResipTerminatedEvent)):
      toCallEvent = DialogToCallTerminatedEvent(self._stateMachine.dialogId,
                                                self._stateMachine.callId)
      if (not self.enqueueEventForCall(toCallEvent)):
        return False
      
      out_pystrNewStateName.set("DISCONNECTED")
      return True
    
    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)
    
    return True

#
# CallNotifiedEarlyMedia state
#

class SipCallEngineCallNotifiedEarlyMediaStateOut(SipCallEngineStateOut):
  def __init__(self, in_stateMachine):
    SipCallEngineStateOut.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(),
               "SipCallEngineCallNotifiedEarlyMediaStateOut.__init__")
  
  def getName(self):
    return "CALL_NOTIFIED_EARLY_MEDIA"
  
  def processEvent(self, in_event, out_pystrNewStateName):
    __pychecker__ = 'maxreturns=0'
    
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    out_pystrNewStateName.set(self.getName())
    
    if (isinstance(in_event, ResipFailedEvent)):
      toCallEvent = DialogToCallFailedEvent(self._stateMachine.dialogId,
                                            self._stateMachine.callId,
                                            in_event.getSipMessageCopy())
      if (not self.enqueueEventForCall(toCallEvent)):
        return False
      
      out_pystrNewStateName.set("DISCONNECTING")
      return True
    
    elif (isinstance(in_event, CallToDialogHangupEvent)):
      self._stateMachine.sipLeg.hangup()
      
      out_pystrNewStateName.set("DISCONNECTING")
      return True
    
    elif (isinstance(in_event, ResipProvisionalEvent)):
      toCallEvent = OutDialogToCallProvisionalEvent(self._stateMachine.dialogId,
                                                    self._stateMachine.callId,
                                                    in_event.getSipMessageCopy())
      if (not self.enqueueEventForCall(toCallEvent)):
        return False
      
      # Stay in the same state
      return True
    
    elif (isinstance(in_event, ResipSdpAnsweredEvent)):
      toCallEvent = DialogToCallSdpAnsweredEvent(self._stateMachine.dialogId,
                                                 self._stateMachine.callId,
                                                 in_event.getSipMessageCopy(),
                                                 in_event.getSdpContentsCopy())
      if (not self.enqueueEventForCall(toCallEvent)):
        return False
      
      # Stay in the same state
      return True
    
    elif (isinstance(in_event, ResipConnectedEvent)):
      toCallEvent = DialogToCallConnectedEvent(self._stateMachine.dialogId,
                                               self._stateMachine.callId,
                                               in_event.getSipMessageCopy())
      if (not self.enqueueEventForCall(toCallEvent)):
        return False
      
      out_pystrNewStateName.set("CONNECTED")
      return True
    
    elif (isinstance(in_event, ResipTerminatedEvent)):
      toCallEvent = DialogToCallTerminatedEvent(self._stateMachine.dialogId,
                                                self._stateMachine.callId)
      if (not self.enqueueEventForCall(toCallEvent)):
        return False
      
      out_pystrNewStateName.set("DISCONNECTED")
      return True
    
    elif (isinstance(in_event, ResipEarlyMediaEvent)):
      # Retransmission: ignore (bug 1018)
      PyLoggingFunctions.log_info(self.getLogger(),
          "SipCallEngineCallNotifiedEarlyMediaStateOut.processEvent: "\
          "Discarding 183 Session Progress duplicate response received")
      # Stay in the same state
      return True;

    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)
    
    return True

#
# CallNotifiedSdpReceived state
#

class SipCallEngineCallNotifiedSdpReceivedStateOut(SipCallEngineStateOut):
  def __init__(self, in_stateMachine):
    SipCallEngineStateOut.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(),
               "SipCallEngineCallNotifiedSdpReceivedStateOut.__init__")
  
  def getName(self):
    return "CALL_NOTIFIED_SDP_RECEIVED"
  
  def processEvent(self, in_event, out_pystrNewStateName):
    __pychecker__ = 'maxreturns=0'
    
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    out_pystrNewStateName.set(self.getName())
    
    if (isinstance(in_event, ResipFailedEvent)):
      toCallEvent = DialogToCallFailedEvent(self._stateMachine.dialogId,
                                            self._stateMachine.callId,
                                            in_event.getSipMessageCopy())
      if (not self.enqueueEventForCall(toCallEvent)):
        return False
      
      out_pystrNewStateName.set("DISCONNECTING")
      return True
    
    elif (isinstance(in_event, CallToDialogHangupEvent)):
      self._stateMachine.sipLeg.hangup()
      
      out_pystrNewStateName.set("DISCONNECTING")
      return True
    
    elif (isinstance(in_event, ResipConnectedEvent)):
      toCallEvent = DialogToCallConnectedEvent(self._stateMachine.dialogId,
                                               self._stateMachine.callId,
                                               in_event.getSipMessageCopy())
      if (not self.enqueueEventForCall(toCallEvent)):
        return False
      
      out_pystrNewStateName.set("CONNECTED")
      return True
    
    elif (isinstance(in_event, ResipTerminatedEvent)):
      toCallEvent = DialogToCallTerminatedEvent(self._stateMachine.dialogId,
                                                self._stateMachine.callId)
      if (not self.enqueueEventForCall(toCallEvent)):
        return False
      
      out_pystrNewStateName.set("DISCONNECTED")
      return True
    
    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)
    
    return True

## !!MIRROR FILE, DO NOT MODIFY!!
