#
#  @file   CpaOM.py
#  @author Stephan Goulet
#  date    April 6, 2008
#
#  Copyright (C) 2007 Paraxip Technologies.
#  All Rights Reserved.
#

import datetime
import os

#Log imports
from netborderPyInfra import TraceScope, Logger  
import paraxip.infra.PyLoggingFunctions as PyLoggingFunctions

fileScopeLogger = Logger("netborder.cpa.%s" % os.path.split(__file__)[1])

class CpaOM:

  # The list of possible cpa OMs (Operational Measurement)
  OM_CALL_ID = 'om.call_id'
  OM_CPA_RESULT='om.cpa.result'
  OM_CPA_DATE='om.cpa.date'
  OM_CPA_CAMPAIGN_ID='om.cpa.campaign_id'
  OM_CPA_CAMPAIGN_NAME='om.cpa.campaign_name'  
  OM_CPA_TIME_DIALING='om.cpa.time.dialing'
  OM_CPA_TIME_CONNECTED='om.cpa.time.connected'
  OM_CPA_TIME_CPA_COMPLETED='om.cpa.time.cpa.completed'
  OM_CPA_TIME_QUEUED='om.cpa.time.queued'
  OM_CPA_TIME_AGENT_CONNECTED='om.cpa.time.agent.connected'
  OM_SPECIAL_CALL_INFORMATION='om.cpa.special_call_information'
  
  def isOM(self, in_OMType):
    return self.__omDict.has_key(in_OMType)
  
  def getOM(self, in_OMType):
    return self.__omDict[in_OMType]
  
  def addOM(self, in_strOMName, in_value):
    
    #Only output log (info) messages when handling Cpa result
    if in_strOMName == CpaOM.OM_CPA_RESULT:
      PyLoggingFunctions.log_debug(fileScopeLogger, "addOM[%s]:%s" %(in_strOMName, in_value))
      
      if self.__omDict.has_key(in_strOMName):
        PyLoggingFunctions.log_debug(fileScopeLogger, "addOM[%s] already exists; "
          "'addOM[%s]:%s' will be replaced by 'addOM[%s]:%s'." % \
            (in_strOMName, in_strOMName, self.__omDict[in_strOMName], in_strOMName, in_value) )
    
    self.__omDict[in_strOMName]=in_value

  def addCurrentTimeOM (self, in_strOMName):
    # Utility to add OM that requires current time 
    now=datetime.datetime.now()
    # strftime does not support millisecond, so we need to do the formatting ourself.
    strNowTime='%s.%03d'%(now.strftime("%H:%M:%S") ,now.microsecond/1000)
    self.addOM(in_strOMName,strNowTime)

  def __init__(self):            
    __pychecker__ = 'unusednames=t'
    self.__omDict={}
  
  def reset(self): 
    __pychecker__ = 'unusednames=t'    
    self.__omDict.clear();
## !!MIRROR FILE, DO NOT MODIFY!!
