#
#  @file   AppCall.py
#  @author Stephan Goulet
#  date    May 02, 2008
#  
#  Description:
#
#  Type your description here
#
#  Copyright (C) 2007 Paraxip Technologies.
#  All Rights Reserved.
#

from paraxip.sipua.internal.SipCallEngineSMCall import *
from paraxip.sipua.internal.ConnectionStatus import *
import paraxip.infra.PyLoggingFunctions as PyLoggingFunctions
from netborderPyInfra import *
from PySipMediaServer import *
from RTSPClientExtending import *
from SipCallEngineExtending import *
import SdpHelperFunctions

import sys

# Import can be done in various way. For now I see three ways:
# 
# option 1:
# Example: import JSR309Sample.CallStates as jsrStates.
# Usage: idleState = jsrStates.IdleState(self)
# Comment: Probably the safest in term of name clashing. 
# No relative path supported.
# 
# option 2:
# Example: from .CallStates import IdleState 
# Usage: idleState = IdleState(self)
# Comment: Safe and takes advantage of the relative path. Something
# not available in pure import (option 1). Requires to explicit 
# the list of import. from .CallStates import * is NOT supported.
# produces: SyntaxError: 'import *' not allowed with 'from .'
# 
# option 3:
# Example: from JSR309Sample.CallStates import *
# Usage: idleState = IdleState(self)
# Comment: Least safe but ok if no risk of name clashing.
# 

import JSR309Sample.CallStates as jsrStates
import jsr309.States

#
# JSR309 Sample Application state machine
#

class Call(SipCallEngineSMCall):
  def __init__(self):
    SipCallEngineSMCall.__init__(self)
    
    self.setLogger(CallLogger("netborder.JSR309Sample.Call.py"))
    
    TraceScope(self.getLogger(), "AppCall.__init__")

    self.resetDataMembers()    

    # Create and register the IDLE state    
    idleState = jsrStates.IdleState(self)
    self.setInitialStateName(idleState.getName())
    self.setInitialState(idleState)

    inDialogConnectedState = jsrStates.InDialogConnectedState(self)
    self.addState(inDialogConnectedState.getName(), inDialogConnectedState)
       
    disconnectingState = jsrStates.DisconnectingState(self)
    self.addState(disconnectingState.getName(), disconnectingState) 

    # Programmable state to handle incoming INVITE with SDP
    self._handlingInviteWithSDPState = \
      jsr309.States.HandlingInviteWithSDPState(self, 
          successStateName=inDialogConnectedState.getName(),
          failureStateName=disconnectingState.getName(),
          disconnectStateName=disconnectingState.getName())
    self.addState(self._handlingInviteWithSDPState.getName(), \
        self._handlingInviteWithSDPState)
    
    # Programmable state to handle incoming INVITE without SDP
    self._handlingInviteWithoutSDPState = \
      jsr309.States.HandlingInviteWithoutSDPState(self, 
          successStateName=inDialogConnectedState.getName(),
          failureStateName=disconnectingState.getName(),
          disconnectStateName=disconnectingState.getName())
    self.addState(self._handlingInviteWithoutSDPState.getName(), \
        self._handlingInviteWithoutSDPState)

    # Create and register the final state
    disconnectedState = jsrStates.DisconnectedState(self)
    self.addFinalState(disconnectedState.getName(), disconnectedState)

  def initialize(self, in_refApplication, in_refGlobalContext, in_szCallId):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "AppCall.initialize")

    SipCallEngineSMCall.initialize(self,
                                   in_refApplication,
                                   in_refGlobalContext,
                                   in_szCallId)

    return True

  def getHandlingInviteWithSDPState(self):
    return self._handlingInviteWithSDPState

  def getHandlingInviteWithoutSDPState(self):
    return self._handlingInviteWithoutSDPState

  def resetDataMembers(self): 
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "AppCall.resetDataMembers")
    self.callerSdpOffered = None

    self.inDialogId = None
   
    
    # Data Members retrieved from the invite's announcement
    self.strMediaUrl = None
    self.strMediaFile = None
    
   
    self.audioRTPport = None
    self.sdpAnswer = None
    self.destIp = None

    # SDP and Anouncements parsers
    self.stringUtils = None
    self.myAnnouncementParser = None
    
    self.bIsMediaPlayingOver = False
    self.bcheckFileName = False    
    self.bProvideAnswer = False
    self.inDialogRequestLine = None
    self.callRejectReason = None   
    self.inDialogStatus = ConnectionStatus.Unknown
    self.inDialogToHeader = None
    self.bCallerOnHold = False
    self.bReceivedInfoMsg = False
    self.uiCallerRejectSipErrorCode = None

    #JSR309 data member
    self.jsr309MediaSession = None
    self.jsr309NetworkConnection = None
    self.jsr309Player = None
    self.jsr309MediaGroup = None
    self.jsr309SignalDetector = None

  def reset(self):
    SipCallEngineSMCall.reset(self)
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "AppCall.reset")
    
    SipCallEngineSMCall.reset(self)
    self.resetDataMembers()
    
  def addInDialog(self, in_szDialogId, in_sipCallEngineSMIn):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "AppCall.addInDialog")
    
    SipCallEngineSMCall.addInDialog(self, in_szDialogId, in_sipCallEngineSMIn)
    
    # Save in dialog id
    self.inDialogId = in_szDialogId
  
  def addOutDialog(self, in_szDialogId, in_sipCallEngineSMOut):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "AppCall.addOutDialog")
    
    SipCallEngineSMCall.addOutDialog(self, in_szDialogId,
                                    in_sipCallEngineSMOut)
    
    # Save out dialog id
    self.outDialogId = in_szDialogId

  def scenarioName(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(),
               "AppCall.scenarioName")
    return "Netborder JSR309 Sample Application"
    
  def resetMediaParameters(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "AppCall.resetMediaParameters")

    self.mediaBegin = None
    self.mediaEnd = None
    self.iStartedPlayingAtTime = 0
    self.iPausedAtTime = 0
    self.iMediaBeginTimeSec = 0


  def ParseUrl(self, in_url):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(),"AppCall.ParseUrl")

    if  self.myAnnouncementParser is not None:
      self.myAnnouncementParser.reset()
    else:
      self.myAnnouncementParser = PyAnnouncementParser()

    if not self.myAnnouncementParser.ParseAnnouncement(in_url):
      self.callRejectReason = 'Announcement content could not be retrieved'
      return False

    self.strMediaUrl =  self.myAnnouncementParser.getfmediaUrl()
    self.strStreamingServerAddress =  self.myAnnouncementParser.getfRTSPServerAddress() 
    self.iPauseDuration =  self.myAnnouncementParser.getfpauseTimeOutMs()
    self.iPlayDurationBeforePause =  self.myAnnouncementParser.getfplayTimeOutMs()
    self.iPlayDurationMs = self.myAnnouncementParser.getfdurationMs()
    self.bIsEarlyMediaMode =  self.myAnnouncementParser.getfearlyMedia()
    self.bTrigEarlyMedia =   self.myAnnouncementParser.getfearlyMedia()
    self.bPauseMedia =   self.myAnnouncementParser.getfpause()
    self.strEarlyMediaFirstPrompt =  self.myAnnouncementParser.getfirstPrompt()
    self.iEarlyMediaDurationMs =  self.myAnnouncementParser.getfearlyMediaTimeOutMs()
    self.strEarlyMediaSecondPrompt =   self.myAnnouncementParser.getfsecondPrompt()
    
    self.bDirectConnectFromEarlyMedia = \
      self.myAnnouncementParser.getfdirectConnectFromEarlyMedia()

    self.iWaitBeforeConnectMs = self.myAnnouncementParser.getfwaitBeforeConnectMs()

    #self.strMediaFile = self.myAnnouncementParser.getfmediaFileName()
        
    # Grabing the file name from the end (e.g. uri:rtsp://192.168.11.155/a.wav
    # should give us a.wav
    #tmpTuple = self.strMediaUrl.rpartition("/")
    # Remove any ">" sometimes found when uri is in to header
    #self.strMediaFile = tmpTuple[2].strip(">")

     # Grabing the file name from the end (e.g. uri:file://a.wav
    # should give us a.wav
    tmpTuple = self.strMediaUrl.partition("//")
    # Remove any ">" sometimes found when uri is in to header
    self.strMediaFile = tmpTuple[2].strip(">")

    self.bHangUpAtMediaEnd = self.myAnnouncementParser.getfhangUpAtEnd()

    self.bcheckFileName =  self.myAnnouncementParser.getfcheckFileName()
  

    if(self.bTrigEarlyMedia):

      # set the mediaUrl to the first prompt
      self.strMediaUrl = \
              'rtsp://'+ self.strStreamingServerAddress + '/' + self.strEarlyMediaFirstPrompt
      
    elif(self.strMediaFile is not None and self.strMediaFile != ''):
        self.strMediaUrl = \
             'rtsp://'+ self.strStreamingServerAddress + '/' + self.strMediaFile

        
    return True

  def VerifyInvite(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "AppCall.VerifyInvite")

    self.stringUtils = PyStringUtils()
    # if the request line was not transmitted in url format, dedode
    # the request line
    request = self.stringUtils.decodedURL(self.inDialogRequestLine)
    
    PyLoggingFunctions.log_info(self.getLogger(),"request line : %s", request)
      
    # 2. Verify if the INVITE request line contains a play request
    p = re.compile('(?<=play=).+')
    m = re.search(p, request)

    # we had a play request in the received invite request line
    if not m is None:
      return self.ParseUrl(m.group(0))

    else:
      toHeader = self.stringUtils.decodedURL(self.inDialogToHeader)
      PyLoggingFunctions.log_info(self.getLogger(),"To header : %s", toHeader)
      # 2. Verify if the INVITE To header contains a play request
      p = re.compile('(?<=play=).+')
      m = re.search(p,toHeader)

      if m is not None:
        return  self.ParseUrl(m.group(0))

      # No play request in the INVITE
      else:
        self.callRejectReason = 'Mandatory play parameter missing'
        p = None
        m = None
        PyLoggingFunctions.log_error(self.getLogger(),
                  "AppCall."
                  "Verify Invite no play request in the caller sdp offered")
        return False

  def getRejectMessage(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "AppCall.getRejectMessage")
    
    sipMessage = PySipMessage()
    self.getInDialog(self.inDialogId).constructRejectMessage(sipMessage)

    # Get the sip reject response code
    uiRejectResponseCode = 603 # Decline
    if (self.uiCallerRejectSipErrorCode is not None):
      uiRejectResponseCode = self.uiCallerRejectSipErrorCode
  
    # Add header
    if (self.callRejectReason is not None):
      assert sipMessage.addCustomHeader("MEDIA-RESULT",
                                 self.callRejectReason)
    
    return uiRejectResponseCode, sipMessage

  def createMediaObject(self):    
    __pychecker__ = 'unusednames=t'

    if(self.jsr309MediaSession is None):
      PyLoggingFunctions.log_trace(self.getLogger(), "Creating new media session")
      mediaSessionFactory = self.application.getMediaSessionFactory()
      self.jsr309MediaSession = \
        mediaSessionFactory.createMediaSession(None)
      isLogAssert(self.getLogger(),\
                  self.jsr309MediaSession is not None)

    if(self.jsr309MediaGroup is None):
      PyLoggingFunctions.log_trace(self.getLogger(), "Creating new media group")
      self.jsr309MediaGroup = self.jsr309MediaSession.createMediaGroup()
      isLogAssert(self.getLogger(),\
                  self.jsr309MediaGroup is not None)
    

  def getAudioFilePath (self):
    return self.application.getAudioFilePath()

## !!MIRROR FILE, DO NOT MODIFY!!
