#
#  @file   CallStates.py
#  @author Stephan Goulet
#  date    May 01 2008
#  
#  Description:
#
#  Small sample application to test media proxy code.
#  This example consists in a Mickey Mouse rfc 4240 server implementation
#
#
#  Copyright (C) 2007 Paraxip Technologies.
#  All Rights Reserved.
#

from paraxip.sipua.internal.SipCallEngineState import *
from paraxip.sipua.internal.SipCallEngineEvent import *
from paraxip.sipua.internal.ConnectionStatus import *
from paraxip.sipua.internal.SMTimeoutProcessor import *
import paraxip.infra.PyLoggingFunctions as PyLoggingFunctions
from netborderPyInfra import *
from PySipMediaServer import *
from SipCallEngineExtending import *
from paraxip.infra.Assert import *
import netborderPyInfra

# TODO: do we want to simply import from them, will produce very long name ...
from JSR309APICommonPythonModules import *
from JSR309ClientAPIPythonModules import *
from jsr309 import JSR309Event, JSR309Listener # under python-scripts/internal
import jsr309.States

from time import clock
import re # Regular Expressions

fileScopeLogger = netborderPyInfra.Logger("netborder.JSR309Sample.CallStates.py")

#
# Call states
#

#
# Base class
#
class CallStates(SipCallEngineState):
  def __init__(self, in_stateMachine):
    SipCallEngineState.__init__(self, in_stateMachine)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(),"CallStates.__init__")

  def warnUnexpectedEvent(self, in_strStateName, in_event):
    if (isinstance(in_event, JSR309Event)):
      PyLoggingFunctions.log_warn(self.getLogger(),
             "%s received an invalid JSR309Event %s",
             in_strStateName, in_event.mediaEvent)
    else:      
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               in_event.__class__.__name__)
#
# Idle state
#
class IdleState(CallStates):
  def __init__(self, in_stateMachine):
    CallStates.__init__(self, in_stateMachine)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "IdleState.__init__")

  def getName(self):
    return "IDLE"

  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)

    # Create new JSR309 main media object (i.e. MediaSession and MediaGroup)
    self._stateMachine.createMediaObject()
    
    out_pystrNewStateName.set(self.getName())
        
  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())

    if (isinstance(in_event, InitialEvent)):
      return True
    
    elif (isinstance(in_event, InDialogToCallCalledEvent)):
      self._stateMachine.inDialogStatus = ConnectionStatus.Connecting

      # Save the request line from the in dialog
      self._stateMachine.inDialogRequestLine = \
        in_event.getSipMessage().getRequestUri()
      
      PyLoggingFunctions.log_debug(self.getLogger(),
                "IdleState::processEvent request: %s",
                self._stateMachine.inDialogRequestLine)
       # Save the te To header from the in dialog
      self._stateMachine.inDialogToHeader = \
        in_event.getSipMessage().getHeaderValue("To")
      PyLoggingFunctions.log_debug(self.getLogger(),
                "IdleState::processEvent To: %s",
                 self._stateMachine.inDialogToHeader)

      # Get a JSR309 Network Connection and add a listener
      self._stateMachine.jsr309NetworkConnection = \
                  self._stateMachine.jsr309MediaSession.createNetworkConnection()

      if (isLogAssert(self.getLogger(),\
                self._stateMachine.jsr309NetworkConnection is not None)):
        return True

      myNClistener = JSR309Listener(self._stateMachine.globalContext ,
                                self._stateMachine)
      self._stateMachine.jsr309NetworkConnection.addListener(myNClistener)
      # TODO: life duration needs to be clarified.
      myNClistener = None
      
      # Now wait for SDP offer or request
      return True
    
    elif (isinstance(in_event, DialogToCallSdpOfferedEvent)):      
      # INCOMING INVITE WITH SDP
      
      # Parse the caller sdp offered to retrieve play information
      # and rtp destination ip and ports
      if(self._stateMachine.VerifyInvite()):
        PyLoggingFunctions.log_debug(self.getLogger(),
                  "IdleState::processEvent receives a play request for uri:%s, file:%s",
                  self._stateMachine.strMediaUrl, self._stateMachine.strMediaFile)

        # Retrieve the programmable state to handle INVITEs with an SDP offer 
        #   using the NetworkConnection, set it up, and transition to it
        handlingInviteWithSDPState = \
            self._stateMachine.getHandlingInviteWithSDPState()

        handlingInviteWithSDPState.setNetworkConnection(
            self._stateMachine.jsr309NetworkConnection)
        handlingInviteWithSDPState.setDialogId(in_event.getDialogId())
        handlingInviteWithSDPState.setSDPOffer(in_event.getSdpContents())

        out_pystrNewStateName.set(handlingInviteWithSDPState.getName())
        return True

      # No play Request in the received sdp or sdp invalid
      else:
        PyLoggingFunctions.log_warn(self.getLogger(),
                 "IdleState::processEvent receives an invalid sdp")
        self._stateMachine.uiCallerRejectSipErrorCode = 400
        out_pystrNewStateName.set("DISCONNECTING")
        return True

    elif (isinstance(in_event, DialogToCallSdpOfferRequiredEvent)):
      # Received an INVITE without an SDP

      if(self._stateMachine.VerifyInvite()):

        # Retrieve the programmable state to handle INVITEs without an SDP offer 
        #   using the NetworkConnection, set it up, and transition to it
        handlingInviteWithoutSDPState = \
            self._stateMachine.getHandlingInviteWithoutSDPState()

        handlingInviteWithoutSDPState.setNetworkConnection(
            self._stateMachine.jsr309NetworkConnection)
        handlingInviteWithoutSDPState.setDialogId(in_event.getDialogId())

        out_pystrNewStateName.set(handlingInviteWithoutSDPState.getName())
        return True
      else:
        PyLoggingFunctions.log_warn(self.getLogger(),
                 "IdleState::processEvent receives an invalid invite")
        # Reject the call with an appropriate message
        self._stateMachine.uiCallerRejectSipErrorCode = 400
        out_pystrNewStateName.set("DISCONNECTING")
        return True
        
      return True

    elif (isinstance(in_event, DialogToCallTerminatedEvent)):
      self._stateMachine.inDialogStatus = ConnectionStatus.Disconnected
      out_pystrNewStateName.set("DISCONNECTING")
      return True
    
    elif (isinstance(in_event, StopEvent)):
      out_pystrNewStateName.set("DISCONNECTING")
      return True
    
    else:
      CallStates.warnUnexpectedEvent(self,self.__class__.__name__, in_event)
      
    return True

#
# InDialogConnected state
#
class InDialogConnectedState(CallStates):
  def __init__(self, in_stateMachine):
    CallStates.__init__(self, in_stateMachine)
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "InDialogConnectedState.__init__")
    
  def getName(self):
    return "INDIALOG_CONNECTED"

  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())

    self._stateMachine.inDialogStatus = ConnectionStatus.Connected

    mediaGroup=self._stateMachine.jsr309MediaGroup

    # Get a new JSR309 Player 
    #     
    self._stateMachine.jsr309Player = mediaGroup.getPlayer()
    
    if (isLogAssert(self.getLogger(),\
        self._stateMachine.jsr309Player is not None)):
      return True

    # TODO: Life duration needs to be clarified. We could always reuse the
    #       same listener
    myPlaylistener = JSR309Listener(self._stateMachine.globalContext ,
                                self._stateMachine)
    self._stateMachine.jsr309Player.addListener(myPlaylistener)
    myPlaylistener = None

    # Joining player with NC
    self._stateMachine.jsr309Player.join(Mode.eSEND, \
                                     self._stateMachine.jsr309NetworkConnection)

    if(self._stateMachine.config["jsr309.enableSignalDetection"] == "True"):
      PyLoggingFunctions.log_trace(self.getLogger(), "Enabling signal detection")
      self._stateMachine.jsr309SignalDetector = mediaGroup.getSignalDetector()

      # TODO: Life duration needs to be clarified. We could always reuse the
      #       same listener
      mySDListener = JSR309Listener(self._stateMachine.globalContext ,
                                self._stateMachine)
      self._stateMachine.jsr309SignalDetector.addListener(mySDListener)
      mySDListener = None
      
      params = PyParameters()
      symbol = SignalDetector.p_PatternsAllDigits()
      value = String()
      params.put(symbol,value)
      self._stateMachine.jsr309SignalDetector.setParameters(params)
      
      self._stateMachine.jsr309SignalDetector.join(Mode.eRECEIVE,
                                    self._stateMachine.jsr309NetworkConnection)
      # Installing RTC      
      myRTCArray = RTCArray()
      trigger = SignalDetector.p_PatternsAllDigits()
      action = Player.rtca_stop()
      myRTC = RTC(trigger, action)
      myRTCArray.addRTC(myRTC)    
      self._stateMachine.jsr309SignalDetector.installRTC(myRTCArray,
                                               self._stateMachine.jsr309Player)
  
    # Now ready to request play operation
    strFullFileName = self._stateMachine.getAudioFilePath()
    strFullFileName += "/"
    strFullFileName += self._stateMachine.strMediaFile

    PyLoggingFunctions.log_info(self.getLogger(), "Requesting a play of file:%s ", 
             strFullFileName)

    uri = URI(strFullFileName)

    self._stateMachine.jsr309Player.play(uri)
    
  def processEvent(self, in_event, out_pystrNewStateName):
    __pychecker__ = 'maxreturns=0 maxlines=0'
    
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())
    
    if (isinstance(in_event, DialogToCallTerminatedEvent)):
      #stop the player
      self._stateMachine.jsr309Player.stop()
      self._stateMachine.inDialogStatus = ConnectionStatus.Disconnected          
      out_pystrNewStateName.set("DISCONNECTING")
      return True
    
    elif (isinstance(in_event, StopEvent)):
      out_pystrNewStateName.set("DISCONNECTING")
      return True
      
    else:
      if (isinstance(in_event, JSR309Event) and 
          isinstance(in_event.mediaEvent, PlayFinishedEvent)):
        # Play operation has completed normally. We don't care
        return True
      else:
        CallStates.warnUnexpectedEvent(self,self.__class__.__name__, in_event)
     
    return True
  
#
# Disconnecting state
#
class DisconnectingState(CallStates):
  def __init__(self, in_stateMachine):
    CallStates.__init__(self, in_stateMachine)
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "DisconnectingState.__init__")
  
  def getName(self):
    return "DISCONNECTING"
  
  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    out_pystrNewStateName.set(self.getName())
    
    # Release media session. This will stop anything that might have 
    # been started (playing, signal detection, streaming, etc.)
    # However, this will not release the memory. Memory will be released 
    # only when the all object will have been set to none. In this
    # case when a call to self._stateMachine.resetDataMembers() is made.
    # See DisconnectedState.entryAction()
    self._stateMachine.jsr309MediaSession.release()
        
    if (self._stateMachine.inDialogStatus == ConnectionStatus.Disconnected or \
        self._stateMachine.inDialogStatus == ConnectionStatus.Unknown):
      out_pystrNewStateName.set("DISCONNECTED")
      return

    # Reject the caller if it's connecting
    if (self._stateMachine.inDialogStatus == ConnectionStatus.Connecting):
      rejectResponseCode, rejectSipMessage = self._stateMachine.getRejectMessage()
      if (not self._stateMachine.rejectDialog(
            self._stateMachine.inDialogId,
            rejectResponseCode,
            rejectSipMessage)):
        return
      self._stateMachine.inDialogStatus = ConnectionStatus.Disconnecting

    # Hanging up the caller if it's connected
    elif (self._stateMachine.inDialogStatus == ConnectionStatus.Answering or
          self._stateMachine.inDialogStatus == ConnectionStatus.Connected):
      if (not self._stateMachine.hangupDialog(
            self._stateMachine.inDialogId)):
        return
      self._stateMachine.inDialogStatus = ConnectionStatus.Disconnecting

  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())

    if (isinstance(in_event, DialogToCallTerminatedEvent) and
        (in_event.getDialogId() == self._stateMachine.inDialogId)):
      self._stateMachine.inDialogStatus = ConnectionStatus.Disconnected
      out_pystrNewStateName.set("DISCONNECTED")
      return True

    elif (isinstance(in_event, StopEvent)):
      # Stay in the same state
      return True

    elif (isinstance(in_event, KillEvent)):
      # Go directly in final state
      out_pystrNewStateName.set("DISCONNECTED")
      return True

    else:
      if (isinstance(in_event, JSR309Event) and 
          isinstance(in_event.mediaEvent, PlayFinishedEvent)):
        # Play operation has completed normally. We don't care
        return True
      else:
        CallStates.warnUnexpectedEvent(self,self.__class__.__name__, in_event)
      
    return True

#
# Disconnected state
#
class DisconnectedState(CallStates):
  def __init__(self, in_stateMachine):
    CallStates.__init__(self, in_stateMachine)
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "DisconnectedState.__init__")

  def getName(self):
    return "DISCONNECTED"

  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    out_pystrNewStateName.set(self.getName())

    # Reset data member: this will release JSR309 objects
    self._stateMachine.resetDataMembers()    
  
  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())

    if (isinstance(in_event, JSR309Event) and 
        isinstance(in_event.mediaEvent, PlayFinishedEvent)):
      # Play operation has completed normally. We don't care
      return True
    else:
      CallStates.warnUnexpectedEvent(self,self.__class__.__name__, in_event)

    return True

## !!MIRROR FILE, DO NOT MODIFY!!
