

from paraxip.sipua.user.NetBorderApp import NetBorderApp
from SipCallEngineExtending import PyGetCallIdResult, CallConfigLoader 
import netborderPyInfra 
import paraxip.infra.PyLoggingFunctions as PyLoggingFunctions
from JSR309APICommonPythonModules import *
from JSR309ClientAPIPythonModules import *

import re # Regular Expressions module

# Call is a class part of the agent_bridge.Call module
import paraxip.sipua.user.agent_bridge.CommonCall as Call

fileScopeLogger = netborderPyInfra.Logger(
    "netborder.AgentBridge.CommonApp.py")

class RegistrarErrorAlarm(AlarmImpl):

    def __init__(self):
        AlarmImpl.__init__(self, error)

    def getName(self):
        return "Could not contact registrar at last attempt"

    def getAlarmId(self):
        return self.getName() + " : Agent Bridge"


class CommonApp(NetBorderApp):
  def __init__(self):
    NetBorderApp.__init__(self)
    self._MsControlFactory = None
    self._MediaSesionFactory = None
    self._strAudioFilePath = None

  def onStartup(self, in_refGlobalContext):
    if (not NetBorderApp.onStartup(self, in_refGlobalContext)):
      return False

    __pychecker__ = 'unusednames=t'
    t = netborderPyInfra.TraceScope(fileScopeLogger, 
        "CommonApp.onStartup")
    # Set up the AlarmRegistry and add the RegistrarErrorAlarm
    alarmRegistry = AlarmRegistry.getInstance()
    self._RegErrAlarmHandle = alarmRegistry.addAlarm(RegistrarErrorAlarm())

    # Start JSR309 Server
    self._MsControlFactory = LocalMsControlFactory.getInstance()
    
    self._MsControlFactory.configure()
    self._MsControlFactory.start()

    self._MediaSessionFactory = self._MsControlFactory.\
                                createMediaSessionFactory()

    if self._MediaSessionFactory is None:
        PyLoggingFunctions.log_error(fileScopeLogger,"Cannot get a MediaSessionFactory")
        return False

    return True

  def onShutdownCompleted(self):
    __pychecker__ = 'unusednames=t'
    t = netborderPyInfra.TraceScope(fileScopeLogger, 
                                    "CommonApp.onShutdownCompleted")
    # Stop JSR309 Server
    self._MsControlFactory.stop()
    self._MsControlFactory.waitUntilDone()
    return NetBorderApp.onShutdownCompleted(self)

  def onInviteGetCallId(self, in_sipMessage):
    """
    USER CONFIGURATION 
      Returns the SMCall call id to use as the call of the new dialog (in leg) 
      to be created. Giving the call id of an existing SMCall adds the new in leg
      to it, giving a new call-id creates a new SMCall, returning a SIP error code
      to rejects the call
    """
    __pychecker__ = 'unusednames=t'
    t = netborderPyInfra.TraceScope(fileScopeLogger, 
        "CommonApp.onInviteGetCallId")

    # See if call-id of parked agent is given thru rinstance parameter
    p = re.compile('rinstance=([^ ;>]+)[ ;>]')
    m = p.search(in_sipMessage.getRequestLine())
    if (m is not None):
      strCallIDFromCaller = m.group(1) # Yes
    else:
      strCallIDFromCaller = "" # No, this is an agent wanting to park itself

    if (strCallIDFromCaller.find("NetBorder-Call-Id") == -1):
      # Create a new SMCall by giving out a new UUID
      return PyGetCallIdResult('NetBorder-Call-Id' + netborderPyInfra.UUID().getAsString() + '.agent-bridge')
    else:
      if (self.isCallIdActive(strCallIDFromCaller)):
        return PyGetCallIdResult(strCallIDFromCaller)
      else:
        # Rejecting the call with 500 Internal Server Error
        return PyGetCallIdResult('NetBorder-Call-Id' + netborderPyInfra.UUID().getAsString() + '.agent-bridge',
                                 500, # Internal Server Error
                                 "Server Internal Error (NetBorder-Call-Id can't be found)")
  
  def onClientRegistrationSuccess(self, in_event):
    __pychecker__ = 'unusednames=in_event'
    # appmanager.startTakingCalls()
    return

  def onClientRegistrationFailure(self, in_event):
    __pychecker__ = 'unusednames=in_event'
    # log()
    # exit()
    return

  def onExternalCommand(self, in_event):
    __pychecker__ = 'unusednames=in_event'
    # if event.__name__ == scenario.PredictiveCall.QuiesceCommand.__name__:
    # External Quiesce request
    # appmanager.stopTakingCalls()
    return

  def getMediaSessionFactory(self):
    return self._MediaSessionFactory

  def getAudioFilePath(self):
    # All file are stored relative to the $PARAXIP environment variable
    # Only one call to os.getenv("PARAXIP") is done as it might 
    # be slow
    return self._strAudioFilePath

  def getRegistrarAlarmHandle(self):
    # We can only have on instance of the Registrar Alarm in the application
    # It is acccessed here throught the application
    return self._RegErrAlarmHandle
## !!MIRROR FILE, DO NOT MODIFY!!
