

from paraxip.sipua.internal.SipCallEngineSMCall import *
from paraxip.sipua.internal.ConnectionStatus import *
from paraxip.sipua.user.agent_bridge.CommonStates import *
import paraxip.sipua.user.b2bua.B2buaStates
import paraxip.infra.PyLoggingFunctions as PyLoggingFunctions
from netborderPyInfra import *
from SipCallEngineExtending import *
import ParameterInfo
import JSR309APICommonPythonModules as jsr309common

import CommonCall

import os

# Force declaration of class attributes in __init__ method
__pychecker__ = "initattr"

#
# Prompt Parameters that are assigned in the sub-classes 
# (DTMFEntry and XMLBased)

# Return parameter information for this Call 
#def getParameterInfoDB():
#
#  parameterInfoDB = ParameterInfo.ParameterInfoDB()
#
#
#
#  return parameterInfoDB


#
# Agent-bridge state machine
#

class CommonCall(SipCallEngineSMCall):

# Valid parameter names for this Call
  PARAM_AGENT_REALM =                           'agent.realm'
  PARAM_AGENT_PASSWORD =                        'agent.password'
  PARAM_AGENT_REGISTRATION_TIME_SEC =           'agent.registration.time.sec'
  PARAM_SIP_SEND_RINGING_BEFORE_ANSWER =        'sip.sendRingingBeforeAnswer'
  
  PARAM_PROMPT_WELCOME         = 'prompt.welcome'
  PARAM_PROMPT_REGISTERFAIL    = 'prompt.registerFail'
  PARAM_PROMPT_REGISTERSUCCESS = 'prompt.registerSuccess'
  PARAM_PROMPT_REGISTERREMOVED = 'prompt.registerRemoved'
  PARAM_PROMPT_PARKING         = 'prompt.parking'
  PARAM_PROMPT_UNPARKING       = 'prompt.unParking'

  # Return parameter information for this Call 
  @staticmethod
  def getParameterInfoDB():
    parameterInfoDB = ParameterInfo.ParameterInfoDB()
  
    parameterInfoDB.addParameterInfoForName(
        CommonCall.PARAM_AGENT_REALM,
        type=str,
        mandatory=False)
    parameterInfoDB.addParameterInfoForName(
        CommonCall.PARAM_AGENT_PASSWORD,
        type=str,
        mandatory=False)
    parameterInfoDB.addParameterInfoForName(
        CommonCall.PARAM_AGENT_REGISTRATION_TIME_SEC,
        type=int,
        defaultValue=3600)
    parameterInfoDB.addParameterInfoForName(
        CommonCall.PARAM_SIP_SEND_RINGING_BEFORE_ANSWER,
        type=bool,
        defaultValue=False)

    return parameterInfoDB



  def __init__(self):
    SipCallEngineSMCall.__init__(self)
    
    self.setLogger(CallLogger("netborder.AgentBridge.CommonCall.py"))
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "CommonCall.__init__")

    self.resetDataMembers()

    # IDLE state
    idleState = IdleState(self)
    self.setInitialStateName(idleState.getName())
    self.setInitialState(idleState)

    # DISCONNECTING state
    self.disconnectingState = DisconnectingState(self)
    self.addState(self.disconnectingState.getName(),self.disconnectingState)

    # DISCONNECTED state
    disconnectedState = DisconnectedState(self)
    self.addFinalState(disconnectedState.getName(),disconnectedState)

    # REGISTERING_AGENT state
    registeringAgentState = RegisteringAgentState(self)
    self.addState(registeringAgentState.getName(), registeringAgentState)
  
    # REGISTER_REMOVED state
    registerRemovedState = RegisterRemovedState(self)
    self.addState(registerRemovedState.getName(), registerRemovedState)

    # AGENT_PARKED state
    agentParkedState = AgentParkedState(self)
    self.addState(agentParkedState.getName(), agentParkedState)

    # HANDLE_AGENT_REINVITE_WITH_SDP
    # Programmable state to handle incoming agent re-INVITEs with SDP while
    #  it is parked
    self._handlingAgentReInviteWithSDPState = \
      jsr309.States.HandlingReInviteWithSDPState(self, 
          successStateName=agentParkedState.getName(),
          failureStateName=self.disconnectingState.getName(),
          disconnectStateName=self.disconnectingState.getName())
    self._handlingAgentReInviteWithSDPState.setName(
        'HANDLE_AGENT_REINVITE_WITH_SDP')
    self.addState(self._handlingAgentReInviteWithSDPState.getName(), \
        self._handlingAgentReInviteWithSDPState)
    
    # HANDLE_AGENT_REINVITE_WITHOUT_SDP
    # Programmable state to handle incoming re-INVITE without SDP while
    #  it is parked
    # TODO on INVITE without SDP, make sure the offer is still sendonly
    self._handlingAgentReInviteWithoutSDPState = \
      jsr309.States.HandlingReInviteWithoutSDPState(self, 
          successStateName=agentParkedState.getName(),
          failureStateName=self.disconnectingState.getName(),
          disconnectStateName=self.disconnectingState.getName())
    self._handlingAgentReInviteWithoutSDPState.setName(
        'HANDLE_AGENT_REINVITE_WITHOUT_SDP')
    self.addState(self._handlingAgentReInviteWithoutSDPState.getName(), \
        self._handlingAgentReInviteWithoutSDPState)

    # PARKING_AGENT
    # Programmable state performing hold following RFC3264
    self._parkingAgentState = \
      jsr309.States.PuttingSessionOnHoldRFC3264State(self, 
          successStateName=agentParkedState.getName(),
          failureStateName=self.disconnectingState.getName(),
          disconnectStateName=self.disconnectingState.getName())
    self._parkingAgentState.setName('PARKING_AGENT')
    self.addState(self._parkingAgentState.getName(), self._parkingAgentState)

    # ESTABLISHING_SESSION_WITH_AGENT_REQUESTOR state
    establishingSessionWithAgentRequestorState = \
      EstablishingSessionWithAgentRequestorState(self)
    self.addState(establishingSessionWithAgentRequestorState.getName(),
                  establishingSessionWithAgentRequestorState)

    # AGENT_AND_REQUESTOR_IN_SESSION state
    agentAndRequestorInSessionState = \
      AgentAndRequestorInSessionState(self)
    self.addState(agentAndRequestorInSessionState.getName(),
                  agentAndRequestorInSessionState)

    # REINVITED_WHILE_AGENT_AND_REQUESTOR_IN_SESSION derived form B2BUA 
    #   Reinvited programmable state
    self._reInvitedWhileAgentAndRequestorInSessionState = \
        ReinvitedState(self,
          successStateName=agentAndRequestorInSessionState.getName(),
          failureStateName=self.disconnectingState.getName(),
          disconnectStateName=self.disconnectingState.getName())
    self._reInvitedWhileAgentAndRequestorInSessionState.setName(
        "REINVITED_WHILE_AGENT_AND_REQUESTOR_IN_SESSION")
    self.addState(self._reInvitedWhileAgentAndRequestorInSessionState.getName(), 
        self._reInvitedWhileAgentAndRequestorInSessionState)

    # ON_HOLD state
    # Agent and Requestor were put on hold following 'hold' NOTIFY from the 
    #   Requestor (Broadsoft extensions)
    onHoldState = OnHoldState(self)
    self.addState(onHoldState.getName(), onHoldState)

    # PUTTING_AGENT_REQUESTOR_ON_HOLD state
    self._puttingAgentRequestorOnHoldState = \
      b2bua.B2buaStates.PuttingSessionOnHoldRFC2543(self)
    self._puttingAgentRequestorOnHoldState.setName(
        "PUTTING_AGENT_REQUESTOR_ON_HOLD")
    self._puttingAgentRequestorOnHoldState.setSuccessStateName(
        onHoldState.getName())
    self._puttingAgentRequestorOnHoldState.setFailureStateName(
        self.disconnectingState.getName())
    self._puttingAgentRequestorOnHoldState.setDisconnectStateName(
        self.disconnectingState.getName())
    self.addState(self._puttingAgentRequestorOnHoldState.getName(), \
        self._puttingAgentRequestorOnHoldState)

    # PUTTING_AGENT_ON_HOLD state
    self._puttingAgentOnHoldState = \
      b2bua.B2buaStates.PuttingSessionOnHoldRFC2543(self)
    self._puttingAgentOnHoldState.setName("PUTTING_AGENT_ON_HOLD")
    self._puttingAgentOnHoldState.setSuccessStateName(
        self._puttingAgentRequestorOnHoldState.getName())
    self._puttingAgentRequestorOnHoldState.setFailureStateName(
        self.disconnectingState.getName())
    self._puttingAgentRequestorOnHoldState.setDisconnectStateName(
        self.disconnectingState.getName())
    self.addState(self._puttingAgentOnHoldState.getName(), \
        self._puttingAgentOnHoldState)

    # PUTTING_AGENT_AND_REQUESTOR_OFF_HOLD state
    puttingAgentAndRequestorOffHoldState = PuttingAgentAndRequestorOffHold(self)
    self.addState(puttingAgentAndRequestorOffHoldState.getName(),
        puttingAgentAndRequestorOffHoldState)

  # State accessors
  def getHandlingAgentInviteWithSDPState(self):
    return self._handlingAgentInviteWithSDPState

  def getHandlingAgentInviteWithoutSDPState(self):
    return self._handlingAgentInviteWithoutSDPState

  def getParkingAgentState(self):
    return self._parkingAgentState

  def getHandlingAgentReInviteWithSDPState(self):
    return self._handlingAgentReInviteWithSDPState

  def getHandlingAgentReInviteWithoutSDPState(self):
    return self._handlingAgentReInviteWithoutSDPState

  def getReInvitedWhileAgentAndRequestorInSessionState(self):
    return self._reInvitedWhileAgentAndRequestorInSessionState

  def getPuttingAgentOnHoldState(self):
    return self._puttingAgentOnHoldState

  def getPuttingAgentRequestorOnHoldState(self):
    return self._puttingAgentRequestorOnHoldState

  # Call I/F
  def initialize(self, in_refApplication, in_refGlobalContext, in_szCallId):
    SipCallEngineSMCall.initialize(self,
                                   in_refApplication,
                                   in_refGlobalContext,
                                   in_szCallId)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "CommonCall.initialize")

    return True

  # Call I/F
  def configure(self, in_dictConfig):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "CommonCall.configure")
    
    # realm and password for registration must be set both or not set at all
    realmDefined = in_dictConfig.has_key(CommonCall.PARAM_AGENT_REALM)
    passwordDefined = in_dictConfig.has_key(CommonCall.PARAM_AGENT_PASSWORD)
    if (realmDefined and not passwordDefined) or \
        (not realmDefined and passwordDefined):
      PyLoggingFunctions.log_error(self.getLogger(), 'Parameters %s and %s must both be set to'
          ' provide credentials for registration' %
          (CommonCall.PARAM_AGENT_REALM, CommonCall.PARAM_AGENT_PASSWORD))
      return False

    SipCallEngineSMCall.configure(self, in_dictConfig)

    return True

  def resetDataMembers(self): 
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "CommonCall.resetDataMembers")
   
    #
    # Agent
    #
    self.agentDialogId = None
    self.agentLegStatus = ConnectionStatus.Unknown

    self.agentDN = ""

    self.uiRejectAgentSipErrorCode = None
    self.strRejectAgentReason = None

    self.agentRegistration = None

    #
    # Agent Requestor
    #
    self.agentRequestorDialogId = None
    self.agentRequestorLegStatus = ConnectionStatus.Unknown
    self.agentRequestorSdp = None   

    #
    # Others
    #
    self.dialogIdInitiatingDisconnect = None
    self.inviteSipMessage = None

    self.ReinvitedByAgent = False
    self.ReinvitedByAgentRequestor = False

    self.uiCallerRejectSipErrorCode = None
    self.callToRejectDialogId = None
    self.callToRejectLegStatus = ConnectionStatus.Unknown
    
    #
    #JSR309 data members
    # 
    self.jsr309MediaSession = None
    self.jsr309NetworkConnection = None
    self.jsr309SignalDetector = None
    self.jsr309Player = None
    self.jsr309MediaGroup = None

  def reset(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(),"CommonCall.reset")

    SipCallEngineSMCall.reset(self)
    self.resetDataMembers()

  def addInDialog(self, in_szDialogId, in_sipCallEngineSMIn):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "CommonCall.addInDialog")

    SipCallEngineSMCall.addInDialog(self, in_szDialogId, in_sipCallEngineSMIn)

    # First incoming call is from the agent
    if (self.agentDialogId == None):
      # Save in the agent dialog id
      self.agentDialogId = in_szDialogId
      
  def addOutDialog(self, in_szDialogId, in_sipCallEngineSMOut):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(),"CommonCall.addOutDialog")

    SipCallEngineSMCall.addOutDialog(self, in_szDialogId,in_sipCallEngineSMOut)

  def scenarioName(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(),
               "CommonCall.scenarioName")
    return "Netborder XMLBased Agent Bridge"
 
  def getAudioFilePath (self):
    return self.application.getAudioFilePath()

  # Helpers
  def getRejectMessage(self, in_szDialogId):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(),"CommonCall.getRejectMessage")
    sipMessage = PySipMessage()

    # Get the sip reject response code
    uiRejectResponseCode = 603 # Decline

    self.getInDialog(in_szDialogId).constructRejectMessage(sipMessage)
    if (self.uiRejectAgentSipErrorCode is not None):
      uiRejectResponseCode = self.uiRejectAgentSipErrorCode
   
    # Add agent-bridge header
    if (self.strRejectAgentReason is not None):
      assert sipMessage.addCustomHeader('NetBorder-ABS-Result', self.strRejectAgentReason)

    return uiRejectResponseCode, sipMessage

  def canGoToDisconnected(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(),"CommonCall.canGoToDisconnected")
    PyLoggingFunctions.log_trace(self.getLogger(), "canGoToDisconnected: self.agentLegStatus=" +
        str(self.agentLegStatus) + " self.agentRequestorLegStatus=" + 
        str(self.agentRequestorLegStatus) + "  self.callToRejectLegStatus=" +
        str(self.callToRejectLegStatus))

    if (self.agentLegStatus == ConnectionStatus.Disconnected or \
        self.agentLegStatus == ConnectionStatus.Unknown) and \
       (self.agentRequestorLegStatus == ConnectionStatus.Disconnected or \
        self.agentRequestorLegStatus  == ConnectionStatus.Unknown) and \
       (self.callToRejectLegStatus == ConnectionStatus.Disconnected or \
        self.callToRejectLegStatus == ConnectionStatus.Unknown):
      return True

    return False

  def isDialogOnHold(self, in_szSdp):

    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(),"CommonCall.isDialogOnHold")

    p = re.compile('(?<=c=).+')
    m = re.search(p, in_szSdp)
    
    if m is not None:
      k = re.split(' ',m.group(0))
      if k[2] is not None:
        p = re.compile('(?<=)[0-9.]+')
        m = re.search(p,k[2])
        if m is not None and m.group(0) == '0.0.0.0':
          return True
  
    return False

  def getMediaSession(self):    
    __pychecker__ = 'unusednames=t'
    if(self.jsr309MediaSession is None):
      PyLoggingFunctions.log_trace(self.getLogger(), "Creating new media session")
      mediaSessionFactory = self.application.getMediaSessionFactory()
      self.jsr309MediaSession = \
        mediaSessionFactory.createMediaSession(None)
      isLogAssert(self.getLogger(),\
                  self.jsr309MediaSession is not None)

    return self.jsr309MediaSession

  def getMediaGroup(self):
    __pychecker__ = 'unusednames=t'
    if(self.jsr309MediaGroup is None):
      PyLoggingFunctions.log_trace(self.getLogger(), "Creating new media group")
      self.jsr309MediaGroup = self.getMediaSession().createMediaGroup()
      isLogAssert(self.getLogger(),\
                  self.jsr309MediaGroup is not None)
    return self.jsr309MediaGroup

  def startPlayback(self, promptFileParamName):
    strFullFileName = self.getAudioFilePath()
    strFullFileName += self.config[promptFileParamName]
    PyLoggingFunctions.log_info(self.getLogger(), "Requesting playback of file: %s ", 
        strFullFileName)
    # URI resip class doesn't accept relative paths
    uri = jsr309common.URI(os.path.abspath(strFullFileName))
    self.jsr309Player.play(uri)

  def getPlayedFileURI(self, promptFileParamName):
    strFullFileName = self.getAudioFilePath()
    strFullFileName += self.config[promptFileParamName]
    strUri = os.path.abspath(strFullFileName)
    return strUri

  def transitionToParkingAgentState(self, out_pystrNewStateName):
    parkingAgentState = self.getParkingAgentState()
    parkingAgentState.setNetworkConnection(
        self.jsr309NetworkConnection)
    parkingAgentState.setDialogId(self.agentDialogId)
    out_pystrNewStateName.set(parkingAgentState.getName())

  def getRegistrarAlarmHandle(self):
    return self.application.getRegistrarAlarmHandle()
## !!MIRROR FILE, DO NOT MODIFY!!
