

from paraxip.sipua.internal.SipCallEngineState import *
from paraxip.sipua.internal.SipCallEngineEvent import *
from paraxip.sipua.internal.ConnectionStatus import *
from paraxip.sipua.internal.SMTimeoutProcessor import *
import paraxip.infra.PyLoggingFunctions as PyLoggingFunctions
from SipCallEngineExtending import *
from paraxip.infra.Assert import *
import netborderPyInfra
from paraxip.sipua.internal.jsr309.States import JSR309Event, JSR309Listener # under python-scripts/internal
import paraxip.sipua.user.b2bua.B2buaStates

import CommonCall as Call

import JSR309APICommonPythonModules as jsr309common
from JSR309ClientAPIPythonModules import PyParameters

fileScopeLogger = netborderPyInfra.Logger(
    "netborder.AgentBridge.CommonStates.py")

# Force declaration of class attributes in __init__ method
__pychecker__ = "initattr"

#
# Call states
#

#
# Base class
#

class CommonState(SipCallEngineState):
  def __init__(self, in_stateMachine):
    SipCallEngineState.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "CommonState.__init__")

  def warnUnexpectedEvent(self, in_strStateName, in_event):
    if (isinstance(in_event, JSR309Event)):
      PyLoggingFunctions.log_warn(self.getLogger(),
             "%s received an unexpected JSR309Event %s",
             in_strStateName, in_event.mediaEvent)
    else:      
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               in_strStateName, in_event.__class__.__name__)

#
# Idle state
#

class IdleState(CommonState):
  def __init__(self, in_stateMachine):
    CommonState.__init__(self, in_stateMachine)
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "IdleState._init__")
  
  def getName(self):
    return "IDLE"

  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)

    # Get a new JSR309 Media Session
    self._stateMachine.getMediaSession()

    out_pystrNewStateName.set(self.getName())

  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())

    if (isinstance(in_event, InitialEvent)):
      return True

    elif (isinstance(in_event, InDialogToCallCalledEvent)): #AGENT
      self._stateMachine.agentLegStatus = ConnectionStatus.Connecting

      # Save the initial invite message
      self._stateMachine.inviteSipMessage = in_event.getSipMessage()

      # Get a JSR309 Network Connection and add a listener
      self._stateMachine.jsr309NetworkConnection = \
                  self._stateMachine.jsr309MediaSession.createNetworkConnection()

      if (isLogAssert(self.getLogger(),\
                self._stateMachine.jsr309NetworkConnection is not None)):
        return True

      myNClistener = JSR309Listener(self._stateMachine.globalContext,
                                    self._stateMachine)
      self._stateMachine.jsr309NetworkConnection.addListener(myNClistener)

      # Stay in the same state, sdp offer or need for one coming up
      return True

    elif isinstance(in_event, DialogToCallSdpOfferedEvent): #AGENT

      # INCOMING AGENT INVITE WITH SDP

      # Retrieve the programmable state to handle INVITEs with an SDP offer 
      #   using the NetworkConnection, set it up, and transition to it
      handlingAgentInviteWithSDPState = \
          self._stateMachine.getHandlingAgentInviteWithSDPState()

      handlingAgentInviteWithSDPState.setNetworkConnection(
          self._stateMachine.jsr309NetworkConnection)
      handlingAgentInviteWithSDPState.setDialogId(in_event.getDialogId())
      handlingAgentInviteWithSDPState.setSDPOffer(in_event.getSdpContents())

      out_pystrNewStateName.set(handlingAgentInviteWithSDPState.getName())
      return True

    elif isinstance(in_event, DialogToCallSdpOfferRequiredEvent): #AGENT

      # INCOMING AGENT INVITE WITHOUT SDP

      # Retrieve the programmable state to handle INVITEs with an SDP offer 
      #   using the NetworkConnection, set it up, and transition to it
      handlingAgentInviteWithoutSDPState = \
          self._stateMachine.getHandlingAgentInviteWithoutSDPState()

      handlingAgentInviteWithoutSDPState.setNetworkConnection(
          self._stateMachine.jsr309NetworkConnection)
      handlingAgentInviteWithoutSDPState.setDialogId(in_event.getDialogId())

      out_pystrNewStateName.set(handlingAgentInviteWithoutSDPState.getName())
      return True

    elif (isinstance(in_event, DialogToCallTerminatedEvent)):
      self._stateMachine.dialogIdInitiatingDisconnect = in_event.getDialogId()
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    elif (isinstance(in_event, StopEvent)):
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)
    
    return True

#
# RegisteringAgent state
#

class RegisteringAgentState(CommonState):
  def __init__(self, in_stateMachine):
    CommonState.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "RegisteringAgentState.__init__")
  
  def getName(self):
    return "REGISTERING_AGENT"

  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())
    
    # Reset the new call parameters only if we don't have a pending
    #   agent requestor call
    if (self._stateMachine.callToRejectLegStatus == ConnectionStatus.Unknown or \
        self._stateMachine.callToRejectLegStatus == ConnectionStatus.Disconnected):
      self._stateMachine.callToRejectDialogId = None
      self._stateMachine.callToRejectLegStatus = ConnectionStatus.Unknown

    strRegistrar = self._stateMachine.registrarURL
    strAgentDN = self._stateMachine.agentDN

    PyLoggingFunctions.log_info(self.getLogger(), "About to register agent %s on registrar %s"
                               % (strAgentDN, strRegistrar))

    PyLoggingFunctions.log_debug(self.getLogger(), "The Call Id for this agent is: %s",
                                self._stateMachine._strCallId)

    # Contact: sip:netborder-callid-agentCallId@agent-bridge
    contactHeaderList = [self._stateMachine._strCallId]

    # Address of Record:
    # To: sip:agentDN@registrar
    strToHeader = "sip:" + strAgentDN + "@" + strRegistrar
    strRegistrarUri = "sip:" + strRegistrar

    # set the default registration time in the userProfile
    userProfile = self._stateMachine.globalContext.getMasterUserProfileCopy()
    userProfile.setDefaultRegistrationTime(
      self._stateMachine.config[Call.CommonCall.PARAM_AGENT_REGISTRATION_TIME_SEC])

    # Set up credentials if provided
    # We already verified in configure() that realm and password 
    # are either both set or none set
    if self._stateMachine.config.has_key(Call.CommonCall.PARAM_AGENT_REALM):
      digestCredential = PyDigestCredential(
          self._stateMachine.config[Call.CommonCall.PARAM_AGENT_REALM], 
          strAgentDN, 
          self._stateMachine.config[Call.CommonCall.PARAM_AGENT_PASSWORD])
      userProfile.setDigestCredential(digestCredential)
    
    self._stateMachine.globalContext.makeRegistration(
      self._stateMachine.getCallId(),
      strRegistrarUri,
      strToHeader,
      userProfile,
      contactHeaderList,
      {},
      self._stateMachine._strCallId) # rInstance = netborder-callid-agentCallId

  def processEvent(self, in_event, out_pystrNewStateName):
    __pychecker__ = 'maxreturns=0 maxlines=0'
    
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())
    
    if (isinstance(in_event, ResipRegistrationSucceededEvent)):
      # Keep a reference on the registration client callback object
      self._stateMachine.agentRegistration = in_event.getClientRegistration()
      self._stateMachine.agentLegStatus = ConnectionStatus.Connected
      self._stateMachine.getRegistrarAlarmHandle().reset()
      # play success prompt
      self._stateMachine.startPlayback(Call.CommonCall.PARAM_PROMPT_REGISTERSUCCESS)

      # Stay in same state and wait for PlayFinishedEvent
      return True

    elif (isinstance(in_event, JSR309Event) and 
          isinstance(in_event.mediaEvent, jsr309common.PlayFinishedEvent) and
            in_event.mediaEvent.getStreamID() == \
              self._stateMachine.getPlayedFileURI(
                Call.CommonCall.PARAM_PROMPT_REGISTERSUCCESS)):
      # Play operation has completed normally, now park the agent
      self._stateMachine.transitionToParkingAgentState(out_pystrNewStateName)
      return True
    
    elif (isinstance(in_event, JSR309Event) and 
          isinstance(in_event.mediaEvent, jsr309common.PlayFinishedEvent)):
      # We received a Play Finished Event that was started in another State 
      # We ignore it and stay in the same state
      return True
      
    elif (isinstance(in_event, ResipRegistrationFailureEvent)):
      self._stateMachine.agentRegistration = None
      self._stateMachine.getRegistrarAlarmHandle().set()
      out_pystrNewStateName.set("REGISTER_FAILED")
      return True
    
    elif (isinstance(in_event, ResipRegistrationRemovedEvent)):
      self._stateMachine.agentRegistration = None

      out_pystrNewStateName.set("REGISTER_REMOVED")
      return True

    elif (isinstance(in_event, InDialogToCallCalledEvent)): #agent requestor
      self._stateMachine.callToRejectLegStatus = ConnectionStatus.Connecting
      # agent requestor trying to contact us while we are not ready yet to 
      #   accept calls. Save the dialogId in order to reject the call with a 
      #   480 Temporarily Unavailable response code when the 
      #   DialogToCallSdpOfferedEvent will be received.
      self._stateMachine.callToRejectDialogId = in_event.getDialogId()

      # Stay in the same state
      return True

    elif (isinstance(in_event, DialogToCallSdpOfferedEvent) and
          (in_event.getDialogId() == self._stateMachine.callToRejectDialogId)):
      #AGENT REQUESTOR second call
      # Reject the call from the agent requestor
      if (not self._stateMachine.rejectDialog(
            self._stateMachine.callToRejectDialogId,
            480)): # Temporarily Unavailable
        return False
      self._stateMachine.callToRejectLegStatus =  ConnectionStatus.Disconnecting

      # stay in the same state
      return True

    elif (isinstance(in_event, DialogToCallSdpOfferRequiredEvent) and
          (in_event.getDialogId() ==  self._stateMachine.callToRejectDialogId)):
      #AGENT REQUESTOR second call

      # Reject the second call from the agent requestor
      if (not self._stateMachine.rejectDialog(
            self._stateMachine.callToRejectDialogId,
            480)): # Temporarily Unavailable
        return False
      self._stateMachine.callToRejectLegStatus =  ConnectionStatus.Disconnecting

      # stay in the same state
      return True

    elif (isinstance(in_event, DialogToCallTerminatedEvent) and
          (in_event.getDialogId() == self._stateMachine.callToRejectDialogId)): #AGENT REQUESTOR second call
      self._stateMachine.callToRejectLegStatus =  ConnectionStatus.Disconnected

      # stay in the same state
      return True

    elif (isinstance(in_event, DialogToCallTerminatedEvent) and
          (in_event.getDialogId() == self._stateMachine.agentDialogId)):#AGENT
      self._stateMachine.dialogIdInitiatingDisconnect = in_event.getDialogId()
      out_pystrNewStateName.set("DISCONNECTING")
      return True
      
    elif (isinstance(in_event, StopEvent)):
      out_pystrNewStateName.set("DISCONNECTING")
      return True
      
    else:
      CommonState.warnUnexpectedEvent(self,self.__class__.__name__, in_event)
       
    return True

  #-----------------------------------------------------------------------------#

#
# AgentParked state
#

class AgentParkedState(CommonState):
  def __init__(self, in_stateMachine):
    CommonState.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "AgentParkedState.__init__")

  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)

    if self._stateMachine.config.has_key(Call.CommonCall.PARAM_PROMPT_PARKING):
      # we only play the parking prompt if its defined in properties
      self._stateMachine.startPlayback(Call.CommonCall.PARAM_PROMPT_PARKING)


    out_pystrNewStateName.set(self.getName())

    # go to the ESTABLISHING_SESSION_WITH_AGENT_REQUESTOR state
    # if we have a pending agent requestor call
    if self._stateMachine.agentRequestorLegStatus == ConnectionStatus.Connecting :
      out_pystrNewStateName.set("ESTABLISHING_SESSION_WITH_AGENT_REQUESTOR")

  def getName(self):
    return "AGENT_PARKED"
  
  def processEvent(self, in_event, out_pystrNewStateName):
    __pychecker__ = 'maxreturns=0 maxlines=0'
    
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())

    if isinstance(in_event, DialogToCallSdpOfferedEvent) and \
        in_event.getDialogId() == self._stateMachine.agentDialogId: #AGENT

      # RE-INVITE WITH SDP FROM AGENT

      # Retrieve the programmable state to handle re-INVITEs with an SDP offer 
      #   using the NetworkConnection, set it up, and transition to it
      handlingAgentReInviteWithSDPState = \
          self._stateMachine.getHandlingAgentReInviteWithSDPState()

      handlingAgentReInviteWithSDPState.setNetworkConnection(
          self._stateMachine.jsr309NetworkConnection)
      handlingAgentReInviteWithSDPState.setDialogId(in_event.getDialogId())
      handlingAgentReInviteWithSDPState.setSDPOffer(in_event.getSdpContents())

      out_pystrNewStateName.set(handlingAgentReInviteWithSDPState.getName())
      return True

    elif isinstance(in_event, DialogToCallSdpOfferRequiredEvent) and \
        in_event.getDialogId() == self._stateMachine.agentDialogId: #AGENT

      # RE-INVITE WITHOUT SDP FROM AGENT

      # Retrieve the programmable state to handle INVITEs with an SDP offer 
      #   using the NetworkConnection, set it up, and transition to it
      handlingAgentReInviteWithoutSDPState = \
          self._stateMachine.getHandlingAgentReInviteWithoutSDPState()

      handlingAgentReInviteWithoutSDPState.setNetworkConnection(
          self._stateMachine.jsr309NetworkConnection)
      handlingAgentReInviteWithoutSDPState.setDialogId(in_event.getDialogId())

      out_pystrNewStateName.set(handlingAgentReInviteWithoutSDPState.getName())
      return True

    elif (isinstance(in_event, InDialogToCallCalledEvent)): #AGENT REQUESTOR
      self._stateMachine.agentRequestorLegStatus = ConnectionStatus.Connecting

      # Request from agent requestor to reach an agent
      # Save the agent requestor dialog Id
      self._stateMachine.agentRequestorDialogId = in_event.getDialogId()
      inDialogSM = self._stateMachine.getInDialog(in_event.getDialogId())
      if (inDialogSM is not None):
        inDialogSM.sipLeg.useDefaultLocalContact()

      # Stay in the same state, sdp offer or need for one coming up
      return True

    elif (isinstance(in_event, DialogToCallSdpOfferedEvent) and
          (in_event.getDialogId() == self._stateMachine.agentRequestorDialogId)): 
      #AGENT REQUESTOR

      # INVITE with SDP from agent requestor. Save the SDP offered then 
      #   transition to ESTABLISHING_SESSION_WITH_AGENT_REQUESTOR
      self._stateMachine.agentRequestorSdp = in_event.getSdpContents()

      if self._stateMachine.config.has_key(Call.CommonCall.PARAM_PROMPT_UNPARKING):
        # we only play the parking prompt if its defined in properties
        # and we wait for the prompt to finish before moving to the next state
        self._stateMachine.startPlayback(Call.CommonCall.PARAM_PROMPT_UNPARKING)
      else:
        out_pystrNewStateName.set("ESTABLISHING_SESSION_WITH_AGENT_REQUESTOR")

      return True

    elif (isinstance(in_event, DialogToCallSdpOfferRequiredEvent) and
          (in_event.getDialogId() == self._stateMachine.agentRequestorDialogId)): 
      #AGENT REQUESTOR

      # INVITE without SDP from agent requestor: will re-invite the agent w/o SDP in
      # ESTABLISHING_SESSION_WITH_AGENT_REQUESTOR
      self._stateMachine.agentRequestorSdp = None
      
      if self._stateMachine.config.has_key(Call.CommonCall.PARAM_PROMPT_UNPARKING):
        # we only play the parking prompt if its defined in properties
        # and we wait for the prompt to finish before moving to the next state
        self._stateMachine.startPlayback(Call.CommonCall.PARAM_PROMPT_UNPARKING)
      else:
        out_pystrNewStateName.set("ESTABLISHING_SESSION_WITH_AGENT_REQUESTOR")

      return True

    elif (isinstance(in_event, ResipRegistrationRemovedEvent)):
      self._stateMachine.agentRegistration = None
      out_pystrNewStateName.set("REGISTER_REMOVED")
      return True

    elif (isinstance(in_event, ResipRegistrationFailureEvent)):
      if self._stateMachine.agentRegistration is not None:
        self._stateMachine.agentRegistration.end()
        self._stateMachine.agentRegistration = None
      out_pystrNewStateName.set("REGISTER_REMOVED")
      return True

    elif (isinstance(in_event, JSR309Event) and 
      isinstance(in_event.mediaEvent, jsr309common.PlayFinishedEvent) and
      in_event.mediaEvent.getStreamID() == \
        self._stateMachine.getPlayedFileURI(
          Call.CommonCall.PARAM_PROMPT_PARKING)):
      # Parking prompt finished playing, nothing to do
      return True

    elif (isinstance(in_event, JSR309Event) and 
      isinstance(in_event.mediaEvent, jsr309common.PlayFinishedEvent) and
      in_event.mediaEvent.getStreamID() == \
        self._stateMachine.getPlayedFileURI(
          Call.CommonCall.PARAM_PROMPT_UNPARKING)):
      # UnParking prompt finished playing, go to EstablishSessionwithRequestor
      out_pystrNewStateName.set("ESTABLISHING_SESSION_WITH_AGENT_REQUESTOR")
      return True

    elif isinstance(in_event, DialogToCallTerminatedEvent) and \
        in_event.getDialogId() == self._stateMachine.agentDialogId:#AGENT
      self._stateMachine.dialogIdInitiatingDisconnect = in_event.getDialogId()
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    elif (isinstance(in_event, DialogToCallTerminatedEvent) and
          (in_event.getDialogId() == self._stateMachine.callToRejectDialogId)): #AGENT REQUESTOR second call
      # stay in the same state
      self._stateMachine.callToRejectLegStatus = ConnectionStatus.Disconnected
      return True

    elif (isinstance(in_event, DialogToCallTerminatedEvent) and
          (in_event.getDialogId() == self._stateMachine.agentRequestorDialogId)): #AGENT REQUESTOR
      # agent requestor tryin to reach an agent who is onhold. The call is
      # rejected and the terminated event for the agent requestor leg might
      # be received in that case

      # stay in the same state
      self._stateMachine.agentRequestorLegStatus = ConnectionStatus.Disconnected
      return True

    elif (isinstance(in_event, DialogToCallAckNotReceivedEvent)):
      # Can happen here after answering 200 Ok to the agent's INVITE
      # TODO use mutable logger for this warning
      PyLoggingFunctions.log_warn(self.getLogger(), "The SIP UAC did not answer our " \
        "SIP 200 response with a ACK request as specified by RFC 3261. " \
        "Terminating the call. (State: %s)",self.getName())
      
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    elif (isinstance(in_event, StopEvent)):
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    else:
      CommonState.warnUnexpectedEvent(self,self.__class__.__name__, in_event)

    return True


#
# Reinvited state
# 
# NOTE: This class is derived from a B2BUA state
#

class ReinvitedState(b2bua.B2buaStates.ReinvitedState):
  
  def __init__(self, in_stateMachine, successStateName, 
      failureStateName, disconnectStateName):
    b2bua.B2buaStates.ReinvitedState.__init__(self, in_stateMachine,
      successStateName, failureStateName, disconnectStateName)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "ReinvitedState.__init__")
  
  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())
    
    if (isinstance(in_event, InDialogToCallCalledEvent)): #agent requestor
      self._stateMachine.callToRejectLegStatus = ConnectionStatus.Connecting
      # agent requestor tryin to contact us while we are not ready yet to accept calls.
      # Save the dialogId in order to reject the call with a 480 Temporarily
      # Unavailable response code when the DialogToCallSdpOfferedEvent will be received.
      self._stateMachine.callToRejectDialogId = in_event.getDialogId()

      # Stay in the same state
      return True

    elif ((isinstance(in_event, DialogToCallSdpOfferedEvent) or \
           isinstance(in_event, DialogToCallSdpOfferRequiredEvent)) and \
          (in_event.getDialogId() ==  self._stateMachine.callToRejectDialogId)): #AGENT REQUESTOR second call
      # Reject the second call from the agent requestor
      if (not self._stateMachine.rejectDialog(
            self._stateMachine.callToRejectDialogId,
            480)): # Temporarily Unavailable
        return False
      self._stateMachine.callToRejectLegStatus =  ConnectionStatus.Disconnecting

      # stay in the same state
      return True

    else:
      # processEvent from based class (B2BUA)
      return b2bua.B2buaStates.ReinvitedState.processEvent(
        self, in_event, out_pystrNewStateName)

    return True

#
# EstablishingSessionWithAgentRequestor state
#

class EstablishingSessionWithAgentRequestorState(CommonState):
  def __init__(self, in_stateMachine):
    CommonState.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "EstablishingSessionWithAgentRequestorState.__init__")
  
  def getName(self):
    return "ESTABLISHING_SESSION_WITH_AGENT_REQUESTOR"
  
  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())
    
    # TODO_IBM unparking prompt playback

    # Reset the new call parameters only if we don't have a pending
    # agent requestor call
    if self._stateMachine.callToRejectLegStatus == ConnectionStatus.Unknown or \
           self._stateMachine.callToRejectLegStatus == ConnectionStatus.Disconnected:
      self._stateMachine.callToRejectDialogId = None
      self._stateMachine.callToRejectLegStatus =  ConnectionStatus.Unknown

    # Re-invite the agent with the agent requestor sdp
    if (not self._stateMachine.reInviteDialog(
      self._stateMachine.agentDialogId,
      self._stateMachine.agentRequestorSdp)):
      return False
    
  def processEvent(self, in_event, out_pystrNewStateName):
    __pychecker__ = 'maxreturns=0 maxlines=0'
    
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())

    if (isinstance(in_event, DialogToCallFailedEvent) and
          (in_event.getDialogId() == self._stateMachine.agentDialogId)): #AGENT
      # Agent refused the re-INVITE with the new SDP
      PyLoggingFunctions.log_warn(self.getLogger(), 'Media capabilities requested by incoming'\
          ' call not supported by Agent SIP device')
      # Send back 488 Not acceptable here
      self._stateMachine.agentRequestorLegStatus = ConnectionStatus.Disconnecting
      if (not self._stateMachine.getInDialog(
              self._stateMachine.agentRequestorDialogId).reject(488)):
        PyLoggingFunctions.log_error(self.getLogger(),
                  "EstablishingSessionWithAgentRequestorState re-invite rejected")
        return False

      # Move the agent back in Parked state
      self._stateMachine.transitionToParkingAgentState(out_pystrNewStateName)
      return True

    elif (isinstance(in_event, DialogToCallSdpAnsweredEvent) and
          (in_event.getDialogId() == self._stateMachine.agentDialogId)): #AGENT
      # Agent accepted the re-INVITE with the new SDP
      agentSdp = in_event.getSdpContents()

      # Send an 180 Ringing message before the 200 Ok if configured to do so
      if(self._stateMachine.config[
          Call.CommonCall.PARAM_SIP_SEND_RINGING_BEFORE_ANSWER]):
        if (not self._stateMachine.ringInDialog(
              self._stateMachine.agentRequestorDialogId):
          return False

      #  Send a 200 ok to the agent requestor with the agent sdp
      if (not self._stateMachine.answerInDialog(
            self._stateMachine.agentRequestorDialogId,
            agentSdp)):
        return False
      self._stateMachine.agentRequestorLegStatus = ConnectionStatus.Answering

      # Stay in the same state
      return True

    elif (isinstance(in_event, DialogToCallSdpOfferedEvent) and
          (in_event.getDialogId() == self._stateMachine.agentDialogId)): #AGENT
      # Agent requestor called without SDP
      # Save the agent SDP offered
      agentSdp = in_event.getSdpContents()

      sipMessage = PySipMessage()
      inDialogSM = self._stateMachine.getInDialog(self._stateMachine.agentRequestorDialogId)
      inDialogSM.constructAnswerMessage(sipMessage)
    
      # Answer agent requestor with the agent SDP offer
      if (not self._stateMachine.answerInDialog(
            self._stateMachine.agentRequestorDialogId,
            agentSdp,
            True)):
        return False

      self._stateMachine.agentRequestorLegStatus = ConnectionStatus.Answering
      
      # Don't change state - DialogToCallSdpAnsweredEvent coming in
      return True
    
    elif (isinstance(in_event, DialogToCallSdpAnsweredEvent) and
          (in_event.getDialogId() == self._stateMachine.agentRequestorDialogId)):#AGENT REQUESTOR
      # Send the agent requestor SDP answer (in ACK) as the agent SDP answer (in ACK)
      # (agent requestor sent INVITE w/o SDP)
      # NOTE this is behaviour common to the b2bua
      self._stateMachine.agentRequestorSdp = in_event.getSdpContents()
      if (not self._stateMachine.provideSdpAnswerToDialog(
            self._stateMachine.agentDialogId,
            self._stateMachine.agentRequestorSdp)):
        return False
      
      # Do not change state: DialogToCallConnectedEvent coming just after
      return True

    elif (isinstance(in_event, DialogToCallConnectedEvent) and
          (in_event.getDialogId() == self._stateMachine.agentRequestorDialogId)):#AGENT REQUESTOR
      self._stateMachine.agentRequestorLegStatus = ConnectionStatus.Connected

      # Done, incoming call from agent requestor connected
      out_pystrNewStateName.set("AGENT_AND_REQUESTOR_IN_SESSION")
      return True

    elif (isinstance(in_event, InDialogToCallCalledEvent)): #AGENT REQUESTOR
      self._stateMachine.callToRejectLegStatus = ConnectionStatus.Connecting
      # agent requestor tryin to contact us while we are not ready yet to accept calls.
      # Save the dialogId in order to reject the call with a 480 Temporarily
      # Unavailable response code when the DialogToCallSdpOfferedEvent will be received.
      self._stateMachine.callToRejectDialogId = in_event.getDialogId()

      # Stay in the same state
      return True

    elif (isinstance(in_event, DialogToCallSdpOfferedEvent) and
          (in_event.getDialogId() ==  self._stateMachine.callToRejectDialogId)): 
      #AGENT REQUESTOR simultaneous call to same agent

      # Reject the second call from the agent requestor
      if (not self._stateMachine.rejectDialog(
            self._stateMachine.callToRejectDialogId,
            480)): # Temporarily Unavailable
        return False
      self._stateMachine.callToRejectLegStatus =  ConnectionStatus.Disconnecting

      # stay in the same state
      return True

    elif (isinstance(in_event, DialogToCallSdpOfferRequiredEvent) and
          (in_event.getDialogId() ==  self._stateMachine.callToRejectDialogId)):
      #AGENT REQUESTOR simultaneous call to same agent
      # Reject the second call from the agent requestor
      if (not self._stateMachine.rejectDialog(
            self._stateMachine.callToRejectDialogId,
            480)): # Temporarily Unavailable
        return False
      self._stateMachine.callToRejectLegStatus = ConnectionStatus.Disconnecting

      # stay in the same state
      return True

    elif (isinstance(in_event, ResipRegistrationRemovedEvent)):
      self._stateMachine.agentRegistration = None

      out_pystrNewStateName.set("REGISTER_REMOVED")
      return True

    elif (isinstance(in_event, ResipRegistrationFailureEvent)):
      if self._stateMachine.agentRegistration is not None:
        self._stateMachine.agentRegistration.end()
        self._stateMachine.agentRegistration = None

      out_pystrNewStateName.set("REGISTER_REMOVED")
      return True  

    elif (isinstance(in_event, DialogToCallTerminatedEvent) and
          (in_event.getDialogId() == self._stateMachine.callToRejectDialogId)): #AGENT REQUESTOR second call
      self._stateMachine.callToRejectLegStatus =  ConnectionStatus.Disconnected

      # stay in the same state
      return True

    elif (isinstance(in_event, DialogToCallTerminatedEvent) and
          (in_event.getDialogId() == self._stateMachine.agentDialogId)): #AGENT
      self._stateMachine.dialogIdInitiatingDisconnect = in_event.getDialogId()

      out_pystrNewStateName.set("DISCONNECTING")
      return True

    elif (isinstance(in_event, DialogToCallTerminatedEvent) and
          (in_event.getDialogId() == self._stateMachine.agentRequestorDialogId)):
          #AGENT REQUESTOR
      # Agent requestor hung up before we where done exchanging SDPs with the 
      #   agent
      self._stateMachine.agentRequestorLegStatus = ConnectionStatus.Disconnected      
      # Park the agent
      self._stateMachine.transitionToParkingAgentState(out_pystrNewStateName)
      return True

    elif (isinstance(in_event, StopEvent)):
      out_pystrNewStateName.set("DISCONNECTING")
      return True
    
    else:
      CommonState.warnUnexpectedEvent(self,self.__class__.__name__, in_event)

    return True


#
# AgentAndRequestorInSession state
#

class AgentAndRequestorInSessionState(CommonState):
  def __init__(self, in_stateMachine):
    CommonState.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "AgentAndRequestorInSessionState.__init__")
  
  def getName(self):
    return "AGENT_AND_REQUESTOR_IN_SESSION"
  
  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())

    # Reset the new call parameters only if we don't have a pending
    # agent requestor call
    if self._stateMachine.callToRejectLegStatus == ConnectionStatus.Unknown or \
           self._stateMachine.callToRejectLegStatus == ConnectionStatus.Disconnected:
      self._stateMachine.callToRejectDialogId = None
      self._stateMachine.callToRejectLegStatus =  ConnectionStatus.Unknown

    if self._stateMachine.ReinvitedByAgent:
      agentSdp = self._stateMachine.\
          getReInvitedWhileAgentAndRequestorInSessionState().getReinvitedSdp()
      self._stateMachine.agentRequestorSdp = self._stateMachine.\
          getReInvitedWhileAgentAndRequestorInSessionState().getPeerSdp()
      self._stateMachine.ReinvitedByAgent = False

    elif self._stateMachine.ReinvitedByAgentRequestor:
      agentSdp = self._stateMachine.\
          getReInvitedWhileAgentAndRequestorInSessionState().getPeerSdp()
      self._stateMachine.agentRequestorSdp = self._stateMachine.\
          getReInvitedWhileAgentAndRequestorInSessionState().getReinvitedSdp()
      self._stateMachine.ReinvitedByAgentRequestor = False
    
  def processEvent(self, in_event, out_pystrNewStateName):
    __pychecker__ = 'maxreturns=0 maxlines=0'
    
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())
    
    if ((isinstance(in_event, DialogToCallSdpOfferedEvent) or \
         isinstance(in_event, DialogToCallSdpOfferRequiredEvent)) and \
        in_event.getDialogId() == self._stateMachine.agentDialogId):
      # re-INVITE received from Agent (with SDP on DialogToCallSdpOfferedEvent, 
      #   without SDP on OFFER_REQUIRED)
      # The REINVITED state is generic; tell the state what is the peer dialog
      # where the re-INVITE must be relayed (here agent requestor)
      reInvitedState = self._stateMachine.\
          getReInvitedWhileAgentAndRequestorInSessionState()
      reInvitedState.setPeerDialogId(self._stateMachine.agentRequestorDialogId)
      self._stateMachine.ReinvitedByAgent = True
      out_pystrNewStateName.set(reInvitedState.getName())
      return True

    elif ((isinstance(in_event, DialogToCallSdpOfferedEvent) or \
           isinstance(in_event, DialogToCallSdpOfferRequiredEvent)) and \
          in_event.getDialogId() == self._stateMachine.agentRequestorDialogId):
      # re-INVITE received from agent requestor 
      # The REINVITED state is generic; tell the state what is the peer dialog
      # where the re-INVITE must be relayed (here the Agent)
      reInvitedState = self._stateMachine.\
          getReInvitedWhileAgentAndRequestorInSessionState()
      reInvitedState.setPeerDialogId(self._stateMachine.agentDialogId)
      self._stateMachine.ReinvitedByAgentRequestor = True
      out_pystrNewStateName.set(reInvitedState.getName())
      return True

    elif (isinstance(in_event, InDialogToCallCalledEvent)): #agent requestor
      self._stateMachine.callToRejectLegStatus = ConnectionStatus.Connecting
      # agent requestor trying to contact us while we are already in a session
      # Save the dialogId in order to reject the call with a 480 Temporarily
      # Unavailable response code when the DialogToCallSdpOfferedEvent will be 
      #   received.
      self._stateMachine.callToRejectDialogId = in_event.getDialogId()

      # Stay in the same state
      return True

    elif (isinstance(in_event, DialogToCallSdpOfferedEvent) and
          (in_event.getDialogId() == self._stateMachine.callToRejectDialogId)):
      #AGENT REQUESTOR second call
      # Reject the second call from the agent requestor (agent already in session)
      if (not self._stateMachine.rejectDialog(
            self._stateMachine.callToRejectDialogId,
            480)): # Temporarily Unavailable
        return False
      self._stateMachine.callToRejectLegStatus =  ConnectionStatus.Disconnecting
      PyLoggingFunctions.log_info(self.getLogger(), "Request for an agent already in session")

      return True

    elif (isinstance(in_event, DialogToCallSdpOfferRequiredEvent) and
          (in_event.getDialogId() ==  self._stateMachine.callToRejectDialogId)): 
      #AGENT REQUESTOR second call

      # Reject the second call from the agent requestor
      if (not self._stateMachine.rejectDialog(
            self._stateMachine.callToRejectDialogId,
            480)): # Temporarily Unavailable
        return False
      self._stateMachine.callToRejectLegStatus =  ConnectionStatus.Disconnecting

      # stay in the same state
      return True

    elif (isinstance(in_event, DialogToCallNotifyEvent) and \
          in_event.getDialogId() == self._stateMachine.agentRequestorDialogId):

      # NOTIFY received from the Agent Requestor: check is for which event
      eventName = in_event.getEventName()
      PyLoggingFunctions.log_debug(self.getLogger(), "Received NOTIFY message with event \"%s\"" % 
          eventName)

      if eventName == "talk":
        # SIP Broadsoft extension event "talk" (click-to-answer)
        # Nothing to do as the agent and requestor are already in session
        pass 
      elif eventName == "hold":
        # SIP Broadsoft extension event "hold" (click-to-hold)
        # Set the session IDs for legs that will be put on hold
        self._stateMachine.getPuttingAgentOnHoldState().setDialogId(
            self._stateMachine.agentDialogId)
        self._stateMachine.getPuttingAgentRequestorOnHoldState().setDialogId(
            self._stateMachine.agentRequestorDialogId)
        # Next state is putting the agent on hold
        out_pystrNewStateName.set(
            self._stateMachine.getPuttingAgentOnHoldState().getName())
      else:
        PyLoggingFunctions.log_info(self.getLogger(), 
            "Received NOTIFY message with unknown event \"%s\"" % eventName)
      return True

    elif (isinstance(in_event, ResipRegistrationRemovedEvent)):
      self._stateMachine.agentRegistration = None
      out_pystrNewStateName.set("REGISTER_REMOVED")
      return True

    elif (isinstance(in_event, ResipRegistrationFailureEvent)):
      if self._stateMachine.agentRegistration is not None:
        self._stateMachine.agentRegistration.end()
        self._stateMachine.agentRegistration = None
      out_pystrNewStateName.set("REGISTER_REMOVED")
      return True  
   
    elif (isinstance(in_event, DialogToCallTerminatedEvent) and
          (in_event.getDialogId() ==  self._stateMachine.callToRejectDialogId)): #AGENT REQUESTOR second call
      # stay in the same state
      self._stateMachine.callToRejectLegStatus =  ConnectionStatus.Disconnected
      return True

    elif (isinstance(in_event, DialogToCallTerminatedEvent) and
          (in_event.getDialogId() == self._stateMachine.agentDialogId)): #AGENT
      self._stateMachine.dialogIdInitiatingDisconnect = in_event.getDialogId()
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    elif (isinstance(in_event, DialogToCallTerminatedEvent) and
          (in_event.getDialogId() == self._stateMachine.agentRequestorDialogId)): 
      #AGENT REQUESTOR
      self._stateMachine.agentRequestorLegStatus = ConnectionStatus.Disconnected

      # Agent Requestor hung up, park the agent
      self._stateMachine.transitionToParkingAgentState(out_pystrNewStateName)
      return True

    elif (isinstance(in_event, StopEvent)):
      out_pystrNewStateName.set("DISCONNECTING")
      return True
      
    else:
      CommonState.warnUnexpectedEvent(self,self.__class__.__name__, in_event)

    return True

#
# RegisterRemoved state
#

class RegisterRemovedState(CommonState):
  def __init__(self, in_stateMachine):
    CommonState.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "RegisterRemovedState.__init__")
  
  def getName(self):
    return "REGISTER_REMOVED"

  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())
    
    # advertise the agent that he has been unregistered
    # play system error prompt
    self._stateMachine.startPlayback(Call.CommonCall.PARAM_PROMPT_REGISTERREMOVED)
    
  def processEvent(self, in_event, out_pystrNewStateName):
    __pychecker__ = 'maxreturns=0 maxlines=0'
    
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())

    if (isinstance(in_event, JSR309Event) and 
          isinstance(in_event.mediaEvent, jsr309common.PlayFinishedEvent) and
            in_event.mediaEvent.getStreamID() == \
              self._stateMachine.getPlayedFileURI(
                Call.CommonCall.PARAM_PROMPT_REGISTERREMOVED)):
      # Play operation has completed normally.
      # go to DISCONNECTING state
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    elif (isinstance(in_event, ResipRegistrationRemovedEvent)):
      self._stateMachine.agentRegistration = None
      return True

    elif (isinstance(in_event, DialogToCallTerminatedEvent)):
      self._stateMachine.dialogIdInitiatingDisconnect = in_event.getDialogId()
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    elif (isinstance(in_event, StopEvent)):
      out_pystrNewStateName.set("DISCONNECTING")
      return True
    
    else:
      CommonState.warnUnexpectedEvent(self,self.__class__.__name__, in_event)

    return True

#
# Disconnecting state
#

class DisconnectingState(CommonState):
  def __init__(self, in_stateMachine):
    CommonState.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "DisconnectingState.__init__")
    self.waitForRegistrationRemoved = False
  
  def getName(self):
    return "DISCONNECTING"
  
  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)
    out_pystrNewStateName.set(self.getName())

    self.waitForRegistrationRemoved = False

    if(self._stateMachine.agentRegistration is not None):
      self._stateMachine.agentRegistration.end()
      self._stateMachine.agentRegistration = None
      self.waitForRegistrationRemoved = True

    if (self._stateMachine.dialogIdInitiatingDisconnect is not None):
      if (self._stateMachine.dialogIdInitiatingDisconnect == \
          self._stateMachine.agentDialogId):
        self._stateMachine.agentLegStatus = ConnectionStatus.Disconnected
      else:
        self._stateMachine.agentRequestorLegStatus = ConnectionStatus.Disconnected

    if(self._stateMachine.canGoToDisconnected() and not self.waitForRegistrationRemoved):
      out_pystrNewStateName.set("DISCONNECTED")
      return

    # Reject the second call from the agent requestor if it's connecting
    if self._stateMachine.callToRejectLegStatus == ConnectionStatus.Connecting:
      if (not self._stateMachine.rejectDialog(
            self._stateMachine.callToRejectDialogId,
            480)): # Temporarily Unavailable
        return
      self._stateMachine.callToRejectLegStatus = ConnectionStatus.Disconnecting

    # Reject the agent if it's connecting
    if (self._stateMachine.agentLegStatus == ConnectionStatus.Connecting):
      rejectResponseCode, rejectSipMessage = \
               self._stateMachine.getRejectMessage(self._stateMachine.agentDialogId)
      if (not self._stateMachine.rejectDialog(
            self._stateMachine.agentDialogId,
            rejectResponseCode,
            rejectSipMessage)):
        return
      self._stateMachine.agentLegStatus = ConnectionStatus.Disconnecting

    # Hanging up the agent if it's connected
    elif (self._stateMachine.agentLegStatus == ConnectionStatus.Answering or
          self._stateMachine.agentLegStatus == ConnectionStatus.Connected):
      if (not self._stateMachine.hangupDialog(
            self._stateMachine.agentDialogId)):
        return
      self._stateMachine.agentLegStatus = ConnectionStatus.Disconnecting

    # Reject the agent requestor if it's connecting
    if (self._stateMachine.agentRequestorLegStatus == ConnectionStatus.Connecting):
      rejectResponseCode, rejectSipMessage = \
        self._stateMachine.getRejectMessage(self._stateMachine.agentRequestorDialogId)
      if (not self._stateMachine.rejectDialog(
            self._stateMachine.agentRequestorDialogId,
            rejectResponseCode,
            rejectSipMessage)):
        return
      self._stateMachine.agentRequestorLegStatus = ConnectionStatus.Disconnecting

    # Hanging up the agent requestor if it's connected
    elif (self._stateMachine.agentRequestorLegStatus == ConnectionStatus.Answering or
          self._stateMachine.agentRequestorLegStatus == ConnectionStatus.Connected):
      if (not self._stateMachine.hangupDialog(
            self._stateMachine.agentRequestorDialogId)):
        return
      self._stateMachine.agentRequestorLegStatus = ConnectionStatus.Disconnecting
    
    out_pystrNewStateName.set(self.getName())

  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())

    if (isinstance(in_event, DialogToCallTerminatedEvent)):
      if ((in_event.getDialogId() == self._stateMachine.agentDialogId)): #AGENT
        self._stateMachine.agentLegStatus = ConnectionStatus.Disconnected
      elif ((in_event.getDialogId() == self._stateMachine.agentRequestorDialogId)): #AGENT REQUESTOR
        self._stateMachine.agentRequestorLegStatus = ConnectionStatus.Disconnected
      elif ((in_event.getDialogId() == self._stateMachine.callToRejectDialogId)): #SECOND AGENT REQUESTOR CALL
        self._stateMachine.callToRejectLegStatus = ConnectionStatus.Disconnected

      if (self._stateMachine.canGoToDisconnected() and not self.waitForRegistrationRemoved):
        out_pystrNewStateName.set("DISCONNECTED")
        
      return True

    elif (isinstance(in_event, ResipRegistrationRemovedEvent)):
      self._stateMachine.agentRegistration = None
      self.waitForRegistrationRemoved = False
      if (self._stateMachine.canGoToDisconnected()):
        out_pystrNewStateName.set("DISCONNECTED")
   
      return True
      
    elif (isinstance(in_event, JSR309Event) and 
      isinstance(in_event.mediaEvent, jsr309common.PlayFinishedEvent)):
      # Prompt finished playing, nothing to do
      return True  

    elif (isinstance(in_event, StopEvent)):
      # Stay in the same state
      return True

    elif (isinstance(in_event, KillEvent)):
      # Go directly in final state
      out_pystrNewStateName.set("DISCONNECTED")
      return True
    
    else:
      CommonState.warnUnexpectedEvent(self,self.__class__.__name__, in_event)
    
    return True

#
# Disconnected state
#

class DisconnectedState(CommonState):
  def __init__(self, in_stateMachine):
    CommonState.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "DisconnectedState.__init__")
  
  def getName(self):
    return "DISCONNECTED"
  
  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    out_pystrNewStateName.set(self.getName())

    # IMPORTANT Reset data member: this will release JSR309 objects
    self._stateMachine.resetDataMembers()    

  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    out_pystrNewStateName.set(self.getName())
    
    # No event should be received in this state
    CommonState.warnUnexpectedEvent(self,self.__class__.__name__, in_event)
    
    return True

#
# OnHoldState state
#

class OnHoldState(CommonState):
  def __init__(self, in_stateMachine):
    CommonState.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "OnHoldState.__init__")
  
  def getName(self):
    return "ON_HOLD"
  
  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    out_pystrNewStateName.set(self.getName())
    
    if isinstance(in_event, DialogToCallNotifyEvent) and \
        in_event.getDialogId() == self._stateMachine.agentRequestorDialogId:

      # NOTIFY received from the Agent Requestor: check is for which event
      eventName = in_event.getEventName()
      PyLoggingFunctions.log_debug(self.getLogger(), "Received NOTIFY message with event \"%s\"" % 
          eventName)

      if eventName == "talk":
        # SIP Broadsoft extension event "talk" (click-to-answer)
        out_pystrNewStateName.set("PUTTING_AGENT_AND_REQUESTOR_OFF_HOLD")
        pass 
      elif eventName == "hold":
        # SIP Broadsoft extension event "hold" (click-to-hold)
        # Nothing to do as the agent and requestor are already on hold 
        #  (unexpected)
        PyLoggingFunctions.log_info(self.getLogger(), "Received NOTIFY message with event 'hold' "
            "when already on hold") 
      else:
        PyLoggingFunctions.log_info(self.getLogger(), 
            "Received NOTIFY message with unknown event \"%s\"")
      return True

    elif (isinstance(in_event, ResipRegistrationRemovedEvent)):
      self._stateMachine.agentRegistration = None
      out_pystrNewStateName.set("REGISTER_REMOVED")
      return True

    elif (isinstance(in_event, ResipRegistrationFailureEvent)):
      if self._stateMachine.agentRegistration is not None:
        self._stateMachine.agentRegistration.end()
        self._stateMachine.agentRegistration = None
      out_pystrNewStateName.set("REGISTER_REMOVED")
      return True  

    elif (isinstance(in_event, DialogToCallTerminatedEvent) and
          (in_event.getDialogId() == self._stateMachine.agentDialogId)): #AGENT
      self._stateMachine.dialogIdInitiatingDisconnect = in_event.getDialogId()
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    elif (isinstance(in_event, DialogToCallTerminatedEvent) and
          (in_event.getDialogId() == self._stateMachine.agentRequestorDialogId)): 
      #AGENT REQUESTOR
      self._stateMachine.agentRequestorLegStatus = ConnectionStatus.Disconnected

      # Agent Requestor hung up, park the agent
      self._stateMachine.transitionToParkingAgentState(out_pystrNewStateName)
      return True

    elif (isinstance(in_event, StopEvent)):
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)
    
    return True

#
# PuttingAgentAndRequestorOffHold state
# TODO_IBM some overlap with ReEstablishingMediaWithMediaServer3pcc1State 
#   from AgentBridgeStates, consider a generic ReInvite3pccFlow1 state
#

class PuttingAgentAndRequestorOffHold(CommonState):
  def __init__(self, in_stateMachine):
    CommonState.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(),
                   "PuttingAgentAndRequestorOffHold.__init__")
  
  def getName(self):
    return "PUTTING_AGENT_AND_REQUESTOR_OFF_HOLD"
 
  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())

    # RE-INVITE the Media Server without SDP
    if (not self._stateMachine.reInviteDialog(
          self._stateMachine.agentDialogId)):
      return

  def processEvent(self, in_event, out_pystrNewStateName):
    __pychecker__ = 'maxreturns=0 maxlines=0'
    
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())
    
    if (isinstance(in_event, DialogToCallFailedEvent) and
        (in_event.getDialogId() == self._stateMachine.agentDialogId)):
      # Agent refused the re-INVITE without SDP (unexpected)
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    elif (isinstance(in_event, DialogToCallSdpOfferedEvent) and
          (in_event.getDialogId() == self._stateMachine.agentDialogId)):
      # Agent offered a SDP to the re-INVITE without SDP
      agentSdp = in_event.getSdpContents()

      # RE-INVITE the Requestor with the Agent SDP
      if (not self._stateMachine.reInviteDialog(
            self._stateMachine.agentRequestorDialogId,
            agentSdp)):
        return False

      # Stay in the same state
      return True

    elif (isinstance(in_event, DialogToCallFailedEvent) and
          (in_event.getDialogId() == self._stateMachine.agentRequestorDialogId)):
      # Requestor refused the re-INVITE with the new SDP
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    elif (isinstance(in_event, DialogToCallSdpAnsweredEvent) and
          (in_event.getDialogId() == self._stateMachine.agentRequestorDialogId)):
      # Requestor accepted the re-INVITE with the new SDP
      # Save the Requestor SDP
      self._stateMachine.agentRequestorSdp = in_event.getSdpContents()

      # Answer the Agent with the Requestor SDP
      if (not self._stateMachine.provideSdpAnswerToDialog(
            self._stateMachine.agentDialogId,
            self._stateMachine.agentRequestorSdp)):
        return False

      # All done
      out_pystrNewStateName.set("AGENT_AND_REQUESTOR_IN_SESSION")
      return True

    elif (isinstance(in_event, ResipRegistrationRemovedEvent)):
      self._stateMachine.agentRegistration = None
      out_pystrNewStateName.set("REGISTER_REMOVED")
      return True

    elif (isinstance(in_event, ResipRegistrationFailureEvent)):
      if self._stateMachine.agentRegistration is not None:
        self._stateMachine.agentRegistration.end()
        self._stateMachine.agentRegistration = None
      out_pystrNewStateName.set("REGISTER_REMOVED")
      return True

    elif (isinstance(in_event, DialogToCallTerminatedEvent) and
          (in_event.getDialogId() == self._stateMachine.agentRequestorDialogId)): #AGENT REQUESTOR 
      # stay in the same state
      self._stateMachine.agentRequestorLegStatus = ConnectionStatus.Disconnected
      return True

    # TODO_IBM what is the use of this?
    elif (isinstance(in_event, DialogToCallTerminatedEvent) and
          (in_event.getDialogId() == self._stateMachine.callToRejectDialogId)): #AGENT REQUESTOR second call
      # stay in the same state
      self._stateMachine.callToRejectLegStatus =  ConnectionStatus.Disconnected
      return True

    elif (isinstance(in_event, DialogToCallTerminatedEvent) and 
          (in_event.getDialogId() == self._stateMachine.agentDialogId)): #AGENT
      self._stateMachine.dialogIdInitiatingDisconnect = in_event.getDialogId()
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    elif (isinstance(in_event, StopEvent)):
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)

    return True

#-----------------------------------------------------------------------------#
# Programmable states subclasses to handle Disconnect events
# TODO_IBM should be handled by base classes directly

import jsr309.States # under python-scripts/internal

# Handles incoming INVITE with SDP using an NC
class HandlingInviteWithSDPState(jsr309.States.HandlingInviteWithSDPState):

  def __init__(self, in_stateMachine, successStateName, 
      failureStateName, disconnectStateName):
    jsr309.States.HandlingInviteWithSDPState.__init__(self, in_stateMachine,
        successStateName, failureStateName, disconnectStateName)

  def processEvent(self, in_event, out_pystrNewStateName):
    __pychecker__ = 'maxreturns=0'
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    if isinstance(in_event, DialogToCallTerminatedEvent):
      self._stateMachine.dialogIdInitiatingDisconnect = in_event.getDialogId()
      out_pystrNewStateName.set(self._disconnectStateName)
      return True
    else:
      return jsr309.States.HandlingInviteWithSDPState.processEvent(self,
          in_event, out_pystrNewStateName)

# Handles incoming INVITE without SDP using an NC
class HandlingInviteWithoutSDPState(jsr309.States.HandlingInviteWithoutSDPState):

  def __init__(self, in_stateMachine, successStateName, 
      failureStateName, disconnectStateName):
    jsr309.States.HandlingInviteWithoutSDPState.__init__(self, in_stateMachine,
        successStateName, failureStateName, disconnectStateName)

  def processEvent(self, in_event, out_pystrNewStateName):
    __pychecker__ = 'maxreturns=0'
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    if isinstance(in_event, DialogToCallTerminatedEvent):
      self._stateMachine.dialogIdInitiatingDisconnect = in_event.getDialogId()
      out_pystrNewStateName.set(self._disconnectStateName)
      return True
    else:
      return jsr309.States.HandlingInviteWithoutSDPState.processEvent(self,
          in_event, out_pystrNewStateName)

# Puts session on hold following RFC 3264 section 8.4
class PuttingSessionOnHoldRFC3264State(jsr309.States.PuttingSessionOnHoldRFC3264State):

  def __init__(self, in_stateMachine, successStateName, 
      failureStateName, disconnectStateName):
    jsr309.States.PuttingSessionOnHoldRFC3264State.__init__(self, in_stateMachine,
        successStateName, failureStateName, disconnectStateName)

  def processEvent(self, in_event, out_pystrNewStateName):
    __pychecker__ = 'maxreturns=0'
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    if isinstance(in_event, DialogToCallTerminatedEvent):
      self._stateMachine.dialogIdInitiatingDisconnect = in_event.getDialogId()
      out_pystrNewStateName.set(self._disconnectStateName)
      return True
    else:
      return jsr309.States.PuttingSessionOnHoldRFC3264State.processEvent(self,
          in_event, out_pystrNewStateName)
## !!MIRROR FILE, DO NOT MODIFY!!
