
from paraxip.sipua.internal.SipCallEngineSMCall import *
from paraxip.sipua.internal.ConnectionStatus import *
from agent_bridge.DTMFEntryStates import *
from agent_bridge.CommonCall import *
import paraxip.sipua.user.b2bua.B2buaStates
import paraxip.infra.PyLoggingFunctions as PyLoggingFunctions
from netborderPyInfra import *
from SipCallEngineExtending import *
import ParameterInfo
import JSR309APICommonPythonModules as jsr309common

import os

# Force declaration of class attributes in __init__ method
__pychecker__ = "initattr"

#
# Agent-bridge state machine
#

class DTMFEntryCall(CommonCall):

  # Valid parameter names for this Call
  PARAM_AGENT_REGISTRAR_ADDRESS =               'agent.registrar.address'
  PARAM_AGENT_ENTERID_NUMDIGITS =               'agent.enterID.numDigits'
  PARAM_AGENT_TERMINATION_CHARACTER =           'agent.enterID.terminationCharacter'
  PARAM_AGENT_ENTERID_INTERDIGITTIMEOUT_MS =    'agent.enterID.interDigitTimeout.ms'
  PARAM_AGENT_MAXIMUM_LOGIN_ATTEMPTS =          'agent.enterID.maxAttempts'

  PARAM_PROMPT_ENTERIDERROR    = 'prompt.enterIDError'

  # Return parameter information for this Call 
  @staticmethod
  def getParameterInfoDB():
    # Call parent class function
    parameterInfoDB = CommonCall.getParameterInfoDB()

    # No default for registrar address, realm and password
    parameterInfoDB.addParameterInfoForName(
        DTMFEntryCall.PARAM_AGENT_REGISTRAR_ADDRESS,
        type=str)
    parameterInfoDB.addParameterInfoForName(
        DTMFEntryCall.PARAM_AGENT_ENTERID_NUMDIGITS,
        type=int,
        defaultValue=8)
    parameterInfoDB.addParameterInfoForName(
        DTMFEntryCall.PARAM_AGENT_TERMINATION_CHARACTER,
        type=str,
        defaultValue='')
    parameterInfoDB.addParameterInfoForName(
        DTMFEntryCall.PARAM_AGENT_ENTERID_INTERDIGITTIMEOUT_MS,
        type=int,
        defaultValue=10000)
    parameterInfoDB.addParameterInfoForName(
        DTMFEntryCall.PARAM_AGENT_MAXIMUM_LOGIN_ATTEMPTS,
        type=int,
        defaultValue=3)
    parameterInfoDB.addParameterInfoForName(
        CommonCall.PARAM_PROMPT_WELCOME,
        type=str,
        defaultValue='/g729/enterID.wav')
    parameterInfoDB.addParameterInfoForName(
        DTMFEntryCall.PARAM_PROMPT_ENTERIDERROR,
        type=str,
        defaultValue='/g729/enterIDError.wav')
    parameterInfoDB.addParameterInfoForName(
        CommonCall.PARAM_PROMPT_REGISTERFAIL,
        type=str,
        defaultValue='/g729/enterIDFail.wav')
    parameterInfoDB.addParameterInfoForName(
        CommonCall.PARAM_PROMPT_REGISTERSUCCESS,
        type=str,
        defaultValue='/g729/enterIDSuccess.wav')
    parameterInfoDB.addParameterInfoForName(
        CommonCall.PARAM_PROMPT_REGISTERREMOVED,
        type=str,
        defaultValue='/g729/unexpectedSystemError.wav')
    parameterInfoDB.addParameterInfoForName(
        CommonCall.PARAM_PROMPT_PARKING,
        type=str,
        mandatory=False)
    parameterInfoDB.addParameterInfoForName(
        CommonCall.PARAM_PROMPT_UNPARKING,
        type=str,
       mandatory=False)

    return parameterInfoDB

  def __init__(self):
    CommonCall.__init__(self)
    
    self.setLogger(CallLogger("netborder.AgentBridge.DTMFEntryCall.py"))
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "DTMFEntryCall.__init__")

    self.resetDataMembers()

    # WELCOME_AGENT state
    welcomeAgentState = WelcomeAgentState(self)
    self.addState(welcomeAgentState.getName(), welcomeAgentState)
   
    # REGISTER_FAILED state
    registerFailedState = RegisterFailedState(self)
    self.addState(registerFailedState.getName(), registerFailedState)

    # HANDLE_AGENT_INVITE_WITH_SDP
    # Programmable state to handle incoming INVITE with SDP
    self._handlingAgentInviteWithSDPState = \
      jsr309.States.HandlingInviteWithSDPState(self, 
          successStateName=welcomeAgentState.getName(),
          failureStateName=self.disconnectingState.getName(),
          disconnectStateName=self.disconnectingState.getName())
    self._handlingAgentInviteWithSDPState.setName(
        'HANDLE_AGENT_INVITE_WITH_SDP')
    self.addState(self._handlingAgentInviteWithSDPState.getName(), \
        self._handlingAgentInviteWithSDPState)
    
    # HANDLE_AGENT_INVITE_WITHOUT_SDP
    # Programmable state to handle incoming INVITE without SDP
    self._handlingAgentInviteWithoutSDPState = \
      jsr309.States.HandlingInviteWithoutSDPState(self, 
          successStateName=welcomeAgentState.getName(),
          failureStateName=self.disconnectingState.getName(),
          disconnectStateName=self.disconnectingState.getName())
    self._handlingAgentInviteWithoutSDPState.setName(
        'HANDLE_AGENT_INVITE_WITHOUT_SDP')
    self.addState(self._handlingAgentInviteWithoutSDPState.getName(), \
        self._handlingAgentInviteWithoutSDPState)



  # Call I/F
  def configure(self, in_dictConfig):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "DTMFEntryCall.configure")

    # Validate the parameters against the Call parameter DB
    # TODO make it happen in base class (SipCallEngineSMCall)
    # TODO don't recreate the parameterInfoDB each time

    # dtmf termination character must be either * or # or '' (ie none)
    termChar = in_dictConfig[DTMFEntryCall.PARAM_AGENT_TERMINATION_CHARACTER]
    if (termChar != '#' and termChar != '*' and termChar != ''):
      PyLoggingFunctions.log_error(self.getLogger(), 'Parameter %s can only be set to'
          ' the value  \'#\' or \'*\', current value is \'%s\'' % 
          (DTMFEntryCall.PARAM_AGENT_TERMINATION_CHARACTER,termChar))
      return False  

    if not DTMFEntryCall.getParameterInfoDB().validateConfig(in_dictConfig):
      return False
   
    CommonCall.configure(self, in_dictConfig)

    return True

  def resetDataMembers(self): 
    CommonCall.resetDataMembers(self)
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "DTMFEntryCall.resetDataMembers")
      
    # Maximum number of digits to retrieve from Signal Detector
    self.sdMaxDigits = -1
    self.sdDTMFCounter = 0
    self.sdDTMFFailedAttempts = 0
    

  def reset(self):
    CommonCall.reset(self)
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(),"DTMFEntryCall.reset")

    self.resetDataMembers()

  def scenarioName(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(),
               "DTMFEntryCall.scenarioName")
    return "Netborder DTMF Agent Bridge"
 
  def getAudioFilePath (self):
    return self.application.getAudioFilePath()

  # Helpers
  def getRejectMessage(self, in_szDialogId):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(),"DTMFEntryCall.getRejectMessage")
    sipMessage = PySipMessage()

    # Get the sip reject response code
    uiRejectResponseCode = 603 # Decline

    # Validate that the DialogId still exists 
    dialog = self.getInDialog(in_szDialogId)
    if (dialog is None):
      return None, None

    dialog.constructRejectMessage(sipMessage)

    if (self.uiRejectAgentSipErrorCode is not None):
      uiRejectResponseCode = self.uiRejectAgentSipErrorCode
   
    # Add agent-bridge header
    if (self.strRejectAgentReason is not None):
      assert sipMessage.addCustomHeader('NetBorder-ABS-Result', self.strRejectAgentReason)

    return uiRejectResponseCode, sipMessage

  
## !!MIRROR FILE, DO NOT MODIFY!!
