
from paraxip.sipua.internal.SipCallEngineState import *
from paraxip.sipua.internal.SipCallEngineEvent import *
from paraxip.sipua.internal.ConnectionStatus import *
from paraxip.sipua.internal.SMTimeoutProcessor import *
import paraxip.infra.PyLoggingFunctions as PyLoggingFunctions
from SipCallEngineExtending import *
from paraxip.infra.Assert import *
import netborderPyInfra
from jsr309 import JSR309Event, JSR309Listener # under python-scripts/internal
import paraxip.sipua.user.b2bua.B2buaStates

import DTMFEntryCall as Call

import JSR309APICommonPythonModules as jsr309common
from JSR309ClientAPIPythonModules import PyParameters

fileScopeLogger = netborderPyInfra.Logger(
    "netborder.AgentBridge.DTMFEntryState.py")

# Force declaration of class attributes in __init__ method
__pychecker__ = "initattr"

#
# Call states
#

#
# Base class
#

class DTMFEntryState(SipCallEngineState):
  def __init__(self, in_stateMachine):
    SipCallEngineState.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "DTMFEntryState.__init__")

  def warnUnexpectedEvent(self, in_strStateName, in_event):
    if (isinstance(in_event, JSR309Event)):
      PyLoggingFunctions.log_warn(self.getLogger(),
             "%s received an unexpected JSR309Event %s",
             in_strStateName, in_event.mediaEvent)
    else:      
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               in_strStateName, in_event.__class__.__name__)

#
# WelcomeAgentEntry state
#
class WelcomeAgentState(DTMFEntryState):
  def __init__(self, in_stateMachine):
    DTMFEntryState.__init__(self, in_stateMachine)
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "WelcomeAgent.__init__")

    self._signalDetectorTimerId = None

  def getName(self):
    return "WELCOME_AGENT"

  def startSignalDetectorTimer(self):
    timeoutEvent = TimeoutEvent()
    signalDetectorTimeoutProcessor = SMTimeoutProcessor(
        self._stateMachine,
        timeoutEvent)
    signalDetectorTimeoutMs = \
      self._stateMachine.config[Call.DTMFEntryCall.PARAM_AGENT_ENTERID_INTERDIGITTIMEOUT_MS]
    self._signalDetectorTimerId = self._stateMachine.globalContext.scheduleTimer(
      signalDetectorTimeoutProcessor,
      signalDetectorTimeoutMs, 0)
    timeoutEvent.timerId = self._signalDetectorTimerId

  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)

    self._stateMachine.agentLegStatus = ConnectionStatus.Connected 
    
    out_pystrNewStateName.set(self.getName())
    
    if (self._stateMachine.sdDTMFFailedAttempts  == 0):
  
      # Get a new JSR309 Media Group
      mediaGroup=self._stateMachine.getMediaGroup()

      # Get a new JSR309 Player 
      #     
      self._stateMachine.jsr309Player = mediaGroup.getPlayer()
      
  
      if (isLogAssert(self.getLogger(),\
          self._stateMachine.jsr309Player is not None)):
        return True
  
      # TODO: Life duration needs to be clarified. We could always reuse the
      #       same listener
      myPlaylistener = JSR309Listener(self._stateMachine.globalContext ,
                                  self._stateMachine)
      self._stateMachine.jsr309Player.addListener(myPlaylistener)
      myPlaylistener = None
  
      # Joining player with NC
      self._stateMachine.jsr309Player.join(jsr309common.Mode.eSEND, \
                                       self._stateMachine.jsr309NetworkConnection)
  
      PyLoggingFunctions.log_trace(self.getLogger(), "Enabling signal detection")
      self._stateMachine.jsr309SignalDetector = mediaGroup.getSignalDetector()
  
      self._stateMachine.sdMaxDigits = \
          self._stateMachine.\
          config[Call.DTMFEntryCall.PARAM_AGENT_ENTERID_NUMDIGITS] 
      self._stateMachine.sdTermChar = \
          self._stateMachine.\
          config[Call.DTMFEntryCall.PARAM_AGENT_TERMINATION_CHARACTER]


      # TODO: Life duration needs to be clarified. We could always reuse the
      #       same listener
      mySDListener = JSR309Listener(self._stateMachine.globalContext ,
                                self._stateMachine)
      self._stateMachine.jsr309SignalDetector.addListener(mySDListener)
      mySDListener = None
      
      params = PyParameters()
      symbol = jsr309common.SignalDetector.p_PatternsAllDigits()
      value = jsr309common.String()
      params.put(symbol,value)
      self._stateMachine.jsr309SignalDetector.setParameters(params)
      
      self._stateMachine.jsr309SignalDetector.join(jsr309common.Mode.eRECEIVE,
                                    self._stateMachine.jsr309NetworkConnection)
      # Installing RTC      
      myRTCArray = jsr309common.RTCArray()
      trigger = jsr309common.SignalDetector.p_PatternsAllDigits()
      action = jsr309common.Player.rtca_stop()
      myRTC = jsr309common.RTC(trigger, action)
      myRTCArray.addRTC(myRTC)    
      self._stateMachine.jsr309SignalDetector.installRTC(myRTCArray,
                                               self._stateMachine.jsr309Player)
    
      # Start playback of initial prompt
      self._stateMachine.startPlayback(Call.DTMFEntryCall.PARAM_PROMPT_WELCOME)
    else:
      # we playback the enterIDError prompt, reset DTMF obtained so far 
      self._stateMachine.startPlayback(Call.DTMFEntryCall.PARAM_PROMPT_ENTERIDERROR)
      self._stateMachine.sdDTMFCounter = 0
      self._stateMachine.agentDN = ''            
    
  def processEvent(self, in_event, out_pystrNewStateName):
    __pychecker__ = 'maxreturns=0 maxlines=0'
    
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())
    
    if (isinstance(in_event, DialogToCallTerminatedEvent)):
      self._stateMachine.agentLegStatus = ConnectionStatus.Disconnected          
      out_pystrNewStateName.set("DISCONNECTING")
      return True
    
    elif (isinstance(in_event, StopEvent)):
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    elif (isinstance(in_event, TimeoutEvent)):
      PyLoggingFunctions.log_info(self.getLogger(), "Signal Detector Timeout: %s expired",
          Call.DTMFEntryCall.PARAM_AGENT_ENTERID_INTERDIGITTIMEOUT_MS)
      # increment the number of failed attempts
      self._stateMachine.sdDTMFFailedAttempts += 1;
      # if the number of failed attempts exceeds the maximum allowed
      # we go to the register fail state
      if  (self._stateMachine.sdDTMFFailedAttempts >= 
           self._stateMachine.config[Call.DTMFEntryCall.PARAM_AGENT_MAXIMUM_LOGIN_ATTEMPTS]):
        out_pystrNewStateName.set("REGISTER_FAILED")
      else:
        # Stop the signal Detector timer
        if (self._signalDetectorTimerId is not None):
          self._stateMachine.globalContext.cancelTimer(self._signalDetectorTimerId)
          self._signalDetectorTimerId = None
        # we playback the enterIDError prompt, reset DTMF obtained so far 
        self._stateMachine.startPlayback(Call.DTMFEntryCall.PARAM_PROMPT_ENTERIDERROR)
        self._stateMachine.sdDTMFCounter = 0
        self._stateMachine.agentDN = ''
      return True

    elif (isinstance(in_event, JSR309Event) and 
        isinstance(in_event.mediaEvent, jsr309common.PlayFinishedEvent) and
          in_event.mediaEvent.getStreamID() == \
            self._stateMachine.getPlayedFileURI(
              Call.DTMFEntryCall.PARAM_PROMPT_WELCOME)):
      # Play operation has completed normally. 
      # Start the signal dectector timer
      if (self._signalDetectorTimerId is not None):
        self._stateMachine.globalContext.cancelTimer(self._signalDetectorTimerId)
      self._signalDetectorTimerId = None
      self.startSignalDetectorTimer()
      return True

    elif (isinstance(in_event, JSR309Event) and 
        isinstance(in_event.mediaEvent, jsr309common.PlayFinishedEvent) and
          in_event.mediaEvent.getStreamID() == \
            self._stateMachine.getPlayedFileURI(
              Call.DTMFEntryCall.PARAM_PROMPT_ENTERIDERROR)):
      # Play operation has completed normally.
      # Start the signal dectector timer
      if (self._signalDetectorTimerId is not None):
        self._stateMachine.globalContext.cancelTimer(self._signalDetectorTimerId)
      self._signalDetectorTimerId = None
      self.startSignalDetectorTimer()
      return True

    elif (isinstance(in_event, JSR309Event) and 
        isinstance(in_event.mediaEvent, jsr309common.DtmfStartEvent)):
      # Reset the signal Detector timer
      if (self._signalDetectorTimerId is not None):
        self._stateMachine.globalContext.cancelTimer(self._signalDetectorTimerId)
        self._signalDetectorTimerId = None
        self.startSignalDetectorTimer()
      # If the termination character is entered, we stop the DTMF entry       
      if (self._stateMachine.sdTermChar == in_event.mediaEvent.getDtmfDigit()):
          PyLoggingFunctions.log_debug(self.getLogger(), "Agent DN is :%s",
                  self._stateMachine.agentDN)
          out_pystrNewStateName.set("REGISTERING_AGENT")
          return True
      # Capture the number if it's index is still within the established range
      if (self._stateMachine.sdDTMFCounter < self._stateMachine.sdMaxDigits):
        self._stateMachine.sdDTMFCounter += 1
        self._stateMachine.agentDN += in_event.mediaEvent.getDtmfDigit()
        PyLoggingFunctions.log_debug(self.getLogger(), "DTMF Counter is :%s",
                  self._stateMachine.sdDTMFCounter)
        # if we have captured all the required digits 
        # or if  received the termination character  
        # cancel the timer and move on to the next state
        if (self._stateMachine.sdDTMFCounter == self._stateMachine.sdMaxDigits):
          PyLoggingFunctions.log_debug(self.getLogger(), "Agent DN is :%s",
                  self._stateMachine.agentDN)
          out_pystrNewStateName.set("REGISTERING_AGENT")
      else:
        PyLoggingFunctions.log_warn(self.getLogger(),
             "%s::processEvent invalid DTMF counter (%s) value for event %s",
             self.__class__.__name__,  self._stateMachine.sdDTMFCounter , 
             in_event.__class__.__name__)
      return True

    else:
      DTMFEntryState.warnUnexpectedEvent(self,self.__class__.__name__, in_event)
     
    return True

  def onStateExit(self, in_event, in_strDestStateName):
    __pychecker__ = 'unusednames=in_event,in_strDestStateName'
    # Retrieve registrar from config (already validated in DTMFEntryCall.configure())
    self._stateMachine.registrarURL = self._stateMachine.config[Call.DTMFEntryCall.PARAM_AGENT_REGISTRAR_ADDRESS]
    # Make sure the DTMF timeout is cancelled before changing states
    if (self._signalDetectorTimerId is not None):
      self._stateMachine.globalContext.cancelTimer(self._signalDetectorTimerId)
      self._signalDetectorTimerId = None


#
# RegisterFailed state
#

class RegisterFailedState(DTMFEntryState):
  def __init__(self, in_stateMachine):
    DTMFEntryState.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "RegisterFailedState.__init__")
  
  def getName(self):
    return "REGISTER_FAILED"
  
  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)

    PyLoggingFunctions.log_debug(self.getLogger(), "Register Failed Event. Failed Attempt count: %s",
              self._stateMachine.sdDTMFFailedAttempts)     
    # increment the number of failed attempts
    self._stateMachine.sdDTMFFailedAttempts += 1;
    # if the number of failed attempts does not exceed the maximum allowed
    # we go back to the WELCOME_AGENT state
    if  (self._stateMachine.sdDTMFFailedAttempts < 
         self._stateMachine.config[Call.DTMFEntryCall.PARAM_AGENT_MAXIMUM_LOGIN_ATTEMPTS]):
      out_pystrNewStateName.set("WELCOME_AGENT")
    else:
      # play register failure prompt
      self._stateMachine.startPlayback(Call.DTMFEntryCall.PARAM_PROMPT_REGISTERFAIL)
      out_pystrNewStateName.set(self.getName())
    
  def processEvent(self, in_event, out_pystrNewStateName):
     __pychecker__ = 'maxreturns=0'

     PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
               self.__class__.__name__, in_event.__class__.__name__)

     out_pystrNewStateName.set(self.getName())

     if (isinstance(in_event, JSR309Event) and 
          isinstance(in_event.mediaEvent, jsr309common.PlayFinishedEvent) and
            in_event.mediaEvent.getStreamID() == \
              self._stateMachine.getPlayedFileURI(
                Call.DTMFEntryCall.PARAM_PROMPT_REGISTERFAIL)):
         # Play operation has completed normally.
         # go to DISCONNECTING state
         out_pystrNewStateName.set("DISCONNECTING")
         return True
     elif (isinstance(in_event, DialogToCallAckNotReceivedEvent)):
       # Can happen here after answering 200 Ok to the agent's INVITE
       # TODO use mutable logger for this warning
       PyLoggingFunctions.log_warn(self.getLogger(), "The SIP UAC did not answer our " \
         "SIP 200 response with a ACK request as specified by RFC 3261. " \
         "Terminating the call. (State: %s)",self.getName())
        
       out_pystrNewStateName.set("DISCONNECTING")
       return True

     else:
       DTMFEntryState.warnUnexpectedEvent(self,self.__class__.__name__, in_event)

     return True


## !!MIRROR FILE, DO NOT MODIFY!!
