
from paraxip.sipua.internal.SipCallEngineSMCall import *
from paraxip.sipua.internal.ConnectionStatus import *
from agent_bridge.XMLBasedEntryStates import *
from agent_bridge.CommonCall import *
import paraxip.sipua.user.b2bua.B2buaStates
import paraxip.infra.PyLoggingFunctions as PyLoggingFunctions
from netborderPyInfra import *
from SipCallEngineExtending import *
import ParameterInfo
import JSR309APICommonPythonModules as jsr309common

import os
from httplib2 import *
from xml.dom.minidom import *
from HttpFileCache import *
import sys


# Force declaration of class attributes in __init__ method
__pychecker__ = "initattr"

httpCache = ParaxipFileCache()

REGISTER_CONFIG = None

#
# Agent-bridge state machine
#

class XMLBasedEntryCall(CommonCall):

  # Valid parameter names for this Call
  PARAM_REGISTER_TABLE_URL =     'register.table.httpurl'

  # Return parameter information for this Call 
  @staticmethod
  def getParameterInfoDB():
    # Call parent class function
    parameterInfoDB = CommonCall.getParameterInfoDB()
    
    parameterInfoDB.addParameterInfoForName(
        CommonCall.PARAM_PROMPT_WELCOME,
        type=str,
        defaultValue='/alaw/welcome.wav')
    parameterInfoDB.addParameterInfoForName(
        CommonCall.PARAM_PROMPT_REGISTERFAIL,
        type=str,
        defaultValue='/alaw/registerFail.wav')
    parameterInfoDB.addParameterInfoForName(
        CommonCall.PARAM_PROMPT_REGISTERSUCCESS,
        type=str,
        defaultValue='/alaw/registerSuccess.wav')
    parameterInfoDB.addParameterInfoForName(
        CommonCall.PARAM_PROMPT_REGISTERREMOVED,
        type=str,
        defaultValue='/alaw/unexpectedSystemError.wav')
    parameterInfoDB.addParameterInfoForName(
        CommonCall.PARAM_PROMPT_PARKING,
        type=str,
        mandatory=False)
    parameterInfoDB.addParameterInfoForName(
        CommonCall.PARAM_PROMPT_UNPARKING,
        type=str,
        mandatory=False)
    parameterInfoDB.addParameterInfoForName(
        XMLBasedEntryCall.PARAM_REGISTER_TABLE_URL,
        type=str,
        defaultValue='http://127.0.0.1/agentMapping.xml')

    return parameterInfoDB

  def __init__(self):
    CommonCall.__init__(self)
    
    self.setLogger(CallLogger("netborder.AgentBridge.XMLBasedEntryCall.py"))
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "XMLBasedEntryCall.__init__")

    self.resetDataMembers()


    # WELCOME_AGENT PROMPT
    welcomeAgentState = WelcomeAgentState(self)
    self.addState(welcomeAgentState.getName(), welcomeAgentState)
 
    # REGISTER_FAILED state
    registerFailedState = RegisterFailedState(self)
    self.addState(registerFailedState.getName(), registerFailedState)

    # HANDLE_AGENT_INVITE_WITH_SDP
    # Programmable state to handle incoming INVITE with SDP
    self._handlingAgentInviteWithSDPState = \
      jsr309.States.HandlingInviteWithSDPState(self, 
          successStateName=welcomeAgentState.getName(),
          failureStateName=self.disconnectingState.getName(),
          disconnectStateName=self.disconnectingState.getName())
    self._handlingAgentInviteWithSDPState.setName(
        'HANDLE_AGENT_INVITE_WITH_SDP')
    self.addState(self._handlingAgentInviteWithSDPState.getName(), \
        self._handlingAgentInviteWithSDPState)
    
    # HANDLE_AGENT_INVITE_WITHOUT_SDP
    # Programmable state to handle incoming INVITE without SDP
    self._handlingAgentInviteWithoutSDPState = \
      jsr309.States.HandlingInviteWithoutSDPState(self, 
          successStateName=welcomeAgentState.getName(),
          failureStateName=self.disconnectingState.getName(),
          disconnectStateName=self.disconnectingState.getName())
    self._handlingAgentInviteWithoutSDPState.setName(
        'HANDLE_AGENT_INVITE_WITHOUT_SDP')
    self.addState(self._handlingAgentInviteWithoutSDPState.getName(), \
        self._handlingAgentInviteWithoutSDPState)

  # Call I/F
  def initialize(self, in_refApplication, in_refGlobalContext, in_szCallId):
    SipCallEngineSMCall.initialize(self,
                                   in_refApplication,
                                   in_refGlobalContext,
                                   in_szCallId)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "XMLBasedEntryCall.initialize")

    global httpCache # This only works for single threaded applications
    self.httpObject = Http(httpCache)
    
    return True

  # Call I/F
  def configure(self, in_dictConfig):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "XMLBasedEntryCall.configure")

    # Validate the parameters against the Call parameter DB
    # TODO make it happen in base class (SipCallEngineSMCall)
    # TODO don't recreate the parameterInfoDB each time
    

    if not XMLBasedEntryCall.getParameterInfoDB().validateConfig(in_dictConfig):
      return False
   
    CommonCall.configure(self, in_dictConfig)

    return True


  def resetDataMembers(self): 
    CommonCall.resetDataMembers(self)
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "XMLBasedEntryCall.resetDataMembers")
    #
    # No Members to reset
    #
    
  def reset(self):
    CommonCall.reset(self)
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(),"XMLBasedEntryCall.reset")

    self.resetDataMembers()
  
  def scenarioName(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(),
               "XMLBasedEntryCall.scenarioName")
    return "Netborder XML Agent Bridge"
 
  def getTable(self, in_szhttpUrl):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(),"XMLBasedEntryCall.getTable")

    try:
      resp, content = self.httpObject.request(in_szhttpUrl)
    except:
      PyLoggingFunctions.log_error(self.getLogger(), "HTTP GET(%s) failed: %s",
                                  in_szhttpUrl,
                                  sys.exc_info()[0])
      return False
      
    if (resp.status != 200):
      PyLoggingFunctions.log_error(self.getLogger(),
                "Failed to get SIP registrar mapping table at %s (register."
                "table.httpurl property): HTTP response %s",
                in_szhttpUrl,
                resp.status)
      return False

    """ 
    TEMP no need to force correct mime type
    if resp['content-type'] != 'text/xml':
      PyLoggingFunctions.log_error(self.getLogger(),
                "AgentBridgeCall.getTable: SIP registrar mapping table at %s (register"
                ".table.httpurl property) is of invalid type %s "
                "(expected type is text/xml)", in_szhttpUrl, resp['content-type'])
      return False
    """

    if (not resp.fromcache):
      global  REGISTER_CONFIG # This only works for single threaded applications
      REGISTER_CONFIG = parseString(content)
      PyLoggingFunctions.log_debug(self.getLogger(), "XMLBasedEntryCall.getTable: the table is from the cache")
    else:
      PyLoggingFunctions.log_debug(self.getLogger(), "XMLBasedEntryCall.getTable: the table have been reloaded")

    return True

  def getRegistrar(self, in_szDN):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(),"XMLBasedEntryCall.getRegistrar")

    global  REGISTER_CONFIG # This only works for single threaded applications
    x = REGISTER_CONFIG.getElementsByTagName("sipserverbinding")
    nbservers = x.length

    i = 0
    while i < nbservers:
      agents = x.item(i).getElementsByTagName('extension')
      j = 0
      nbagents = agents.length
      while j < nbagents:
        if (agents.item(j).getAttribute('DN') == in_szDN):
          return str(x.item(i).getAttribute('addr'))
        else:
          j = j+1
      i = i + 1

    return None

  def GetXMLTableAlarmHandle(self):
    return self.application.GetXMLTableAlarmHandle()
## !!MIRROR FILE, DO NOT MODIFY!!
