

from paraxip.sipua.internal.SipCallEngineState import *
from paraxip.sipua.internal.SipCallEngineEvent import *
from paraxip.sipua.internal.ConnectionStatus import *
from paraxip.sipua.internal.SMTimeoutProcessor import *
import paraxip.infra.PyLoggingFunctions as PyLoggingFunctions
from SipCallEngineExtending import *
from paraxip.infra.Assert import *
import netborderPyInfra
from jsr309 import JSR309Event, JSR309Listener # under python-scripts/internal
import paraxip.sipua.user.b2bua.B2buaStates

import XMLBasedEntryCall as Call

import JSR309APICommonPythonModules as jsr309common
from JSR309ClientAPIPythonModules import PyParameters

fileScopeLogger = netborderPyInfra.Logger(
    "netborder.AgentBridge.XMLBasedEntryStates.py")

# Force declaration of class attributes in __init__ method
__pychecker__ = "initattr"

#
# Call states
#

#
# Base class
#

class XMLBasedEntryState(SipCallEngineState):
  def __init__(self, in_stateMachine):
    SipCallEngineState.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "XMLBasedEntryState.__init__")

  def warnUnexpectedEvent(self, in_strStateName, in_event):
    if (isinstance(in_event, JSR309Event)):
      PyLoggingFunctions.log_warn(self.getLogger(),
             "%s received an unexpected JSR309Event %s",
             in_strStateName, in_event.mediaEvent)
    else:      
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               in_strStateName, in_event.__class__.__name__)

#
# WelcomeAgentEntry state
#
class WelcomeAgentState(XMLBasedEntryState):
  def __init__(self, in_stateMachine):
    XMLBasedEntryState.__init__(self, in_stateMachine)
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "WelcomeAgent.__init__")

    self._signalDetectorTimerId = None

  def getName(self):
    return "WELCOME_AGENT"

  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)

    self._stateMachine.agentLegStatus = ConnectionStatus.Connected 

    out_pystrNewStateName.set(self.getName())

    # Get the agent DN from the initial sip message
    self._stateMachine.agentDN = \
      self._stateMachine.inviteSipMessage.getUserPart("From")

    # Get a new JSR309 Media Group
    mediaGroup=self._stateMachine.getMediaGroup()

    # Get a new JSR309 Player 
    #     
    self._stateMachine.jsr309Player = mediaGroup.getPlayer()
    
    if (isLogAssert(self.getLogger(),\
        self._stateMachine.jsr309Player is not None)):
      return True

    # TODO: Life duration needs to be clarified. We could always reuse the
    #       same listener
    myPlaylistener = JSR309Listener(self._stateMachine.globalContext ,
                                self._stateMachine)
    self._stateMachine.jsr309Player.addListener(myPlaylistener)
    myPlaylistener = None

    # Joining player with NC
    self._stateMachine.jsr309Player.join(jsr309common.Mode.eSEND, \
                                     self._stateMachine.jsr309NetworkConnection)
   
    # Start playback of initial prompt
    self._stateMachine.startPlayback(Call.XMLBasedEntryCall.PARAM_PROMPT_WELCOME)

    # Check for the DN in our table and retrieve the sip server to register to
    resTable = self._stateMachine.getTable( \
      self._stateMachine.config[Call.XMLBasedEntryCall.PARAM_REGISTER_TABLE_URL])
    if (not resTable):
      # turn on the XML table alarm
      self._stateMachine.GetXMLTableAlarmHandle().set()
      self._stateMachine.agentRegistration = None
      out_pystrNewStateName.set("REGISTER_FAILED")
      return
    else:
      # turn off the XML table alarm
      self._stateMachine.GetXMLTableAlarmHandle().reset()
      self._stateMachine.registrarURL = \
        self._stateMachine.getRegistrar(self._stateMachine.agentDN)

      if (self._stateMachine.registrarURL is not None):
        PyLoggingFunctions.log_info(self.getLogger(), "This agent is on server: %s",
                                   self._stateMachine.registrarURL)
      else:
        PyLoggingFunctions.log_error(self.getLogger(),
                  "%s: Found no Registrar to register agent: \"%s\"",
                  self.__class__.__name__,
                  self._stateMachine.agentDN)

        self._stateMachine.agentRegistration = None
        out_pystrNewStateName.set("REGISTER_FAILED") 
   
  def processEvent(self, in_event, out_pystrNewStateName):
    __pychecker__ = 'maxreturns=0 maxlines=0'
    
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())
    
    if (isinstance(in_event, DialogToCallTerminatedEvent)):
      self._stateMachine.agentLegStatus = ConnectionStatus.Disconnected          
      out_pystrNewStateName.set("DISCONNECTING")
      return True
    
    elif (isinstance(in_event, StopEvent)):
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    elif (isinstance(in_event, JSR309Event) and 
        isinstance(in_event.mediaEvent, jsr309common.PlayFinishedEvent) and
        in_event.mediaEvent.getStreamID() == \
          self._stateMachine.getPlayedFileURI(
            Call.XMLBasedEntryCall.PARAM_PROMPT_WELCOME)):
      # Play operation has completed normally. Go to Register state
      out_pystrNewStateName.set("REGISTERING_AGENT")
      return True
          
    else:
      XMLBasedEntryState.warnUnexpectedEvent(self,self.__class__.__name__, in_event)
     
    return True
 
#
# RegisterFailed state
#

class RegisterFailedState(XMLBasedEntryState):
  def __init__(self, in_stateMachine):
    XMLBasedEntryState.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "RegisterFailedState.__init__")
  
  def getName(self):
    return "REGISTER_FAILED"
  
  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)

    # play register failure prompt
    self._stateMachine.startPlayback(Call.XMLBasedEntryCall.PARAM_PROMPT_REGISTERFAIL)

    out_pystrNewStateName.set(self.getName())
    
  def processEvent(self, in_event, out_pystrNewStateName):
     __pychecker__ = 'maxreturns=0'

     PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
               self.__class__.__name__, in_event.__class__.__name__)

     out_pystrNewStateName.set(self.getName())
   
     if (isinstance(in_event, JSR309Event) and 
          isinstance(in_event.mediaEvent, jsr309common.PlayFinishedEvent) and
          in_event.mediaEvent.getStreamID() == \
            self._stateMachine.getPlayedFileURI(
              Call.XMLBasedEntryCall.PARAM_PROMPT_REGISTERFAIL)):
       # Play operation has completed normally.
       # go to DISCONNECTING state
       out_pystrNewStateName.set("DISCONNECTING")
       return True

     elif (isinstance(in_event, JSR309Event) and 
          isinstance(in_event.mediaEvent, jsr309common.PlayFinishedEvent)):  
       # The Welcome prompt finished playing, disregard event
       return True

     else:
      XMLBasedEntryState.warnUnexpectedEvent(self,self.__class__.__name__, in_event)

     return True
## !!MIRROR FILE, DO NOT MODIFY!!
