
from paraxip.sipua.user.cpa import CallEvent
from paraxip.sipua.user.cpa import QuickLogger

class TerminaCallStateException(Exception):
  pass

class CallEventStack:
  """
  A list of the CallEvents that occurred during the call. Those CallEvents
  describe the terminal state of the call.
  
  This class can be used to determine what will be sent back to the caller/
  caller (SIP-wise) and what will be logged in the permanent call database
  (Cpa-Stats.csv).  
  """
  
  _Trait_CallEventStack = None
  
  def __init__(self, in_CallEventsInit = None):
    if in_CallEventsInit is not None:
      assert isinstance(in_CallEventsInit, list)
      self.__CallEvents = in_CallEventsInit
    else:
      self.__CallEvents = []
  
  def notifyCallEvent(self, in_CallEvent):
    assert hasattr(in_CallEvent, '_Trait_isCallEvent')
    self.__CallEvents.append(in_CallEvent)
    
  def getCallEventList(self):
    return self.__CallEvents
  
  def getCpaResult(self):
    """Return a CpaResult if one contained in CallState memory, 'None' if not."""
    cpaResult = None
    for callEvent in self.__CallEvents:
      if hasattr(callEvent, '_Trait_isCallCpaResult'):
        if cpaResult is not None:
          #This can happen in normal cases:
          #  ['CallCpaResult: CPA_MACHINE', 'CallCpaResult: EAMD_BEEP']
          lstEv = []
          for event in self.__CallEvents:
            lstEv.append("%s" % event)
          strExcept = "More than one CpaResult in this call: %s, " \
                      "only using the last one." % lstEv
          QuickLogger.log_debug(strExcept)

        cpaResult = callEvent
    return cpaResult
  
  def getCallProblems(self):
    callProblems = []
    for callEvent in self.__CallEvents:
      if hasattr(callEvent, '_Trait_isSipError') or \
         hasattr(callEvent, '_Trait_isTimeout'):
        callProblems.append(callEvent)
    return callProblems
  
  def getLastCallProblem(self):
    callProblems = self.getCallProblems()
    if len(callProblems) > 0:
      return callProblems[-1]
    else:
      return None
  
  def hadPostConnectTimeout(self):
    for callEvent in self.__CallEvents:
      if hasattr(callEvent, '_Trait_isTimeout'):
        if callEvent.getTimeoutType() == CallEvent.Timeout.Postconnect:
          return True
    return False
  
  def hadSipError(self):
    for callEvent in self.__CallEvents:
      if hasattr(callEvent, 'getSipErrorCodeAsInteger'):
        return True
    return False
  
  def __str__(self):
    
    if len(self.getCallEventList())==0:
      return self.__class__.__name__ + ": Empty"
    
    x = 0
    rStr = self.__class__.__name__ + ": {"
    for event in self.__CallEvents:
      rStr += "%s: %s, " % (x, event)
      x += 1
    return rStr.strip(", ") + "}"

  ## !!MIRROR FILE, DO NOT MODIFY!!
