
from paraxip.sipua.user.cpa import CpaCall
from paraxip.sipua.user.cpa import CpdResult
from paraxip.sipua.user.cpa import CallEvent
from paraxip.sipua.user.cpa import CpaResult
from paraxip.sipua.user.cpa import DetailedCpdResult

def test_CpaCallReset():
  
  #Initial state
  cpaCall = CpaCall.CpaCall()
  assert len( cpaCall.getCallStack().getCallEventList() ) == 0
  assert hasattr(cpaCall, 'cpaSdpAnswered')
  assert cpaCall.cpaSdpAnswered is None
  
  #Mod
  cpaCall.notifyCallEvent( CallEvent.Factory_CallCpaResult( CpaResult.CpaResult.Human ) )
  cpaCall.cpaSdpAnswered = {'fake_val':'for_test'} 
  assert len( cpaCall.getCallStack().getCallEventList() ) == 1
  
  #Reset, confirm we are back to initial state
  cpaCall.reset()
  assert len( cpaCall.getCallStack().getCallEventList() ) == 0
  assert cpaCall.cpaSdpAnswered is None
  
  #Mod, reset again (in case the cache gets corrupted)
  for x in range(0,100):
    cpaCall.notifyCallEvent( CallEvent.Factory_CallCpaResult( CpaResult.CpaResult.Human ) )
    cpaCall.cpaSdpAnswered = {'fake_val':'for_test %x' % x} 
    assert len( cpaCall.getCallStack().getCallEventList() ) == 1
    cpaCall.reset()
    assert len( cpaCall.getCallStack().getCallEventList() ) == 0
    assert cpaCall.cpaSdpAnswered is None
  

def test_CallResults():
  cpaCall = CpaCall.CpaCall()
  cpdResult = cpaCall.getCpdResultFromStateVariables()
  assert cpdResult == CpdResult.CpdResult.Unknown
  
  cpaCall = CpaCall.CpaCall()
  cpaCall.notifyCallEvent( CallEvent.Factory_CallCpaResult( CpaResult.CpaResult.Human ) ) 
  cpdResult = cpaCall.getCpdResultFromStateVariables()
  assert cpdResult == CpdResult.CpdResult.Voice
  
  # Test related to bug 3501 (http://bugzilla/show_bug.cgi?id=3501):
  #   "The CPD-result should be answering machine even if EAMD timeout occurs."
  cpaCall = CpaCall.CpaCall()
  cpaCall.notifyCallEvent( CallEvent.Factory_CallCpaResult( CpaResult.CpaResult.AnsweringMachine ) ) 
  cpaCall.notifyCallEvent( CallEvent.Timeout( CallEvent.Timeout.Postconnect_Eamd) )
  cpdResult = cpaCall.getCpdResultFromStateVariables()  
  assert cpdResult == CpdResult.CpdResult.AnsweringMachine
  cpdDetailedResult = cpaCall.getDetailedCpdResultFromStateVariables()
  assert cpdDetailedResult == DetailedCpdResult.DetailedCpdResult.Timeout_Eamd
  sipErrCode = cpaCall.getIntSipRejectResponseCodeFromStateVariables()
  assert sipErrCode == 408
  
  # Test the cpdResult after non-eamd timeouts
  # (see http://intranet/pmwiki/pmwiki.php?n=Engineering.HeaderConvention)
  
  # 1. Timeout (Preconnect) -> CpdResult = No-Answer
  cpaCall = CpaCall.CpaCall()
  cpaCall.notifyCallEvent( CallEvent.Factory_CallCpaResult( CpaResult.CpaResult.AnsweringMachine ) ) 
  cpaCall.notifyCallEvent( CallEvent.Timeout( CallEvent.Timeout.Preconnect) )
  cpdResult = cpaCall.getCpdResultFromStateVariables()  
  assert cpdResult == CpdResult.CpdResult.NoAnswer
  cpdDetailedResult = cpaCall.getDetailedCpdResultFromStateVariables()
  assert cpdDetailedResult == DetailedCpdResult.DetailedCpdResult.Timeout_PreConnect
  sipErrCode = cpaCall.getIntSipRejectResponseCodeFromStateVariables()
  assert sipErrCode == 408
  
  # 2. Timeout (Postconnect) -> CpdResult = Unknown
  cpaCall = CpaCall.CpaCall()
  cpaCall.notifyCallEvent( CallEvent.Factory_CallCpaResult( CpaResult.CpaResult.AnsweringMachine ) ) 
  cpaCall.notifyCallEvent( CallEvent.Timeout( CallEvent.Timeout.Postconnect) )
  cpdResult = cpaCall.getCpdResultFromStateVariables()  
  assert cpdResult == CpdResult.CpdResult.Unknown
  cpdDetailedResult = cpaCall.getDetailedCpdResultFromStateVariables()
  assert cpdDetailedResult == DetailedCpdResult.DetailedCpdResult.Timeout_PostConnect
  sipErrCode = cpaCall.getIntSipRejectResponseCodeFromStateVariables()
  assert sipErrCode == 408
  
  # Preconnect Timeout
  # ... scenario yet to be defined
  
  # SipErrorIn -> SipErrorOut! (getIntSipRejectResponseCodeFromStateVariables)
  cpaCall = CpaCall.CpaCall()
  ir = cpaCall.getIntSipRejectResponseCodeFromStateVariables()
  assert ir is None
  sipErrCode = 27182 #Also validate that we do not check the values
                     #of sip error codes
  cpaCall.notifyCallEvent( CallEvent.SipError( CallEvent.SipErrorOriginator.Callee, 
                           CallEvent.SipErrorCode(sipErrCode)))
  ir = cpaCall.getIntSipRejectResponseCodeFromStateVariables()
  assert ir == sipErrCode

def run_CpdResultOnSipError(in_IntSipErrorCode, in_CpdResultExpected):
  cpaCall = CpaCall.CpaCall()
  cpaCall.notifyCallEvent( CallEvent.Factory_CallCpaResult( CpaResult.CpaResult.AnsweringMachine ) ) 
  sipErrCode = in_IntSipErrorCode
  cpaCall.notifyCallEvent( CallEvent.SipError( CallEvent.SipErrorOriginator.Callee, 
                           CallEvent.SipErrorCode(sipErrCode)))
  cpdResult = cpaCall.getCpdResultFromStateVariables()  
  assert cpdResult == in_CpdResultExpected
  sipErrCode = cpaCall.getIntSipRejectResponseCodeFromStateVariables()
  assert sipErrCode == in_IntSipErrorCode
  
def test_CpdErrorStrings():
  run_CpdResultOnSipError(404, CpdResult.CpdResult.InvalidNumber)
  run_CpdResultOnSipError(408, CpdResult.CpdResult.NoAnswer)
  run_CpdResultOnSipError(480, CpdResult.CpdResult.ServiceUnavailable)
  run_CpdResultOnSipError(486, CpdResult.CpdResult.Busy)
  run_CpdResultOnSipError(600, CpdResult.CpdResult.AllTrunksBusy)

def test_OmBug_3885():
  from paraxip.sipua.ui.model.cpaOM import CpaOM 
  cpaCall = CpaCall.CpaCall()
  
  class fake_CallerII:
    def getRequestUri(self):
      return ""
  
  cpaCall.callerInitialInvite = fake_CallerII()
  cpaCall.cpaStats = CpaOM()
  cpaCall.dumpCpaStats()

def test_module():
  test_CpdErrorStrings()
  test_CallResults()
  test_CpaCallReset()
  test_OmBug_3885()
  
if __name__ == '__main__':
  test_module()## !!MIRROR FILE, DO NOT MODIFY!!
