#
#  @file   CpaEamdCall.py
#
#  Copyright (C) 2010 Paraxip Technologies.
#  All Rights Reserved.
#

#Python Core Libraries
import os.path

#Logs
import paraxip.infra.PyLoggingFunctions as PyLoggingFunctions
from netborderPyInfra import TraceScope, CallLogger

#Sip-Related
from SipCallEngineExtending import PySipMessage

#Cpa-Data Related
from paraxip.sipua.user.cpa import CpaCall
from paraxip.sipua.user.cpa import CpaEamdStates
from paraxip.sipua.user.cpa import CpaResult
from paraxip.sipua.user.cpa import DetailedCpdResult

# Force declaration of class attributes in __init__ method
__pychecker__ = "initattr"

class CpaEamdCall(CpaCall.CpaCall):

  PARAM_CPA_EAMD_TIMEOUT_MS = 'cpa.eamd.timeout.ms'
  PARAM_CPA_RESULT_THRESHOLD_EAMD_BEEP = 'cpa.result.threshold.EAMD_BEEP'
  PARAM_CPA_RESULT_THRESHOLD_EAMD_SILENCE = 'cpa.result.threshold.EAMD_SILENCE'

  # Provide parameter information for this Call 
  @staticmethod
  def getParameterInfoDB():

    # Start with inherited params
    parameterInfoDB = CpaCall.CpaCall.getParameterInfoDB()

    # Parameters that must be set explicitly (mandatory, no defaults)
    parameterInfoDB.addParameterInfoForName(
        CpaEamdCall.PARAM_CPA_EAMD_TIMEOUT_MS,
        type=int)

    # Parameters with defaults
    parameterInfoDB.addParameterInfoForName(
        CpaEamdCall.PARAM_CPA_RESULT_THRESHOLD_EAMD_BEEP,
        type=float,
        defaultValue=0.7)
    parameterInfoDB.addParameterInfoForName(
        CpaEamdCall.PARAM_CPA_RESULT_THRESHOLD_EAMD_SILENCE,
        type=float,
        defaultValue=0.9)

    return parameterInfoDB

  def __init__(self):
    __pychecker__ = 'unusednames=t'

    CpaCall.CpaCall.__init__(self)
    self.setLogger(CallLogger("netborder.cpa.%s" % os.path.split(__file__)[1]))
    
    t = TraceScope(self.getLogger(), "CpaEamdCall.__init__")
    
    #
    # Public Members
    #
    
    #None! :)
  
  #@override
  def addStates(self):
    CpaCall.CpaCall.addStates(self)
    
    # WAITING_EAMD_RESULT state
    waitForCpaEamdResultState = CpaEamdStates.WaitForCpaEamdResultState(self)
    self.addState(waitForCpaEamdResultState.getName(), 
                  waitForCpaEamdResultState)
  
  #@override
  def initialize(self, in_refApplication, in_refGlobalContext, in_szCallId):
    
    CpaCall.CpaCall.initialize(self,
                             in_refApplication,
                             in_refGlobalContext,
                             in_szCallId)
    
    PyLoggingFunctions.log_debug(self.getLogger(), self.__class__.__name__ + " initialized with config: " + str(self.config))
    
    return True
  
  #@override
  def prepareCpaEndpointMsg(self):
    inviteMsg = CpaCall.CpaCall.prepareCpaEndpointMsg(self)
    inviteMsg.addCustomHeader("CPA-IsEamdEnabled", 'True')    
    return inviteMsg
  
  #@override
  def transitionFromPreOrPostDetectionDone(self, out_pystrNewStateName):
    
    strCpaResult = self.getCallStack().getCpaResult().getStr()
    
    #Check for unexpected CpaResult
    if not CpaResult.isPreOrPostConnect(strCpaResult):
      PyLoggingFunctions.log_warn(self.getLogger(), 
                                  "Unexpected CpaResult in "
                                  "CpaEamdCall::transitionFromPreOrPostDetectionDone")
    
    if (strCpaResult == CpaResult.CpaResult.AnsweringMachine):
      out_pystrNewStateName.set('WAITING_EAMD_RESULT')
    else:
      self._reintegrateCallFlowAfterCPADone(out_pystrNewStateName)
    
    return True

def test_classInit():
  from paraxip.sipua.user.cpa import CallEvent
  from paraxip.sipua.user.cpa import CallEventStack
  
  cpaEamdCall = CpaEamdCall()
  print len( cpaEamdCall.getCallStack().getCallEventList() )
  assert len( cpaEamdCall.getCallStack().getCallEventList() ) == 0
  
  cpaEamdCall.notifyCallEvent(CallEvent.Factory_CallCpaResult( CpaResult.CpaResult.Eamd_Beep ))
  print len( cpaEamdCall.getCallStack().getCallEventList() )
  assert len( cpaEamdCall.getCallStack().getCallEventList() ) == 1
  
  print 'cpaEamdCall should be a new empty object now'
  cpaEamdCall = CpaEamdCall()
  print len( cpaEamdCall.getCallStack().getCallEventList() )
  assert len( cpaEamdCall.getCallStack().getCallEventList() ) == 0 
  
  cpaEamdCall = CpaEamdCall()
  cpaEamdCall.notifyCallEvent(CallEvent.Factory_CallCpaResult( CpaResult.CpaResult.Eamd_Beep ))
  
  # Make sure the class can be 'reset' by assigning to a new instance
  from paraxip.sipua.user.cpa import CallEvent
  from paraxip.sipua.user.cpa import CallEventStack
  a = CallEventStack.CallEventStack( [CallEvent.Factory_CallCpaResult( CpaResult.CpaResult.Human )] )
  a = CallEventStack.CallEventStack()
  assert len(a.getCallEventList()) == 0 

def test_stateTransition():
  # false positive on __init__ in classes below
  __pychecker__ = "no-reuseattr"

  "Test state transition in the case of Eamd"
  
  from netborderPyInfra import PyString
  from paraxip.sipua.user.cpa import CallEvent
  
  class FakeSM:
    
    def __init__(self):
      self.logger = CallLogger("netborder.cpa.FakeSm")
    
    def getLogger(self):
      return self.logger

  # When it should transition to WAITING_EAMD_RESULT
  cpaEamdCall = CpaEamdCall()
  cpaEamdCall.notifyCallEvent( CallEvent.Factory_CallCpaResult( CpaResult.CpaResult.AnsweringMachine ) )
  nextState = PyString('TEST_CASE_NO_MATTER_SHOULD_GO_TO_EAMD_STATE')
  cpaEamdCall.transitionFromPreOrPostDetectionDone(nextState)
  print nextState
  from paraxip.sipua.user.cpa.CpaEamdStates import WaitForCpaEamdResultState
  waitEamdState = WaitForCpaEamdResultState(FakeSM())
  assert str(nextState) == waitEamdState.getName()
  
  # Cannot be tested: need a valid dialog
  #  # When it should transition to REINVITE
  #  print 'When it should transition to the previously set state'
  #  cpaEamdCall = CpaEamdCall()
  #  cpaEamdCall.notifyCallEvent( CallEvent.Factory_CallCpaResult( CpaResult.CpaResult.Human ) )  
  #  fakeState = FakeState("TEST_CASE_NO_MATTER_SHOULD_REINVITE")
  #  from paraxip.sipua.internal.ConnectionStatus import ConnectionStatus
  #  cpaEamdCall.calleeLegStatus == ConnectionStatus.Connected
  #  cpaEamdCall.transitionFromPreOrPostDetectionDone(fakeState)
  #  print fakeState.strStateName
  #  from paraxip.sipua.user.cpa.CpaEamdStates import WaitForCpaEamdResultState
  #  waitEamdState = WaitForCpaEamdResultState(FakeSM())
  #  print "!"+fakeState.strStateName
  #  #assert fakeState.strStateName == expectedTransition
  
  # Should work with pre-connect results
  #@tag Find a way to check there was NO error(s)/warn(s) logged
  cpaEamdCall = CpaEamdCall()
  cpaEamdCall.notifyCallEvent( CallEvent.Factory_CallCpaResult( CpaResult.CpaResult.ReOrder ) )
  nextState = PyString("TEST_CASE")
  cpaEamdCall.transitionFromPreOrPostDetectionDone(nextState)
  
  # Should not work with eamd results
  #@tag Find a way to check there was an error/warn logged
  cpaEamdCall = CpaEamdCall()
  cpaEamdCall.notifyCallEvent( CallEvent.Factory_CallCpaResult( CpaResult.CpaResult.Eamd_Beep ) )
  nextState = PyString("TEST_CASE")
  cpaEamdCall.transitionFromPreOrPostDetectionDone(nextState)

def test_detailedCpdResult():
  from paraxip.sipua.user.cpa import CallEvent
  
  cpaEamdCall = CpaEamdCall()
  detailedCpdR = cpaEamdCall.getDetailedCpdResultFromStateVariables()
  assert detailedCpdR == DetailedCpdResult.DetailedCpdResult.NoResult
  
  cpaEamdCall = CpaEamdCall()
  cpaEamdCall.notifyCallEvent( CallEvent.Factory_CallCpaResult( CpaResult.CpaResult.Eamd_Beep ) )
  detailedCpdR = cpaEamdCall.getDetailedCpdResultFromStateVariables()
  print detailedCpdR
  assert detailedCpdR == DetailedCpdResult.DetailedCpdResult.Eamd_Beep

def test_module():
  #@tag Move to a different file
  test_classInit()
  test_stateTransition()
  test_detailedCpdResult()

if __name__ == '__main__':
  test_module()


## !!MIRROR FILE, DO NOT MODIFY!!
