#
#  @file   CpaEamdStates.py
#
#  Copyright (C) 2010 Sangoma Technologies.
#  All Rights Reserved.
#

#
# Our imports
#
from paraxip.infra.Assert import logAssert
import paraxip.infra.PyLoggingFunctions as PyLoggingFunctions
from netborderPyInfra import TraceScope

from paraxip.sipua.ui.model.cpaOM import CpaOM 
from paraxip.sipua.internal.ConnectionStatus import ConnectionStatus

from paraxip.sipua.user.cpa import CpaStates
from paraxip.sipua.user.cpa import CpaResult
from paraxip.sipua.user.cpa import CpdResult
from paraxip.sipua.user.cpa import DetailedCpdResult
from paraxip.sipua.user.cpa import CallEvent

# Force declaration of class attributes in __init__ method
__pychecker__ = "initattr"

  
class WaitForCpaEamdResultState(CpaStates.WaitForCpaResultCommonState):
  """
  Wait for Cpa result of EAMD_BEEP or EAMD_SILENCE.
  Reintegrate call flow with no result if the Eamd timeout expires.
  """
  
  def __init__(self, in_stateMachine):
    super(WaitForCpaEamdResultState, self).__init__(in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "WaitForCpaEamdResultState.__init__")
  
  def reset(self):
    CpaStates.WaitForCpaResultCommonState.reset(self)
  
  def getName(self):
    #@tag Should be a @staticmethod
    return "WAITING_EAMD_RESULT"
  
  def entryAction(self, in_event, out_pystrNewStateName):
    __pychecker__ = 'unusednames=in_event'
    out_pystrNewStateName.set(self.getName())
    uiTimeoutMs = self._stateMachine.config[self._stateMachine.PARAM_CPA_EAMD_TIMEOUT_MS]
    self.getTimerManager().startTimer("EamdTimeout", uiTimeoutMs)
    
  def onStateExit(self, in_event, in_strDestStateName):
    PyLoggingFunctions.log_debug(self.getLogger(), "%s::onStateExit %s",
              self.__class__.__name__, in_event.__class__.__name__)

    # Class should NOT have more than 1 parent
    logAssert(self.getLogger(), len(self.__class__.__bases__) <= 1)
    
    super(WaitForCpaEamdResultState, self).onStateExit(in_event, in_strDestStateName)
    
    ###
    ### @see bug no.3480: Need to update the CpdResult in case there was a CANCEL
    ###      after receiving a CPA_MACHINE but before receiving the EAMD_RESULT.
    ###
    
    #Transform the Cpa to Cpd result
    strCpdResult = self._stateMachine.getCpdResultFromStateVariables()
                                                      
    PyLoggingFunctions.log_debug(self.getLogger(),
      "Setting CpaOM.OM_CPA_RESULT to %s." % strCpdResult)
    #Store the result in class that ends-up in Cpa-Stats.csv. Note that this
    #should be the ONLY place where CpaOM.OM_CPA_RESULT is written.
    self._stateMachine.cpaStats.addOM(CpaOM.OM_CPA_RESULT, strCpdResult)
    self._stateMachine.cpaStats.addCurrentTimeOM(CpaOM.OM_CPA_TIME_CPA_COMPLETED)
    
    specialEvent = self._stateMachine.getDetailedCpdResultFromStateVariables()    
    self._stateMachine.cpaStats.addOM(CpaOM.OM_SPECIAL_CALL_INFORMATION, specialEvent)
    ###
    
  
  def _handle_TimeoutEvent_Custom(self, in_event):
    __pychecker__ = 'unusednames=in_event'
    
    self._stateMachine.notifyCallEvent( CallEvent.Timeout( CallEvent.Timeout.Postconnect_Eamd, 
                                        'Timeout_Eamd'))
  
  # @override
  def _handle_DialogToCallInfoEvent(self, in_event, out_pystrNewStateName):
    """Where the CPA-Eamd outcome is received from the CPA as a SIP message"""
    #
    # Extract SIP message from event and let the state machine decide
    # if there is any relevant information to be processed.
    #
    sipMsg = in_event.getSipMessage()
    PyLoggingFunctions.log_debug(self.getLogger(), "CPA state " \
        "'WaitForCpaEamdResultState' got SIP message:\n" \
              + sipMsg.getBodyText() )
    strEamdOutcome = self._stateMachine.getFinalCPAOutcomeFromInfoMsg(
        sipMsg,
        in_bCalleeIsConnected = \
        self._stateMachine.calleeLegStatus == ConnectionStatus.Connected)
    
    #
    # Wait for an acceptable Eamd result.
    #
    if strEamdOutcome is not None:
      if not CpaResult.isEamd(strEamdOutcome):
        # Should not happen, it means that the CPA is sending the 
        # wrong type of message.
        PyLoggingFunctions.log_error(self.getLogger(), 
          "Invalid INFO message received from CPA, not an EamdOutcome: %s." % strEamdOutcome)
        
      PyLoggingFunctions.log_info(self.getLogger(), "Received CPA-Eamd result of %s; reintegrating call flow." \
                                  % strEamdOutcome)
      
      # Commit the value back to the state machine
      self._stateMachine.notifyCallEvent( CallEvent.Factory_CallCpaResult(strEamdOutcome) )
      
      # Done with CPA analysis
      return self._stateMachine._reintegrateCallFlowAfterCPADone(\
          out_pystrNewStateName)
    
    sipMsg = in_event.getSipMessage()
    
    # Received a SIP INFO without a CpaResult. The message should be
    # forwarded to the caller/callee. For now, log a warning and drop
    # the INFO.
    
    # A message for the client
    PyLoggingFunctions.log_warn(self.getLogger(), "SIP INFO message not supported.")
    
    # A message for the developer
    PyLoggingFunctions.log_info(self.getLogger(), "Received an unexpected INFO message "
                                                  "in current state (%s). Sip message " 
                                                  "received: %s." % (self.getName(), sipMsg.getBodyText()))
    
    return True
## !!MIRROR FILE, DO NOT MODIFY!!
