#
#  @file   CpaResult.py
#  @author Francis Moreau
#  date    Mon July 30 2007
#
#  Copyright (C) 2007 Paraxip Technologies.
#  All Rights Reserved.
#

class CpaResult:
  """The list of possible CPA results returned by the CpaEngineImpl.{cpp, hpp}
  
  @tag Maybe should rename to CpaEngineImplInterface.py ??
  @tag This should be a C++ class exposed in Python.
  @see cpa/ml/src/CPAResult.hpp contains most of the original
       C++ annotations.
  @see cpa/src/CPAAnnotation.hpp also contains the annotation,
       but they are not used in the code that this class interacts
       with (I think they are used to parse .wav files).
  """
  
  ## Pre-connect Results
  Busy =             "CPA_BUSY"
  ReOrder =          "CPA_REORDER" # Fast busy
  SIT_Temporary = 'CPA_SIT_TEMPORARY'
  SIT_Permanent = 'CPA_SIT_PERMANENT'
  All_PreConnect = [Busy, ReOrder, SIT_Temporary, SIT_Permanent]
  
  ## Post-connect Results
  Human =            "CPA_HUMAN"
  AnsweringMachine = "CPA_MACHINE"
  Fax =              "CPA_FAX"
  # Special tag when no result happens. Not set by the Cpa
  # but set by us meaning 'we did  not get anything from the'
  # Cpa.
  NoResult = 'CPA_NO_RESULT'
  # @tag Maybe Eamd are part of PostConnect?
  All_PostConnect = [Human, AnsweringMachine, Fax, NoResult]
  
  ## Eamd Results
  Eamd_Beep    =     "EAMD_BEEP"
  Eamd_Silence =     "EAMD_SILENCE"
  All_Eamd = [Eamd_Beep, Eamd_Silence]
  
  ## All results, used for test
  AllCpaResults = All_PreConnect + All_PostConnect + All_Eamd
  
  ## Types
  Type_PreConnect = 'Type_PreConnect'
  Type_PostConnect = 'Type_PostConnect'
  Type_Eamd = 'Type_Eamd'
  Type_Unknown = 'Type_Unknown'
  
  @staticmethod
  def getListOfAllCpaResult():
    return CpaResult.AllCpaResults

#
#@tag The is<X> functions should probably be @staticmethod members
#     of CpaResult.
#

def isPreConnect(in_CpaResult):
  assert isinstance(in_CpaResult, basestring)
  if in_CpaResult in CpaResult.All_PreConnect:
    return True
  else:
    return False

def isPostConnect(in_CpaResult):
  assert isinstance(in_CpaResult, basestring)
  if in_CpaResult in CpaResult.All_PostConnect:
    return True
  else:
    return False

def isPreOrPostConnect(in_CpaResult):
    return ( isPreConnect(in_CpaResult) or isPostConnect(in_CpaResult) )
    
def isEamd(in_CpaResult):
  assert isinstance(in_CpaResult, basestring)
  if in_CpaResult in CpaResult.All_Eamd:
    return True
  else:
    return False

def getCpaResultType(in_CpaResult):
  if in_CpaResult is None:
    return CpaResult.Type_Unknown
  
  if isPreConnect(in_CpaResult):
    return CpaResult.Type_PreConnect
  elif isPostConnect(in_CpaResult):
    return CpaResult.Type_PostConnect
  elif isEamd(in_CpaResult):
    return CpaResult.Type_Eamd
  else:
    return CpaResult.Type_Unknown

def test_Module():
  """
  Is run automatically from sip/hub/src/runtest.pl
  """
  cpaResult = CpaResult.Busy
  assert isPreConnect(cpaResult)
  cpaResult = CpaResult.AnsweringMachine
  assert not isPreConnect(cpaResult)
  assert isPostConnect(cpaResult)
  cpaResult = CpaResult.Eamd_Silence
  assert not isPreConnect(cpaResult)
  assert not isPostConnect(cpaResult)
  assert isEamd(cpaResult)
  assert getCpaResultType(cpaResult) == CpaResult.Type_Eamd
  assert getCpaResultType("Invalid_Str") == CpaResult.Type_Unknown

 
if __name__ == '__main__':
  test_Module()  
  
  
## !!MIRROR FILE, DO NOT MODIFY!!
