

from netborderPyInfra import Logger 
import paraxip.infra.PyLoggingFunctions as PyLoggingFunctions

from paraxip.sipua.ui.model.cpaOM import CpaOM 

import re

cpaStatsLogger = Logger('netborder.cpa.stats')

def _appendStatsToList(in_cpaOM, in_list, in_key):
  
  if in_cpaOM.isOM(in_key):
    item = in_cpaOM.getOM(in_key)
    if not isinstance(item, basestring):
      item = 'PyNotStr_' + str(item)
    in_list.append(item)
  else:
    in_list.append("")

def WriteToCsv(in_cpaOM, in_strCallId, in_strCallerRequestURL):
  """
   Write CPA stats using the following CSV format:
     - Paraxip Call-id  
     - Date of the call 
     - Campaign DBID (empty if it does not exist) 
     - Campaign Name (empty if it does not exist)
     - Phone number  (exclude public access code)
     - CPA Result 
     - Time Dialing 
     - Time Connected 
     - Time CPA Completed 
     - Time Queued (if relevant, empty if it doesn't exist) 
     - Time Connected to agent
  """
  
  cpaStatsList = []
  
  # Call-id
  in_cpaOM.addOM(CpaOM.OM_CALL_ID, in_strCallId)
  _appendStatsToList(in_cpaOM, cpaStatsList, CpaOM.OM_CALL_ID)
  
  # Date
  _appendStatsToList(in_cpaOM, cpaStatsList, CpaOM.OM_CPA_DATE)
  
  # Campaign ID
  _appendStatsToList(in_cpaOM, cpaStatsList, CpaOM.OM_CPA_CAMPAIGN_ID)
  
  # Campaign Name
  _appendStatsToList(in_cpaOM, cpaStatsList, CpaOM.OM_CPA_CAMPAIGN_NAME)
  
  # Phone number
  strCallerRequestURL = in_strCallerRequestURL
  phoneNumberRegEx = re.compile('sip:(\S+)@')
  if strCallerRequestURL:
    m = phoneNumberRegEx.match(strCallerRequestURL)
    if m:
      cpaStatsList.append(m.group(1))
    else:
      cpaStatsList.append("")
  else:
    cpaStatsList.append("")
  
  # CPA result
  _appendStatsToList(in_cpaOM, cpaStatsList, CpaOM.OM_CPA_RESULT)
  
  # Time Dialing
  _appendStatsToList(in_cpaOM, cpaStatsList, CpaOM.OM_CPA_TIME_DIALING)

  # Time Connected
  _appendStatsToList(in_cpaOM, cpaStatsList, CpaOM.OM_CPA_TIME_CONNECTED)

  # Time CPA completed
  _appendStatsToList(in_cpaOM, cpaStatsList, CpaOM.OM_CPA_TIME_CPA_COMPLETED)

  # Time callee was queued before talking to an agent
  _appendStatsToList(in_cpaOM, cpaStatsList, CpaOM.OM_CPA_TIME_QUEUED)

  # Time agent was connected
  _appendStatsToList(in_cpaOM, cpaStatsList, CpaOM.OM_CPA_TIME_AGENT_CONNECTED)
  
  # New special call information field
  _appendStatsToList(in_cpaOM, cpaStatsList, CpaOM.OM_SPECIAL_CALL_INFORMATION)

  # Fast way to concatenate string list using "," as separator
  strCpaStats = ','.join(cpaStatsList)
  
  # Finally logging the CSV line
  PyLoggingFunctions.log_info(cpaStatsLogger, strCpaStats)
  
  
def test_module():
  from paraxip.sipua.ui.model.cpaOM import CpaOM
  cpaOM = CpaOM()
  WriteToCsv(cpaOM, 'test_call_ID', 'test_caller_url')

if __name__ == '__main__':
  test_module()## !!MIRROR FILE, DO NOT MODIFY!!
