#
#  @file   CpaUASStates.py
#  @author Francis Moreau
#  date    Mon Jun 11 2007
#  
#  Description:
#
#  Type your description here
#
#  Copyright (C) 2007 Paraxip Technologies.
#  All Rights Reserved.
#

import paraxip.infra.PyLoggingFunctions as PyLoggingFunctions
from netborderPyInfra import TraceScope

from netborderPyML import *

from SipCallEngineExtending import \
    PyCPAClassifierType, PySipMessage, MediaSdpOfferGeneratedEvent, \
    MediaCPAUpdateEvent, InitialEvent, StopEvent, KillEvent

from PyResipStack import *

#@tag FIX THE import *, use from <x> import <y>
from paraxip.sipua.internal.SipCallEngineState import *
from paraxip.sipua.internal.SipCallEngineEvent import *
from paraxip.sipua.internal.ConnectionStatus import *
from paraxip.sipua.internal.SMTimeoutProcessor import *

import ComputeCpaResult
import CpaResult
import CpaMsgsSentMemory


#
# Call states
#

#
# Base class
#

class CpaUASStates(SipCallEngineState):
  def __init__(self, in_stateMachine):
    SipCallEngineState.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(),
               "CpaUASStates.__init__")
               

#
# Idle state
#

class IdleState(CpaUASStates):
  def __init__(self, in_stateMachine):
    CpaUASStates.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "IdleState.__init__")
  
  def getName(self):
    return self.getIdleName()

  def getIdleName(self):
    """ 
    Returns the name of Idle state, no matter what is the
    most-derived class of the object 'self' 
    
    """
    return "IDLE"
  

  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)

    self._stateMachine._cpaMsgsSentMemory = CpaMsgsSentMemory.CpaMsgsSentMemory()

    out_pystrNewStateName.set(self.getIdleName())
    return True

  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    out_pystrNewStateName.set(self.getIdleName())
    
    if (isinstance(in_event, InitialEvent)):
      return True
    
    elif (isinstance(in_event, InDialogToCallCalledEvent)):
      """
      Received an INVITE with SDP
      """
      self._stateMachine.callerLegStatus = ConnectionStatus.Connecting
      
      sipMessage = in_event.getSipMessage()
      CustomHeader = 'CPA-Threshold'
      
      # Retrieve cpa threshold passed in SIP custom header
      strCPAThresholds = sipMessage.getHeaderValue(CustomHeader)
      
      #Will be something like:
      #CPA-Threshold: EAMD_BEEP=0.4;global=0.9;CPA_MACHINE=0.9
      PyLoggingFunctions.log_debug(self.getLogger(), "Received Thresholds: '%s'." % strCPAThresholds)
      
      if(strCPAThresholds == ''):
        
        PyLoggingFunctions.log_warn(self.getLogger(),
                  "The NCA Engine should not receive a SIP INVITE without "
                  "a custom header. Maybe the user meant to contact the NCA "
                  "Service and accidentally used the NCA Engine port?")
        
        self._stateMachine._CallRejectReason = "SIP INVITE without custom header"
        out_pystrNewStateName.set("DISCONNECTING")
        return True
      
      # Create the computeCPAResult object out of the threshold string retrieved
      self._stateMachine._cpaResultFilter = ComputeCpaResult.CpaResultFilter(self.getLogger())
      self._stateMachine._cpaResultFilter. \
        configureCpaThresholdsFromString(strCPAThresholds)
      
      # Stay in same state - a request for an SDP offer is coming right up
      return True
    
    elif (isinstance(in_event, DialogToCallSdpOfferRequiredEvent)):
      # INVITE without SDP -> this is what's expected by hub in CPA scenario
      
      # An SDP offer is required, instantiate a media termination entity and ask
      #   it for an initial offer
      self._stateMachine.media = \
        self._stateMachine.globalContext.acquireMedia(self._stateMachine.getCallId())
      if (self._stateMachine.media == None):
        PyLoggingFunctions.log_error(self.getLogger(),
                  "IdleState acquireMedia failed")
        return False
      
      out_pystrNewStateName.set("GENERATING_SDP_OFFER")

      return True
    
    elif (isinstance(in_event, DialogToCallTerminatedEvent)):
      self._stateMachine.dialogIdInitiatingDisconnect = \
        in_event.getDialogId()
      
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    elif (isinstance(in_event, StopEvent)):
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    elif (isinstance(in_event, MediaCPAUpdateEvent)):
      # Stray message coming way too late, just drop it and stay in same state
      return True


    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)
    
    return True

#
# GeneratingSdpOffer state
#

class GeneratingSdpOfferState(IdleState):
  def __init__(self, in_stateMachine):
    IdleState.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "GeneratingSdpOfferState.__init__")

  def getName(self):
    return "GENERATING_SDP_OFFER"

  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)

    if (self._stateMachine.media.generateSdpOffer()):
      out_pystrNewStateName.set(self.getName())

    else:
      # Not able to obtain SDP offer: reject incoming call
      # Stay in IDLE state
      self._stateMachine.uiRejectCallerWithInternalSipErrorCode = 500 # Server Internal Error
      # TODO: Reason header
      # self._stateMachine.strHangupWithInternalReasonHeader = ""
      out_pystrNewStateName.set("DISCONNECTING")

    return True

  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    out_pystrNewStateName.set(self.getName())
    
    if (isinstance(in_event, MediaSdpOfferGeneratedEvent)):
      # Get a copy of provided SDP offer
      sdpOffer = in_event.getSdpOfferRef()

      # Acknowledge the Sdp Offer is to be used
      if (not self._stateMachine.media.ackGeneratedSdpOffer(sdpOffer)):
        out_pystrNewStateName.set("DISCONNECTING")

      # Anwser call while providing our SDP offer
      else:
        if (not self._stateMachine.answerInDialog(
            self._stateMachine.callerDialogId,
            sdpOffer,
            True, # SDP is an offer
            self._stateMachine.getAnswerMessage())):
          out_pystrNewStateName.set("DISCONNECTING")
          return False
        self._stateMachine.callerLegStatus = ConnectionStatus.Answering

      out_pystrNewStateName.set("ANSWERING")

    else:
      return IdleState.processEvent(self, in_event, out_pystrNewStateName)

    return True

#
# Answering state
#

class AnsweringState(CpaUASStates):
  def __init__(self, in_stateMachine):
    CpaUASStates.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "AnsweringState.__init__")
  
  def getName(self):
    return "ANSWERING"
  
  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    out_pystrNewStateName.set(self.getName())
    
    if (isinstance(in_event, DialogToCallSdpAnsweredEvent)):
      # TODO: Validate the SDP answer and terminated the call with a Reason: header
      #  if not acceptable
      
      # Stay in same state
      return True
    
    elif (isinstance(in_event, DialogToCallConnectedEvent)):
      self._stateMachine.callerLegStatus = ConnectionStatus.Connected
      
      out_pystrNewStateName.set("CONNECTED")
      return True
    
    elif (isinstance(in_event, DialogToCallAckNotReceivedEvent)):
      # TODO use mutable logger for this warning
      PyLoggingFunctions.log_warn(self.getLogger(), "The SIP UAC did not answer our " \
        "SIP 200 response with a ACK request as specified by RFC 3261. " \
        "Terminating the call. (State: %s)",self.getName())
      
      # TODO: Reason header
      # self._stateMachine.strHangupWithInternalReasonHeader = ""
      out_pystrNewStateName.set("DISCONNECTING")
      return True;
    
    elif (isinstance(in_event, DialogToCallTerminatedEvent)):
      self._stateMachine.dialogIdInitiatingDisconnect = \
        in_event.getDialogId()
      
      out_pystrNewStateName.set("DISCONNECTING")
      return True
    
    elif (isinstance(in_event, StopEvent)):
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    elif (isinstance(in_event, MediaCPAUpdateEvent)):
      # Seen under load where audio has been received and processed by 
      # CPAMediaEngine before we received the ACK for our 200 Ok with SDP
      return True


    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)
    
    return True

#
# Connected state
#

class ConnectedState(CpaUASStates):
  def __init__(self, in_stateMachine):
    CpaUASStates.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "ConnectedState.__init__")    
  
  def getName(self):
    return "CONNECTED"
  
  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    if not self._stateMachine.bMediaComputeCPAResultsCalled:
      if (self._stateMachine._cpaResultFilter is None):
        PyLoggingFunctions.log_error(self.getLogger(),
                  "ConnectedState self._stateMachine._cpaResultFilter not set")
        return False
      
      # Set the initial classifier
      self._stateMachine.media.switchToClassifier(PyCPAClassifierType(PyCPAClassifierType.Preconnect))
      
      # Do this only once per call.
      self._stateMachine.bMediaComputeCPAResultsCalled = True
    
    out_pystrNewStateName.set(self.getName())
    return True
  
  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    out_pystrNewStateName.set(self.getName())
    
    if (isinstance(in_event, DialogToCallInfoEvent)):
      self._stateMachine.handleIncomingInfoMsg(in_event)
      # Stay in the same state
      return True        
    
    elif (isinstance(in_event, MediaCPAUpdateEvent)):
      out_pystrNewStateName.set("SENDING_CPA_UPDATE")
      return True
    
    elif (isinstance(in_event, DialogToCallTerminatedEvent)):
      self._stateMachine.dialogIdInitiatingDisconnect = \
        in_event.getDialogId()
      
      out_pystrNewStateName.set("DISCONNECTING")
      return True
    
    elif (isinstance(in_event, StopEvent)):
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)
    
    return True


class SendingCPAUpdateState(CpaUASStates):
  def __init__(self, in_stateMachine):
    CpaUASStates.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "SendingCPAUpdateState.__init__")
  
  def getName(self):
    return "SENDING_CPA_UPDATE"
  
  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    out_pystrNewStateName.set( self.getName() )
    
    # The event is assumed to be of type MediaCPAUpdateEvent
    assert isinstance(in_event, MediaCPAUpdateEvent)
    
    pyMlClassifierResult_Cpa = in_event.getResults()
    # No need to sort, already sorted. Leave the CPU to other more meaningful task.
    # cpaResults.sort()        
    cpaResultsListSorted = pyMlClassifierResult_Cpa.getResultListCopy()
    strCpaResults = pyMlClassifierResult_Cpa.getResultsAsString()
    
    if (self._stateMachine._cpaResultFilter is None):
      PyLoggingFunctions.log_error(self.getLogger(),
                "ConnectedState self._stateMachine._cpaResultFilter not set.")
      return False
    
    strCpaResult = self._stateMachine._cpaResultFilter.\
        getHighestCpaResultAboveThreshold(
          cpaResultsListSorted,
          self._stateMachine.cpaCalleeAnswered)
    
    PyLoggingFunctions.log_debug(self.getLogger(), "SendingCPAUpdateState strCpaResult:%s." % strCpaResult)
    
    if strCpaResult == None:
      out_pystrNewStateName.set("CONNECTED")
      return True
    
    msgType = CpaResult.getCpaResultType(strCpaResult)
    isAlreadySent = self._stateMachine._cpaMsgsSentMemory.isTypeOfMessageAlreadySent(msgType)
    
    if isAlreadySent:
      PyLoggingFunctions.log_debug(self.getLogger(), 
                                   "Result type was sent already, skiping sending of %s." % strCpaResult)
      out_pystrNewStateName.set("CONNECTED")
      return True
    else:
      #
      # Send SIP info message
      #
      if (not self._stateMachine.sendInfoMsgToDialog(
            self._stateMachine.callerDialogId, "application",
            "cpa-update", strCpaResults)):
        return False
      
      self._stateMachine._cpaMsgsSentMemory.notifySipMessageSent(msgType)
	  
      self._stateMachine.media.switchToClassifier(PyCPAClassifierType(PyCPAClassifierType.Eamd))
      
      # Stay in the same state and wait for 200 OK from INFO
      return True
    
    # Stay in the same sate
    return True
  
  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    out_pystrNewStateName.set(self.getName())
    
    if (isinstance(in_event, DialogToCallInfoSentSucceededEvent)):
      #Received 200 OK ?
      out_pystrNewStateName.set("CONNECTED")
      return True

    elif (isinstance(in_event, DialogToCallInfoSentFailedEvent)): 
      # Race condition seen under load, the 200 Ok of our INFO did not
      # arrive fast enough and resiprocate timed out
      # Could try to resend the INFO...
      # For now, as the INFO msg was not saved, it is not possible to 
      # resend, so hope for the best and try to continue the call.
      # Worst case, NCA service will timeout and drop the call.
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent, Event: %s : The reception of the previous "
               "INFO message with CPA result was not confirmed in a timely fashion. "
               "Trying to continue the call anyway.",
               self.__class__.__name__, in_event.__class__.__name__)
      out_pystrNewStateName.set("CONNECTED")
      return True
    
    elif (isinstance(in_event, DialogToCallInfoEvent)):
      self._stateMachine.handleIncomingInfoMsg(in_event)
      # Stay in the same state
      return True
    
    elif (isinstance(in_event, MediaCPAUpdateEvent)):
      # We could receive intermediate update while sending the info, 
      # just trash them and stay in the same state
      return True
    
    elif (isinstance(in_event, DialogToCallTerminatedEvent)):
      self._stateMachine.dialogIdInitiatingDisconnect = \
        in_event.getDialogId()
      
      out_pystrNewStateName.set("DISCONNECTING")
      return True
    
    elif (isinstance(in_event, StopEvent)):
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)
    
    return True

#
# Disconnecting state
#

class DisconnectingState(CpaUASStates):
  def __init__(self, in_stateMachine):
    CpaUASStates.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "DisconnectingState.__init__")
  
  def getName(self):
    return "DISCONNECTING"
  
  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    out_pystrNewStateName.set(self.getName())
    
    if (self._stateMachine.dialogIdInitiatingDisconnect is not None):
      self._stateMachine.callerLegStatus = ConnectionStatus.Disconnected
    
    if (self._stateMachine.callerLegStatus == ConnectionStatus.Disconnected):
      out_pystrNewStateName.set("DISCONNECTED")
      return
    
    # Reject the caller if it's connecting
    if (self._stateMachine.callerLegStatus == ConnectionStatus.Connecting):
      rejectResponseCode, rejectSipMessage = \
        self._stateMachine.getRejectMessage()
      if (not self._stateMachine.rejectDialog(
            self._stateMachine.callerDialogId,
            rejectResponseCode,
            rejectSipMessage)):
        return
      self._stateMachine.callerLegStatus = ConnectionStatus.Disconnecting
    
    # Hanging up the caller if it's connected
    elif (self._stateMachine.callerLegStatus == ConnectionStatus.Answering or
          self._stateMachine.callerLegStatus == ConnectionStatus.Connected):
      if (not self._stateMachine.hangupDialog(
            self._stateMachine.callerDialogId)):
        return
      self._stateMachine.callerLegStatus = ConnectionStatus.Disconnecting
  
  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    out_pystrNewStateName.set(self.getName())
    
    if (isinstance(in_event, MediaCPAUpdateEvent)):
      # Cpa update received, but too late: stay in same state
      return True
    
    elif (isinstance(in_event, DialogToCallSdpAnsweredEvent)):
      # Already been cancelled in entryAction function. This event
      # was probably already pending in the queue. Then stay in the same state
      # since DialogToCallTerminatedEvent will follow.
      return True
    
    elif (isinstance(in_event, DialogToCallSdpOfferedEvent)):
      # Already been cancelled in entryAction function. This event
      # was probably already pending in the queue. Then stay in the same state
      # since DialogToCallTerminatedEvent will follow.
      return True
    
    elif (isinstance(in_event, DialogToCallInfoEvent)):
      # Too late to handle received INFO (cannot answer as dialog is tearing
      # down): ignore and stay in same state (bug 1018)
      # Stay in the same state
      return True

    elif (isinstance(in_event, DialogToCallTerminatedEvent)):
      self._stateMachine.callerLegStatus = ConnectionStatus.Disconnected

      out_pystrNewStateName.set("DISCONNECTED")
      return True
    
    elif (isinstance(in_event, StopEvent)):
      # Stay in the same state
      return True

    elif (isinstance(in_event, KillEvent)):
      # Go directly in final state
      out_pystrNewStateName.set("DISCONNECTED")
      return True

    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)
    
    return True

#
# Disconnected state
#

class DisconnectedState(CpaUASStates):
  def __init__(self, in_stateMachine):
    CpaUASStates.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "DisconnectedState.__init__")
  
  def getName(self):
    return "DISCONNECTED"
  
  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)

    if (self._stateMachine.media is not None):
      # Stop and delete media endpoint created following last SDP exchange
      self._stateMachine.media.stopAndDeleteEndpoints()
      
      # It's important to set the media to None since it contains a proxy on
      # the call state machine
      self._stateMachine.media = None
      
    out_pystrNewStateName.set(self.getName())
    return True
  
  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    out_pystrNewStateName.set(self.getName())
    
    if (isinstance(in_event, MediaCPAUpdateEvent)):
      # Cpa update received, but too late: stay in same state
      return True
    
    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)
    
    return True

## !!MIRROR FILE, DO NOT MODIFY!!
