
from paraxip.sipua.user.cpa import CpaResult

class DetailedCpdResultException(Exception):
  pass

def isPythonDecorated(in_str):
  if len(in_str)>=4:
    if in_str[:2] == '__' and in_str[-2:] == '__':
      return True
  return False
  
class DetailedCpdResult:
  """
  The results logged in the 'Detailed Cpd Result' section of the 
  Cpa-Stats.csv file. Also the result sent in the SIP messages under 
  the header 'X-Netborder-Detailed-CPD-Result-v2-0'.
  
  These results indicate what can be inferred from the call events.
  
  @see http://intranet/pmwiki/pmwiki.php?n=Engineering.HeaderConvention
  """
  
  _Trait_DetailedCpdResult = None
  
  NoResult = '???'
  
  # Pre-connect results
  Sit_Temporary = 'Sit-Temporary'
  Sit_Permanent = 'Sit-Permanent'
  
  # Post-connect results
  Human = 'Human'
  Fax = 'Fax'
  AnsweringMachine = 'Answering Machine'
  
  # Errors/Timeouts
  Timeout_PostConnect = 'Unknown'
  Timeout_PreConnect = 'No-Answer'
  Cancelled = 'Cancelled'
  Reject = 'Reject'  
  Busy = 'Busy'
  Reorder = 'Reorder'
  
  # Eamd-Specific
  Eamd_Beep = 'End of Greeting - Beep'
  Eamd_Silence = 'End of Greeting - Silence'
  Timeout_Eamd = 'End of Greeting - Timeout' # Also an error/timeout
  
  @staticmethod
  def isStrADetailedCpdResult(in_str):
    strPublicStaticMembers = [static_member 
                              for static_key, static_member in DetailedCpdResult.__dict__.items() 
                              if (isinstance(static_member, str) and not isPythonDecorated(static_key))] 
    
    if in_str in strPublicStaticMembers:
      return True
    else:
      return False
  
  @staticmethod
  def getSipHeaderName():
    return 'X-Netborder-Detailed-CPD-Result-v2-0'
  
def test_module():
  assert isPythonDecorated('abd__abc__as__') == False
  assert isPythonDecorated('testtext') == False
  assert isPythonDecorated('__privateVar') == False
  assert isPythonDecorated('_privateVar__') == False
  assert isPythonDecorated('_privateVar') == False
  assert isPythonDecorated('__doc__') == True
  assert isPythonDecorated('__item__') == True
  assert isPythonDecorated('__anything__') == True
  assert isPythonDecorated('__anything_long__') == True
  
  assert DetailedCpdResult.isStrADetailedCpdResult( DetailedCpdResult.getSipHeaderName() ) == False
  assert DetailedCpdResult.isStrADetailedCpdResult( "Not A Detailed Cpd Result" ) == False
  assert DetailedCpdResult.isStrADetailedCpdResult( None ) == False
  assert DetailedCpdResult.isStrADetailedCpdResult( 0 ) == False
  assert DetailedCpdResult.isStrADetailedCpdResult( DetailedCpdResult.Eamd_Beep ) == True
  assert DetailedCpdResult.isStrADetailedCpdResult( DetailedCpdResult.Busy ) == True
  assert DetailedCpdResult.isStrADetailedCpdResult( DetailedCpdResult.Reorder ) == True
  assert DetailedCpdResult.isStrADetailedCpdResult( DetailedCpdResult.AnsweringMachine ) == True
  
if __name__ == '__main__':
  test_module()
  
  
## !!MIRROR FILE, DO NOT MODIFY!!
