"""
Allows one to use the logger with just one import.
It creates one logger per function call.
Logger name: netborder.cpa.<Caller_Module>.<Caller_Function>.
"""

import paraxip.infra.PyLoggingFunctions as PyLoggingFunctions
from netborderPyInfra import TraceScope, Logger  

import inspect
import os

def getCallerInfo():
  """
  Assumes that the caller is the second 'function call parent'.
  call stack: <Whatever>() -> Caller_fn() -> log_<X>() -> getCallerInfo()
  """
  stack = inspect.stack()
  caller_fn = stack[2][3]
  caller_filename = stack[2][1]
  caller_module = os.path.splitext(os.path.split(caller_filename)[1])[0]
  return (caller_module, caller_fn)

def log_error(in_strErr):
  caller_module, caller_fn = getCallerInfo()
  targetStr = "netborder.cpa." + caller_module + "." + caller_fn
  logger = Logger(targetStr)
  PyLoggingFunctions.log_error(logger, in_strErr)

def log_warn(in_strErr):
  caller_module, caller_fn = getCallerInfo()
  targetStr = "netborder.cpa." + caller_module + "." + caller_fn
  logger = Logger(targetStr)
  PyLoggingFunctions.log_warn(logger, in_strErr)

def log_info(in_strErr):
  caller_module, caller_fn = getCallerInfo()
  targetStr = "netborder.cpa." + caller_module + "." + caller_fn
  logger = Logger(targetStr)
  PyLoggingFunctions.log_info(logger, in_strErr)

def log_debug(in_strErr):
  caller_module, caller_fn = getCallerInfo()
  targetStr = "netborder.cpa." + caller_module + "." + caller_fn
  logger = Logger(targetStr)
  PyLoggingFunctions.log_debug(logger, in_strErr)

def test_module():
  log_error("test_QuickLogger")
  log_warn("test_QuickLogger")
  log_info("test_QuickLogger")
  log_debug("test_QuickLogger")

if __name__ == '__main__':
  test_module()
## !!MIRROR FILE, DO NOT MODIFY!!
