import paraxip.sipua.CheckImport
paraxip.sipua.CheckImport.CheckImport(__file__)

import paraxip.infra.PyLoggingFunctions as PyLoggingFunctions
from netborderPyInfra import Logger

fileScopeLogger = Logger('netborder.cpa.__init__')

# NOTE do not import anything locally without aliases, as the imports will be visible in .sipua.user.cpa for subsequent calls
from paraxip.sipua.user.NetBorderApp import NetBorderApp as _NetBorderApp

###############################################################################
# Call Analyzer Engine (ie CPA UAS process)

class Engine(_NetBorderApp):

  def onInviteGetCall(self, in_sipMessage):
    __pychecker__ = 'unusednames=in_sipMessage'
    """
    USER CONFIGURATION 
      Returns a tuple ([SM class:Class], [config:dict]) to tell which scenario \
      to run, and with what configuration.
      Must make sure that the named class is known (ie was imported)

    """
    from paraxip.sipua.user.cpa.CpaUASCall import CpaUASCall
    return (CpaUASCall,
            {
              #Nothing here; part of the configuration is received from
              #the B2BUA as a SIP nessage (see InDialogToCallCalledEvent)
              #in CpaUASStates.py.
            })

###############################################################################
# Call Analyzer Service base class

class CallAnalyzerApp(_NetBorderApp):
  
  # Valid parameter names for this App
  PARAM_NCA_ENDOFGREETING_ENABLE = 'nca.EndOfGreeting.enable'
  PARAM_NCA_REQUIRE_CPD_ON_TO_PERFORM_CPA = \
      'nca.RequireCpdOnToPerformCpa'
  PARAM_NCA_ACCEPT_CALLS_WHEN_CPA_NOT_AVAIL = \
      'nca.AcceptCallsWhenCPANotAvailable'

  # Aliases for Call properties - call properties have precedence over these
  PARAM_NCA_PRECONNECT_TIMEOUT_MS = \
      'nca.PreConnectTimeout.ms'
  PARAM_NCA_POSTCONNECT_TIMEOUT_MS = \
      'nca.PostConnectTimeout.ms'
  PARAM_NCA_END_OF_GREETING_TIMEOUT_MS = \
      'nca.EndOfGreetingTimeout.ms'
  PARAM_NCA_HUMAN_THRESHOLD = \
      'nca.HumanThreshold'
  PARAM_NCA_MACHINE_THRESHOLD = \
      'nca.MachineThreshold'
  PARAM_NCA_FAX_THRESHOLD = \
      'nca.FaxThreshold'
  PARAM_NCA_ENGINE_HOST = \
      'nca.EngineHost'
  PARAM_NCA_ENGINE_PORT = \
      'nca.EnginePort'
  
  # Provide parameter information for this App
  @staticmethod
  def getParameterInfoDB():
    from paraxip.sipua.internal.ParameterInfo import ParameterInfoDB
    parameterInfoDB = ParameterInfoDB()
    
    # Parameters that must be set explicitly (mandatory, no defaults)
    parameterInfoDB.addParameterInfoForName(
        CallAnalyzerApp.PARAM_NCA_ENDOFGREETING_ENABLE,
        type=bool)
    
    # Parameters with default values
    parameterInfoDB.addParameterInfoForName(
        CallAnalyzerApp.PARAM_NCA_REQUIRE_CPD_ON_TO_PERFORM_CPA,
        type=bool,
        defaultValue=False)
    parameterInfoDB.addParameterInfoForName(
        CallAnalyzerApp.PARAM_NCA_ACCEPT_CALLS_WHEN_CPA_NOT_AVAIL,
        type=bool,
        defaultValue=False)

    
    # Parameters that are aliases to Call parameters
    from paraxip.sipua.user.cpa.CpaCall import CpaCall 
    # Ending comma for aliases is to create a tuple
    # http://docs.python.org/reference/expressions.html#grammar-token-expression_list
    parameterInfoDB.addParameterInfoForName(
        CallAnalyzerApp.PARAM_NCA_PRECONNECT_TIMEOUT_MS,
        type=int,
        mandatory=False,
        aliases=(CpaCall.PARAM_CPA_PRECONNECT_TIMEOUT_MS,))
    parameterInfoDB.addParameterInfoForName(
        CallAnalyzerApp.PARAM_NCA_POSTCONNECT_TIMEOUT_MS,
        type=int,
        mandatory=False,
        aliases=(CpaCall.PARAM_CPA_POSTCONNECT_TIMEOUT_MS,))
    parameterInfoDB.addParameterInfoForName(
        CallAnalyzerApp.PARAM_NCA_HUMAN_THRESHOLD,
        type=float,
        mandatory=False,
        aliases=(CpaCall.PARAM_CPA_RESULT_THRESHOLD_HUMAN,))
    parameterInfoDB.addParameterInfoForName(
        CallAnalyzerApp.PARAM_NCA_MACHINE_THRESHOLD,
        type=float,
        mandatory=False,
        aliases=(CpaCall.PARAM_CPA_RESULT_THRESHOLD_ANSWERING_MACHINE,))
    parameterInfoDB.addParameterInfoForName(
        CallAnalyzerApp.PARAM_NCA_FAX_THRESHOLD,
        type=float,
        mandatory=False,
        aliases=(CpaCall.PARAM_CPA_RESULT_THRESHOLD_FAX,))
    parameterInfoDB.addParameterInfoForName(
        CallAnalyzerApp.PARAM_NCA_ENGINE_HOST,
        type=str,
        mandatory=False,
        aliases=(CpaCall.PARAM_CPA_ENGINE_HOST,))
    parameterInfoDB.addParameterInfoForName(
        CallAnalyzerApp.PARAM_NCA_ENGINE_PORT,
        type=int,
        mandatory=False,
        aliases=(CpaCall.PARAM_CPA_ENGINE_PORT,))

    from paraxip.sipua.user.cpa.CpaEamdCall import CpaEamdCall 
    parameterInfoDB.addParameterInfoForName(
        CallAnalyzerApp.PARAM_NCA_END_OF_GREETING_TIMEOUT_MS,
        type=int,
        mandatory=False,
        aliases=(CpaEamdCall.PARAM_CPA_EAMD_TIMEOUT_MS,))

    return parameterInfoDB

  ##########################################################################

  def __init__(self):
    super(CallAnalyzerApp, self).__init__()
    # Application level configuration
    self.__config = None
    return

  #@abstract
  def getNonCpaCallCls(self):
    assert False
    
  #accessor
  # Application level configuration
  def getConfig(self):
    return self.__config

  #@override
  def onStartup(self, in_refGlobalContext):
    # Call base class method
    assert super(CallAnalyzerApp, self).onStartup(in_refGlobalContext)

    # Get app config from defaults + config URL
    import paraxip.sipua.internal.call_config_loader as call_config_loader
    self.__config = call_config_loader.getAppConfig(self.__class__)
    if self.__config is None:
      # Error already logged by function
      return False

    # Verify that config is valid for app (errors logged by function if any)
    fileName = call_config_loader.getConfigURLForApp(self.__class__)
    return self.__class__.getParameterInfoDB().validateConfig(\
        self.__config, fileName)

  # helper to extract custom headers from the initial INVITE message from callee
  def _addAllCustomHeadersToCPAStats(self, config, in_sipINVITEMessage):
    #Extract custom headers from SIP INVITE's header.
    dict_inviteSIPMsgCustomHeaders = {}
    in_sipINVITEMessage.getCustomHeaders(dict_inviteSIPMsgCustomHeaders)
    from paraxip.sipua.user.cpa.CpaCall import CpaCall
    config[CpaCall.PARAM_CPA_STATS_CUSTOM_HEADERS] = \
      dict_inviteSIPMsgCustomHeaders

  #@override
  def onInviteGetCall(self, in_sipMessage):
    """
    USER CONFIGURATION 
      Returns a tuple ([SM class:Class], [config:dict]) to tell which scenario \
      to run, and with what configuration.
    """
    
    # Select a Call subclass following the incoming INVITE
    callCls = None
    config = {}
    
    doCPA = True
    if self.getConfig()[\
        CallAnalyzerApp.PARAM_NCA_REQUIRE_CPD_ON_TO_PERFORM_CPA]:
      doCPA = (in_sipMessage.getRequestLine().find("cpd=on") != -1)

    if doCPA:
      self._addAllCustomHeadersToCPAStats(config, in_sipMessage)
      if self.getConfig()[\
          CallAnalyzerApp.PARAM_NCA_ENDOFGREETING_ENABLE]:
        from paraxip.sipua.user.cpa.CpaEamdCall import CpaEamdCall 
        callCls = CpaEamdCall 
      else:
        from paraxip.sipua.user.cpa.CpaCall import CpaCall
        callCls = CpaCall
    else:
      callCls = self.getNonCpaCallCls()

    if self.getConfig()[\
        CallAnalyzerApp.PARAM_NCA_ACCEPT_CALLS_WHEN_CPA_NOT_AVAIL]:
      errorMsg = 'Setting parameter %s to true is not yet supported' % \
        CallAnalyzerApp.PARAM_NCA_ACCEPT_CALLS_WHEN_CPA_NOT_AVAIL
      PyLoggingFunctions.log_error(fileScopeLogger, errorMsg)
      return self.rejectCall(500, errorMsg)

    return self.updateAndValidateConfigForAppAndCall(config, self, callCls)

###############################################################################
# Call Analyzer Service, targeted as being an outbound proxy, using b2bua Call
#   for non-cpa calls

class CallAnalyzerAsOutboundProxy(CallAnalyzerApp):
  
  #@override
  def getNonCpaCallCls(self):
    from paraxip.sipua.user.b2bua.B2buaCall import B2buaCall
    return B2buaCall

###############################################################################
# Call Analyzer Service, NOT targeted as being an outbound proxy, using redirect for non-cpa calls

class CallAnalyzerGenesysOCS(CallAnalyzerApp):
  
  PARAM_NCA_RELAY_SERVER_HOST = 'nca.RelayServerHost'
  PARAM_NCA_RELAY_SERVER_PORT = 'nca.RelayServerPort'

  #@override
  # Provide parameter information for this App
  @staticmethod
  def getParameterInfoDB():
    from paraxip.sipua.user.b2bua.B2buaCall import B2buaCall
    from paraxip.sipua.user.redirect.RedirectCall import RedirectCall

    parameterInfoDB = CallAnalyzerApp.getParameterInfoDB()
    parameterInfoDB.addParameterInfoForName(
        CallAnalyzerGenesysOCS.PARAM_NCA_RELAY_SERVER_HOST,
        type=str,
        mandatory=False,
        aliases=(\
          B2buaCall.PARAM_B2BUA_OUTBOUND_REQUESTURI_HOST,
          RedirectCall.PARAM_REDIRECT_CONTACT_HOST
          ))
    parameterInfoDB.addParameterInfoForName(
        CallAnalyzerGenesysOCS.PARAM_NCA_RELAY_SERVER_PORT,
        type=int,
        mandatory=False,
        aliases=(\
          B2buaCall.PARAM_B2BUA_OUTBOUND_REQUESTURI_PORT,
          RedirectCall.PARAM_REDIRECT_CONTACT_PORT
          ))
    return parameterInfoDB

  #@override
  def getNonCpaCallCls(self):
    from paraxip.sipua.user.redirect.RedirectCall import RedirectCall
    return RedirectCall

###############################################################################

# remove aliases no longer needed
del _NetBorderApp
## !!MIRROR FILE, DO NOT MODIFY!!
