
from paraxip.sipua.user.cpa.calldata_converter import CallDataConverter
from paraxip.sipua.user.cpa.calldata_converter import CpaResultToCpdResult

from paraxip.sipua.user.cpa import QuickLogger
from paraxip.sipua.user.cpa import CpdResult
from paraxip.sipua.user.cpa import CallEvent
from paraxip.sipua.user.cpa import CallEventStack

class CallEventStackToCpdResult(CallDataConverter.CallDataConverter):
  """
  Interface in between what happened in the call and the legacy 'CpdResult'
  provided to clients. Make sure the output of this module match and is 
  restricted to the set of values described in the following document:
  svn://mtl-svn1/cymric/branches/int/netborder-platform-2.0/sip/hub/
  installer/doc/CallAnalyzer/cpa_user_guide.pdf, r16216, p.50.  
  """
  
  
  #@tag This is a somewhat ugly implementation, search for simpler alternative after release 2.0
  __sipErrorCodeToCpdResult = {
  CallEvent.SipError(CallEvent.SipErrorOriginator.Cpa,    CallEvent.SipErrorCode(404)): CpdResult.CpdResult.InvalidNumber,
  CallEvent.SipError(CallEvent.SipErrorOriginator.Caller, CallEvent.SipErrorCode(404)): CpdResult.CpdResult.InvalidNumber,
  CallEvent.SipError(CallEvent.SipErrorOriginator.Callee, CallEvent.SipErrorCode(404)): CpdResult.CpdResult.InvalidNumber,
  CallEvent.SipError(CallEvent.SipErrorOriginator.Cpa,    CallEvent.SipErrorCode(408)): CpdResult.CpdResult.Reject,
  CallEvent.SipError(CallEvent.SipErrorOriginator.Caller, CallEvent.SipErrorCode(408)): CpdResult.CpdResult.NoAnswer,
  CallEvent.SipError(CallEvent.SipErrorOriginator.Callee, CallEvent.SipErrorCode(408)): CpdResult.CpdResult.NoAnswer,
  CallEvent.SipError(CallEvent.SipErrorOriginator.Cpa,    CallEvent.SipErrorCode(480)): CpdResult.CpdResult.ServiceUnavailable,
  CallEvent.SipError(CallEvent.SipErrorOriginator.Caller, CallEvent.SipErrorCode(480)): CpdResult.CpdResult.ServiceUnavailable,
  CallEvent.SipError(CallEvent.SipErrorOriginator.Callee, CallEvent.SipErrorCode(480)): CpdResult.CpdResult.ServiceUnavailable,
  CallEvent.SipError(CallEvent.SipErrorOriginator.Cpa,    CallEvent.SipErrorCode(486)): CpdResult.CpdResult.Busy,
  CallEvent.SipError(CallEvent.SipErrorOriginator.Caller, CallEvent.SipErrorCode(486)): CpdResult.CpdResult.Busy,
  CallEvent.SipError(CallEvent.SipErrorOriginator.Callee, CallEvent.SipErrorCode(486)): CpdResult.CpdResult.Busy,
  CallEvent.SipError(CallEvent.SipErrorOriginator.Cpa,    CallEvent.SipErrorCode(600)): CpdResult.CpdResult.AllTrunksBusy,
  CallEvent.SipError(CallEvent.SipErrorOriginator.Caller, CallEvent.SipErrorCode(600)): CpdResult.CpdResult.AllTrunksBusy,
  CallEvent.SipError(CallEvent.SipErrorOriginator.Callee, CallEvent.SipErrorCode(600)): CpdResult.CpdResult.AllTrunksBusy,
  }
  
  def canYouHandle(self, in_instToConver, in_TypeToConvertTo):
    if isinstance(in_instToConver, CallEventStack.CallEventStack) and \
       in_TypeToConvertTo.__name__ == CpdResult.CpdResult.__name__:
      return True
    else:
      return False
  
  def __timeoutToCpdResult(self, in_CallEventTimeout):
    rv = None
    
    if in_CallEventTimeout.getTimeoutType() == CallEvent.Timeout.Preconnect:
      rv = CpdResult.CpdResult.NoAnswer
    elif in_CallEventTimeout.getTimeoutType() == CallEvent.Timeout.Postconnect:
      rv = CpdResult.CpdResult.Unknown
    elif in_CallEventTimeout.getTimeoutType() == CallEvent.Timeout.Postconnect_Eamd:
      rv = CpdResult.CpdResult.AnsweringMachine
    
    return rv
  
  def __sipErrorToCpdResult(self, in_SipError):
    
    if self.__sipErrorCodeToCpdResult.has_key(in_SipError):
      return self.__sipErrorCodeToCpdResult[in_SipError]
    else:
      return CpdResult.CpdResult.SipErrorGeneric
  
  def _convert(self, in_CallEventStack):
    
    #The eventual Return Value.
    rv = None
    
    #Translate CpaResult to CpdResult
    cpaResult = in_CallEventStack.getCpaResult()
    callProblem = in_CallEventStack.getLastCallProblem() 

    if callProblem is not None:
      if hasattr(callProblem, '_Trait_isCallCpaResult'):
        cpaCpdConcerter = CpaResultToCpdResult.CpaResultToCpdResult()
        cpdResult = cpaCpdConcerter._convert(cpaResult.getStr())
        rv = cpdResult
      elif hasattr(callProblem, '_Trait_isTimeout'):
        #
        # This is a special case for the CpdResult: the value of a timeout
        # transforms the result. Note that this is not a 'call problem' (the
        # behavior is expected in normal call flow).
        # @see http://intranet/pmwiki/pmwiki.php?n=Engineering.HeaderConvention
        #
        rv = self.__timeoutToCpdResult(callProblem)
      elif hasattr(callProblem, 'getSipErrorCodeAsInteger'):
        rv = self.__sipErrorToCpdResult(callProblem)
      else:
        raise CallDataConverter.CallDataConverterException(
                                "Unexpected CallProblem type: %s." % callProblem)
    elif cpaResult is not None:
      cpaCpdConcerter = CpaResultToCpdResult.CpaResultToCpdResult()
      cpdResult = cpaCpdConcerter._convert(cpaResult.getStr())
      rv = cpdResult
    
    #If NOTHING has happened in the call
    if len(in_CallEventStack.getCallEventList()) == 0:
      QuickLogger.log_debug("No event in call, check state transitions")
      rv = CpdResult.CpdResult.Unknown
    
    if rv == None:
      raise CallDataConverter.CallDataConverterException(
                                "CallEventStackToCpdResult.convert failed "
                                "with input: %s." % in_CallEventStack)
    return rv
## !!MIRROR FILE, DO NOT MODIFY!!
