
from paraxip.sipua.user.cpa.calldata_converter import CallDataConverter

from paraxip.sipua.user.cpa import DetailedCpdResult
from paraxip.sipua.user.cpa import CallEventStack
from paraxip.sipua.user.cpa import CpaResult
from paraxip.sipua.user.cpa import QuickLogger
from paraxip.sipua.user.cpa import CallEvent

class CallEventStackToDetailedCpdResult(CallDataConverter.CallDataConverter):
  
  __cpa_to_detailedCpdResult = {
    
    CpaResult.CpaResult.Human : DetailedCpdResult.DetailedCpdResult.Human,
    CpaResult.CpaResult.Fax : DetailedCpdResult.DetailedCpdResult.Fax,
    CpaResult.CpaResult.AnsweringMachine : DetailedCpdResult.DetailedCpdResult.AnsweringMachine,

    CpaResult.CpaResult.Eamd_Beep : DetailedCpdResult.DetailedCpdResult.Eamd_Beep,
    CpaResult.CpaResult.Eamd_Silence : DetailedCpdResult.DetailedCpdResult.Eamd_Silence,
        
    CpaResult.CpaResult.Busy : DetailedCpdResult.DetailedCpdResult.Busy,
    CpaResult.CpaResult.ReOrder : DetailedCpdResult.DetailedCpdResult.Reorder,
    
    CpaResult.CpaResult.SIT_Permanent : DetailedCpdResult.DetailedCpdResult.Sit_Permanent,
    CpaResult.CpaResult.SIT_Temporary : DetailedCpdResult.DetailedCpdResult.Sit_Temporary    
  }
  
  __timeout_to_detailedCpdResult = {
    CallEvent.Timeout.Postconnect_Eamd : DetailedCpdResult.DetailedCpdResult.Timeout_Eamd,
    CallEvent.Timeout.Preconnect       : DetailedCpdResult.DetailedCpdResult.Timeout_PreConnect,
    CallEvent.Timeout.Postconnect      : DetailedCpdResult.DetailedCpdResult.Timeout_PostConnect
  }
    
  def canYouHandle(self, in_instToConver, in_TypeToConvertTo):
    if hasattr(in_instToConver, '_Trait_CallEventStack') and \
       hasattr(in_TypeToConvertTo, '_Trait_DetailedCpdResult'):
      return True
    else:
      return False
  
  def __timeoutToDetailedCpdResult(self, in_timeout):
    rv = None
    if hasattr(in_timeout, '_Trait_isTimeout'):
      rv = self.__timeout_to_detailedCpdResult.get(in_timeout.getTimeoutType())
      if rv == None:
        QuickLogger.log_warn("Unhandled Timeout: %s" % in_timeout)
    return rv
  
  def __sipErrorToDetailedCpdResult(self, in_sipError):
    rv = None
    if hasattr(in_sipError, 'getErrorCodeOriginator'):
      sipErrorOriginator = in_sipError.getErrorCodeOriginator()
      if sipErrorOriginator == CallEvent.SipErrorOriginator.Caller:
        rv = DetailedCpdResult.DetailedCpdResult.Cancelled
      elif sipErrorOriginator == CallEvent.SipErrorOriginator.Callee:
        rv = DetailedCpdResult.DetailedCpdResult.Reject
      elif sipErrorOriginator == CallEvent.SipErrorOriginator.Cpa:
        rv = DetailedCpdResult.DetailedCpdResult.Reject
      else:
        QuickLogger.log_warn("Unhandled SipErrorOriginator: %s" % sipErrorOriginator)
    
    #
    #  Allow predefined SipError context strings 
    #  to be appended to the DetailedCpdResult.
    #    
    if rv is not None:
      if in_sipError.getContextStr() != None:
        if "licensing capacity exceeded" in in_sipError.getContextStr().lower():
          rv = rv + " - " + "licensing capacity exceeded"
    return rv
  
  def _convert(self, in_CallEventStack):
    
    # If a known call problem has happened (priority over Cpa result)
    callProblem = in_CallEventStack.getLastCallProblem()
    if callProblem != None:
      
      # Sip Errors (priority over timeouts)
      rv = self.__sipErrorToDetailedCpdResult(callProblem)
      if rv is not None:
        return rv
      
      # Timeouts
      rv = self.__timeoutToDetailedCpdResult(callProblem)  
      if rv is not None:
        return rv  
      
    # If there is a valid CpaResult that is interpretable
    cpaCallResult = in_CallEventStack.getCpaResult()
    if cpaCallResult is not None:
      #If the detailedCpdResult is directly translatable from a cpaCallResult, return this value.
      if self.__cpa_to_detailedCpdResult.has_key(cpaCallResult.getStr()):
        return self.__cpa_to_detailedCpdResult[cpaCallResult.getStr()]
    
    # No DetailedCpdResult. http://intranet/pmwiki/pmwiki.php?n=Engineering.HeaderConvention
    # specifies that 'X-Netborder-Detailed-CPD-Result-v2-0: None' should be present in the SIP 
    # message in the case that no 'DetailedCpdResult' event occurred.    
    return DetailedCpdResult.DetailedCpdResult.NoResult

## !!MIRROR FILE, DO NOT MODIFY!!
