
from paraxip.sipua.user.cpa.calldata_converter import CpaResultToCpdResult
from paraxip.sipua.user.cpa.calldata_converter import CallEventStackToCpdResult
from paraxip.sipua.user.cpa.calldata_converter import CallEventStackToDetailedCpdResult
from paraxip.sipua.user.cpa.calldata_converter import CallDataConverter

from paraxip.sipua.user.cpa import QuickLogger

class SmartConverterException(CallDataConverter.CallDataConverterException):
  pass

class NoAppropriateConverter(SmartConverterException):
  pass

class MoreThanOneConverter(SmartConverterException):
  pass

__ConverterRepository = [CallEventStackToDetailedCpdResult.CallEventStackToDetailedCpdResult(),
                         CallEventStackToCpdResult.CallEventStackToCpdResult(),
                         CpaResultToCpdResult.CpaResultToCpdResult()]

def do(in_instToConver, in_TypeToConvertTo):
  """
  Convert the instance of the first type into an instance of the
  second type.
  """
  
  QuickLogger.log_debug("Trying to convert (%s) to (%s)." % (in_instToConver, in_TypeToConvertTo))
  
  validConverters = []
  for converter in __ConverterRepository:
    if converter.canYouHandle(in_instToConver, in_TypeToConvertTo):
      validConverters.append(converter)
  
  if len(validConverters) == 0:
    raise NoAppropriateConverter("Could not find a converter for types: (%s, %s)." %
                                 (in_instToConver, in_TypeToConvertTo))
  
  if len(validConverters) > 1:
    raise MoreThanOneConverter("More than one converter for types: (%s, %s), cannot" 
                               "decide which one to use." %
                                 (in_instToConver, in_TypeToConvertTo))
  
  rv = validConverters[0].convert(in_instToConver, in_TypeToConvertTo)
  
  QuickLogger.log_debug("Conversion result: %s." % rv)
  
  return rv

def test_module():
  
  # Check that invalid types are not converted
  gotExcept = False
  try:
    do(1, 2)
  except NoAppropriateConverter, e:
    print "Got expected exception: %s" % e
    gotExcept = True
  assert gotExcept
  
#  # Test conversion of: (<X>, <Y>).
#  from paraxip.sipua.user.cpa import CpaCall
#  from paraxip.sipua.user.cpa import CpdResult
#  from paraxip.sipua.user.cpa import CallEventStack
#  
#  #callEventStack should have no event logged
#  cpaCallStateSnapshot = CpaCallStateSnapshot.CpaCallStateSnapshot()
#  callEventStack = do(cpaCallStateSnapshot, CallEventStack.CallEventStack)
#  assert len(callEventStack.getCallEventList())==0
#  
#  #Check cpdResult
#  cpdResult = do(callEventStack, CpdResult.CpdResult)
#  print cpdResult
  
if __name__ == '__main__':
  test_module()
## !!MIRROR FILE, DO NOT MODIFY!!
