#
#  @file   DNCRedirectorStates.py
#  
#  Description:
#
#  Type your description here
#
#  Copyright (C) 2008 Paraxip Technologies.
#  All Rights Reserved.
#

from paraxip.sipua.internal.SipCallEngineState import *
from paraxip.sipua.internal.SipCallEngineEvent import *
import paraxip.infra.PyLoggingFunctions as PyLoggingFunctions
from netborderPyInfra import *
from SipCallEngineExtending import *

import re

# Force declaration of class attributes in __init__ method
__pychecker__ = "initattr"

#
# Call states
#

#
# Base class
#

class DNCRedirectorStates(SipCallEngineState):
  def __init__(self, in_stateMachine):
    SipCallEngineState.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "DNCRedirectorStates.__init__")

#
# Idle state
#

class IdleState(DNCRedirectorStates):
  def __init__(self, in_stateMachine):
    DNCRedirectorStates.__init__(self, in_stateMachine)
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "IdleState._init__")
  
  def getName(self):
    return "IDLE"
  
  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())

    if (isinstance(in_event, InitialEvent)):
      return True

    elif (isinstance(in_event, InDialogToCallCalledEvent)):
      # Isolate the phone number from the request URL
      requestUrl = in_event.getSipMessage().getRequestUri()
      # Only straight digits string supported
      p = re.compile('sip:(\d+)@')
      m = p.match(requestUrl)
      if not m is None:

        strPhoneNumber = m.group(1)
        if self._stateMachine.isPhoneNumberAcceptable(strPhoneNumber):
          # The phone number is acceptable, redirect using
          # sip:[phoneNumber]@[relay_address]:[relay_port] as the contact
          contacts = ['sip:%s@%s:%i' % (strPhoneNumber, 
            self._stateMachine.config['sip.request.relay.server.address'], 
            self._stateMachine.config['sip.request.relay.server.port'])]
          if (not self._stateMachine.redirectInDialog(
                in_event.getDialogId(), contacts)):
            return False

        else:
          # Phone number is blocked by the DNC list
          if (not self._stateMachine.rejectDialog(
                in_event.getDialogId(), 403)): # 403 Forbidden
            return False

      else:
        # Malformed phone number
        if (not self._stateMachine.rejectDialog(
              in_event.getDialogId(), 404)): # 404 Not Found
          return False
        PyLoggingFunctions.log_error(self.getLogger(), 
                  "Request URL %s is malformed (user part must be digits only)",
                  requestUrl)
        return True

      return True # Do not change state, DialogToCallTerminatedEvent coming up

    elif (isinstance(in_event, DialogToCallSdpOfferedEvent)):
      return True # Stay in same state

    elif (isinstance(in_event, DialogToCallTerminatedEvent)):
      out_pystrNewStateName.set("DISCONNECTED")
      return True

    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)
    
    return True

#
# Disconnected state
#
# TODO this state is the same for all scenarios

class DisconnectedState(DNCRedirectorStates):
  def __init__(self, in_stateMachine):
    DNCRedirectorStates.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "DisconnectedState.__init__")
  
  def getName(self):
    return "DISCONNECTED"
  
  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    out_pystrNewStateName.set(self.getName())
    
    # No event should be received in this state
    PyLoggingFunctions.log_warn(self.getLogger(),
             "%s::processEvent received an invalid event %s",
             self.__class__.__name__, in_event.__class__.__name__)
    
    return True

## !!MIRROR FILE, DO NOT MODIFY!!
