#
#  @file   MediaServerCall.py
#  @author Alkassoum Ahmoud
#  date    Thursday August 09 2007
#  
#  Description:
#
#  Type your description here
#
#  Copyright (C) 2007 Paraxip Technologies.
#  All Rights Reserved.
#

from paraxip.sipua.internal.SipCallEngineSMCall import *
from media_server.MediaServerStates import *
from paraxip.sipua.internal.ConnectionStatus import *
import paraxip.infra.PyLoggingFunctions as PyLoggingFunctions
from netborderPyInfra import *
from PySipMediaServer import *
from RTSPClientExtending import *
import SdpHelperFunctions

import sys

#
# MediaServer state machine
#

class MediaServerCall(SipCallEngineSMCall):
  def __init__(self):
    SipCallEngineSMCall.__init__(self)
    
    self.setLogger(CallLogger("netborder.media-server.call.py"))
    
    TraceScope(self.getLogger(), "MediaServerCall.__init__")

    self.initDataMembers()    

    # Create and register the IDLE state
    idleState = IdleState(self)
    self.setInitialStateName(idleState.getName())
    self.setInitialState(idleState)
    
    # Create and register the intermediate states
    configureRTSPClientState = ConfigureRTSPClientState(self)
    self.addState(configureRTSPClientState.getName(), configureRTSPClientState)

    inviteWithoutSdpState = InviteWithoutSDPState(self)
    self.addState(inviteWithoutSdpState.getName(), inviteWithoutSdpState)

    sendDescribeState = SendDescribeState(self)
    self.addState(sendDescribeState.getName(), sendDescribeState)

    sendSetUpState = SendSetUpState(self)
    self.addState(sendSetUpState.getName(), sendSetUpState)

    resendSetUpState = ReSendSetUpState(self)
    self.addState(resendSetUpState.getName(), resendSetUpState)

    inDialogEarlyMediaState = InChilEarlyMediaState(self)
    self.addState(inDialogEarlyMediaState.getName(), inDialogEarlyMediaState)

    inDialogAfterEarlyMediaState = InChilAfterEarlyMediaState(self)
    self.addState(inDialogAfterEarlyMediaState.getName(),
                  inDialogAfterEarlyMediaState)

    waitBeforeConnectState = WaitBeforeConnectState(self)
    self.addState(waitBeforeConnectState.getName(), waitBeforeConnectState)

    inDialogAnsweringState = InDialogAnsweringState(self)
    self.addState(inDialogAnsweringState.getName(), inDialogAnsweringState)
    
    inDialogConnectedState = InDialogConnectedState(self)
    self.addState(inDialogConnectedState.getName(), inDialogConnectedState)
    
    connectedMediaEndedState = ConnectedMediaEndedState(self)
    self.addState(connectedMediaEndedState.getName(), connectedMediaEndedState)

    reinvitedByCallerWithSdpState = ReinvitedByCallerWithSdpState(self)
    self.addState(reinvitedByCallerWithSdpState.getName(),
                  reinvitedByCallerWithSdpState)

    reinvitedByCallerWithoutSdpState = ReinvitedByCallerWithoutSdpState(self)
    self.addState(reinvitedByCallerWithoutSdpState.getName(),
                  reinvitedByCallerWithoutSdpState)

    inDialogPausedState = InDialogPausedState(self)
    self.addState(inDialogPausedState.getName(), inDialogPausedState)

    inDialogConnectedWaitBeforPlayState = InDialogConnectedWaitBeforePlayState(self)
    self.addState(inDialogConnectedWaitBeforPlayState.getName(),
                  inDialogConnectedWaitBeforPlayState)
    
    disconnectingState = DisconnectingState(self)
    self.addState(disconnectingState.getName(), disconnectingState) 
    
    # Create and register the final state
    disconnectedState = DisconnectedState(self)
    self.addFinalState(disconnectedState.getName(), disconnectedState)

  def initialize(self, in_refApplication, in_refGlobalContext, in_szCallId):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "MediaServerCall.initialize")

    SipCallEngineSMCall.initialize(self,
                                   in_refApplication,
                                   in_refGlobalContext,
                                   in_szCallId)

    return True

  def initDataMembers(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "MediaServerCall.initDataMembers")
    self.callerSdpOffered = None

    self.inDialogId = None
   

    # Very important to release the UsageEnvironment AFTER rtspClient
    # & MediaSession
    self.live555RtspClient = None
    self.live555MediaSession = None
    self.live555UsageEnvironment = None
    self.live555TaskScheduler = None
    #self.live555MediaSubsessionIterator = None

    # Data Members retrieved from the invite's announcement
    self.strMediaUrl = None
    self.strMediaFile = None
    self.strStreamingServerAddress = None
    self.bPauseMedia = False
    self.bTrigEarlyMedia = False
    self.bIsEarlyMediaMode = False
    self.strEarlyMediaFirstPrompt = None
    self.bHangUpAtMediaEnd = False
    self.strEarlyMediaSecondPrompt = None
    self.iWaitBeforeConnectMs = -1
    self.iPlayDurationMs = -1
    self.bDirectConnectFromEarlyMedia = False
    
    # Data Members retrieved from the media SDP
    self.mediaSDP = None
    self.mediaBegin = None
    self.mediaEnd = None
   
    self.audioRTPport = None
    self.sdpAnswer = None
    self.destIp = None
    self.strRtspServerRtpPort = None

    # Data Members for controlling the playing durations timeouts
    self.iMediaBeginTimeSec = 0
    self.iEarlyMediaDurationMs = -1
    self.iPlayDurationBeforePause = -1
    self.iPauseDuration = -1
    self.iStartedPlayingAtTime = 0
    self.iPausedAtTime = 0

    # SDP and Anouncements parsers
    self.stringUtils = None
    self.mySdpUtils = None
    self.myAnnouncementParser = None
    
    self.bIsMediaPlayingOver = False
    self.bcheckFileName = False
    self.badvertiseAtMediaEnd = False
    self.bProvideAnswer = False
    self.bReinvitedByCaller = False
    self.inDialogRequestLine = None
    self.callRejectReason = None   
    self.inDialogStatus = ConnectionStatus.Unknown
    self.inDialogToHeader = None
    self.bCallerOnHold = False
    self.bReceivedInfoMsg = False
    self.bInvitedWithSdp = False
    self.uiCallerRejectSipErrorCode = None

  def reset(self):
    SipCallEngineSMCall.reset(self)
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "MediaServerCall.reset")
    
    SipCallEngineSMCall.reset(self)
    self.initDataMembers()
    
  def addInDialog(self, in_szDialogId, in_sipCallEngineSMIn):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "MediaServerCall.addInDialog")
    
    SipCallEngineSMCall.addInDialog(self, in_szDialogId, in_sipCallEngineSMIn)
    
    # Save in dialog id
    self.inDialogId = in_szDialogId
  
  def addOutDialog(self, in_szDialogId, in_sipCallEngineSMOut):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "MediaServerCall.addOutDialog")
    
    SipCallEngineSMCall.addOutDialog(self, in_szDialogId,
                                    in_sipCallEngineSMOut)
    
    # Save out dialog id
    self.outDialogId = in_szDialogId

  def scenarioName(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(),
               "MediaServerCall.scenarioName")
    return "Netborder Media Server"
    
  def resetMediaParameters(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "MediaServerCall.resetMediaParameters")

    self.mediaBegin = None
    self.mediaEnd = None
    self.iStartedPlayingAtTime = 0
    self.iPausedAtTime = 0
    self.iMediaBeginTimeSec = 0


  def ParseUrl(self, in_url):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(),"MediaServerCall.ParseUrl")

    if  self.myAnnouncementParser is not None:
      self.myAnnouncementParser.reset()
    else:
      self.myAnnouncementParser = PyAnnouncementParser()

    if not self.myAnnouncementParser.ParseAnnouncement(in_url):
      self.callRejectReason = 'Announcement content could not be retrieved'
      return False

    self.strMediaUrl =  self.myAnnouncementParser.getfmediaUrl()
    self.strStreamingServerAddress =  self.myAnnouncementParser.getfRTSPServerAddress() 
    self.iPauseDuration =  self.myAnnouncementParser.getfpauseTimeOutMs()
    self.iPlayDurationBeforePause =  self.myAnnouncementParser.getfplayTimeOutMs()
    self.iPlayDurationMs = self.myAnnouncementParser.getfdurationMs()
    self.bIsEarlyMediaMode =  self.myAnnouncementParser.getfearlyMedia()
    self.bTrigEarlyMedia =   self.myAnnouncementParser.getfearlyMedia()
    self.bPauseMedia =   self.myAnnouncementParser.getfpause()
    self.strEarlyMediaFirstPrompt =  self.myAnnouncementParser.getfirstPrompt()
    self.iEarlyMediaDurationMs =  self.myAnnouncementParser.getfearlyMediaTimeOutMs()
    self.strEarlyMediaSecondPrompt =   self.myAnnouncementParser.getfsecondPrompt()
    
    self.bDirectConnectFromEarlyMedia = \
      self.myAnnouncementParser.getfdirectConnectFromEarlyMedia()

    self.iWaitBeforeConnectMs = self.myAnnouncementParser.getfwaitBeforeConnectMs()

    self.strMediaFile = self.myAnnouncementParser.getfmediaFileName()

    self.bHangUpAtMediaEnd = self.myAnnouncementParser.getfhangUpAtEnd()

    self.bcheckFileName =  self.myAnnouncementParser.getfcheckFileName()

    self.badvertiseAtMediaEnd = self.myAnnouncementParser.getfadvertiseAtMediaEnd()

    if(self.bTrigEarlyMedia):

      # set the mediaUrl to the first prompt
      self.strMediaUrl = \
              'rtsp://'+ self.strStreamingServerAddress + '/' + self.strEarlyMediaFirstPrompt
      
    elif(self.strMediaFile is not None and self.strMediaFile != ''):
        self.strMediaUrl = \
             'rtsp://'+ self.strStreamingServerAddress + '/' + self.strMediaFile

        
    return True

  def VerifyInvite(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "MediaServerCall.VerifyInvite")

    self.stringUtils = PyStringUtils()
    # if the request line was not transmitted in url format, dedode
    # the request line
    request = self.stringUtils.decodedURL(self.inDialogRequestLine)
    
    PyLoggingFunctions.log_info(self.getLogger(),"request line : %s", request)
      
    # 2. Verify if the INVITE request line contains a play request
    p = re.compile('(?<=play=).+')
    m = re.search(p, request)

    # we had a play request in the received invite request line
    if not m is None:
      return self.ParseUrl(m.group(0))

    else:
      toHeader = self.stringUtils.decodedURL(self.inDialogToHeader)
      PyLoggingFunctions.log_info(self.getLogger(),"To header : %s", toHeader)
      # 2. Verify if the INVITE To header contains a play request
      p = re.compile('(?<=play=).+')
      m = re.search(p,toHeader)

      if m is not None:
        return  self.ParseUrl(m.group(0))

      # No play request in the INVITE
      else:
        self.callRejectReason = 'Mandatory play parameter missing'
        p = None
        m = None
        PyLoggingFunctions.log_error(self.getLogger(),
                  "MediaServerCall."
                  "Verify Invite no play request in the caller sdp offered")
        return False

  def ValidateStream(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "MediaServerCall.ValidateStream")
    
    # get the stream info
    mediaStreamInfo = SdpHelperFunctions.getStreamInfo(self.mediaSDP)
    if mediaStreamInfo is None:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "MediaServerCall."
               "ValidateStream the media stream could not be set")
      self.callRejectReason = \
                 'Announcement content is incompatible with your sdp'
      return False

    for i in range(3,len(mediaStreamInfo),3):
      if(self.mySdpUtils.validateStream(mediaStreamInfo[2],int(mediaStreamInfo[i]))):
        return True

    PyLoggingFunctions.log_warn(self.getLogger(),
             "MediaServerCall.ValidateStream the media stream is incompatible"
             " with the caller sdp")
    self.callRejectReason = \
             'Announcement content is incompatible with your sdp'

    return False

  def ParseCallerSDP(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "MediaServerCall.ParseCallerSDP")

    self.mySdpUtils.resetStreams()

    self.mySdpUtils.ParseSDP(self.callerSdpOffered)

    # get the destination ip adress for the rtp (caller in adress)
    self.destIp =   self.mySdpUtils.getDestIp()

    if ( self.destIp == ''):
      PyLoggingFunctions.log_warn(self.getLogger(),
               "MediaServerCall.ParseCallerSDP could not find any audio "
               "destination adress in the caller sdp offered")
      self.callRejectReason = \
                            'Announcement content could not be retrieved'
      return False
    
    else:
      PyLoggingFunctions.log_info(self.getLogger(),
               "MediaServerCall.ParseCallerSDP destination Ip: %s", self.destIp)
      if(self.destIp == '0.0.0.0'):
        self.bCallerOnHold = True
      else:
        self.bCallerOnHold = False
    
    # get the destination port for the rtp (caller in adress)
    m =   self.mySdpUtils.getSendPort()
    if(m == ''):
      PyLoggingFunctions.log_warn(self.getLogger(),
               "MediaServerCall.ParseCallerSDP could not find any audio rtp port"
               " in the caller sdp offered")
      self.callRejectReason = \
                            'Announcement content could not be retrieved'
      return False
     
    else:
      PyLoggingFunctions.log_info(self.getLogger(),
               "MediaServerCall.ParseCallerSDP destination port : %s",m)
      self.audioRTPport = int(m)
      
    return True

  def getRejectMessage(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "MediaServerCall.getRejectMessage")
    
    sipMessage = PySipMessage()
    self.constructRejectMessageInDialog(self.inDialogId, sipMessage)

    # Get the sip reject response code
    uiRejectResponseCode = 603 # Decline
    if (self.uiCallerRejectSipErrorCode is not None):
      uiRejectResponseCode = self.uiCallerRejectSipErrorCode
  
    # Add MediaServer header
    if (self.callRejectReason is not None):
      assert sipMessage.addCustomHeader("MEDIA-RESULT",
                                 self.callRejectReason)
    
    return uiRejectResponseCode, sipMessage

  def setupMediaSession(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "MediaServerCall.setupMediaSession")

    live555MediaSubsessionIterator =  MediaSubsessionIterator(self.live555MediaSession)
      
    mediaSubsession = live555MediaSubsessionIterator.next()

##    while not mediaSubsession is None:

    mediaSubsession.setClientIP(self.destIp)
    mediaSubsession.setClientPortNum(self.audioRTPport)
    # Uncomment the line bellow to set the size of the streamed rtp
    # packets The set parameter command will tell the streaming
    # server to the the preferred size of rtp packets to 160 and the
    # maximum size to 320. This command works only if you're using
    # the paraxip-live555 streaming server
    mediaSubsession.setRtpPacketsSize(160,320);

    if(not self.live555RtspClient.setupMediaSubsession(mediaSubsession,
                                                     False, False,False)):
      PyLoggingFunctions.log_error(self.getLogger(),
                "%s send SETUP request failed. %s",
                t.getScope(),self.live555UsageEnvironment.getResultMessage())
      return False

    else:
      if self.mySdpUtils is not None:
        self.strRtspServerRtpPort = str(mediaSubsession.serverPortNum)
        self.mySdpUtils.setServerPort(self.strRtspServerRtpPort)
        PyLoggingFunctions.log_info(self.getLogger(),
                 "MediaServerCall.setupMediaSession server port: %s",
                 self.mySdpUtils.getServerPort())
        #mediaSubsession = live555MediaSubsessionIterator.next()
        PyLoggingFunctions.log_debug(self.getLogger(),
                  "MediaServerCall.setupMediaSession error code = 0, "
                  "response code = 200")
    
    live555MediaSubsessionIterator = None    
    mediaSubsession = None
    return True

  def threatInfoMessage(self, in_szmsg):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "MediaServerCall.threatInfoMessage")

    # 2. Verify if the info message contains a new-media request
    p = re.compile('(?<=new-media=).+')
    m = re.search(p,in_szmsg)
    if m is None:
      return False

    k = re.split(';',m.group(0))

    # set the mediaUrl to the prompt
    self.strMediaUrl = k[0]
    if(len(k)>1):
      p = re.compile('(?<=duration=).+')
      m = re.search(p,k[1])
      if m is not None:
        self.iPlayDurationMs = int(m.group(0))
        self.bHangUpAtMediaEnd = False
      else:
        p = re.compile('(?<=hangUpAtEnd=).+')
        m = re.search(p,k[1])
        if m is not None:
          self.bHangUpAtMediaEnd = True
          self.iPlayDurationMs = -1
        else:
          self.bHangUpAtMediaEnd = False
          self.iPlayDurationMs = -1
    
    return True

  def makeRTSPClientDescribe(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "MediaServerCall.makeRTSPClientDescribe")

    self.mediaSDP = self.live555RtspClient.describeURLStringCopy(self.strMediaUrl,
                                                          None,False)

    status = self.live555RtspClient.describeStatus()
    if (self.mediaSDP == '' or status != 0):
      PyLoggingFunctions.log_error(self.getLogger(),
                "%s send DESCRIBE request failed. %s",
                t.getScope(),self.live555UsageEnvironment.getResultMessage())
      self.uiCallerRejectSipErrorCode = 404
      self.mediaSDP = None
      return False

    # Get the media file range
    if(self.live555RtspClient.setMediaRange(self.mediaSDP)):
      self.mediaBegin = \
                  self.live555RtspClient.getMediaBegin()
      self.mediaEnd = \
                self.live555RtspClient.getMediaEnd()
      if self.bHangUpAtMediaEnd and self.iPlayDurationMs == -1:
        self.iPlayDurationMs = int(self.mediaEnd * 1000)
    else:
      self.mediaBegin = 0
      self.mediaEnd = -1
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s could not find range parameter in"
               " media destription",
               t.getScope())

    # create the media session with the received sdp
    self.live555MediaSession = MediaSession.createNew(self.live555UsageEnvironment,
                                                        self.mediaSDP)

    # The SDP must have been misformatted.
    if self.live555MediaSession is None:
      PyLoggingFunctions.log_error(self.getLogger(),
                "%s could not create media session from media sdp. %s",
                t.getScope())
      # close the media session
      self.live555RtspClient.closeSession(self.live555MediaSession)
      self.uiCallerRejectSipErrorCode = 488
      self.mediaSDP = None
      return False

    return True

  def checkMediaFileName(self,in_sdp):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "MediaServerCall.checkMediaFileName")

    # get the stream info
    callerStreamInfo = SdpHelperFunctions.getStreamInfo(in_sdp)
    if callerStreamInfo is None :
      PyLoggingFunctions.log_warn(self.getLogger(),
               "MediaServerCall.checkMediaFileName the media encoding "
               "type could not be set")
      return False

    c = callerStreamInfo[3:len(callerStreamInfo)]
    pcmu = '0' in c
    pcma = '8' in c
    alaw = self.strMediaUrl.find('_alaw.wav')
    if not pcmu and pcma and alaw == -1:
      self.strMediaUrl = self.strMediaUrl.replace('.wav','_alaw.wav')

    elif  pcmu and not pcma and alaw != -1:
      self.strMediaUrl =  self.strMediaUrl.replace('_alaw.wav','.wav')
    
    return True
## !!MIRROR FILE, DO NOT MODIFY!!
