#
#  @file   MediaServerStates.py
#  @author Alkassoum Ahmoud
#  date    Thursday August 09 2007
#  
#  Description:
#
#  Type your description here
#
#  Copyright (C) 2007 Paraxip Technologies.
#  All Rights Reserved.
#

from paraxip.sipua.internal.SipCallEngineState import *
from paraxip.sipua.internal.SipCallEngineEvent import *
from paraxip.sipua.internal.ConnectionStatus import *
from paraxip.sipua.internal.SMTimeoutProcessor import *
import paraxip.infra.PyLoggingFunctions as PyLoggingFunctions
from netborderPyInfra import *
from RTSPClientExtending import *
from PySipMediaServer import *
from SipCallEngineExtending import *
from PyResipStack import *
from time import clock

import re # Regular Expressions

#
# Call states
#

#
# Base class
#

class MediaServerStates(SipCallEngineState):
  def __init__(self, in_stateMachine):
    SipCallEngineState.__init__(self, in_stateMachine)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(),"MediaServerStates.__init__")

#
# Idle state
#

class IdleState(MediaServerStates):
  def __init__(self, in_stateMachine):
    MediaServerStates.__init__(self, in_stateMachine)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "IdleState.__init__")
  
  def getName(self):
    return "IDLE"
  
  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())

    if (isinstance(in_event, InitialEvent)):
      return True

    # SIP_MEDIA_UAS_SPECIFIC START
    elif (isinstance(in_event, InDialogToCallCalledEvent)):
      self._stateMachine.inDialogStatus = ConnectionStatus.Connecting

      # Save the request line from the in dialog
      self._stateMachine.inDialogRequestLine = \
                          in_event.getSipMessage().getRequestUri()
      
      PyLoggingFunctions.log_debug(self.getLogger(),
                "IdleState::processEvent request: %s",
                self._stateMachine.inDialogRequestLine)
       # Save the te To header from the in dialog
      self._stateMachine.inDialogToHeader = \
                              in_event.getSipMessage().getHeaderValue("To")
      PyLoggingFunctions.log_debug(self.getLogger(),
                "IdleState::processEvent To: %s",
                 self._stateMachine.inDialogToHeader)
      # Now wait for SDP offer (expected in INVITE)
      return True
    
    elif (isinstance(in_event, DialogToCallSdpOfferedEvent)):
      self._stateMachine.bInvitedWithSdp = True
      
      # Save the caller sdp offered
      self._stateMachine.callerSdpOffered = in_event.getSdpContents()

      self._stateMachine.mySdpUtils = PySDPUtils()
      
      # Parse the caller sdp offered to retrieve play information
      # and rtp destination ip and ports
      if(self._stateMachine.ParseCallerSDP() and \
         self._stateMachine.VerifyInvite()):        
        PyLoggingFunctions.log_debug(self.getLogger(),
                  "IdleState::processEvent receives a play request for : %s",
                  self._stateMachine.strMediaUrl)
        
        out_pystrNewStateName.set("CONFIGURE_RTSPCLIENT")
        return True

      # No play Request in the received sdp or sdp invalid
      else:
        PyLoggingFunctions.log_warn(self.getLogger(),
                 "IdleState::processEvent receives an invalid sdp")
        self._stateMachine.uiCallerRejectSipErrorCode = 400
        out_pystrNewStateName.set("DISCONNECTING")
        return True

    elif (isinstance(in_event, DialogToCallSdpOfferRequiredEvent)):
      self._stateMachine.bInvitedWithSdp = False
      # Received an INVITE without an SDP
      self._stateMachine.callerSdp = None
      self._stateMachine.mySdpUtils = PySDPUtils()

      if(self._stateMachine.VerifyInvite()):
        out_pystrNewStateName.set("CONFIGURE_RTSPCLIENT")
        # No play Request in the received invite
      else:
        PyLoggingFunctions.log_warn(self.getLogger(),
                 "IdleState::processEvent receives an invalid invite")
        # Reject the call with an appropriate message
        self._stateMachine.uiCallerRejectSipErrorCode = 400
        out_pystrNewStateName.set("DISCONNECTING")
        return True
        
      return True

    # SIP_MEDIA_UAS_SPECIFIC STOP
    elif (isinstance(in_event, DialogToCallTerminatedEvent)):
      self._stateMachine.live555RtspClient = None
      self._stateMachine.inDialogStatus = ConnectionStatus.Disconnected
      out_pystrNewStateName.set("DISCONNECTING")
      return True
    
    elif (isinstance(in_event, StopEvent)):
      out_pystrNewStateName.set("DISCONNECTING")
      return True
    
    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)

    return True

#
# ConfigureTheRTSPClient state
#

class ConfigureRTSPClientState(MediaServerStates):
  def __init__(self, in_stateMachine):
    MediaServerStates.__init__(self, in_stateMachine)
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "ConfigureRTSPClientState.__init__")
    
  def getName(self):
    return "CONFIGURE_RTSPCLIENT"
  
  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)

    try:
      self._stateMachine.live555TaskScheduler =  BasicTaskScheduler.createNew()

      # set the usage environement
      self._stateMachine.live555UsageEnvironment = \
        BasicUsageEnvironment.createNew(self._stateMachine.live555TaskScheduler)

      # create our rtsp client
      self._stateMachine.live555RtspClient = \
        RTSPClient.createNew(self._stateMachine.live555UsageEnvironment,0,None,0)
      
    except MemoryError:
      PyLoggingFunctions.log_error(self.getLogger(),
                "ConfigureRTSPClientState catches the following error: %s",
                sys.exc_info()[0])
      self._stateMachine.live555RtspClient = None
      out_pystrNewStateName.set("DISCONNECTING")
      return
    
    PyLoggingFunctions.log_debug(self.getLogger(),"Server Adress: %s",
              self._stateMachine.strStreamingServerAddress)

    if (self._stateMachine.iWaitBeforeConnectMs != -1 and \
        self._stateMachine.bInvitedWithSdp):
      if (not self._stateMachine.ringInDialog(
            self._stateMachine.inDialogId)):
        return
      out_pystrNewStateName.set("WAIT_BEFORE_CONNECT")
      return

    elif (self._stateMachine.iWaitBeforeConnectMs == -1 and \
          self._stateMachine.bInvitedWithSdp):
      out_pystrNewStateName.set("SEND_DESCRIBE")
      return

    elif (not self._stateMachine.bInvitedWithSdp):
      out_pystrNewStateName.set("INVITE_WITHOUT_SDP")
      return
    
  def processEvent(self, in_event, out_pystrNewStateName):
    __pychecker__ = 'maxreturns=0'

    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())

    if (isinstance(in_event, DialogToCallTerminatedEvent)):
      self._stateMachine.live555RtspClient = None
      self._stateMachine.inDialogStatus = ConnectionStatus.Disconnected
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    elif (isinstance(in_event, StopEvent)):
      out_pystrNewStateName.set("DISCONNECTING")
      return True
    
    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)

    return True

#
# InviteWithoutSDP state
#

class InviteWithoutSDPState(MediaServerStates):
  def __init__(self, in_stateMachine):
    MediaServerStates.__init__(self, in_stateMachine)
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "InviteWithoutSDPState.__init__")
    
  def getName(self):
    return "INVITE_WITHOUT_SDP"
  
  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())

    desc = self._stateMachine.makeRTSPClientDescribe()
    if not desc:
      self._stateMachine.live555RtspClient = None
      out_pystrNewStateName.set("DISCONNECTING")
      return

    self._stateMachine.destIp = '0.0.0.0'
    self._stateMachine.audioRTPport = int(9000)

    if not self._stateMachine.setupMediaSession():
      PyLoggingFunctions.log_error(self.getLogger(),
                "InviteWithoutSDPState media session setup failed")

      self._stateMachine.live555RtspClient.teardownMediaSession(self._stateMachine.live555MediaSession)

      # close the media session
      self._stateMachine.live555RtspClient.closeSession(self._stateMachine.live555MediaSession)
      self._stateMachine.uiCallerRejectSipErrorCode = 500
      self._stateMachine.live555RtspClient = None
      self._stateMachine.live555MediaSession = None
      out_pystrNewStateName.set("DISCONNECTING")
      return

    # get the dialog id from the media sdp
    p = re.compile('(?<=o=).+')
    m = re.search(p,self._stateMachine.mediaSDP)
    u = re.split(' ', m.group(0))
    self._stateMachine.mySdpUtils.setSessionId(u[1])

    errStr = PyString()
    sdpOffer = SDP()
    if sdpOffer.\
           parse(self._stateMachine.mySdpUtils.produceDummySDP()\
                 , errStr) == False:
      PyLoggingFunctions.log_error(self.getLogger(), "failed to parse sdp :\n%s", errStr)
      return
    
    #sdpOffer = PySdpContents(self._stateMachine.mySdpUtils.produceDummySDP())
    if (not self._stateMachine.answerInDialog(
          self._stateMachine.inDialogId,
          sdpOffer, True)):
      return

    self._stateMachine.inDialogStatus = ConnectionStatus.Answering

  def processEvent(self, in_event, out_pystrNewStateName):
    __pychecker__ = 'maxreturns=0'
    
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())

    if (isinstance(in_event, DialogToCallTerminatedEvent)):
      # Send a stop request to the streaming server
      self._stateMachine.live555RtspClient. \
        teardownMediaSession(self._stateMachine.live555MediaSession)

      # close the media session
      self._stateMachine.live555RtspClient.closeSession(self._stateMachine.live555MediaSession)
      self._stateMachine.live555RtspClient = None
      self._stateMachine.live555MediaSession = None
      self._stateMachine.inDialogStatus = ConnectionStatus.Disconnected
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    elif (isinstance(in_event, DialogToCallSdpAnsweredEvent)): #IN
      # Send a stop request to the streaming server
      self._stateMachine.live555RtspClient. \
        teardownMediaSession(self._stateMachine.live555MediaSession)

      # close the media session
      self._stateMachine.live555RtspClient.closeSession(self._stateMachine.live555MediaSession)

      self._stateMachine.live555RtspClient.reset()

      self._stateMachine.resetMediaParameters()
      
      # Save the caller sdp offered
      self._stateMachine.callerSdpOffered = in_event.getSdpContents()
      if self._stateMachine.ParseCallerSDP():
        if self._stateMachine.bcheckFileName:
          self._stateMachine.checkMediaFileName(self._stateMachine.callerSdpOffered.getBodyDataStringCopy())
          self._stateMachine.bcheckFileName = False
          
        out_pystrNewStateName.set("SEND_DESCRIBE")
        return True
      else:
        PyLoggingFunctions.log_warn(self.getLogger(),
                 "InviteWithoutSDPState::processEvent receives an invalid sdp")
        self._stateMachine.uiCallerRejectSipErrorCode = 400
        self._stateMachine.live555RtspClient = None
        self._stateMachine.live555MediaSession = None
        out_pystrNewStateName.set("DISCONNECTING")
      return True

    elif (isinstance(in_event, StopEvent)):
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    elif (isinstance(in_event, DialogToCallAckNotReceivedEvent)): #IN
      # Can happen here after answering 200 Ok to an INVITE without SDP, 
      #   see bug 989
      # TODO use mutable logger for this warning
      PyLoggingFunctions.log_warn(self.getLogger(),
               "InviteWithoutSDPState: "
               "The SIP UAC did not answer our SIP 200 response with a ACK "
               "request as specified by RFC 3261. Terminating the call. "
               "(State: %s)",self.getName())

      # Send a stop request to the streaming server
      self._stateMachine.live555RtspClient. \
        teardownMediaSession(self._stateMachine.live555MediaSession)

      # close the media session
      self._stateMachine.live555RtspClient.closeSession(self._stateMachine.live555MediaSession)

      self._stateMachine.live555RtspClient = None
      self._stateMachine.live555MediaSession = None
      
      out_pystrNewStateName.set("DISCONNECTING")
      return True
    
    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)

    return True

#
# SendDescribeState state
#

class SendDescribeState(MediaServerStates):
  def __init__(self, in_stateMachine):
    MediaServerStates.__init__(self, in_stateMachine)
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "SendDescribeState.__init__")
  
  def getName(self):
    return "SEND_DESCRIBE"
  
  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())

    if not self._stateMachine.makeRTSPClientDescribe():
      self._stateMachine.live555RtspClient = None
      self._stateMachine.uiCallerRejectSipErrorCode = 404
      if self._stateMachine.bReinvitedByCaller and not self._stateMachine.bReceivedInfoMsg:
        rejectResponseCode, rejectSipMessage = self._stateMachine.getRejectMessage()
        if (not self._stateMachine.rejectDialog(
              self._stateMachine.inDialogId,
              rejectResponseCode,
              rejectSipMessage)):
          return
      
      out_pystrNewStateName.set("DISCONNECTING")
      return

    # get the dialog id from the media sdp
    p = re.compile('(?<=o=).+')
    m = re.search(p,self._stateMachine.mediaSDP)
    u = re.split(' ', m.group(0))

    if not self._stateMachine.ValidateStream():
      PyLoggingFunctions.log_error(self.getLogger(),
                "SendDescribeState.entryAction the stream codec isn't "
                "support by the caller sdp")
      self._stateMachine.uiCallerRejectSipErrorCode = 488

      if self._stateMachine.bReinvitedByCaller and not self._stateMachine.bReceivedInfoMsg:
        rejectResponseCode, rejectSipMessage = self._stateMachine.getRejectMessage()
        if (not self._stateMachine.rejectDialog(
              self._stateMachine.inDialogId,
              rejectResponseCode,
              rejectSipMessage)):
          return
      
      out_pystrNewStateName.set("DISCONNECTING")
      return

    # If it is an initial invite, set the dialog id and validate the
    # stream
    if not self._stateMachine.bReinvitedByCaller:
      self._stateMachine.mySdpUtils.setSessionId(u[1])
      out_pystrNewStateName.set("SEND_SET_UP")
    else:
      out_pystrNewStateName.set("RESEND_SET_UP")

  def processEvent(self, in_event, out_pystrNewStateName):
    __pychecker__ = 'maxreturns=0'
    
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())

    if (isinstance(in_event, DialogToCallTerminatedEvent)):
      # close the media session
      self._stateMachine.live555RtspClient.closeSession(self._stateMachine.live555MediaSession)
      self._stateMachine.live555RtspClient = None
      self._stateMachine.live555MediaSession = None
      self._stateMachine.inDialogStatus = ConnectionStatus.Disconnected
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    elif (isinstance(in_event, StopEvent)):
      out_pystrNewStateName.set("DISCONNECTING")
      return True
    
    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)

    return True

#
# SendSetUpState state
#

class SendSetUpState(MediaServerStates):
  def __init__(self, in_stateMachine):
    MediaServerStates.__init__(self, in_stateMachine)

    __pychecker__ = 'unusednames=t'
    
    t = TraceScope(self.getLogger(), "SendSetUpState.__init__")
  
  def getName(self):
    return "SEND_SET_UP"
  
  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())
    
    if not self._stateMachine.setupMediaSession():
      PyLoggingFunctions.log_error(self.getLogger(), "SendSetUpState media session setup failed")
      # close the media session
      self._stateMachine.live555RtspClient.closeSession(self._stateMachine.live555MediaSession)
      self._stateMachine.uiCallerRejectSipErrorCode = 500
      self._stateMachine.live555MediaSession = None
      self._stateMachine.live555RtspClient = None
      out_pystrNewStateName.set("DISCONNECTING")
      return

    errStr = PyString()
    self._stateMachine.sdpAnswer = SDP()
    if self._stateMachine.sdpAnswer.\
           parse(self._stateMachine.mySdpUtils.produceSDP()\
                                          , errStr) == False:
      PyLoggingFunctions.log_error(self.getLogger(), "failed to parse sdp :\n%s", errStr)
      return

    #self._stateMachine.sdpAnswer = PySdpContents(self._stateMachine.mySdpUtils.produceSDP())

    if(not self._stateMachine.bTrigEarlyMedia and self._stateMachine.bInvitedWithSdp):
      if self._stateMachine.bIsEarlyMediaMode:
        sdpContents = None
      else:
        sdpContents = self._stateMachine.sdpAnswer

      if (not self._stateMachine.answerInDialog(
        self._stateMachine.inDialogId,
        sdpContents,
        False): # SDP is an answer
        return
      else:
        self._stateMachine.inDialogStatus = ConnectionStatus.Answering
        out_pystrNewStateName.set("INDIALOG_ANSWERING")
        return

    elif(self._stateMachine.bTrigEarlyMedia and self._stateMachine.bInvitedWithSdp):
      if (not self._stateMachine.sendEarlyMediaToInDialog(
            self._stateMachine.inDialogId,
            self._stateMachine.sdpAnswer)):
        return
      
      out_pystrNewStateName.set("INDIALOG_EARLY_MEDIA")
      return
    
    elif(not self._stateMachine.bInvitedWithSdp):
      self._stateMachine.bInvitedWithSdp = True
      out_pystrNewStateName.set("INDIALOG_ANSWERING")
      return
      
  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    out_pystrNewStateName.set(self.getName())

    if (isinstance(in_event, DialogToCallTerminatedEvent)):
      # Send a stop request to the streaming server
      self._stateMachine.live555RtspClient.teardownMediaSession(self._stateMachine.live555MediaSession)

      # close the media session
      self._stateMachine.live555RtspClient.closeSession(self._stateMachine.live555MediaSession)
      self._stateMachine.live555RtspClient = None
      self._stateMachine.live555MediaSession = None
      self._stateMachine.inDialogStatus = ConnectionStatus.Disconnected

      out_pystrNewStateName.set("DISCONNECTING")
      return True

    elif (isinstance(in_event, StopEvent)):
      out_pystrNewStateName.set("DISCONNECTING")
      return True
    
    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)

    return True

#
# WaitBeforeConnect state
#

class WaitBeforeConnectState(MediaServerStates):
  def __init__(self, in_stateMachine):
    MediaServerStates.__init__(self, in_stateMachine)
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "WaitBeforeConnectState.__init__")

    self._waitTimerId = None

  def getName(self):
    return "WAIT_BEFORE_CONNECT"

  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())

    self._waitTimerId = None

    timeoutEvent = TimeoutEvent()
    timeoutCancelledEvent = TimeoutCancelledEvent()
    
    waitBeforeConnectConnectTimeoutProcessor = SMTimeoutProcessor(
      self._stateMachine,
      timeoutEvent,
      timeoutCancelledEvent)
    self._waitTimerId = self._stateMachine.globalContext.scheduleTimer(
      waitBeforeConnectConnectTimeoutProcessor,
      self._stateMachine.iWaitBeforeConnectMs,
      0)
    
    timeoutCancelledEvent.timerId =  self._waitTimerId

    timeoutEvent.timerId =  self._waitTimerId

  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())

    if (isinstance(in_event, DialogToCallTerminatedEvent)):
      self._stateMachine.live555RtspClient = None
      self._stateMachine.live555MediaSession = None
      self._stateMachine.inDialogStatus = ConnectionStatus.Disconnected
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    elif (isinstance(in_event, TimeoutEvent) and (in_event.timerId == self._waitTimerId)):
      self._stateMachine.iWaitBeforeConnectMs = -1
      out_pystrNewStateName.set(self.getName())
      return True
      
    elif (isinstance(in_event, TimeoutCancelledEvent) and (in_event.timerId == self._waitTimerId)):
      out_pystrNewStateName.set("SEND_DESCRIBE")
        
      self._waitTimerId = None
      return True

    elif (isinstance(in_event, StopEvent)):
      out_pystrNewStateName.set("DISCONNECTING")
      return True
    
    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)

    return True

  def onStateExit(self, in_event, in_strDestStateName):
    __pychecker__ = 'unusednames=in_event,in_strDestStateName,t'
    t = TraceScope(self.getLogger(), "WaitBeforeConnectState.onStateExit")
    if (self._waitTimerId is not None):
      self._stateMachine.globalContext.cancelTimer(self._waitTimerId)

#
# InDialogEarlyMedia state
#

class InChilEarlyMediaState(MediaServerStates):
  def __init__(self, in_stateMachine):
    MediaServerStates.__init__(self, in_stateMachine)
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "InChilEarlyMediaState.__init__")
    
    self._preConnectTimerId = None
  
  def getName(self):
    return "INDIALOG_EARLY_MEDIA"
  
  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())
    
    self._preConnectTimerId = None

    # Set the early media timer
    if(self._stateMachine.iEarlyMediaDurationMs == -1):
      self._stateMachine.iEarlyMediaDurationMs = \
                                       int(self._stateMachine.mediaEnd * 1000)
    
    timeoutEvent = TimeoutEvent()
    timeoutCancelledEvent = TimeoutCancelledEvent()
    preConnectTimeoutProcessor = SMTimeoutProcessor(
      self._stateMachine,timeoutEvent,
      timeoutCancelledEvent)
    self._preConnectTimerId = self._stateMachine.globalContext.scheduleTimer(
      preConnectTimeoutProcessor,
      self._stateMachine.iEarlyMediaDurationMs,
      0)
    timeoutCancelledEvent.timerId = self._preConnectTimerId

    timeoutEvent.timerId = self._preConnectTimerId

    status = self._stateMachine.live555RtspClient.playMediaSession(
               self._stateMachine.live555MediaSession,
               self._stateMachine.iMediaBeginTimeSec,
               self._stateMachine.mediaEnd,
               1)
    if(not status):

      PyLoggingFunctions.log_error(self.getLogger(),
                "InChilEarlyMediaState.entryAction PLAY request failed. %s",
                self._stateMachine.live555UsageEnvironment.getResultMessage())
      # Send the status code to the caller and disconnet
      self._stateMachine.callRejectReason = 'Could not play media announcement'
      self._stateMachine.uiCallerRejectSipErrorCode = 500
      out_pystrNewStateName.set("DISCONNECTING")
      return
    else:
      # save the time at wich we started playing
      self._stateMachine.bIsMediaPlayingOver = False
      self._stateMachine.iStartedPlayingAtTime = clock()

  
  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())
    
    if (isinstance(in_event, DialogToCallTerminatedEvent)):
      # Send a pause request to the streaming server
      self._stateMachine.live555RtspClient.pauseMediaSession(
        self._stateMachine.live555MediaSession)

      # Send a stop request to the streaming server
      self._stateMachine.live555RtspClient.teardownMediaSession(
        self._stateMachine.live555MediaSession)

      # close the media session
      self._stateMachine.live555RtspClient.closeSession(self._stateMachine.live555MediaSession)

      self._stateMachine.live555RtspClient = None
      self._stateMachine.live555MediaSession = None

      self._stateMachine.inDialogStatus = ConnectionStatus.Disconnected
      out_pystrNewStateName.set("DISCONNECTING")
     
      return True

    elif (isinstance(in_event, TimeoutEvent)):
      if (in_event.timerId == self._preConnectTimerId):
        PyLoggingFunctions.log_info(self.getLogger(),
                 "InChilEarlyMediaState::processEvent early media timeout expired")
        if (not self._stateMachine.bDirectConnectFromEarlyMedia):
          # Send a pause request to the streaming server
          self._stateMachine.live555RtspClient.pauseMediaSession(
            self._stateMachine.live555MediaSession)
          # Send a stop request to the streaming server
          self._stateMachine.live555RtspClient.teardownMediaSession(
            self._stateMachine.live555MediaSession)
          # close the media session
          self._stateMachine.live555RtspClient.closeSession(self._stateMachine.live555MediaSession)

        # wait for the TIME_OUT_CANCELED event
      return True

    elif (isinstance(in_event, TimeoutCancelledEvent)):
      if (in_event.timerId == self._preConnectTimerId):
        self._preConnectTimerId = None
        if(not self._stateMachine.bDirectConnectFromEarlyMedia):
          out_pystrNewStateName.set("INDIALOG_AFTER_EARLY_MEDIA")
        else:
          if self._stateMachine.bIsEarlyMediaMode:
            sdpContents = None
          else:
            sdpContents = self._stateMachine.sdpAnswer

          if(not self._stateMachine.answerInDialog(
               self._stateMachine.inDialogId,
               sdpContents,
               False): # SDP is an answer
            return False

          self._stateMachine.inDialogStatus = ConnectionStatus.Answering
          self._stateMachine.iPlayDurationBeforePause = \
            int(self._stateMachine.mediaEnd * 1000) - \
            self._stateMachine.iEarlyMediaDurationMs
          out_pystrNewStateName.set("INDIALOG_ANSWERING")
      return True

    elif (isinstance(in_event, StopEvent)):
      out_pystrNewStateName.set("DISCONNECTING")
      return True
    
    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)
    
    return True

  def onStateExit(self, in_event, in_strDestStateName):
    __pychecker__ = 'unusednames=in_event,in_strDestStateName,t'
    t = TraceScope(self.getLogger(), "InChilEarlyMediaState.onStateExit")
    if (self._preConnectTimerId is not None):
      self._stateMachine.globalContext.cancelTimer(self._preConnectTimerId)

#
# InDialogAfterEarlyMedia state
#

class InChilAfterEarlyMediaState(MediaServerStates):
  def __init__(self, in_stateMachine):
    MediaServerStates.__init__(self, in_stateMachine)
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "InChilAfterEarlyMediaState.__init__")
  
  def getName(self):
    return "INDIALOG_AFTER_EARLY_MEDIA"
  
  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)

    #self._stateMachine.live555MediaSession = None
    #self._stateMachine.live555RtspClient = None
    #self._stateMachine.live555UsageEnvironment = None

    if self._stateMachine.strEarlyMediaSecondPrompt != '':
      # set the mediaUrl to the second prompt
      self._stateMachine.strMediaUrl = \
                'rtsp://'+ self._stateMachine.strStreamingServerAddress + '/' + \
                self._stateMachine.strEarlyMediaSecondPrompt
      self._stateMachine.bTrigEarlyMedia = False
      
      self._stateMachine.live555RtspClient.reset()
      self._stateMachine.resetMediaParameters()
      self._stateMachine.live555MediaSession = None
      self._stateMachine.mediaSDP = None
      out_pystrNewStateName.set("SEND_DESCRIBE")
      
      #out_pystrNewStateName.set("CONFIGURE_RTSPCLIENT")
      return
    else:
      out_pystrNewStateName.set(self.getName())

  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())
    
    if (isinstance(in_event, DialogToCallTerminatedEvent)):
      self._stateMachine.live555MediaSession = None
      self._stateMachine.live555RtspClient = None
      self._stateMachine.inDialogStatus = ConnectionStatus.Disconnected
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    elif (isinstance(in_event, StopEvent)):
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)

    return True

#
# InDialogAnswering state
#

class InDialogAnsweringState(MediaServerStates):
  def __init__(self, in_stateMachine):
    MediaServerStates.__init__(self, in_stateMachine)

    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "InDialogAnsweringState.__init__")
  
  def getName(self):
    return "INDIALOG_ANSWERING"
  
  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())
    
    if (isinstance(in_event, DialogToCallConnectedEvent)):
      self._stateMachine.inDialogStatus = ConnectionStatus.Connected
      if (not self._stateMachine.bDirectConnectFromEarlyMedia):
        # We've received the ACK response from the caller,send the play request
        status = \
          self._stateMachine.live555RtspClient.playMediaSession(self._stateMachine.live555MediaSession,
                                                self._stateMachine.iMediaBeginTimeSec,
                                                self._stateMachine.mediaEnd,1)

        if (not status):
          PyLoggingFunctions.log_error(self.getLogger(),
                    "InDialogAnsweringState.processEvent PLAY request failed. %s",
                    self._stateMachine.live555UsageEnvironment.getResultMessage())
          # Send the status code to the caller and disconnet
          self._stateMachine.callRejectReason = 'Could not play media announcement'
          self._stateMachine.uiCallerRejectSipErrorCode = 500
          out_pystrNewStateName.set("DISCONNECTING")
          return True
      
        else:
          # save the time at wich we started playing
          self._stateMachine.iStartedPlayingAtTime = clock()
          self._stateMachine.bIsMediaPlayingOver = False
          PyLoggingFunctions.log_debug(self.getLogger(),"Send play error code = 0, response code = 200")
          
      # go to the connected state
      out_pystrNewStateName.set("INDIALOG_CONNECTED")
          
      return True

    elif (isinstance(in_event, DialogToCallTerminatedEvent)):
      # Send a stop request to the streaming server
      self._stateMachine.live555RtspClient.teardownMediaSession(
        self._stateMachine.live555MediaSession)
      # close the media session
      self._stateMachine.live555RtspClient.closeSession(self._stateMachine.live555MediaSession)

      self._stateMachine.live555MediaSession = None
      self._stateMachine.live555RtspClient = None

      self._stateMachine.inDialogStatus = ConnectionStatus.Disconnected
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    elif (isinstance(in_event, StopEvent)):
      out_pystrNewStateName.set("DISCONNECTING")
      return True
      
    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)

    return True

#
# InDialogConnected state
#

class InDialogConnectedState(MediaServerStates):
  def __init__(self, in_stateMachine):
    MediaServerStates.__init__(self, in_stateMachine)
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "InDialogConnectedState.__init__")
    
    self._ConnectTimerId = None
    self._DurationTimerId = None
    self._Pause = False
  
  def getName(self):
    return "INDIALOG_CONNECTED"

  def startTimer(self,in_szTimerName ,in_timerDuration):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "InDialogConnectedState.startTimer")

    timeoutEvent = TimeoutEvent()
    timeoutCancelledEvent = TimeoutCancelledEvent()

    timeoutProcessor = SMTimeoutProcessor(
      self._stateMachine, timeoutEvent, timeoutCancelledEvent)

    if in_szTimerName == "CONNECT":
      self._ConnectTimerId = self._stateMachine.globalContext.scheduleTimer(
        timeoutProcessor,
        in_timerDuration,
        0)
      timeoutCancelledEvent.timerId =  self._ConnectTimerId
      timeoutEvent.timerId =  self._ConnectTimerId
    else:
      self._DurationTimerId = self._stateMachine.globalContext.scheduleTimer(
        timeoutProcessor,
        in_timerDuration,
        0)
      timeoutCancelledEvent.timerId =  self._DurationTimerId
      timeoutEvent.timerId =   self._DurationTimerId

  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    self._Pause = False
    self._ConnectTimerId = None
    self._DurationTimerId = None
    
    out_pystrNewStateName.set(self.getName())
    
    if(self._stateMachine.iPlayDurationMs != -1):
      self.startTimer("DURATION",self._stateMachine.iPlayDurationMs)

    if(self._stateMachine.iPlayDurationBeforePause == -1):
      self._stateMachine.iPlayDurationBeforePause = \
                          int(self._stateMachine.mediaEnd * 1000)
    
    elif(self._stateMachine.iPlayDurationBeforePause != -1 and  self._stateMachine.bPauseMedia):
      self._stateMachine.iPlayDurationBeforePause = \
                                   self._stateMachine.mySdpUtils.getfplayTimeOutMs()

    self.startTimer("CONNECT",self._stateMachine.iPlayDurationBeforePause)
  
  def processEvent(self, in_event, out_pystrNewStateName):
    __pychecker__ = 'maxreturns=0 maxlines=0'
    
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())
    
    if (isinstance(in_event, DialogToCallTerminatedEvent)):
      self._stateMachine.live555RtspClient.pauseMediaSession(self._stateMachine.live555MediaSession)
      
      # Send a stop request to the streaming server
      self._stateMachine.live555RtspClient.teardownMediaSession(
        self._stateMachine.live555MediaSession)
  
      # close the media session
      self._stateMachine.live555RtspClient.closeSession(self._stateMachine.live555MediaSession)
      self._stateMachine.live555RtspClient = None
      self._stateMachine.live555MediaSession = None
      
      self._stateMachine.inDialogStatus = ConnectionStatus.Disconnected    
      
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    elif (isinstance(in_event, DialogToCallSdpOfferedEvent) and
          (in_event.getDialogId() == self._stateMachine.inDialogId)):
      # Caller re-inviting us with SDP

      if(not self._stateMachine.bCallerOnHold and self._stateMachine.bInvitedWithSdp):
        self._stateMachine.live555RtspClient.pauseMediaSession(self._stateMachine.live555MediaSession)
        # save the time at wich we paused the playing
        if not self._Pause:
          self._stateMachine.iPausedAtTime = clock()

      # Save the caller sdp offered
      self._stateMachine.callerSdpOffered = in_event.getSdpContents()
      out_pystrNewStateName.set("REINVITED_BY_CALLER_WITH_SDP")
      return True

    elif (isinstance(in_event, DialogToCallSdpOfferRequiredEvent)):
      # Caller re-inviting us without SDP

      if(not self._stateMachine.bCallerOnHold and self._stateMachine.bInvitedWithSdp):
        self._stateMachine.live555RtspClient.pauseMediaSession(self._stateMachine.live555MediaSession)
        # save the time at wich we paused the playing
        if not self._Pause:
          self._stateMachine.iPausedAtTime = clock()
          
      self._stateMachine.bInvitedWithSdp = False
      # Received an INVITE without an SDP
      self._stateMachine.callerSdp = None
      out_pystrNewStateName.set("REINVITED_BY_CALLER_WITHOUT_SDP")
      return True

    elif (isinstance(in_event, DialogToCallInfoEvent)): #IN
      self._stateMachine.bReinvitedByCaller = True
      self._stateMachine.bReceivedInfoMsg = True
      infMsg = in_event.getSipMessage().getBodyText()

      if self._stateMachine.threatInfoMessage(infMsg):

        self._stateMachine.live555RtspClient.pauseMediaSession(self._stateMachine.live555MediaSession)
        
        # Send a stop request to the streaming server
        self._stateMachine.live555RtspClient.teardownMediaSession(self._stateMachine.live555MediaSession)

        # close the media session
        self._stateMachine.live555RtspClient.closeSession(self._stateMachine.live555MediaSession)

        self._stateMachine.live555RtspClient.reset()
        self._stateMachine.resetMediaParameters()
        
        self._stateMachine.live555MediaSession = None
        self._stateMachine.mediaSDP = None

        out_pystrNewStateName.set("SEND_DESCRIBE")     
      return True

    elif (isinstance(in_event, TimeoutEvent) and (in_event.timerId == self._ConnectTimerId)):
      # Send a pause request to the streaming server
      PyLoggingFunctions.log_info(self.getLogger(),
               "InDialogConnectedState::processEvent PlayTimeoutMs expired")
      
      self._stateMachine.live555RtspClient.pauseMediaSession(self._stateMachine.live555MediaSession)
      
      # save the time at wich we paused the playing
      self._stateMachine.iPausedAtTime = clock()

      # wait for the TIME_OUT_CANCELD envent
      out_pystrNewStateName.set(self.getName())
      return True
          
    elif (isinstance(in_event, TimeoutEvent) and (in_event.timerId == self._DurationTimerId)):
      self._stateMachine.live555RtspClient.pauseMediaSession(self._stateMachine.live555MediaSession)
      # Send a stop request to the streaming server
      self._stateMachine.live555RtspClient.teardownMediaSession(self._stateMachine.live555MediaSession)
      # close the media session
      self._stateMachine.live555RtspClient.closeSession(self._stateMachine.live555MediaSession)
      self._stateMachine.live555RtspClient = None
      self._stateMachine.live555MediaSession = None
      # wait for the TIME_OUT_CANCELD envent
      out_pystrNewStateName.set(self.getName())
      return True
      
    elif (isinstance(in_event, TimeoutCancelledEvent) and
          (in_event.timerId == self._ConnectTimerId)):
      self._ConnectTimerId = None
      if(self._stateMachine.bPauseMedia):
        if(self._stateMachine.bDirectConnectFromEarlyMedia):
          self._stateMachine.iMediaBeginTimeSec = \
                       self._stateMachine.iPausedAtTime - self._stateMachine.iStartedPlayingAtTime
        # go to the Paused State
        out_pystrNewStateName.set("INDIALOG_PAUSED")
        PyLoggingFunctions.log_info(self.getLogger(),
                 "InDialogConnectedState::processEvent media paused")
      else:
        PyLoggingFunctions.log_info(self.getLogger(),
                 "InDialogConnectedState::processEvent Media ended")
        out_pystrNewStateName.set("CONNECTED_MEDIA_ENDED")
        return True
        
    elif (isinstance(in_event, TimeoutCancelledEvent) and \
          (in_event.timerId == self._DurationTimerId)):
      self._DurationTimerId = None
      if (not self._stateMachine.hangupDialog(
            self._stateMachine.inDialogId)):
        return False
      self._stateMachine.inDialogStatus = ConnectionStatus.Disconnecting

      out_pystrNewStateName.set("DISCONNECTING")
      return True

    elif (isinstance(in_event, StopEvent)):
      out_pystrNewStateName.set("DISCONNECTING")
      return True
    
    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)
      
    return True

  def onStateExit(self, in_event, in_strDestStateName):
    __pychecker__ = 'unusednames=in_strDestStateName'
    
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::onStateExit %s",
              self.__class__.__name__, in_event.__class__.__name__)

    if(self._ConnectTimerId is not None):
      self._stateMachine.globalContext.cancelTimer(self._ConnectTimerId)
    if(self._DurationTimerId is not None):
      self._stateMachine.globalContext.cancelTimer(self._DurationTimerId)

#
# ConnectedMediaEnded state
#

class ConnectedMediaEndedState(MediaServerStates):
  def __init__(self, in_stateMachine):
    MediaServerStates.__init__(self, in_stateMachine)
    __pychecker__ = 'unusednames=t'
    
    t = TraceScope(self.getLogger(), "ConnectedMediaEndedState.__init__")
  
  def getName(self):
    return "CONNECTED_MEDIA_ENDED"
  
  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)

    self._stateMachine.bIsMediaPlayingOver = True

    if (not self._stateMachine.bReinvitedByCaller and
            self._stateMachine.badvertiseAtMediaEnd):
      # Send an info msg to the caller in order to advertise him that
      # the media has ended
      msg = "SIP/2.0 " +  "MEDIA-RESULT: END"
      if (not self._stateMachine.sendInfoMsgToDialog(
            self._stateMachine.inDialogId,"application","sipfrag", msg)):
        return
    elif self._stateMachine.bReinvitedByCaller:
      self._stateMachine.bReinvitedByCaller = False
      
    out_pystrNewStateName.set(self.getName())

  def processEvent(self, in_event, out_pystrNewStateName):
    __pychecker__ = 'maxreturns=0'

    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())

    if (isinstance(in_event, DialogToCallTerminatedEvent)):
      # Send a stop request to the streaming server
      self._stateMachine.live555RtspClient.teardownMediaSession(self._stateMachine.live555MediaSession)

      # close the media session
      self._stateMachine.live555RtspClient.closeSession(self._stateMachine.live555MediaSession)

      self._stateMachine.live555MediaSession = None
      self._stateMachine.live555RtspClient = None
      self._stateMachine.inDialogStatus = ConnectionStatus.Disconnected
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    elif (isinstance(in_event, DialogToCallInfoSentSucceededEvent) and
          (in_event.getDialogId() == self._stateMachine.inDialogId)):
      return True

    elif (isinstance(in_event, DialogToCallInfoSentFailedEvent) and
          (in_event.getDialogId() == self._stateMachine.inDialogId)):
      # The inDialog might be down
      # disconnect the media server
      PyLoggingFunctions.log_error(self.getLogger(),
                "ConnectedMediaEndedState::processEvent "
                "failed sending info message to the caller")
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    elif (isinstance(in_event, DialogToCallSdpOfferedEvent) and
          (in_event.getDialogId() == self._stateMachine.inDialogId)):
      # Save the caller sdp offered
      self._stateMachine.callerSdpOffered = in_event.getSdpContents()
      out_pystrNewStateName.set("REINVITED_BY_CALLER_WITH_SDP")
      return True

    elif (isinstance(in_event, DialogToCallSdpOfferRequiredEvent)):
      # Caller re-inviting us without SDP
      self._stateMachine.bInvitedWithSdp = False
      # Received an INVITE without an SDP
      self._stateMachine.callerSdp = None
      out_pystrNewStateName.set("REINVITED_BY_CALLER_WITHOUT_SDP")
      return True

    elif (isinstance(in_event, DialogToCallInfoEvent)): #IN
      self._stateMachine.bReceivedInfoMsg = True
      infMsg = in_event.getSipMessage().getBodyText()

      if self._stateMachine.threatInfoMessage(infMsg):

        self._stateMachine.live555RtspClient.pauseMediaSession(self._stateMachine.live555MediaSession)

        # Send a stop request to the streaming server
        self._stateMachine.live555RtspClient.teardownMediaSession(self._stateMachine.live555MediaSession)

        # close the media session
        self._stateMachine.live555RtspClient.closeSession(self._stateMachine.live555MediaSession)

        self._stateMachine.live555RtspClient.reset()
        self._stateMachine.resetMediaParameters()

        self._stateMachine.live555MediaSession = None
        self._stateMachine.mediaSDP = None

        self._stateMachine.bReinvitedByCaller = True

        out_pystrNewStateName.set("SEND_DESCRIBE")  
      return True

    elif (isinstance(in_event, StopEvent)):
      out_pystrNewStateName.set("DISCONNECTING")
      return True
    
    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)

    return True

#
# ReinvitedByCallerWithSdp state
#

class ReinvitedByCallerWithSdpState(MediaServerStates):
  def __init__(self, in_stateMachine):
    MediaServerStates.__init__(self, in_stateMachine)
    __pychecker__ = 'unusednames=t'
    
    t = TraceScope(self.getLogger(), "ReinvitedByCallerWithSdpState.__init__")
  
  def getName(self):
    return "REINVITED_BY_CALLER_WITH_SDP"
  
  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())

    self._stateMachine.bReinvitedByCaller = True
    
    resSdp = self._stateMachine.ParseCallerSDP()

    if(resSdp):
      PyLoggingFunctions.log_debug(self.getLogger(),
                "ReinvitedByCallerWithSdpState::process receives a re-INVITE "
                "for %s", self._stateMachine.strMediaUrl)
      self._stateMachine.bReinvitedByCaller = True

      if(not self._stateMachine.bCallerOnHold):
        self._stateMachine.iMediaBeginTimeSec = \
            self._stateMachine.iPausedAtTime - \
            self._stateMachine.iStartedPlayingAtTime + \
            self._stateMachine.iMediaBeginTimeSec

      if (not self._stateMachine.bIsMediaPlayingOver):
        # Send a stop request to the streaming server
        self._stateMachine.live555RtspClient.teardownMediaSession(self._stateMachine.live555MediaSession)
        # close the media session
        self._stateMachine.live555RtspClient.closeSession(self._stateMachine.live555MediaSession)

        self._stateMachine.live555RtspClient.reset()

        self._stateMachine.live555MediaSession = None
        self._stateMachine.mediaSDP = None
        self._stateMachine.bProvideAnswer = True
        out_pystrNewStateName.set("SEND_DESCRIBE")
      else:
        self._stateMachine.bProvideAnswer = False
        out_pystrNewStateName.set("CONNECTED_MEDIA_ENDED")
        # Answer the caller
        errStr = PyString()
        self._stateMachine.sdpAnswer = SDP()
        if self._stateMachine.sdpAnswer.\
               parse(self._stateMachine.mySdpUtils.produceSDP()\
                     , errStr) == False:
          PyLoggingFunctions.log_error(self.getLogger(), "failed to parse sdp :\n%s", errStr)
          return
        #self._stateMachine.sdpAnswer = PySdpContents(self._stateMachine.mySdpUtils.produceSDP())
        if(not self._stateMachine.provideSdpAnswerToDialog(
             self._stateMachine.inDialogId,
             self._stateMachine.sdpAnswer)):
          return
        
    else:
      # Send a stop request to the streaming server
      self._stateMachine.live555RtspClient.teardownMediaSession(self._stateMachine.live555MediaSession)
      # close the media session
      self._stateMachine.live555RtspClient.closeSession(self._stateMachine.live555MediaSession)
      self._stateMachine.live555MediaSession = None
      self._stateMachine.mediaSDP = None

      rejectResponseCode, rejectSipMessage = self._stateMachine.getRejectMessage()
      if (not self._stateMachine.rejectDialog(
            self._stateMachine.inDialogId,
            rejectResponseCode,
            rejectSipMessage)):
        return
        
      PyLoggingFunctions.log_warn(self.getLogger(),
               "ReinvitedByCallerWithSdpState::processEvent receives "
               "an invalid sdp")

      out_pystrNewStateName.set("DISCONNECTING")
      return

  def processEvent(self, in_event, out_pystrNewStateName):
    __pychecker__ = 'maxreturns=0'
    
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())

    if (isinstance(in_event, DialogToCallTerminatedEvent)):
      self._stateMachine.inDialogStatus = ConnectionStatus.Disconnected
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    elif (isinstance(in_event, StopEvent)):
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)
      
    return True  

#
# ReinvitedByCallerWithoutSdp state
#

class ReinvitedByCallerWithoutSdpState(MediaServerStates):
  def __init__(self, in_stateMachine):
    MediaServerStates.__init__(self, in_stateMachine)
    __pychecker__ = 'unusednames=t'
    
    t = TraceScope(self.getLogger(),
                   "ReinvitedByCallerWithoutSdpState.__init__")
  
  def getName(self):
    return "REINVITED_BY_CALLER_WITHOUT_SDP"
  
  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())

    self._stateMachine.bReinvitedByCaller = True

    if(not self._stateMachine.bCallerOnHold):
      self._stateMachine.iMediaBeginTimeSec = \
                    self._stateMachine.iPausedAtTime - \
                    self._stateMachine.iStartedPlayingAtTime + \
                    self._stateMachine.iMediaBeginTimeSec

    # Dummy Dialog Id to be sent with the dummy SDP
    self._stateMachine.mySdpUtils.setSessionId("2353687637")

    # Dummy RTSP Server RTP port num to be sent with the dummy SDP
    if self._stateMachine.strRtspServerRtpPort is not None:
      self._stateMachine.mySdpUtils.setServerPort(self._stateMachine.strRtspServerRtpPort)
    else:
      self._stateMachine.mySdpUtils.setServerPort("6970")

       
    errStr = PyString()
    sdpOffer = SDP()
    if sdpOffer.\
           parse(self._stateMachine.mySdpUtils.produceSDP()\
                 , errStr) == False:
      PyLoggingFunctions.log_error(self.getLogger(), "failed to parse sdp :\n%s", errStr)
      return

    #sdpOffer = PySdpContents(self._stateMachine.mySdpUtils.produceDummySDP())
    if (not self._stateMachine.provideSdpOfferToDialog(
          self._stateMachine.inDialogId,
          sdpOffer)):
      return

  def processEvent(self, in_event, out_pystrNewStateName):
    __pychecker__ = 'maxreturns=0'
    
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())

    if (isinstance(in_event, DialogToCallTerminatedEvent)):
      # Send a stop request to the streaming server
      self._stateMachine.live555RtspClient. \
                       teardownMediaSession(self._stateMachine.live555MediaSession)

      # close the media session
      self._stateMachine.live555RtspClient.closeSession(self._stateMachine.live555MediaSession)
      self._stateMachine.live555RtspClient = None
      self._stateMachine.live555MediaSession = None
      self._stateMachine.inDialogStatus = ConnectionStatus.Disconnected
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    elif (isinstance(in_event, DialogToCallSdpAnsweredEvent)): #IN
      # Save the caller sdp answered
      self._stateMachine.callerSdpOffered = in_event.getSdpContents()
      if not self._stateMachine.ParseCallerSDP():
        PyLoggingFunctions.log_error(self.getLogger(),
                  "ReinvitedByCallerWithoutSdpState.processEvent receives "
                  "and invalid SDP")
        rejectResponseCode, rejectSipMessage = self._stateMachine.getRejectMessage()
        if (not self._stateMachine.rejectDialog(
              self._stateMachine.inDialogId,
              rejectResponseCode,
              rejectSipMessage)):
          return False

        out_pystrNewStateName.set("DISCONNECTING")
        return True

      if (not self._stateMachine.bIsMediaPlayingOver):
        # Send a stop request to the streaming server
        self._stateMachine.live555RtspClient.teardownMediaSession(self._stateMachine.live555MediaSession)
        # close the media session
        self._stateMachine.live555RtspClient.closeSession(self._stateMachine.live555MediaSession)

        self._stateMachine.live555RtspClient.reset()
        self._stateMachine.live555MediaSession = None
        self._stateMachine.mediaSDP = None
        self._stateMachine.bProvideAnswer = False
        out_pystrNewStateName.set("SEND_DESCRIBE")  
      else:
        if not self._stateMachine.bReceivedInfoMsg:
          self._stateMachine.bProvideAnswer = False
          out_pystrNewStateName.set("CONNECTED_MEDIA_ENDED")
        else:
          self._stateMachine.bProvideAnswer = False
          out_pystrNewStateName.set("SEND_DESCRIBE")

      return True

    elif (isinstance(in_event, DialogToCallInfoEvent)): #IN
      # Fix for bug 1269; Under load,with the acd script the INFO
      # message might be received before the ACK with SDP by the media
      # server. In that case, we set the
      # self._stateMachine.bReceivedInfoMsg flag to true and we remain
      # in this state. When we'll receive the ACK with SDP, if the
      # self._stateMachine.bReceivedInfoMsg flag is true, we directly
      # go to SEND DESCRIBE in order to play the new prompt instead of
      # going to the CONNECTED_MEDIA_ENDED state.
      self._stateMachine.bReceivedInfoMsg = True
      infMsg = in_event.getSipMessage().getBodyText()

      if self._stateMachine.threatInfoMessage(infMsg):

        self._stateMachine.live555RtspClient.pauseMediaSession(self._stateMachine.live555MediaSession)

        # Send a stop request to the streaming server
        self._stateMachine.live555RtspClient.teardownMediaSession(self._stateMachine.live555MediaSession)

        # close the media session
        self._stateMachine.live555RtspClient.closeSession(self._stateMachine.live555MediaSession)

        self._stateMachine.live555RtspClient.reset()
        self._stateMachine.resetMediaParameters()

        self._stateMachine.live555MediaSession = None
        self._stateMachine.mediaSDP = None

        self._stateMachine.bReinvitedByCaller = True
        
      return True
  
    elif (isinstance(in_event, StopEvent)):
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    elif (isinstance(in_event, DialogToCallAckNotReceivedEvent)): #IN
      # Can happen here after answering 200 Ok to an INVITE without SDP, 
      #   see bug 989
      # TODO use mutable logger for this warning
      PyLoggingFunctions.log_warn(self.getLogger(),
               "ReinvitedByCallerWithoutSdpState "
               "The SIP UAC did not answer our SIP 200 response with a ACK "
               "request as specified by RFC 3261. Terminating the call. "
               "(State: %s)",self.getName())

      # Send a stop request to the streaming server
      self._stateMachine.live555RtspClient. \
        teardownMediaSession(self._stateMachine.live555MediaSession)

      # close the media session
      self._stateMachine.live555RtspClient.closeSession(self._stateMachine.live555MediaSession)

      self._stateMachine.live555RtspClient = None
      self._stateMachine.live555MediaSession = None
      
      out_pystrNewStateName.set("DISCONNECTING")
      return True
    
    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)

    return True

#
# ReSendSetUpState state
#

class ReSendSetUpState(MediaServerStates):
  def __init__(self, in_stateMachine):
    MediaServerStates.__init__(self, in_stateMachine)
    __pychecker__ = 'unusednames=t'
    
    t = TraceScope(self.getLogger(), "ReSendSetUpState.__init__")
  
  def getName(self):
    return "RESEND_SET_UP"
  
  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)

    if(not self._stateMachine.bCallerOnHold):
      if not self._stateMachine.setupMediaSession():
        PyLoggingFunctions.log_error(self.getLogger(), "ReSendSetUpState media sesson setup failed")
        # Send a stop request to the streaming server
        self._stateMachine.live555RtspClient.teardownMediaSession(self._stateMachine.live555MediaSession)
        # close the media session
        self._stateMachine.live555RtspClient.closeSession(self._stateMachine.live555MediaSession)
        self._stateMachine.uiCallerRejectSipErrorCode = 500
        self._stateMachine.live555MediaSession = None
        out_pystrNewStateName.set("DISCONNECTING")
        return

    if not self._stateMachine.bReceivedInfoMsg and self._stateMachine.bProvideAnswer:
      # Answer the Re-invite from the caller
      errStr = PyString()
      self._stateMachine.sdpAnswer = SDP()
      if self._stateMachine.sdpAnswer.\
             parse(self._stateMachine.mySdpUtils.produceSDP()\
                   , errStr) == False:
        PyLoggingFunctions.log_error(self.getLogger(), "failed to parse sdp :\n%s", errStr)
        return
      #self._stateMachine.sdpAnswer = PySdpContents(self._stateMachine.mySdpUtils.produceSDP())
      if (not self._stateMachine.provideSdpAnswerToDialog(
            self._stateMachine.inDialogId,
            self._stateMachine.sdpAnswer)):
        return
      self._stateMachine.bProvideAnswer = False
    
    if(not self._stateMachine.bCallerOnHold):
      self._stateMachine.live555RtspClient.playMediaSession(self._stateMachine.live555MediaSession,
                                                   self._stateMachine.iMediaBeginTimeSec,
                                                   self._stateMachine.mediaEnd,1)
      self._stateMachine.iPlayDurationBeforePause = \
                 int((self._stateMachine.mediaEnd - \
                  self._stateMachine.iMediaBeginTimeSec) * 1000)
      self._stateMachine.iStartedPlayingAtTime = clock()
      self._stateMachine.bIsMediaPlayingOver = False
      
    self._stateMachine.bReinvitedByCaller = False
    self._stateMachine.bReceivedInfoMsg = False
    out_pystrNewStateName.set("INDIALOG_CONNECTED")
  
  def processEvent(self, in_event, out_pystrNewStateName):
    __pychecker__ = 'maxreturns=0'
    
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())

    if (isinstance(in_event, DialogToCallTerminatedEvent)):
      self._stateMachine.live555MediaSession = None
      self._stateMachine.live555RtspClient = None
      self._stateMachine.inDialogStatus = ConnectionStatus.Disconnected  
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    elif (isinstance(in_event, StopEvent)):
      out_pystrNewStateName.set("DISCONNECTING")
      return True
    
    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)

    return True

#
# InDialogPaused state
#

class InDialogPausedState(MediaServerStates):
  def __init__(self, in_stateMachine):
    MediaServerStates.__init__(self, in_stateMachine)
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "InDialogPausedState.__init__")
    
    self._PauseTimerId = None
  
  def getName(self):
    return "INDIALOG_PAUSED"
  
  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)

    self._PauseTimerId = None
    timeoutEvent = TimeoutEvent()
    timeoutCancelledEvent = TimeoutCancelledEvent()
    pauseTimeoutProcessor = \
                     SMTimeoutProcessor(self._stateMachine,
                                        timeoutEvent,timeoutCancelledEvent)
    self._PauseTimerId = self._stateMachine.globalContext.scheduleTimer(
      pauseTimeoutProcessor,
      self._stateMachine.iPauseDuration,
      0)

    timeoutCancelledEvent.timerId = self._PauseTimerId

    timeoutEvent.timerId = self._PauseTimerId

    out_pystrNewStateName.set(self.getName())

  def processEvent(self, in_event, out_pystrNewStateName):
    __pychecker__ = 'maxreturns=0 maxlines=0'
    
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())
    
    if (isinstance(in_event, DialogToCallTerminatedEvent)):
      # Send a stop request to the streaming server
      self._stateMachine.live555RtspClient.teardownMediaSession(
        self._stateMachine.live555MediaSession)
      
      # close the media session
      self._stateMachine.live555RtspClient.closeSession(self._stateMachine.live555MediaSession)
      
      self._stateMachine.inDialogStatus = ConnectionStatus.Disconnected
      self._stateMachine.live555MediaSession = None
      self._stateMachine.live555RtspClient = None
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    elif (isinstance(in_event, TimeoutEvent)):
      if (in_event.timerId ==  self._PauseTimerId):
        PyLoggingFunctions.log_info(self.getLogger(), "PauseTimeoutMs expired")

        self._stateMachine.bPauseMedia = False

        # Wait for the TIMEOUT_CANCELED event
      
      return True
      
    elif (isinstance(in_event, TimeoutCancelledEvent)):
      if(in_event.timerId ==  self._PauseTimerId):
        self._PauseTimerId = None
        out_pystrNewStateName.set("INDIALOG_CONNECTED_WAIT_BEFORE_PLAY")
      
      return True

    elif (isinstance(in_event, StopEvent)):
      out_pystrNewStateName.set("DISCONNECTING")
      return True
      
    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)
    
    return True
  
  def onStateExit(self, in_event, in_strDestStateName):
    __pychecker__ = 'unusednames=in_strDestStateName'
    
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::onStateExit %s",
              self.__class__.__name__, in_event.__class__.__name__)

    if(self._PauseTimerId is not None):
      self._stateMachine.globalContext.cancelTimer(self._PauseTimerId)

#
# InDialogConnectedWaitBeforPlay state
#

class InDialogConnectedWaitBeforePlayState(MediaServerStates):
  def __init__(self, in_stateMachine):
    MediaServerStates.__init__(self, in_stateMachine)
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(),
                   "InDialogConnectedWaitBeforePlayState.__init__")
  
  def getName(self):
    return "INDIALOG_CONNECTED_WAIT_BEFORE_PLAY"

  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)

    # send a play request after pause
    self._stateMachine.iMediaBeginTimeSec = \
                                        long(self._stateMachine.iMediaBeginTimeSec + \
                                             self._stateMachine.iPlayDurationBeforePause/1000)
    status = self._stateMachine.live555RtspClient.playMediaSession(
      self._stateMachine.live555MediaSession,
      self._stateMachine.iMediaBeginTimeSec,self._stateMachine.mediaEnd,1)
    if(not status):
      # Send a stop request to the streaming server
      self._stateMachine.live555RtspClient.teardownMediaSession(
        self._stateMachine.live555MediaSession)
      # close the media session
      self._stateMachine.live555RtspClient.closeSession(
        self._stateMachine.live555MediaSession)
      self._stateMachine.live555MediaSession = None
      self._stateMachine.live555RtspClient = None
      # Hang up the caller
      out_pystrNewStateName.set("DISCONNECTING")
      return
    else:
      if(self._stateMachine.iPlayDurationMs != -1):
        self._stateMachine.iPlayDurationMs = self._stateMachine.iPlayDurationMs -\
                                        self._stateMachine.iPlayDurationBeforePause
      self._stateMachine.iPlayDurationBeforePause = int(self._stateMachine.mediaEnd*1000)-\
                                         self._stateMachine.iPlayDurationBeforePause

      self._stateMachine.bIsMediaPlayingOver = False
      out_pystrNewStateName.set("INDIALOG_CONNECTED")
      
  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())
    
    if (isinstance(in_event, DialogToCallTerminatedEvent)):
      self._stateMachine.inDialogStatus = ConnectionStatus.Disconnected
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    elif (isinstance(in_event, StopEvent)):
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)
      
    return True

#
# Disconnecting state
#

class DisconnectingState(MediaServerStates):
  def __init__(self, in_stateMachine):
    MediaServerStates.__init__(self, in_stateMachine)
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "DisconnectingState.__init__")
  
  def getName(self):
    return "DISCONNECTING"
  
  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    out_pystrNewStateName.set(self.getName())
    
    # close the streaming session if it is active
    if (self._stateMachine.live555RtspClient is not None and
        self._stateMachine.live555MediaSession is not None):
      # Send a stop request to the streaming server
      self._stateMachine.live555RtspClient.teardownMediaSession(self._stateMachine.live555MediaSession)
      # close the media session
      self._stateMachine.live555RtspClient.closeSession(self._stateMachine.live555MediaSession)

    if (self._stateMachine.inDialogStatus == ConnectionStatus.Disconnected or \
        self._stateMachine.inDialogStatus == ConnectionStatus.Unknown):
      out_pystrNewStateName.set("DISCONNECTED")
      return

    # Reject the caller if it's connecting
    if (self._stateMachine.inDialogStatus == ConnectionStatus.Connecting):
      rejectResponseCode, rejectSipMessage = self._stateMachine.getRejectMessage()
      if (not self._stateMachine.rejectDialog(
            self._stateMachine.inDialogId,
            rejectResponseCode,
            rejectSipMessage)):
        return
      self._stateMachine.inDialogStatus = ConnectionStatus.Disconnecting

    # Hanging up the caller if it's connected
    elif (self._stateMachine.inDialogStatus == ConnectionStatus.Answering or
          self._stateMachine.inDialogStatus == ConnectionStatus.Connected):
      if (not self._stateMachine.hangupDialog(
            self._stateMachine.inDialogId)):
        return
      self._stateMachine.inDialogStatus = ConnectionStatus.Disconnecting

  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    out_pystrNewStateName.set(self.getName())

    if (isinstance(in_event, DialogToCallTerminatedEvent) and
        (in_event.getDialogId() == self._stateMachine.inDialogId)):
      self._stateMachine.inDialogStatus = ConnectionStatus.Disconnected
      out_pystrNewStateName.set("DISCONNECTED")
      return True

    elif (isinstance(in_event, StopEvent)):
      # Stay in the same state
      return True

    elif (isinstance(in_event, KillEvent)):
      # Go directly in final state
      out_pystrNewStateName.set("DISCONNECTED")
      return True

    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)
      
    return True

#
# Disconnected state
#

class DisconnectedState(MediaServerStates):
  def __init__(self, in_stateMachine):
    MediaServerStates.__init__(self, in_stateMachine)
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "DisconnectedState.__init__")
  
  def getName(self):
    return "DISCONNECTED"
  
  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())
    
    # No event should be received in this state
    PyLoggingFunctions.log_warn(self.getLogger(),
             "%s::processEvent received an invalid event %s",
             self.__class__.__name__, in_event.__class__.__name__)
    
    return True

## !!MIRROR FILE, DO NOT MODIFY!!
