#
#  Copyright (C) 2010 Sangoma Technologies.
#  All Rights Reserved.
#

# Force declaration of class attributes in __init__ method
__pychecker__ = "initattr"

# 
# -----------------------------------------------------------------------------
# RejectCall: call to reject incoming calls immediatly
# -----------------------------------------------------------------------------

#
# Idle state
#

from netborderPyInfra import TraceScope, CallLogger
import paraxip.infra.PyLoggingFunctions as PyLoggingFunctions

from SipCallEngineExtending import InitialEvent

from paraxip.sipua.internal.SipCallEngineState import SipCallEngineState 
from paraxip.sipua.internal.SipCallEngineEvent import \
    InDialogToCallCalledEvent, DialogToCallTerminatedEvent
from paraxip.sipua.internal.SipCallEngineSMCall import SipCallEngineSMCall
from paraxip.sipua.internal.ParameterInfo import ParameterInfoDB

class IdleState(SipCallEngineState):
  def __init__(self, in_stateMachine):
    SipCallEngineState.__init__(self, in_stateMachine)
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "IdleState._init__")
  
  def getName(self):
    return "IDLE"

  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)
    out_pystrNewStateName.set(self.getName())

  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)

    out_pystrNewStateName.set(self.getName())

    if (isinstance(in_event, InitialEvent)):
      return True

    elif (isinstance(in_event, InDialogToCallCalledEvent)): 

      reasonText = None # None means use default text for status Code      
      if RejectCall.PARAM_REJECT_REASON_TEXT in self._stateMachine.config:
        reasonText = self._stateMachine.config[\
            RejectCall.PARAM_REJECT_REASON_TEXT]

      if (not self._stateMachine.rejectDialog(
            in_event.getDialogId(),
            self._stateMachine.config[RejectCall.PARAM_REJECT_STATUS_CODE],
            in_szReason=reasonText)): 
        return False
    
    elif isinstance(in_event, DialogToCallTerminatedEvent):
      out_pystrNewStateName.set("DISCONNECTED")
      return True

    return True

#
# Disconnected state
#

class DisconnectedState(SipCallEngineState):
  def __init__(self, in_stateMachine):
    SipCallEngineState.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "DisconnectedState.__init__")
  
  def getName(self):
    return "DISCONNECTED"
  
  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    out_pystrNewStateName.set(self.getName())

  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    out_pystrNewStateName.set(self.getName())
    
    # No event should be received in this state
    SipCallEngineState.warnUnexpectedEvent(self,self.__class__.__name__, in_event)
    
    return True

#
# RejectCall
#

class RejectCall(SipCallEngineSMCall):

  # Valid parameter names for this Call
  PARAM_REJECT_STATUS_CODE = 'reject.statusCode'
  PARAM_REJECT_REASON_TEXT = 'reject.reasonText'

  # Provide parameter information for this Call 
  @staticmethod
  def getParameterInfoDB():

    parameterInfoDB = ParameterInfoDB()

    # Optional outbound proxy address:port
    parameterInfoDB.addParameterInfoForName(
        RejectCall.PARAM_REJECT_STATUS_CODE,
        type=int)
    parameterInfoDB.addParameterInfoForName(
        RejectCall.PARAM_REJECT_REASON_TEXT,
        type=str,
        mandatory=False)

    return parameterInfoDB

  def __init__(self):
    SipCallEngineSMCall.__init__(self)
    
    self.setLogger(CallLogger("netborder.reject.Call.py"))
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "Call.__init__")

    # IDLE state
    idleState = IdleState(self)
    self.setInitialStateName(idleState.getName())
    self.setInitialState(idleState)

    # DISCONNECTED state
    disconnectedState = DisconnectedState(self)
    self.addFinalState(disconnectedState.getName(),disconnectedState)
## !!MIRROR FILE, DO NOT MODIFY!!
