#
#  @file   Call.py
#  @author Francis Moreau
#  date    Mon Jun 16 2008
#  
#  Description:
#
#  Type your description here
#
#  Copyright (C) 2008 Paraxip Technologies.
#  All Rights Reserved.
#

from netborderPyInfra import *
import paraxip.infra.PyLoggingFunctions as PyLoggingFunctions
from paraxip.infra.Assert import *
from paraxip.sipua.internal.SipCallEngineSMCall import *
import b2bua.B2buaCall
import paraxip.sipua.user.b2bua.B2buaStates
from paraxip.sipua.internal.ConnectionStatus import *
import paraxip.sipua.internal.SipMessageHelperFunctions
from sip_router.States import *
from sip_router.RoutingSession import RoutingSession
from SipCallEngineExtending import PySipMessage

#
# SIP Router State Machine
#

# Force declaration of class attributes in __init__ method
__pychecker__ = "initattr"

# SIP Router Call is derived from B2BUA Call
class Call(b2bua.B2buaCall.B2buaCall):
  def __init__(self):
    paraxip.sipua.user.b2bua.B2buaCall.__init__(self)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "Call.__init__")
    
    # RoutingSession is created on INVITE request
    self.routingSession = None
    
    # A dictionary containing the list of cancelled dialog, 
    # they stay in the list until they are disconnected
    self.dictOutCancelledDialog = {}

  def setSMCallLogger(self):
    self.setLogger(CallLogger("netborder.sip-router.call.py"))
  
  def addStates(self):
    # ROUTING state
    routingState = RoutingState(self)
    self.addState(routingState.getName(), routingState)

    # IDLE state (from B2BUA scenario)
    idleState = b2bua.B2buaStates.IdleState(self, routingState.getName())
    self.setInitialStateName(idleState.getName())
    self.setInitialState(idleState)

    # EARLY_MEDIA state
    earlyMediaState = EarlyMediaState(self)
    self.addState(earlyMediaState.getName(), earlyMediaState)

    # ANSWERING_CALLER state
    answeringCallerState = AnsweringCallerState(self)
    self.addState(answeringCallerState.getName(), answeringCallerState)

    # CALLER_CALLEE_CONNECTED state
    callerCalleeConnectedState = CallerCalleeConnectedState(self)
    self.addState(callerCalleeConnectedState.getName(),
                  callerCalleeConnectedState)

    # DISCONNECTING state
    disconnectingState = DisconnectingState(self)
    self.addState(disconnectingState.getName(), disconnectingState)

    # REINVITED state
    self._reinvitedState = ReinvitedState(self)
    self._reinvitedState.setSuccessStateName(
      callerCalleeConnectedState.getName())
    self._reinvitedState.setFailureStateName(
      callerCalleeConnectedState.getName())
    self._reinvitedState.setDisconnectStateName(
      disconnectingState.getName())
    self.addState(self._reinvitedState.getName(), self._reinvitedState)

    # DISCONNECTED state (from B2BUA scenario)
    disconnectedState = b2bua.B2buaStates.DisconnectedState(self)
    self.addFinalState(disconnectedState.getName(), disconnectedState)

  def initialize(self, in_refApplication, in_refGlobalContext, in_szCallId):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "Call.initialize")

    return paraxip.sipua.user.b2bua.B2buaCall.B2buaCall.initialize(self,
                                                in_refApplication,
                                                in_refGlobalContext,
                                                in_szCallId)
  
  def reset(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "Call.reset")

    self.releaseRoutingSession()

    paraxip.sipua.user.b2bua.B2buaCall.B2buaCall.reset(self)
  
  def scenarioName(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "Call.scenarioName")

    return "Netborder SIP Router"

  def tryRoute(self, in_routeToConfig):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "Call.tryRoute")

    strRequestUriUserPart = self.callerInitialInvite.getUserPart("INVITE")
    if (strRequestUriUserPart is not None):
      self.callerRequestUrl = "sip:%s@%s:%d" % (strRequestUriUserPart,
                                                in_routeToConfig.serverAddress,
                                                in_routeToConfig.serverPort)
    else:
      PyLoggingFunctions.log_error(self.getLogger(), "Cannot find the user part in URL %s",
                                  self.callerRequestUrl)
      return False

    # We generate the dialog id ourselves. This is required since we can
    # have multiple out dialog and we need to differentiate them when 
    # addOutDialog is called
    dialogId = UUID().getAsString()   
    PyLoggingFunctions.log_trace(self.getLogger(), "Call %s attempting to call %s (dialog id= %s)",
              self.getCallId(),self.callerRequestUrl, dialogId)

    # Get an initial SIP INVITE message
    inviteMsg = PySipMessage()
    if (not self.globalContext.constructInviteMessage(
          inviteMsg,
          self.getCallId(),
          self.callerRequestUrl,
          self.callerSdp,
          in_routeToConfig.localOutboundProxy,
          dialogId)):
      # Error logged by constructInviteMessage
      return False

    self.mergeCommonInviteHeaders(self.callerInitialInvite, inviteMsg)

    # Send the SIP INVITE message
    self.globalContext.makeOutCall(inviteMsg)

    self.calleeLegStatus = ConnectionStatus.Connecting
    self.calleeDialogId = dialogId

    return True
  
  def addOutDialog(self, in_szDialogId, in_sipCallEngineSMOut):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "Call.addOutDialog")
    
    SipCallEngineSMCall.addOutDialog(self, in_szDialogId, in_sipCallEngineSMOut)

  def getRoutingRejectMessage(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "Call.getRoutingRejectMessage")
    
    sipMessage = PySipMessage()
    
    # It's possible that this Dialog Id has already been released but the Call
    # did not received the DialogToCallTerminatedEvent from this Dialog yet
    inDialogSM = self.getInDialog(self.callerDialogId)
    if (inDialogSM is None):
      PyLoggingFunctions.log_info(self.getLogger(),
               "getInDialog failed because dialog id %s "
               "does not exist or has been released", self.callerDialogId)
      return None

    inDialogSM.constructRejectMessage(sipMessage)
    
    # Add custom header here
    # assert sipMessage.addCustomHeader("header name", "header value")
    
    return sipMessage

  def createNewRoutingSession(self, in_inviteRequest):
    return self.application.routingEngine.createSession(self.getCallId(),
                                                        in_inviteRequest)

  def releaseRoutingSession(self):
    if (self.routingSession is not None):
      self.application.routingEngine.releaseSession(self.getCallId())
      self.routingSession = None

## !!MIRROR FILE, DO NOT MODIFY!!
