#
#  @file   RoutingAction.py
#  @author Francis Moreau
#  date    Wed June 11 2008
#  
#  Description:
#
#  Type your description here
#
#  Copyright (C) 2008 Paraxip Technologies.
#  All Rights Reserved.
#

import netborderPyInfra
import paraxip.infra.PyLoggingFunctions as PyLoggingFunctions
from paraxip.infra.Assert import *
from sip_router.RouteSet import RouteSet
from cStringIO import StringIO

fileScopeLogger = netborderPyInfra.Logger(
  "netborder.sip-router.RoutingAction.py")

# Force declaration of class attributes in __init__ method
__pychecker__ = "initattr"

class NewResultRoutingAction:
  """
  This action happens when a new Route must be tried. Cancel the current
  outgoing Dialog if needed (e.g. timeout has been reached) and re-start with
  a new outgoing Dialog.
  """
  def __init__(self, in_routeSet):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "NewResultRoutingAction.__init__")

    self._routeSet = in_routeSet

  def getRouteSet(self):
    return self._routeSet

  def __str__(self):
    strIOResult = StringIO()

    strIOResult.write("NewResultRoutingAction members:\n")

    strIOResult.write("%s" % (self._routeSet))

    strResult = strIOResult.getvalue()
    strIOResult.close()
    return strResult

class ContinueRoutingAction:
  """
  This action happens in parallel routing when not all the results has been
  received yet. Continue waiting the other routing results.
  """
  def __init__(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "ContinueRoutingAction.__init__")

  def __str__(self):
    strIOResult = StringIO()

    strIOResult.write("ContinueRoutingAction members:\n")

    strResult = strIOResult.getvalue()
    strIOResult.close()
    return strResult

class RejectRoutingAction:
  """
  This action happens when a RejectConfigEntry entry is reached or when there
  are no more Route to try. Cancel the current outgoing Dialog if needed (e.g.
  timeout has been reached) and reject the incoming Dialog.
  """
  def __init__(self, in_rejectConfigEntry):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "RejectRoutingAction.__init__")

    self._rejectConfigEntry = in_rejectConfigEntry

  def getRejectConfigEntry(self):
    return self._rejectConfigEntry

  def __str__(self):
    strIOResult = StringIO()

    strIOResult.write("RejectRoutingAction members:\n")

    strIOResult.write("%s" % (self._rejectConfigEntry))

    strResult = strIOResult.getvalue()
    strIOResult.close()
    return strResult

class AbandonRoutingAction:
  """
  This action happens when a final response is received (e.g. 486 Busy Here).
  Reject the incoming Dialog with this final response.
  """
  def __init__(self, in_uiRejectSipErrorCode, in_szRejectSipErrorDesc):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "AbandonRoutingAction.__init__")

    self._uiRejectSipErrorCode = in_uiRejectSipErrorCode
    self._strRejectSipErrorDesc = in_szRejectSipErrorDesc

  def getRejectSipErrorCode(self):
    return self._uiRejectSipErrorCode

  def getRejectSipErrorDesc(self):
    return self._strRejectSipErrorDesc

  def __str__(self):
    strIOResult = StringIO()

    strIOResult.write("AbandonRoutingAction members:\n")

    strIOResult.write("SIP Error Code %s\n" % (self._uiRejectSipErrorCode))
    strIOResult.write("SIP Error Description %s\n" % \
                      (self._strRejectSipErrorDesc))

    strResult = strIOResult.getvalue()
    strIOResult.close()
    return strResult

## !!MIRROR FILE, DO NOT MODIFY!!
