#
#  @file   RoutingEngine.py
#  @author Francis Moreau
#  date    Wed June 11 2008
#  
#  Description:
#
#  Type your description here
#
#  Copyright (C) 2008 Paraxip Technologies.
#  All Rights Reserved.
#

import paraxip.infra.PyLoggingFunctions as PyLoggingFunctions
from paraxip.infra.Assert import *
import netborderPyInfra
from sip_router.RoutingConfig import RoutingConfig
from sip_router.RoutingSession import RoutingSession
from sip_router.RouteSet import RouteSet
from sip_router.Route import Route
from sip_router.RoutingConfigEntry import RejectConfigEntry
import sys
import random
import re # Regular Expressions

fileScopeLogger = netborderPyInfra.Logger(
  "netborder.sip-router.RoutingEngine.py")

# Force declaration of class attributes in __init__ method
__pychecker__ = "initattr"

class RoutingEngine:
  def __init__(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "RoutingEngine.__init__")

    self._routingConfig = None

    self._dictSessions = {}

  def initialize(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "RoutingEngine.initialize")

    # Initialize the random number generator
    random.seed()

    # TODO: Keep the list of servers that failed recently to prevent of
    #       re-using them for 5 seconds

    self._routingConfig = RoutingConfig()
    return self._routingConfig.initialize()

  def stop(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "RoutingEngine.stop")

  def createSession(self, in_szCallId, in_inviteRequest):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "RoutingEngine.createSession")

    if (isLogAssert(fileScopeLogger, in_szCallId not in self._dictSessions)):
      return None

    routingSession = RoutingSession(in_szCallId, in_inviteRequest, self)

    self._dictSessions[in_szCallId] = routingSession

    return routingSession

  def releaseSession(self, in_szCallId):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "RoutingEngine.releaseSession")

    if (isLogAssert(fileScopeLogger, in_szCallId in self._dictSessions)):
      return
    del self._dictSessions[in_szCallId]

  def getSessionsIterator(self):
    """
    re = RoutingEngine()
    it = re.getSessionsIterator()
    for sess in it:
      print sess
    it.close()
    """
    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "RoutingEngine.getSessionsIterator")

    if (len(self._dictSessions) > 0):
      for session in self._dictSessions.itervalues():
        yield session

  def isDestServerInactive(self, in_routeToConfig):
    # TODO: Check if the server has been put inactive. This function is also
    #       used to remove server from inactive list when its inactive time
    #       has expired. The server address with its port must be taken. And
    #       when a Local Outbound Proxy is used then it's its address and port
    #       that must be taken.
    return True

  def getApplicableRoutingEntriesPerPriority(self, in_inviteRequest):
    """
    Return a sorted list by priorities where the list's items are also lists
    containing the configuration entries that have the same priority and where
    their pattern match the incoming SIP INVITE message.
    """
    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger,
                   "RoutingEngine.getApplicableRoutingEntriesPerPriority")

    # TODO: Use isDestServerInactive method

    # Look for the configuration entries that are matching the SIP INVITE
    configEntries = None
    bIsConfigValid = False
    try:
      configEntries = self._routingConfig.routeInviteRequest(in_inviteRequest)
      bIsConfigValid = RoutingConfig.isConfigValid(configEntries)
    except:
      PyLoggingFunctions.log_error(fileScopeLogger,
                "Routing Configuration exception: %s", sys.exc_info()[0])

    if ((configEntries is None) or (not bIsConfigValid)):
      # No routing rules are matching the incoming SIP INVITE, reject the Dialog
      listResults = []
      innerListResults = [RejectConfigEntry(\
        480, "Invite message does not match any RoutingConfigEntry")]
      listResults.append(innerListResults)
      return listResults
    else:
      # Put the routing rules that match the incoming SIP INVITE in a dictionary
      # where the key is the priority and the value is a list of matching
      # configuration entries
      dictConfigEntries = {}
      for configEntry in configEntries:
        # TODO: This can fail since it's user dependent
        if (configEntry.matchInvite(in_inviteRequest)):
          if (configEntry.priority in dictConfigEntries):
            listConfigEntries = dictConfigEntries[configEntry.priority]
            listConfigEntries.append(configEntry)
          else:
            listConfigEntries = [configEntry]
            dictConfigEntries[configEntry.priority] = listConfigEntries

      # Get the keys of the dictionary and sort them
      listPrioritiesOrdered = dictConfigEntries.keys()
      listPrioritiesOrdered.sort()

      # Construct and return a sorted list by priorities where the list's items
      # are also lists containing the configuration entries that have the same
      # priority
      listResults = []
      for priority in listPrioritiesOrdered:
        listConfigEntriesPerPriority = dictConfigEntries[priority]
        for configEntry in listConfigEntriesPerPriority:
          if (issubclass(configEntry.__class__, RejectConfigEntry)):
            # If there is a RejectConfigEntry then put it in listResults and
            # return immediately
            listRejectEntry = [configEntry]
            listResults.append(listRejectEntry)
            return listResults
        # There is no RejectConfigEntry for this priority
        listResults.append(listConfigEntriesPerPriority)

      return listResults

## !!MIRROR FILE, DO NOT MODIFY!!
