#
#  @file   States.py
#  @author Francis Moreau
#  date    Mon Jun 16 2008
#  
#  Description:
#
#  Type your description here
#
#  Copyright (C) 2008 Paraxip Technologies.
#  All Rights Reserved.
#

import paraxip.sipua.user.b2bua.B2buaStates
from netborderPyInfra import *
import paraxip.infra.PyLoggingFunctions as PyLoggingFunctions
from paraxip.infra.Assert import *
from SipCallEngineExtending import *
from paraxip.sipua.internal.SipCallEngineState import *
from paraxip.sipua.internal.SipCallEngineEvent import *
from PyResipStack import *
from paraxip.sipua.internal.ConnectionStatus import *
from paraxip.sipua.internal.SMTimeoutProcessor import *
from sip_router.RoutingConfig import *
from sip_router.RoutingSession import RoutingSession
from sip_router.RoutingAction import *
from sip_router.RouteSet import RouteSet
from sip_router.Route import Route
import time
import re

# Force declaration of class attributes in __init__ method
__pychecker__ = "initattr"

#
# Call states
#

#
# Base class
#

class SipRouterState(SipCallEngineState):
  def __init__(self, in_stateMachine):
    SipCallEngineState.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "SipRouterState.__init__")

  def processCancelledDialogEvent(self, in_event):
    # This method is used to process event from formerly cancelled dialog.
    # It will return false if the event was not processed (i.e. if it is not 
    # an event from a cancelled dialog), true if it was processed.
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "processCancelledDialogEvent")
    
    if (not issubclass(in_event.__class__, DialogToCallEvent)):
      # We don't care about non dialog event here. Just return false to tell
      # the caller that this event was not processed.
      return False

    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processCancelledDialogEvent %s for dialog "
              "%s", self.__class__.__name__, in_event.__class__.__name__, 
              in_event.getDialogId())

    if ((in_event.getDialogId() == self._stateMachine.callerDialogId) or
        (in_event.getDialogId() == self._stateMachine.calleeDialogId)):
      # Not a cancelled dialog event. Just return false to tell the caller that 
      # this event was not processed.
      return False

    # If we reach this point, this means that the event must be from a 
    # previously cancelled dialog    
    if (isLogAssert(self.getLogger(),
                    in_event.getDialogId() in 
                      self._stateMachine.dictOutCancelledDialog)):
      return True

    # Dialog terminated (this is the ultimate event we were waiting for)
    if (isinstance(in_event, DialogToCallTerminatedEvent)):
      # Dialog is terminated, just remove it from the list     
      PyLoggingFunctions.log_debug(self.getLogger(), "Cancelled dialog %s terminated",
                  in_event.getDialogId())
      del self._stateMachine.dictOutCancelledDialog[in_event.getDialogId()]

    # Calling event 
    elif (isinstance(in_event, OutDialogToCallCallingEvent)):
      # Dialog is now in a proper state to be disconnected        
      dialogStatus = \
        self._stateMachine.dictOutCancelledDialog[in_event.getDialogId()]
      if (dialogStatus == ConnectionStatus.MustDisconnect):
        # Hanging up the callee
        PyLoggingFunctions.log_debug(self.getLogger(),
                  "Ready to cancel dialog %s ", in_event.getDialogId())
        calleeSM = self._stateMachine.getOutDialog(in_event.getDialogId())
        logAssert(self.getLogger(), calleeSM is not None)
        if (not self._stateMachine.hangupDialog(in_event.getDialogId())):
          return True

      self._stateMachine.dictOutCancelledDialog\
        [in_event.getDialogId()] = ConnectionStatus.Disconnecting

    # Failed event
    elif (isinstance(in_event, DialogToCallFailedEvent)):
      # DialogToCallTerminatedEvent will follow
      self._stateMachine.dictOutCancelledDialog[in_event.getDialogId()] = \
        ConnectionStatus.Disconnecting

    # Just "process" the other event type silently
    return True
       
#
# Routing state
#

class RoutingState(SipRouterState):
  def __init__(self, in_stateMachine):
    SipRouterState.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "RoutingState.__init__")
    
    # TODO: Support parallel routing... with OutDialogToCallCallingEvent and
    #       in_event.getDialogId() we should be able to match a Route with
    #       a DialogId
    self._currentRoute = None

    self._tryingTimerId = None
  
  def getName(self):
    return "ROUTING"
  
  def _startTryingTimer(self, in_iTimeoutDurationMs):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "RoutingState._startTryingTimer")

    logAssert(self.getLogger(), self._tryingTimerId is None)

    timeoutEvent = TimeoutEvent()
    timeoutCancelledEvent = TimeoutCancelledEvent()
    
    tryingTimeoutProcessor = SMTimeoutProcessor(
      self._stateMachine,
      timeoutEvent,
      timeoutCancelledEvent)

    self._tryingTimerId = self._stateMachine.globalContext.scheduleTimer(
      tryingTimeoutProcessor,
      in_iTimeoutDurationMs, 0)
    timeoutEvent.timerId = self._tryingTimerId
    timeoutCancelledEvent.timerId = self._tryingTimerId

  def _cancelTryingTimer(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "RoutingState._cancelTryingTimer")

    if (self._tryingTimerId is not None):
      self._stateMachine.globalContext.cancelTimer(self._tryingTimerId)
      self._tryingTimerId = None

  def _tryRouteSet(self, in_routeSet):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "RoutingState._tryRouteSet")

    if (isLogAssert(self.getLogger(), in_routeSet.size() > 0)):
      return False

    self._cancelTryingTimer()

    # TODO: Support parallel routing... for now take only the first Route
    #       from the RouteSet
    bResult = False
    it = in_routeSet.getIterator()
    for route in it:
      self._currentRoute = route
      if (self._stateMachine.tryRoute(self._currentRoute.getRouteToConfig())):
        # Start the timer for the reception of 180/183 or 200OK
        self._startTryingTimer(self._currentRoute.getTimeoutMs())
        bResult = True
        break
    it.close()

    return bResult

  def _cancelOutDialogs(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "RoutingState._cancelOutDialogs")
    
    if (self._stateMachine.calleeLegStatus == ConnectionStatus.Connecting):
      calleeSM = self._stateMachine.getOutDialog(
        self._stateMachine.calleeDialogId)
      if (calleeSM is not None):
        if (not self._stateMachine.hangupDialog(
              self._stateMachine.calleeDialogId)):
          return

        # Add this outgoing Dialog in the list of Cancelled Dialog 
        PyLoggingFunctions.log_debug(self.getLogger(), "Adding %s to list of cancel dialog",
                                    self._stateMachine.calleeDialogId)
        self._stateMachine.dictOutCancelledDialog\
          [self._stateMachine.calleeDialogId] = ConnectionStatus.Disconnecting

      else:
        # The SIP Dialog has not been created yet. We are between a call to 
        # makeOutCall and the call to addOutDialog. We will need to wait for
        # the dialog to be created in order to cancel it.        
        PyLoggingFunctions.log_debug(self.getLogger(),
                  "Adding %s to list of cancel dialog (must disconnect)",
                  self._stateMachine.calleeDialogId)
        logAssert(self.getLogger(),
                  self._stateMachine.calleeDialogId is not None)
        self._stateMachine.dictOutCancelledDialog\
          [self._stateMachine.calleeDialogId] = ConnectionStatus.MustDisconnect

    elif (self._stateMachine.calleeLegStatus == ConnectionStatus.Disconnecting):
      self._stateMachine.dictOutCancelledDialog\
        [self._stateMachine.calleeDialogId] = ConnectionStatus.Disconnecting

    elif ((self._stateMachine.calleeLegStatus != ConnectionStatus.Disconnected)
      and (self._stateMachine.calleeLegStatus != ConnectionStatus.Unknown)):
      # For now these are the only state that we accept for cancelling.
      PyLoggingFunctions.log_error(self.getLogger(),"Unexpected dialog connection status %s", 
                self._stateMachine.calleeLegStatus) 
      logAssert(self.getLogger(), False)

    # Before we leave the function:
    # Reset all information about the callee. We will track its state 
    # with the _stateMachine.dictOutCancelledDialog dictionary from now on
    self._stateMachine.calleeLegStatus = ConnectionStatus.Unknown
    self._stateMachine.calleeDialogId = None
    self._stateMachine.calleeSdp = None
    self._stateMachine.bCalleeAnswered = False
    self._stateMachine.uiCalleeSipErrorCode = None

  def _rejectIncomingDialog(self, in_uiSipErrorCode, in_szSipErrorDesc):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "RoutingState._rejectIncomingDialog")

    logAssert(self.getLogger(),
              self._stateMachine.callerLegStatus == ConnectionStatus.Connecting)

    rejectSipMessage = self._stateMachine.getRoutingRejectMessage()
    if (not self._stateMachine.rejectDialog(
          self._stateMachine.callerDialogId,
          in_uiSipErrorCode,
          rejectSipMessage,
          in_szSipErrorDesc)):
      return
    self._stateMachine.callerLegStatus = ConnectionStatus.Disconnecting

  def executeRoutingAction(self, in_routingAction, out_pystrNewStateName):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "RoutingState.executeRoutingAction")

    if (issubclass(in_routingAction.__class__, NewResultRoutingAction)):
      # There is a new RouteSet to try, cancel the current outgoing Dialogs if
      # needed (e.g. trying timeout has been reached) and re-start with this
      # new route

      # Cancel the outgoing Dialogs
      self._cancelOutDialogs()

      # Try the new RouteSet
      if (self._tryRouteSet(in_routingAction.getRouteSet())):
        # Stay in this state
        out_pystrNewStateName.set(self.getName())
      else:
        # Reject the incoming Dialog with 500 Server Internal Error
        self._rejectIncomingDialog(500, "Server Internal Error")

        out_pystrNewStateName.set("DISCONNECTING")

    elif (issubclass(in_routingAction.__class__, ContinueRoutingAction)):
      # Continue waiting the other routing results on outgoing Dialogs

      # Stay in this state
      out_pystrNewStateName.set(self.getName())

    elif (issubclass(in_routingAction.__class__, RejectRoutingAction)):
      # Rejecting the incoming Dialog has been requested, but start by
      # cancelling the current outgoing Dialogs if needed (e.g. trying timeout
      # has been reached)

      # Cancel the outgoing Dialogs
      self._cancelOutDialogs()

      # Reject the incoming Dialog
      rejectConfigEntry = in_routingAction.getRejectConfigEntry()
      self._rejectIncomingDialog(rejectConfigEntry.sipErrorCode,
                                 rejectConfigEntry.sipErrorDesc)

      out_pystrNewStateName.set("DISCONNECTING")

    elif (issubclass(in_routingAction.__class__, AbandonRoutingAction)):
      # TODO: Support parallel routing

      # A final response has been received (e.g. 486 Busy Here), then cancel
      # the other outgoing Dialogs if needed and reject the incoming Dialog
      # with this final response

      # Cancel the outgoing Dialogs
      self._cancelOutDialogs()

      # Reject the incoming Dialog
      self._rejectIncomingDialog(in_routingAction.getRejectSipErrorCode(),
                                 in_routingAction.getRejectSipErrorDesc())

      out_pystrNewStateName.set("DISCONNECTING")

    else:
      # TODO: PyLoggingFunctions.log_warn...

      # Reject the incoming Dialog with 500 Server Internal Error
      self._rejectIncomingDialog(500, "Server Internal Error")

      out_pystrNewStateName.set("DISCONNECTING")

  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    # Create a new routing session
    self._stateMachine.routingSession = \
      self._stateMachine.createNewRoutingSession(in_event.getSipMessage())

    # Get the first routing action to try
    routingAction = self._stateMachine.routingSession.route()

    # Execute the routing action
    self.executeRoutingAction(routingAction, out_pystrNewStateName)

    # out_pystrNewStateName has already been set in executeRoutingAction
  
  def processEvent(self, in_event, out_pystrNewStateName):
    __pychecker__ = 'maxreturns=0 maxlines=0'
    
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    out_pystrNewStateName.set(self.getName())

    # Process cancelled dialog event first
    if (self.processCancelledDialogEvent(in_event)):
      return True

    if (isinstance(in_event, OutDialogToCallCallingEvent)):
      # Stay in the same state
      return True
    
    elif (isinstance(in_event, DialogToCallFailedEvent)): #OUT
      # TODO: Support parallel routing... multiple routes could failed. For
      # now we could set calleeLegStatus since there only one route at a time.
      self._stateMachine.calleeLegStatus = ConnectionStatus.Disconnecting
      
      if (isLogAssert(self.getLogger(), self._currentRoute is not None)):
        # Reject the incoming Dialog with 500 Server Internal Error
        self._rejectIncomingDialog(500, "Server Internal Error")
        out_pystrNewStateName.set("DISCONNECTING")
        return True

      # TODO: PyLoggingFunctions.log_info

      # Get the next routing action to try
      routingAction = self._stateMachine.routingSession.onRouteFailure(
        self._currentRoute,
        in_event.getSipMessage())

      # Execute the routing action
      self.executeRoutingAction(routingAction, out_pystrNewStateName)

      # out_pystrNewStateName has already been set in executeRoutingAction
      return True
    
    elif (isinstance(in_event, TimeoutEvent)):
      # TODO: Support parallel routing... multiple routes could failed. For
      # now we could use _currentRoute since there only one route at a time.

      if (isLogAssert(self.getLogger(), self._currentRoute is not None)):
        # Reject the incoming Dialog with 500 Server Internal Error
        self._rejectIncomingDialog(500, "Server Internal Error")
        out_pystrNewStateName.set("DISCONNECTING")
        return True

      # TODO: PyLoggingFunctions.log_info
      
      # Cancel the timer since a new timer could be required and be assigned
      # to _tryingTimerId
      self._cancelTryingTimer()

      # Get the next routing action to try
      routingAction = self._stateMachine.routingSession.onRouteTimeout(
        self._currentRoute)

      # Execute the routing action
      self.executeRoutingAction(routingAction, out_pystrNewStateName)

      # out_pystrNewStateName has already been set in executeRoutingAction
      return True
    
    elif (isinstance(in_event, OutDialogToCallEarlyMediaEvent)):
      # Stop the routing on 180 or 183 since the destination has been reached
      self._cancelTryingTimer()

      # Early SDP from the callee, check if we got an SDP offer from the 
      #   caller yet (ie INVITE with SDP)
      if(self._stateMachine.callerSdp is None):
        # No, callee UAS is not compliant, warn and disconnect
        PyLoggingFunctions.log_warn(self.getLogger(), "Received 183 Session Progress SIP response "\
            "following an INVITE request without an SDP offer, which is not "\
            "compliant to RFC 3261. Disconnecting the call.")
        out_pystrNewStateName.set("DISCONNECTING")
        return True

      # Save the callee SDP answered
      self._stateMachine.calleeSdp = in_event.getSdpContents()
      
      # Send 183 early media to the caller
      if (not self._stateMachine.sendEarlyMediaToInDialog(
            self._stateMachine.callerDialogId,
            self._stateMachine.calleeSdp)):
        return False
      
      out_pystrNewStateName.set("EARLY_MEDIA")
      return True
    
    elif (isinstance(in_event, OutDialogToCallProvisionalEvent)):
      # Received a provisional response from target
      # Bug 1384: All provisional responses must stop route hunting
      self._cancelTryingTimer()

      # Relay provisional response to caller
      if (not self._stateMachine.sendProvisionalToInDialog(
          self._stateMachine.callerDialogId,
          in_event.getSipMessage().getStatusCode(),
          in_event.getSipMessage().getReasonText())):
        return False
      
      # Stay in the same state
      return True
    
    # TODO lot of overlap in following lines with DialingCallee B2BUA state
    elif (isinstance(in_event, DialogToCallSdpAnsweredEvent) and
          (in_event.getDialogId() == self._stateMachine.calleeDialogId)): #OUT
      # Callee has been invited with SDP
      # Save the callee SDP answered
      self._stateMachine.calleeSdp = in_event.getSdpContents()
      
      # Stay in the same state since DialogToCallConnectedEvent is coming
      return True
    
    elif (isinstance(in_event, DialogToCallSdpOfferedEvent) and
          (in_event.getDialogId() == self._stateMachine.calleeDialogId)): #OUT
      # Callee has been invited without SDP
      # Save the callee SDP offered
      self._stateMachine.calleeSdp = in_event.getSdpContents()
      
      # Stay in the same state since DialogToCallConnectedEvent is coming
      return True

    elif (isinstance(in_event, DialogToCallConnectedEvent)):
      # Stop the routing 200OK since the destination has been reached
      self._cancelTryingTimer()

      self._stateMachine.calleeLegStatus = ConnectionStatus.Connected
      self._stateMachine.bCalleeAnswered = True
      
      # Answer the caller with the callee SDP
      if (not self._stateMachine.answerInDialog(
            self._stateMachine.callerDialogId,
            self._stateMachine.calleeSdp,
            not self._stateMachine.bInvitedWithSdp, # SDP is an offer is we were
                                                    # invited without SDP
            self._stateMachine.getAnswerMessage(in_event.getSipMessage()))):
        return False
      self._stateMachine.callerLegStatus = ConnectionStatus.Answering
      
      out_pystrNewStateName.set("ANSWERING_CALLER")
      return True
    
    elif (isinstance(in_event, DialogToCallTerminatedEvent)):
      if (in_event.getDialogId() == self._stateMachine.callerDialogId):
        self._stateMachine.dialogIdInitiatingDisconnect = in_event.getDialogId()

        self._cancelOutDialogs()

        out_pystrNewStateName.set("DISCONNECTING")
        return True
      else:
        # A route that has previously failed or timed out is now terminated.
        # Remove it from the list of Dialogs that are in process of being
        # disconnected
        del self._stateMachine.dictOutCancelledDialog[in_event.getDialogId()]

      # Stay in the same state
      return True
    
    elif (isinstance(in_event, StopEvent)):
      out_pystrNewStateName.set("DISCONNECTING")
      return True

    else:
      PyLoggingFunctions.log_warn(self.getLogger(),
               "%s::processEvent received an invalid event %s",
               self.__class__.__name__, in_event.__class__.__name__)
    
    return True
  
  def onStateExit(self, in_event, in_strDestStateName):
    __pychecker__ = 'unusednames=in_strDestStateName'

    PyLoggingFunctions.log_trace(self.getLogger(), "%s::onStateExit %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    self._cancelTryingTimer()

#
# EarlyMedia state
#
# NOTE: This class is derived from a B2BUA state
#

class EarlyMediaState(SipRouterState,
                      b2bua.B2buaStates.EarlyMediaState):
  def __init__(self, in_stateMachine):
    b2bua.B2buaStates.EarlyMediaState.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "EarlyMediaState.__init__")
  
  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    out_pystrNewStateName.set(self.getName())

    # Process cancelled dialog event first
    if(self.processCancelledDialogEvent(in_event)):
      return True    
    else:
      # processEvent from based class (B2BUA)
      return b2bua.B2buaStates.EarlyMediaState.processEvent(
        self, in_event, out_pystrNewStateName)
        

#
# AnsweringCaller state
#
# NOTE: This class is derived from a B2BUA state
#

class AnsweringCallerState(SipRouterState,
                           b2bua.B2buaStates.AnsweringCallerState):
  def __init__(self, in_stateMachine):
    b2bua.B2buaStates.AnsweringCallerState.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "AnsweringCallerState.__init__")
  
  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    out_pystrNewStateName.set(self.getName())

    # Process cancelled dialog event first
    if(self.processCancelledDialogEvent(in_event)):
      return True    
    else:
      # processEvent from based class (B2BUA)
      return b2bua.B2buaStates.AnsweringCallerState.processEvent(
        self, in_event, out_pystrNewStateName)
        

#
# CallerCalleeConnected state
#
# NOTE: This class is derived from a B2BUA state
#

class CallerCalleeConnectedState(SipRouterState,
                                 b2bua.B2buaStates.CallerCalleeConnectedState):
  def __init__(self, in_stateMachine):
    b2bua.B2buaStates.CallerCalleeConnectedState.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "CallerCalleeConnectedState.__init__")
  
  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    out_pystrNewStateName.set(self.getName())

    # Process cancelled dialog event first
    if(self.processCancelledDialogEvent(in_event)):
      return True    
    else:
      # processEvent from based class (B2BUA)
      return b2bua.B2buaStates.CallerCalleeConnectedState.processEvent(
        self, in_event, out_pystrNewStateName)
    
    return True

#
# Reinvited state
#
# NOTE: This class is derived from a B2BUA state
#

class ReinvitedState(SipRouterState,
                     b2bua.B2buaStates.ReinvitedState):
  def __init__(self, in_stateMachine):
    b2bua.B2buaStates.ReinvitedState.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "ReinvitedState.__init__")
  
  def processEvent(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    out_pystrNewStateName.set(self.getName())

   # Process cancelled dialog event first
    if(self.processCancelledDialogEvent(in_event)):
      return True    
    else:        
      # processEvent from based class (B2BUA)
      return b2bua.B2buaStates.ReinvitedState.processEvent(
        self, in_event, out_pystrNewStateName)
        
#
# Disconnecting state
#
# NOTE: This class is derived from a B2BUA state
#

class DisconnectingState (SipRouterState,
                         b2bua.B2buaStates.DisconnectingState):
  def __init__(self, in_stateMachine):
    b2bua.B2buaStates.DisconnectingState.__init__(self, in_stateMachine)
    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(self.getLogger(), "DisconnectingState.__init__")
    self._bB2BUAReadyToGoInDisconnectedState = False
  
  def entryAction(self, in_event, out_pystrNewStateName):
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::entryAction %s",
              self.__class__.__name__, in_event.__class__.__name__)
   
    # Reset boolean
    self._bB2BUAReadyToGoInDisconnectedState = False

    # Call parent entryAction
    b2bua.B2buaStates.DisconnectingState.entryAction(self,
                                                     in_event,
                                                     out_pystrNewStateName)
    # Note: Cannot write out_pystrNewStateName == "DISCONNECTED" as
    # out_pystrNewStateName is not a string but a pyString
    if (str(out_pystrNewStateName) == "DISCONNECTED"):
      # Keep track of the fact that the b2bua is ready to go in disconnected
      PyLoggingFunctions.log_debug(self.getLogger(),"B2BUA is ready to go in Disconnected.")
      self._bB2BUAReadyToGoInDisconnectedState = True

    PyLoggingFunctions.log_debug(self.getLogger(), "List of dialog still cancelling %s",
              self._stateMachine.dictOutCancelledDialog)

    if (self._bB2BUAReadyToGoInDisconnectedState and
        (len(self._stateMachine.dictOutCancelledDialog) == 0)):
      PyLoggingFunctions.log_debug(self.getLogger(),"All dialogs already disconnected.\
                Ready to go in Disconnected")     
      out_pystrNewStateName.set("DISCONNECTED")
    else:
      # Stay in this state until all dialogs are disconnected      
      out_pystrNewStateName.set(self.getName())
      PyLoggingFunctions.log_debug(self.getLogger(),"Waiting for all dialogs to disconnect")
    
  def processEvent(self, in_event, out_pystrNewStateName):
    t = TraceScope(self.getLogger(), "DisconnectingState.processEvent")
    PyLoggingFunctions.log_trace(self.getLogger(), "%s::processEvent %s",
              self.__class__.__name__, in_event.__class__.__name__)
    
    out_pystrNewStateName.set(self.getName())

    # Process cancelled dialog event first
    if (not self.processCancelledDialogEvent(in_event)):
      # Not a cancelled dialog event, try the B2BUA          
      b2bua.B2buaStates.DisconnectingState.processEvent(
        self, in_event, out_pystrNewStateName)
      
      if (str(out_pystrNewStateName) == "DISCONNECTED"):
        PyLoggingFunctions.log_debug(self.getLogger(),"B2BUA is finally ready to go in "
                  "Disconnected.")
        self._bB2BUAReadyToGoInDisconnectedState = True

    # Are we done?
    if (self._bB2BUAReadyToGoInDisconnectedState and
        len(self._stateMachine.dictOutCancelledDialog) == 0): 
      PyLoggingFunctions.log_debug(self.getLogger(),"All dialogs have been disconnected"
                "Ready to go in Disconnected")     
      out_pystrNewStateName.set("DISCONNECTED")
    else:
      PyLoggingFunctions.log_debug(self.getLogger(),"Still waiting for some dialogs to disconnect")                
      out_pystrNewStateName.set(self.getName())
    
    return True

## !!MIRROR FILE, DO NOT MODIFY!!
